unit e_OverlayGeometryElement;

interface
uses
   e_GeometryElement,
   Elements;

type

TOverlayGeometryElement = class(TGeometryElement)
  private
    FA1: TElement;
    FA2: TElement;
    procedure SetA1(const Value: TElement);
    procedure SetA2(const Value: TElement);
    function GetIsA1: integer;
    function GetIsA2: integer;
    function GetisEmpty: boolean;
    function GetIsFull: boolean;
    function GetArea: double;
    function GetFixedArea: double;
    function GetFixedAreaIndex: integer;
public
   function   HasPropertyByName(aName: string): boolean; override;
   function   GetPropertyStringByName(AName: string): string; override;
   procedure  SetPropertyStringByName(AName: string; aValue: string); override;
   function   GetClearPropertyName(const aName: string; var PreName: string): string;
published
   property A1: TElement read FA1 write SetA1 stored false;
   property A2: TElement read FA2 write SetA2 stored false;
   property IsA1: integer read GetIsA1 stored false;
   property IsA2: integer read GetIsA2 stored false;
   property IsEmpty: boolean read GetisEmpty stored false;
   property IsFull: boolean read GetIsFull stored false;
   property Area: double read GetArea stored false;
   property FixedArea: double read GetFixedArea stored false;
   property FixedAreaIndex: integer read GetFixedAreaIndex stored false;
end;

implementation
uses
   e_Class,
   e_Geometry;

{ TOverlayGeometryElement }

function TOverlayGeometryElement.GetArea: double;
begin
   result := 0;
   if Geometry <> nil then
   if Geometry is TAreaElement then
      result := TAreaElement(Geometry).Area;
end;

function TOverlayGeometryElement.GetClearPropertyName(const aName: string; var PreName: string): string;
var
   S: string;
   ls: integer;
begin
   result := aName;
   PreName := '';
   ls := Length(aName);
   if ls < 3 then exit;
   S := Copy(aName,1,3);
   if not (S[1] in ['A','G']) then exit;
   if not (S[2] in ['1','2']) then exit;
   if not (S[3]='_') then exit;
   PreName := S;
   result := Copy(aName,4,ls-3);
end;

function TOverlayGeometryElement.GetFixedArea: double;
begin
   result := 0;
   if Geometry <> nil then
   if Geometry is TAreaElement then
      result := TAreaElement(Geometry).FixedArea;
end;

function TOverlayGeometryElement.GetFixedAreaIndex: integer;
begin
   result := 0;
   if Geometry <> nil then
   if Geometry is TAreaElement then
      result := TAreaElement(Geometry).FixedAreaIndex;
end;

function TOverlayGeometryElement.GetIsA1: integer;
begin
   result := 0;
   if A1 <> nil then
      result := 1;
end;

function TOverlayGeometryElement.GetIsA2: integer;
begin
   result := 0;
   if A2 <> nil then
      result := 1;
end;

function TOverlayGeometryElement.GetisEmpty: boolean;
begin
   result := (A1 = nil) and (A2 = nil);
end;

function TOverlayGeometryElement.GetIsFull: boolean;
begin
   result := (A1 <> nil) and (A2 <> nil);
end;

function TOverlayGeometryElement.GetPropertyStringByName(AName: string): string;
var
   S1,S2: string;
   GE: TGeometryElement;
   G: TGeometry;
begin
   S2 := GetClearPropertyName(aName,S1);
   result := '';
   if S1 = '' then
   begin
      result := inherited GetPropertyStringByName(aName);
   end
   else
   if S1 = 'A1_' then
   begin
      if A1 <> nil then
         result := A1.GetPropertyStringByName(S2);
   end
   else
   if S1 = 'A2_' then
   begin
      if A2 <> nil then
         result := A2.GetPropertyStringByName(S2);
   end
   else
   if S1 = 'G1_' then
   begin
      if (A1 <> nil) and (A1 is TGeometryElement) then
      begin
         GE := TGeometryElement(A1);
         G := GE.GetSimpleGeometryByXY(X,Y,0);
         if G <> nil then
            result := G.GetPropertyStringByName(S2);
      end;
   end
   else
   if S1 = 'G2_' then
   begin
      if (A2 <> nil) and (A2 is TGeometryElement) then
      begin
         GE := TGeometryElement(A2);
         G := GE.GetSimpleGeometryByXY(X,Y,0);
         if G <> nil then
            result := G.GetPropertyStringByName(S2);
      end;
   end;
end;

function TOverlayGeometryElement.HasPropertyByName(aName: string): boolean;
var
   S1,S2: string;
   GE: TGeometryElement;
   G: TGeometry;
begin
   S2 := GetClearPropertyName(aName,S1);
   result := false;
   if S1 = '' then
   begin
      result := inherited HasPropertyByName(aName);
   end
   else
   if S1 = 'A1_' then
   begin
      if A1 <> nil then
         result := A1.HasPropertyByName(S2);
   end
   else
   if S1 = 'A2_' then
   begin
      if A2 <> nil then
         result := A2.HasPropertyByName(S2);
   end
   else
   if S1 = 'G1_' then
   begin
      if (A1 <> nil) and (A1 is TGeometryElement) then
      begin
         GE := TGeometryElement(A1);
         G := GE.GetSimpleGeometryByXY(X,Y,0);
         if G <> nil then
            result := G.HasPropertyByName(S2);
      end;
   end
   else
   if S1 = 'G2_' then
   begin
      if (A2 <> nil) and (A2 is TGeometryElement) then
      begin
         GE := TGeometryElement(A2);
         G := GE.GetSimpleGeometryByXY(X,Y,0);
         if G <> nil then
            result := G.HasPropertyByName(S2);
      end;
   end;
end;

procedure TOverlayGeometryElement.SetA1(const Value: TElement);
begin
   ExchangeFieldElement(TElement(FA1),Value);
end;

procedure TOverlayGeometryElement.SetA2(const Value: TElement);
begin
   ExchangeFieldElement(TElement(FA2),Value);
end;


procedure TOverlayGeometryElement.SetPropertyStringByName(AName,
  aValue: string);
begin
  inherited;

end;

INITIALIZATION
   RegisterOrbisClass(TOverlayGeometryElement);
END.

