unit e_TileLayer;

interface
uses
   G_Elems;

type

TGis_OrbisTilesMatrix = class(TRectEntity)
  private
    FMatrixWidth: integer;
    FTileHeight: double;
    FMatrixHeight: integer;
    FOverX: double;
    FOverY: double;
    FMatrix_ID: string;
    FTileWidth: double;
    FScale: double;
    procedure SetMatrix_ID(const Value: string);
    procedure SetMatrixHeight(const Value: integer);
    procedure SetMatrixWidth(const Value: integer);
    procedure SetOverX(const Value: double);
    procedure SetOverY(const Value: double);
    procedure SetScale(const Value: double);
    procedure SetTileHeight(const Value: double);
    procedure SetTileWidth(const Value: double);
published
   property Matrix_ID: string read FMatrix_ID write SetMatrix_ID;
   property Scale: double read FScale write SetScale;
   property TileWidth: double read FTileWidth write SetTileWidth;
   property TileHeight: double read FTileHeight write SetTileHeight;
   property OverX: double read FOverX write SetOverX;
   property OverY: double read FOverY write SetOverY;
   property MatrixWidth: integer read FMatrixWidth write SetMatrixWidth;
   property MatrixHeight: integer read FMatrixHeight write SetMatrixHeight;
end;

TGis_OrbisTilesLayer = class(TRectEntity)
  private
    FProject: string;
    FRegistered: boolean;
    FMapName: String;
    FSubject: string;
    FMapDir: string;
    FPlocha: integer;
    FError: integer;
    FMapID: string;
    Fuser: string;
    FExt: string;
    function GetTileMatrix(aIndex: integer): TGis_OrbisTilesMatrix;
    procedure SetError(const Value: integer);
    procedure SetExt(const Value: string);
    procedure SetMapDir(const Value: string);
    procedure SetMapID(const Value: string);
    procedure SetMapName(const Value: String);
    procedure SetPlocha(const Value: integer);
    procedure SetProject(const Value: string);
    procedure SetReadDefinition(const Value: boolean);
    procedure SetRegistered(const Value: boolean);
    procedure SetSubject(const Value: string);
    procedure Setuser(const Value: string); // orig Map in Karkulka
public
   property TileMatrix[aIndex: integer]: TGis_OrbisTilesMatrix read GetTileMatrix;
published
   property MapName: String read FMapName write SetMapName;
   property Ext: string read FExt write SetExt;
   property MapDir: string read FMapDir write SetMapDir;
   property MapID: string read FMapID write SetMapID;
   property Subject: string read FSubject write SetSubject;
   property Project: string read FProject write SetProject;
   property user: string read Fuser write Setuser;
   property Plocha: integer read FPlocha write SetPlocha;
   property Error: integer read FError write SetError;
   property Registered: boolean read FRegistered write SetRegistered;
   property ReadDefinition: boolean Write SetReadDefinition stored false; // direct? or separated tool?
end;

implementation
uses
   System.IOUtils,
   Xml.XMLIntf,
   Xml.XMLDoc,
   SysUtils,
   Elements,
   u_String,
   e_Class;

{ TGis_OrbisTilesMatrix }

procedure TGis_OrbisTilesMatrix.SetMatrixHeight(const Value: integer);
begin
  FMatrixHeight := Value;
end;

procedure TGis_OrbisTilesMatrix.SetMatrixWidth(const Value: integer);
begin
  FMatrixWidth := Value;
end;

procedure TGis_OrbisTilesMatrix.SetMatrix_ID(const Value: string);
begin
  FMatrix_ID := Value;
end;

procedure TGis_OrbisTilesMatrix.SetOverX(const Value: double);
begin
  FOverX := Value;
end;

procedure TGis_OrbisTilesMatrix.SetOverY(const Value: double);
begin
  FOverY := Value;
end;

procedure TGis_OrbisTilesMatrix.SetScale(const Value: double);
begin
  FScale := Value;
end;

procedure TGis_OrbisTilesMatrix.SetTileHeight(const Value: double);
begin
  FTileHeight := Value;
end;

procedure TGis_OrbisTilesMatrix.SetTileWidth(const Value: double);
begin
  FTileWidth := Value;
end;

{ TGis_OrbisTilesLayer }

function TGis_OrbisTilesLayer.GetTileMatrix(aIndex: integer): TGis_OrbisTilesMatrix;
var
   e: TElement;
begin
   result := nil;
   e := Elements[aIndex];
   if not (e is TGis_OrbisTilesMatrix) then
      exit;
   result := TGis_OrbisTilesMatrix(e);
end;

procedure TGis_OrbisTilesLayer.SetError(const Value: integer);
begin
  FError := Value;
end;

procedure TGis_OrbisTilesLayer.SetExt(const Value: string);
begin
  FExt := Value;
end;

procedure TGis_OrbisTilesLayer.SetMapDir(const Value: string);
begin
  FMapDir := IncludeTrailingPathDelimiter(Value);
end;

procedure TGis_OrbisTilesLayer.SetMapID(const Value: string);
begin
  FMapID := Value;
end;

procedure TGis_OrbisTilesLayer.SetMapName(const Value: String);
begin
  FMapName := Value;
end;

procedure TGis_OrbisTilesLayer.SetPlocha(const Value: integer);
begin
  FPlocha := Value;
end;

procedure TGis_OrbisTilesLayer.SetProject(const Value: string);
begin
  FProject := Value;
end;

procedure TGis_OrbisTilesLayer.SetReadDefinition(const Value: boolean);
var
   S,SS: string;
   XMLDOcument: IXMLDocument;//Xml.XMLDoc.TXMLDocument;
   N, TMS: Xml.XMLIntf.IXMLNode;
   MXCount, ii: integer;
   MXRec: TGis_OrbisTilesMatrix;
begin
   FreeAll;
   S := MapDir + 'MapTiles.xml';
   if not System.IOUtils.TFile.Exists(S) then // a pokud neni def soubor.....
   begin
       Error := 1;
       exit;
   end;
   // cteme dataset
   XMLDOcument := Xml.XMLDoc.TXMLDocument.Create(nil);
   XMLDocument.LoadFromFile(S);
//   XMLDOcument.Active := false;
//   XMLDocument.FileName := S;
//   XMLDocument.Active := true;
   N := XMLDocument.DocumentElement;
   //ReadMap
   SS := N.ChildNodes['name'].Text;
   MapName := N.ChildNodes['name'].Text;
   Ext := N.ChildNodes['ext'].Text;
//   if aLocal then
//      MapRec.MapDir := LocalMapDir
//   else
//      MapRec.MapDir := IncludeTrailingPathDelimiter(MapDirectory+ N.ChildNodes['dir'].Text);

   MinX := 1E27;
   MinY := 1E27;
   MaxX := -1E27;
   MaxY := -1E27;
   TMS := N.ChildNodes['TileMatrixSet'];
   MXCount := TMS.ChildNodes.Count;
   for ii := 0 to MXCount - 1 do     // tilematrix
   begin
      N := TMS.ChildNodes[ii];
      MXRec := TGis_OrbisTilesMatrix.Create;
      AddElement(MXRec);
      MXRec.Matrix_ID := N.ChildNodes['ID'].Text;
      MXRec.Name := MXRec.Matrix_ID;
      MXRec.MinX := StrToIntDef(N.ChildNodes['MinX'].Text,0);
      MXRec.MinY := StrToIntDef(N.ChildNodes['MinY'].Text,0);
      MXRec.TileWidth := StrToIntDef(N.ChildNodes['TileWidth'].Text,0);
      MXRec.TileHeight := StrToIntDef(N.ChildNodes['TileHeight'].Text,0);
      MXRec.OverX := StrToExtended(N.ChildNodes['OverX'].Text);
      MXRec.OverY := StrToExtended(N.ChildNodes['OverY'].Text);
      MXRec.MatrixWidth := StrToIntDef(N.ChildNodes['MatrixWidth'].Text,0);
      MXRec.MatrixHeight := StrToIntDef(N.ChildNodes['MatrixHeight'].Text,0);
      MXRec.MaxX := MXRec.MinX + ((MXRec.MatrixWidth * MXRec.TileWidth) - ((MXRec.MatrixWidth - 1) * MXRec.OverX));
      MXRec.MaxY := MXRec.MinY + ((MXRec.MatrixHeight * MXRec.TileHeight) - ((MXRec.MatrixHeight - 1) * MXRec.OverY));
      UpdateByElement := MXRec;
   end;
   Error := 0;
   Registered := true;
   //---
//   XMLDOcument.Free;
end;

procedure TGis_OrbisTilesLayer.SetRegistered(const Value: boolean);
begin
  FRegistered := Value;
end;

procedure TGis_OrbisTilesLayer.SetSubject(const Value: string);
begin
  FSubject := Value;
end;

procedure TGis_OrbisTilesLayer.Setuser(const Value: string);
begin
  Fuser := Value;
end;

initialization
   RegisterOrbisClass(TGis_OrbisTilesLayer);
   RegisterOrbisClass(TGis_OrbisTilesMatrix);
end.
