unit f_TM_Editor;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, f_OrbisForm, ExtCtrls, ComCtrls, ImgList, System.ImageList;

type
  TTematicMap_Editor_Form = class(TORBISForm)
    MapList: TListView;
    Splitter1: TSplitter;
    Panel1: TPanel;
    Panel_Props: TPanel;
    Splitter2: TSplitter;
    ListLayers: TListView;
    ImageList1: TImageList;
    ImageList2: TImageList;
    ColorDialog: TColorDialog;
    procedure MapListSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure ListLayersChanging(Sender: TObject; Item: TListItem;
      Change: TItemChange; var AllowChange: Boolean);
    procedure ListLayersChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure ListLayersMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ListLayersCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure ListLayersCustomDrawSubItem(Sender: TCustomListView;
      Item: TListItem; SubItem: Integer; State: TCustomDrawState;
      var DefaultDraw: Boolean);
    procedure MapListCustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure ListLayersDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure ListLayersDragDrop(Sender, Source: TObject; X, Y: Integer);
  private
    FCHangedItem: TListItem;
    FChangedItemChecked: boolean;
    FIgnoreChanges: boolean;
    procedure SetIgnoreChanges(const Value: boolean);
  public
    property  IgnoreChanges: boolean read FIgnoreChanges write SetIgnoreChanges;
  end;


implementation
uses
   Elements,
   e_ThematicMap;


{$R *.dfm}

procedure TTematicMap_Editor_Form.MapListSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
var
   Map: TThematicMap;
   TL: TThematicLayer;
   i: integer;
   L,SL: TListItem;
   E: TElement;
   S: string;
begin
//   if IgnoreChanges then exit;
   Map := TThematicMap(Item.Data);
   IgnoreChanges := true;
   ListLayers.Items.BeginUpdate;
   ListLayers.ViewStyle := vsList;
   ListLayers.Items.Clear;
   //ListLayers.Column[0].Width := -2;
   SL := nil;
   for i := 0 to Map.LastIndex do
   begin
      L := ListLayers.Items.Add;
      E := Map.Elements[i];
      TL := TThematicLayer(E);
      L.Data := E;
      L.Caption := E.Name;
      S := UpperCase(Copy(E.Name,1,3));
      if TL.AsText then
         L.ImageIndex := 5
      else if (S = 'LIN') or (S = 'SEL') or (S = 'FOU') then
         L.ImageIndex := 1
      else if S = 'ARE' then
         L.ImageIndex := 2
      else if S = 'PNT' then
         L.ImageIndex := 3
      else if S = 'TXT' then
         L.ImageIndex := 4
      else if S = 'RAS' then
         L.ImageIndex := 6
      else
         L.ImageIndex := 0;
      E := TL.FindElementClass(TElement);
      if (E = nil) or (TEntity(E).Count = 0) or TL.SwitchedOff then
         L.ImageIndex := L.ImageIndex + 7;
      L.Checked := not (TL.SwitchedOff);
      //L.StateIndex := 0;
   end;
   ListLayers.ViewStyle := vsReport;
   ListLayers.Items.EndUpdate;
   if Visible then
      IgnoreChanges := false;
   //Form.AddCaption := AddCaption;
end;

procedure TTematicMap_Editor_Form.MapListCustomDrawItem(
  Sender: TCustomListView; Item: TListItem; State: TCustomDrawState;
  var DefaultDraw: Boolean);
begin
   if Item.Selected then
      MapList.Canvas.Font.Style := ListLayers.Canvas.Font.Style + [fsBold]
   else
      MapList.Canvas.Font.Style := ListLayers.Canvas.Font.Style - [fsBold];
end;

procedure TTematicMap_Editor_Form.ListLayersChanging(Sender: TObject;
  Item: TListItem; Change: TItemChange; var AllowChange: Boolean);
begin
   if IgnoreChanges then exit;
   if not (CHange = ctState) then exit;
   FChangedItem := Item;
   FChangedItemChecked := Item.Checked;
end;

procedure TTematicMap_Editor_Form.ListLayersCustomDrawItem(
  Sender: TCustomListView; Item: TListItem; State: TCustomDrawState;
  var DefaultDraw: Boolean);
var
   TL: TThematicLayer;
   E: TEntity;
begin
   if Item.Data = nil then
      ListLayers.Canvas.Font.Color := 255
   else
   begin
      TL := TThematicLayer(Item.Data);
      E := TEntity(TL.FindElementClass(TElement));
      if E = nil then
         ListLayers.Canvas.Font.Color := 255
      else
      if E.Count = 0 then
         ListLayers.Canvas.Font.Color := $AAAAAA
      else
         ListLayers.Canvas.Font.Color := clBlack;
      if Item.Selected then
         ListLayers.Canvas.Font.Style := ListLayers.Canvas.Font.Style + [fsBold]
      else
         ListLayers.Canvas.Font.Style := ListLayers.Canvas.Font.Style - [fsBold];
   end;
   SetBkMode( ListLayers.Canvas.Handle, TRANSPARENT );
//   DefaultDraw := false;
end;


procedure TTematicMap_Editor_Form.ListLayersCustomDrawSubItem(
  Sender: TCustomListView; Item: TListItem; SubItem: Integer;
  State: TCustomDrawState; var DefaultDraw: Boolean);
var
   TL: TThematicLayer;
   R: TRect;
   ri: integer;
begin
   if SubItem = 1 then
   begin
      TL := TThematicLayer(Item.Data);
      if TL.UseColor then
         ri := 2
      else
         ri := 7;
      R := Item.DisplayRect(drBounds);
      R.Left := R.Left + ListLayers.Column[0].Width + ri;
      R.Top := R.Top + ri;
      R.Right := R.Left + 25 - ri *2;
      R.Bottom := R.Bottom -ri;
//      ListLayers.Canvas.Brush.Style := bsSolid;
      ListLayers.Canvas.Brush.Color := TL.Color;
      ListLayers.Canvas.Rectangle(R);
      ListLayers.Canvas.Brush.Color := ListLayers.Color;
//      ListLayers.Canvas.Brush.Style := bsClear;
      DefaultDraw := false;
   end;
end;

procedure TTematicMap_Editor_Form.ListLayersDragDrop(Sender,
  Source: TObject; X, Y: Integer);
var
  DragItem, DropItem, CurrentItem, NextItem: TListItem;
  Map: TThematicMap;
  CurrentE,DropE: TElement;
begin
  if (Sender = Source) and (Sender = ListLayers) then
  begin
      if MapList.Selected <> nil then
         Map := TThematicMap(MapList.Selected.Data)
      else
         exit;
      DropItem    := ListLayers.GetItemAt(X, Y);
      CurrentItem := ListLayers.Selected;
      if DropItem <> nil then
         DropE := TElement(DropItem.Data)
      else
         DropE := nil;
      CurrentE := TElement(CurrentItem.Data);
      Map.RemoveElement(CurrentE);
//      NextItem := ListLayers.GetNextItem(CurrentItem, SdAll, [IsSelected]);
      if DropItem = nil then
      begin
         DragItem := ListLayers.Items.Add;
         Map.AddElement(CurrentE);
      end
      else
      begin
        DragItem := ListLayers.Items.Insert(DropItem.Index);
        Map.InsertElement(Map.IndexOf(DropE),CurrentE);
      end;
      DragItem.Assign(CurrentItem);
      CurrentItem.Free;
  end;
end;

procedure TTematicMap_Editor_Form.ListLayersDragOver(Sender,
  Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
//   inherited;
   Accept := Sender = ListLayers;
end;

procedure TTematicMap_Editor_Form.ListLayersChange(Sender: TObject;
  Item: TListItem; Change: TItemChange);
var
   TL: TThematicLayer;
begin
   if IgnoreChanges then exit;
   if not (CHange = ctState) then exit;
   if FChangedItem <> Item then exit;
   if FChangedItemChecked = Item.Checked then exit;
   TL := TThematicLayer(Item.Data);
   if TL = nil then exit;
   TL.SwitchedOff := not Item.Checked;
   if Item.Checked then
   begin
      if Item.ImageIndex > 7 then
         Item.ImageIndex := Item.ImageIndex - 7;
   end
   else
   if Item.ImageIndex < 7 then
      Item.ImageIndex := Item.ImageIndex + 7;
   if Tool <> nil then
      Tool.SetPropertyBooleanByName('Review',true);
   //ListLayers.Invalidate;
end;

procedure TTematicMap_Editor_Form.SetIgnoreChanges(const Value: boolean);
begin
  FIgnoreChanges := Value;
end;

procedure TTematicMap_Editor_Form.ListLayersMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   aItem,aRItem: TListItem;
   i: integer;
   R: TRect;
   di,di1: integer;
   TL: TThematicLayer;
begin
   di  := ListLayers.Column[0].Width;
   di1 := ListLayers.Column[1].Width;
   aRItem := nil;
   for i := 0 to ListLayers.Items.Count - 1 do
   begin
      aItem := ListLayers.Items[i];
      R := aItem.DisplayRect(drBounds);
      if (Y > R.Top) and (Y < R.Bottom) then
      begin
         if (x > (R.Left + di)) and  (x < (R.Left + di + di1))then
         begin
            aRItem := aItem;
         end;
         break;
      end;
   end;
   if aRItem = nil then exit;
   //ShowMessage(aRItem.Caption);
   TL := TThematicLayer(aRItem.Data);
   if Button = mbRight then
   begin
      ColorDialog.Color := TL.Color;
      if ColorDialog.Execute then
         TL.Color := ColorDialog.Color;
   end
   else
   begin
      TL.UseColor := not TL.UseColor;
   end;
   aRItem.Update;
   if aRItem.Checked then
   if Tool <> nil then
      Tool.SetPropertyBooleanByName('Review',true);
end;

end.
