unit t_MapSpacesFrame;

interface
uses
   Elements,
   e_MSpace,
   e_SKMap,
   m_Intera,
   f_MapSpaceForm,
   r_MapSpacesFrame,
   t_OrbisFrame;

type

TMapSpacesFrame = class(TOrbisFrame)
  private
    FSMO: TSMO;
    FGusterberg: TMaps_SK;
    FStepan: TMaps_SK;
    FSKInterpret: TInterpretMap_SK;
    FForm: TMapSpaces_Form;
    FFormLeft: integer;
    FFormTop: integer;
    FShowSMO: boolean;
    FShowSMOOverlay: boolean;
    FShowSMOOverlayH: double;
    FShowSMOOverlayV: double;
    FShowSMOOverlayColor: integer;
    FShowSMOColor: integer;
    FShowSMOScale: integer;
    FWaitForDone: boolean;
    FShowGusterberg: boolean;
    FShowStepan: boolean;
    FShowStepanColor: integer;
    FShowGusterbergColor: integer;
    FShowStepanSections: boolean;
    FShowGusterbergSections: boolean;
    FShowGusterbergSectionsColor: integer;
    FShowStepanSectionsColor: integer;
    function GetMSFrame: TMapSpaces_Frame;
    procedure SetForm(const Value: TMapSpaces_Form);
    function GetActive: boolean;
    procedure SetActive(const Value: boolean);
    procedure SetFormLeft(const Value: integer);
    procedure SetFormTop(const Value: integer);
    function GetSMO: TSMO;
    procedure SetShowSMO(const Value: boolean);
    procedure SetShowSMOColor(const Value: integer);
    procedure SetShowSMOOverlay(const Value: boolean);
    procedure SetShowSMOOverlayColor(const Value: integer);
    procedure SetShowSMOOverlayH(const Value: double);
    procedure SetShowSMOOverlayV(const Value: double);
    procedure SetShowSMOScale(const Value: integer);
    procedure SetWaitForDone(const Value: boolean);
    function GetGusterberg: TMaps_SK;
    function GetStepan: TMaps_SK;
    procedure SetShowGusterberg(const Value: boolean);
    procedure SetShowGusterbergColor(const Value: integer);
    procedure SetShowStepan(const Value: boolean);
    procedure SetShowStepanColor(const Value: integer);
    function GetSKInterpret: TInterpretMap_SK;
    procedure SetShowGusterbergSections(const Value: boolean);
    procedure SetShowGusterbergSectionsColor(const Value: integer);
    procedure SetShowStepanSections(const Value: boolean);
    procedure SetShowStepanSectionsColor(const Value: integer);
    procedure SetFindGusterbergName(const Value: string);
    procedure SetFindStepanName(const Value: string);
    procedure SetFindSMOName(const Value: string);
public
   constructor Create; override;
   destructor  Destroy; override;
   procedure   CreateFrame; override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   on_tm_Hide(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   property    MSFrame: TMapSpaces_Frame read GetMSFrame;
   property    Form: TMapSpaces_Form read FForm write SetForm;
   property    SMO: TSMO read GetSMO;
   property    Gusterberg: TMaps_SK read GetGusterberg;
   property    Stepan: TMaps_SK read GetStepan;
   property    SKInterpret: TInterpretMap_SK read GetSKInterpret;
   property    WaitForDone: boolean read FWaitForDone write SetWaitForDone;
published
   property    Active: boolean Read GetActive write SetActive;
   property    FormLeft: integer read FFormLeft write SetFormLeft;
   property    FormTop: integer read FFormTop write SetFormTop;
   property    ShowSMO: boolean read FShowSMO write SetShowSMO;
   property    ShowSMOScale: integer read FShowSMOScale write SetShowSMOScale;
   property    ShowSMOColor: integer read FShowSMOColor write SetShowSMOColor;
   property    ShowSMOOverlay: boolean read FShowSMOOverlay write SetShowSMOOverlay;
   property    ShowSMOOverlayColor: integer read FShowSMOOverlayColor write SetShowSMOOverlayColor;
   property    ShowSMOOverlayH: double read FShowSMOOverlayH write SetShowSMOOverlayH;
   property    ShowSMOOverlayV: double read FShowSMOOverlayV write SetShowSMOOverlayV;
   property    ShowGusterberg: boolean read FShowGusterberg write SetShowGusterberg;
   property    ShowGusterbergSections: boolean read FShowGusterbergSections write SetShowGusterbergSections;
   property    ShowGusterbergColor: integer read FShowGusterbergColor write SetShowGusterbergColor;
   property    ShowGusterbergSectionsColor: integer read FShowGusterbergSectionsColor write SetShowGusterbergSectionsColor;
   property    ShowStepan: boolean read FShowStepan write SetShowStepan;
   property    ShowStepanSections: boolean read FShowStepanSections write SetShowStepanSections;
   property    ShowStepanSectionsColor: integer read FShowStepanSectionsColor write SetShowStepanSectionsColor;
   property    ShowStepanColor: integer read FShowStepanColor write SetShowStepanColor;
   property    FindGusterbergName: string write SetFindGusterbergName;
   property    FindStepanName: string write SetFindStepanName;
   property    FindSMOName: string write SetFindSMOName;
end;

implementation
uses
   Classes,
   Forms,
   Graphics,
   SysUtils,
   e_Class,
   e_SpTree,
   g_Elems,
   i_Uni,
   m_Orbis,
   t_GraphicForm,
   t_Loader,
   u_String, t_Device, Tools;

{ TMapSpacesFrame }

constructor TMapSpacesFrame.Create;
begin
  inherited;

end;

procedure TMapSpacesFrame.CreateFrame;
begin
   aFrame := TMapSpaces_Frame.Create(nil);
   MSFrame.Tool := Self;
end;

destructor TMapSpacesFrame.Destroy;
begin
  if Active then
     Form.Free;
  inherited;
end;

function TMapSpacesFrame.GetActive: boolean;
begin
   result := Form <> nil;
end;

function TMapSpacesFrame.GetGusterberg: TMaps_SK;
var
   S: string;
   Load: TLoad;
begin
   if FGusterberg = nil then
   begin
      Busy := true;
      S := GetAbsolutePath('LED\Klady\Gusterberg.orb');
      Load := TLoad.CreateAll(S);
      Load.DoIt;
      FGusterberg := TMaps_SK(Load.Output);
      Load.Free;
      AddElement(FGusterberg);
      Busy := false;
   end;
   result := FGusterberg;
end;

function TMapSpacesFrame.GetMSFrame: TMapSpaces_Frame;
begin
   result := TMapSpaces_Frame(aFrame);
end;


function TMapSpacesFrame.GetSKInterpret: TInterpretMap_SK;
begin
   if FSKInterpret = nil then
   begin
      FSKInterpret := TInterpretMap_SK.Create;
      FSKInterpret.Device := Device;
      FSKInterpret.NoSelect := true;
      AddElement(FSKInterpret);
   end;
   result := FSKInterpret;
end;

function TMapSpacesFrame.GetSMO: TSMO;
var
   S: string;
begin
   if FSMO = nil then
   begin
      Busy := true;
      FSMO := TSMO.Create;
      S := GetAbsolutePath('MAPLSMO.TXT');
      FSMO.Create_SMO_50_ByScript(S);
      AddElement(FSMO);
      Busy := false;
   end;
   result := FSMO;
end;

function TMapSpacesFrame.GetStepan: TMaps_SK;
var
   S: string;
   Load: TLoad;
begin
   if FStepan = nil then
   begin
      Busy := true;
      S := GetAbsolutePath('LED\Klady\Stepan.orb');
      Load := TLoad.CreateAll(S);
      Load.DoIt;
      FStepan := TMaps_SK(Load.Output);
      Load.Free;
      AddElement(FStepan);
      Busy := false;
   end;
   result := FStepan;
end;

procedure TMapSpacesFrame.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_Hide:   on_tm_Hide(aMessage,Handled);
   end;
end;

procedure TMapSpacesFrame.on_tm_Done(var aMessage: TOrbisMessage;
  var Handled: boolean);
var
   R: TDoubleRect;
   D: double;
   H,dH: double;
   V,dV: double;
   aX,aY,StartX,StartY,XX,YY: double;
   ISK: TInterpretMap_SK;

begin
   if not (aMessage.Sender is TUniversalInterpret) then exit;
   if not WaitForDone then exit;
   WaitForDone := false;
   if Device = nil then exit;
   if ShowGusterberg then
   begin
      ISK := SKInterpret;
      ISK.Input := Gusterberg;
      ISK.ShowSections := ShowGusterbergSections;
      ISK.Color := ShowGusterbergColor;
      ISK.SectionsColor := ShowGusterbergSectionsColor;
      ISK.Reset(Device);
      ISK.DoIt;
   end;
   if ShowStepan then
   begin
      ISK := SKInterpret;
      ISK.Input := Stepan;
      ISK.ShowSections := ShowStepanSections;
      ISK.Color := ShowStepanColor;
      ISK.SectionsColor := ShowStepanSectionsColor;
      ISK.Reset(Device);
      ISK.DoIt;
   end;
   if ShowSMO then
   begin
      R := Device.Projection.CLipModelView;
      Case ShowSMOScale of
         50000:
         begin
            H := 25000;
            V := 20000;
         end;
         25000:
         begin
            H := 12500;
            V := 10000;
         end;
         10000:
         begin
            H :=  5000;
            V :=  4000;
         end;
         else
         begin
            H  :=  2500;
            V  :=  2000;
         end;
      end;
      StartX:= trunc(R.MinX / H) * H;
      StartY:= trunc(R.MinY / V) * V;
      if ShowSMOOverlay then
      begin
         dH := ShowSMOOverlayH;
         dV := ShowSMOOverlayV;
         Device.Pen.Style:= psSolid;
         Device.Pen.Color:= ShowSMOOverlayColor;
         Device.PenWidth:= 1;
         aX:= StartX;
         aY:= StartY;
         if (StartX - H + dH) > R.MinX then
            aX := StartX -H;
         if (StartY - V + dV) > R.MinY then
            aY := StartY -V;
         while (aX - dH <= R.MaxX) do
         begin
            XX := aX - dH;
            if XX >= R.MinX then
            begin
               Device.MoveTo(XX, R.MinY);
               Device.LineTo(XX, R.MaxY);
            end;
            XX := aX + dH;
            if XX <= R.MaxX then
            begin
               Device.MoveTo(XX, R.MinY);
               Device.LineTo(XX, R.MaxY);
            end;
            aX:= aX + H;
         end;

        while (aY - dV <= R.MaxY) do
        begin
           YY := aY - dV;
           if YY >= R.MinY then
           begin
              Device.MoveTo(R.MinX, YY);
              Device.LineTo(R.MaxX, YY);
           end;
           YY := aY + dV;
           if YY <= R.MaxY then
           begin
              Device.MoveTo(R.MinX, YY);
              Device.LineTo(R.MaxX, YY);
           end;
           aY:= aY + V;
        end;
      end;
      begin
         Device.Pen.Style:= psSolid;
         Device.Pen.Color:= ShowSMOColor;
         Device.PenWidth:= 1;
         aX:= StartX;
         aY:= StartY;
         while (aX  <= R.MaxX) do
         begin
            Device.MoveTo(aX, R.MinY);
            Device.LineTo(aX, R.MaxY);
            aX:= aX + H;
         end;

         while (aY <= R.MaxY) do
         begin
            Device.MoveTo(R.MinX, aY);
            Device.LineTo(R.MaxX, aY);
            aY:= aY + V;
         end;
         //PostMsgToOwners(tm_Repaint,nil,false);
         //Application.ProcessMessages
      end
   end;
end;

procedure TMapSpacesFrame.on_tm_Hide(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   if aMessage.Sender = Self then
   begin
      Active := false;
      Handled := true;
   end;
end;

procedure TMapSpacesFrame.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   aX,aY: double;
   N_50,N_25,N_10,N_5,N_2: string;
   MSK: TMaps_SK;
   M: TMap_SK;
begin
   if not Active then exit;
   if Busy  then exit;
   if Device = nil then exit;
   aX:= Device.Projection.GetModelX(aMessage.x);
   aY:= Device.Projection.GetModelY(aMessage.y);
   case Form.MSPages.ActivePageIndex of
      0:
      begin
         SMO.GetSMONamesByCoord(aX, aY, N_50, N_25, N_10, N_5, N_2);
         Form.l_50.Caption := N_50;
         Form.l_10.Caption := N_10;
         Form.l_5.Caption := N_5;
         Form.l_25.Caption := N_25;
         Form.l_2.Caption := N_2;
      end;
      1:
      begin
         MSK := Gusterberg;
         if MSK = nil then
         begin
            Form.l_Gusterberg.Caption := 'Error';
            exit;
         end;
         M := MSK.GetMapByXY(aX,aY);
         if M = nil then
            Form.l_Gusterberg.Caption := 'Mimo klad'
         else
         begin
            if ShowGusterbergSections then
               Form.l_Gusterberg.Caption := M.FullName
            else
               Form.l_Gusterberg.Caption := M.Name;
         end;
      end;
      2:
      begin
         MSK := Stepan;
         if MSK = nil then
         begin
            Form.l_Stepan.Caption := 'Error';
            exit;
         end;
         M := MSK.GetMapByXY(aX,aY);
         if M = nil then
            Form.l_Stepan.Caption := 'Mimo klad'
         else
         begin
            if ShowStepanSections then
               Form.l_Stepan.Caption := M.FullName
            else
               Form.l_Stepan.Caption := M.Name;
         end;
      end;
   end;
end;

procedure TMapSpacesFrame.On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   WaitForDone := true;
end;

procedure TMapSpacesFrame.SetActive(const Value: boolean);
var
   SKM: TMaps_SK;
   i,j: integer;
   E: TElement;
   SL: TStringList;
   Col: TSpaces;
   aSMO: TSMO;
begin
   if Value then
   begin
      if Form = nil then
         Form := TMapSpaces_Form.Create(Application);
      Form.Left := FormLeft;
      Form.Top := FormTop;
      Form.Tool := Self;
      Form.Setting := true;
      aSMO := SMO;
      SL := TStringList.Create;
      SL.Sorted := true;
      SL.Duplicates := dupAccept;
      for i := 0 to aSMO.ColumnCount - 1 do
      begin
         Col := aSMO.Column[i];
         for j := 0 to Col.Count - 1 do
         begin
            SL.Add(TSMO_50(Col.Space[j]).Abbrev);
         end;
      end;
      Form.Combo_SMO_50.Items.AddStrings(SL);
      Form.Combo_SMO_50.ItemIndex := 0;
      SL.Free;
      Form.cb_ShowSMO.Checked := ShowSMO;
      Form.c_SMOScale.ItemIndex := Form.c_SMOScale.Items.IndexOf(IntToStr(ShowSMOScale));
      Form.SMOColorPanel.Color := ShowSMOColor;
      Form.cb_ShowSMOOverlay.Checked := ShowSMOOverlay;
      Form.SMOOverlayColorPanel.Color := ShowSMOOverlayColor;
      Form.Ed_SMOOverlayH.Text := GetPropertyStringByName('ShowSMOOverlayH');
      Form.Ed_SMOOverlayV.Text := GetPropertyStringByName('ShowSMOOverlayV');
      Form.SetEnabling;
      Form.cb_ShowGusterberg.Checked := ShowGusterberg;
      Form.cb_ShowGusterbergSections.Checked := ShowGusterbergSections;
      Form.cb_ShowGusterbergSectionsClick(nil);
      Form.GusterbergColorPanel.Color := ShowGusterbergColor;
      Form.GusterbergsectionColorPanel.Color := ShowGusterbergSectionsColor;
      Form.cb_ShowStepan.Checked := ShowStepan;
      Form.cb_ShowStepanSections.Checked := ShowStepanSections;
      Form.cb_ShowStepanSectionsClick(nil);
      Form.StepanColorPanel.Color := ShowStepanColor;
      Form.StepansectionColorPanel.Color := ShowStepanSectionsColor;
      SKM := Gusterberg;
      for i := 0 to SKM.LastIndex do
      begin
         E := SKM.Elements[i];
         Form.Combo_G_Q.Items.AddObject(E.Name,E);
         Form.Combo_G_Q.ItemIndex := 0;
         Form.Combo_G_QClick(nil);
      end;
      SKM := Stepan;
      for i := 0 to SKM.LastIndex do
      begin
         E := SKM.Elements[i];
         Form.Combo_S_Q.Items.AddObject(E.Name,E);
         Form.Combo_S_Q.ItemIndex := 0;
         Form.Combo_S_QClick(nil);
      end;
      Form.Setting := False;
      Form.Visible := true;
      MSFrame.But.Down := true;
   end
   else
   begin
      if Form <> nil then
      begin
         FormLeft := Form.Left;
         FormTop := Form.Top;
         Form.Free;
      end;
      Form := nil;
      MSFrame.But.Down := false;
   end;
end;

procedure TMapSpacesFrame.SetFindGusterbergName(const Value: string);
var
   M: TMap_SK;
   GF: TGraphicForm;
begin
   if Value = '' then exit;
   M := Gusterberg.GetMapByName(Value);
   if M = nil then exit;
   GF := TGraphicForm(FindClassInOwners(TGraphicForm));
   if GF = nil then exit;
   if ShowGusterbergSections then
   begin
      M := M.CreateSection(StrToInt(Form.Combo_G_S.Text));
      if M <> nil then
      begin
         GF.ElementForProjection := M;
         Application.ProcessMessages;
         M.Free;
      end;
   end
   else
      GF.ElementForProjection := M;
end;

procedure TMapSpacesFrame.SetFindSMOName(const Value: string);
var
   aSMO: TSMO;
   SMO50: TSMO_50;
   Col,Row: integer;
   GF: TGraphicForm;
   R: TRectEntity;
begin
   aSMO := SMO;
   SMO50 := aSMO.Get_SMO_50_ByAbbrev(Value);
   if SMO50 = nil then exit;
   Col := StrToIntDef(Form.Combo_SMO_Col.Text,0);
   Row := StrToIntDef(Form.Combo_SMO_Row.Text,0);
   GF := TGraphicForm(FindClassInOwners(TGraphicForm));
   if GF = nil then exit;
   R := TRectEntity.Create;
   R.MaxX := SMO50.MaxX - col*2500;
   R.MaxY := SMO50.MaxY - row*2000;
   R.MinX := R.MaxX - 2500;
   R.MinY := R.MaxY - 2000;
   GF.ElementForProjection := R;
   Application.ProcessMessages;
   R.Free;
end;

procedure TMapSpacesFrame.SetFindStepanName(const Value: string);
var
   M: TMap_SK;
   GF: TGraphicForm;
begin
   if Value = '' then exit;
   M := Stepan.GetMapByName(Value);
   if M = nil then exit;
   GF := TGraphicForm(FindClassInOwners(TGraphicForm));
   if GF = nil then exit;
   if ShowStepanSections then
   begin
      M := M.CreateSection(StrToInt(Form.Combo_S_S.Text));
      if M <> nil then
      begin
         GF.ElementForProjection := M;
         Application.ProcessMessages;
         M.Free;
      end;
   end
   else
      GF.ElementForProjection := M;
end;

procedure TMapSpacesFrame.SetForm(const Value: TMapSpaces_Form);
begin
  FForm := Value;
end;

procedure TMapSpacesFrame.SetFormLeft(const Value: integer);
begin
  FFormLeft := Value;
end;

procedure TMapSpacesFrame.SetFormTop(const Value: integer);
begin
  FFormTop := Value;
end;

procedure TMapSpacesFrame.SetShowGusterberg(const Value: boolean);
begin
  FShowGusterberg := Value;
end;

procedure TMapSpacesFrame.SetShowGusterbergColor(const Value: integer);
begin
  FShowGusterbergColor := Value;
end;

procedure TMapSpacesFrame.SetShowGusterbergSections(const Value: boolean);
begin
  FShowGusterbergSections := Value;
  Gusterberg.Sections := Value;
end;

procedure TMapSpacesFrame.SetShowGusterbergSectionsColor(
  const Value: integer);
begin
  FShowGusterbergSectionsColor := Value;
end;

procedure TMapSpacesFrame.SetShowSMO(const Value: boolean);
begin
  FShowSMO := Value;
end;

procedure TMapSpacesFrame.SetShowSMOColor(const Value: integer);
begin
  FShowSMOColor := Value;
end;

procedure TMapSpacesFrame.SetShowSMOOverlay(const Value: boolean);
begin
  FShowSMOOverlay := Value;
end;

procedure TMapSpacesFrame.SetShowSMOOverlayColor(const Value: integer);
begin
  FShowSMOOverlayColor := Value;
end;

procedure TMapSpacesFrame.SetShowSMOOverlayH(const Value: double);
begin
  FShowSMOOverlayH := Value;
end;

procedure TMapSpacesFrame.SetShowSMOOverlayV(const Value: double);
begin
  FShowSMOOverlayV := Value;
end;

procedure TMapSpacesFrame.SetShowSMOScale(const Value: integer);
begin
  FShowSMOScale := Value;
end;

procedure TMapSpacesFrame.SetShowStepan(const Value: boolean);
begin
  FShowStepan := Value;
end;

procedure TMapSpacesFrame.SetShowStepanColor(const Value: integer);
begin
  FShowStepanColor := Value;
end;

procedure TMapSpacesFrame.SetShowStepanSections(const Value: boolean);
begin
  FShowStepanSections := Value;
  Stepan.Sections := Value;
end;

procedure TMapSpacesFrame.SetShowStepanSectionsColor(const Value: integer);
begin
  FShowStepanSectionsColor := Value;
end;

procedure TMapSpacesFrame.SetWaitForDone(const Value: boolean);
begin
  FWaitForDone := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TMapSpacesFrame);
END.
