unit t_RasterTransformation;

interface
uses
   Controls,
   Graphics,
   Elements,
   t_InterfaceTool,
   m_Intera,
   e_Transformation,
   t_Transformation,
   f_RasterTransformer;

const
   crNoSelect = 0;
   crRas      = 1;
   crHandFlat = 2;
   crHandGrab = 3;

type

TRasterTransformerStatus=(rtsWaitForFile,rtsPosition,rtsIDPoint,rtsTransformation);

TDragRecord = record
    StartX: integer;
    StartY: integer;
    CenterX: integer;
    CenterY: integer;
end;

TTransformerStatus = record
    Status: TRasterTransformerStatus;
    MapX: Double;
    MapY: double;
    MapDone: boolean;
    RasX: double;
    RasY: double;
    RasDone: boolean;
end;

TRasterTransformer = class(TInterfaceTool)
private
    FForm: TRasterTransformer_Form;
    FFileName: string;
    FBM: TBitmap;
    FLocalCursor: integer;
    FTransformation: TTransformation_Afinni2D;
    FTransformationTable: TTransformationTable;
    function GetForm: TRasterTransformer_Form;
    procedure SetFileName(const Value: string);
    procedure SetDoOpen(const Value: boolean);
    procedure SetZoom(const Value: string);
    procedure SetLocalCursor(const Value: integer);
    function GetTransformation: TTransformation_Afinni2D;
    function GetTransformationTable: TTransformationTable;
    procedure SetTransformation(const Value: TTransformation_Afinni2D);
    procedure SetTransformationTable(const Value: TTransformationTable);
    procedure SetWaitForCoords(const Value: boolean);
    procedure SetText(const Value: string);
    procedure SetDoTransform(const Value: boolean);
    procedure SetIDPointCheck(const Value: boolean);
protected
   property Form: TRasterTransformer_Form read GetForm;
   procedure DrawCross(aX,aY: integer);
   procedure DrawR(aX,aY: integer; aText: string);
   procedure DrawCrossRas(aX,aY: integer);
   procedure DrawRRas(aX,aY: integer; aText: string);
   procedure ShowTrafoMap;
   procedure ShowTrafoRas;
public
   Drag: boolean;
   PageColor: Integer;
   CenterX: integer;
   CenterY: integer;
   Scale: double;
   AddX: integer;
   AddY: integer;
   Ready: boolean;
   DragR: TDragRecord;
   TrStatus: TTransformerStatus;
   property  Text: string write SetText;
   property  Transformation: TTransformation_Afinni2D read GetTransformation write SetTransformation;
   property  TransformationTable: TTransformationTable read GetTransformationTable write SetTransformationTable;
   property LocalCursor: integer read FLocalCursor write SetLocalCursor;
   procedure On_Activate  ; override;
   procedure On_DeActivate; override;
   function  IsPossibleToDeactivate: boolean; override;
   procedure RegisterCursors; override;
   function LoadFile: boolean;
   procedure ChangeProjection(aX,aY: integer; aScale: double);
   function  GetPageX(const ModelX: integer): integer;
   function  GetPageY(const ModelY: integer): integer;
   function  GetModelX(PageX: integer): integer;
   function  GetModelY(PageY: integer): integer;
   function  GetPageLength(const ModelLength: integer): integer;
   function  GetModelLength(const PageLength: integer): integer;
   procedure On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_DragDrop(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_DragEnd(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseWheelDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseWheelUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_Resize(var Handled: boolean); override;
   procedure On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure TextByStatus;
   procedure ClearStatusPoint;
   procedure SetupCursor;
   procedure Clear;
   procedure SmartClear;
   procedure ViewAll;
   procedure ZoomIn;
   procedure ZoomOut;
   procedure Review;
   procedure ViewTable;
published
   property FileName: string read FFileName write SetFileName;
   property DoOpen: boolean write SetDoOpen;
   property Zoom: string write SetZoom;
   property WaitForCoords: boolean write SetWaitForCoords;
   property DoTransform: boolean write SetDoTransform;
   property IDPointCheck: boolean write SetIDPointCheck;
end;

implementation
uses
   Dialogs,
   SysUtils,
   Types,
   Forms,
   u_Ini,
   Worlds,
   m_Orbis,
   t_Main,
   t_ConfirmDialog,
   MathLib,
   d_Raster,
   g_Elems,
   e_Class, E_device;

{ TRasterTransformer }

procedure TRasterTransformer.ChangeProjection(aX, aY: integer; aScale: double);
var
   CX, CY: integer;
begin
   Scale   := aScale;
   CenterX := aX;
   CenterY := aY;
   CX := Form.Image.Picture.Bitmap.Width div 2;
   CY := Form.Image.Picture.Bitmap.Height div 2;

   AddX := CenterX - Round(CX * Scale);
   AddY := CenterY - Round(CY * Scale);

end;

procedure TRasterTransformer.Clear;
var
   C: TCanvas;
begin
   C := Form.Image.Picture.Bitmap.Canvas;
   C.Brush.Color:= 0;
   C.Brush.Style:= bsSolid;
   C.FillRect(Rect(0,0,Form.Image.Width,Form.Image.Height));
end;


procedure TRasterTransformer.ClearStatusPoint;
begin
   TrStatus.MapX := 0;
   TrStatus.MapY := 0;
   TrStatus.MapDone := false;
   TrStatus.RasX := 0;
   TrStatus.RasY := 0;
   TrStatus.RasDone := false;
end;

procedure TRasterTransformer.DrawCross(aX, aY: integer);
begin
   Device.Pen.Style := psSolid;
   Device.Pen.Width := 1;
   Device.Pen.Color := clBlack;
   Device.Pen.Mode:= pmCopy;
   Device.MoveToDevice(aX-10,aY);
   Device.LineToDevice(aX+10,aY);
   Device.MoveToDevice(aX,aY-10);
   Device.LineToDevice(aX,aY+10);
   Application.processMessages;
end;

procedure TRasterTransformer.DrawCrossRas(aX, aY: integer);
var
   C: TCanvas;
begin
   C := Form.Image.Picture.Bitmap.Canvas;
   C.Pen.Style := psSolid;
   C.Pen.Width := 1;
   C.Pen.Color := clBlack;
   C.Pen.Mode:= pmCopy;
   C.MoveTo(aX-10,aY);
   C.LineTo(aX+10,aY);
   C.MoveTo(aX,aY-10);
   C.LineTo(aX,aY+10);
   //Application.processMessages;
end;

procedure TRasterTransformer.DrawR(aX, aY: integer; aText: string);
begin
   Device.Font.Color := clRed;
   Device.Font.Size := 20;
   Device.Font.Name := 'Arial';
   Device.TextOutDevice(aX+5,aY,aText);
   Application.processMessages;
end;

procedure TRasterTransformer.DrawRRas(aX, aY: integer; aText: string);
var
   C: TCanvas;
begin
   C := Form.Image.Picture.Bitmap.Canvas;
   C.Font.Color := clRed;
   C.Font.Size := 20;
   C.Font.Name := 'Arial';
   C.Brush.Style:= bsClear;
   C.TextOut(aX+5,aY,aText);
   //Application.processMessages;
end;

function TRasterTransformer.GetForm: TRasterTransformer_Form;
begin
   result := FForm;
   if result = nil then
   begin
      FForm := TRasterTransformer_Form.Create(Application);
      FForm.Tool := Self;
      result := FForm;
      Ready := false;
      Drag := false;
      FForm.Caption := 'Transformace .... nen vybrn dn soubor';
      TrStatus.Status := rtsWaitForFile;
      Text := 'Vyberte rastrov soubor pro transformaci';
      //LocalCursor := 0;
      //CursorIndex := 2;
   end;
end;


function TRasterTransformer.GetModelLength(const PageLength: integer): integer;
begin
   Result:= Round(PageLength * Scale);
end;

function TRasterTransformer.GetModelX(PageX: integer): integer;
begin
   Result:= Round(Scale*PageX + AddX) ;
end;

function TRasterTransformer.GetModelY(PageY: integer): integer;
begin
   Result := Round(Scale*PageY + AddY);
end;

function TRasterTransformer.GetPageLength(const ModelLength: integer): integer;
begin
  result:= 0;
  if Scale = 0 then exit;
  try
    Result:= round(ModelLength / Scale);
  except
  end
end;

function TRasterTransformer.GetPageX(const ModelX: integer): integer;
begin
  result:= 0;
  if Scale = 0 then exit;
  try
    Result:= Round((ModelX - AddX)/Scale);
  except
  end
end;

function TRasterTransformer.GetPageY(const ModelY: integer): integer;
begin
  result:= 0;
  if Scale = 0 then exit;
  try
    Result:= Round((ModelY - AddY)/Scale);
  except
  end
end;


function TRasterTransformer.GetTransformation: TTransformation_Afinni2D;
begin
   if FTransformation = nil then
   begin
      FTransformation := TTransformation_Afinni2D.Create;
      AddElement(FTransformation);
   end;
   result := FTransformation;
end;

function TRasterTransformer.GetTransformationTable: TTransformationTable;
begin
   if FTransformationTable = nil then
   begin
      FTransformationTable := TTransformationTable.Create;
      AddElement(FTransformationTable);
   end;
   result := FTransformationTable;
end;

function TRasterTransformer.IsPossibleToDeactivate: boolean;
begin
   result := True;
   if Ready  then
      result := Confirm('Opravdu chcete opustit transformaci rastru?');
end;

function TRasterTransformer.LoadFile: boolean;
begin

end;

procedure TRasterTransformer.On_Activate;
begin
  inherited;
  Form.Show;
  SetupCursor;
end;

procedure TRasterTransformer.On_DeActivate;
begin
  inherited;
  FForm.Free;
  FForm := nil;
end;


procedure TRasterTransformer.On_tm_DragDrop(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
  //Form.Caption := 'Drop ' + IntToStr(aMessage.X) + 'x' + IntToStr(aMessage.Y);

end;

procedure TRasterTransformer.On_tm_DragEnd(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
  //Form.Caption := 'End ' + IntToStr(aMessage.X) + 'x' + IntToStr(aMessage.Y);
   DragR.StartX := 0;
   DragR.StartY := 0;
   Drag := false;
   SetupCursor;
end;

procedure TRasterTransformer.On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   dX, dY: integer;
begin
  //Form.Caption := 'Over ' + IntToStr(aMessage.X) + 'x' + IntToStr(aMessage.Y);
   if aMessage.Sender <> Self then exit;
   if not Drag then
   begin
      Drag := true;
      DragR.StartX := aMessage.X;
      DragR.StartY := aMessage.Y;
      DragR.CenterX := CenterX;
      DragR.CenterY := CenterY;
      exit;
   end;
   dX := GetModelLength(DragR.StartX - aMessage.X);
   dy := GetModelLength(DragR.StartY - aMessage.Y);
   ChangeProjection(DragR.CenterX + dx,DragR.CenterY + dy,Scale);
   SmartClear;
   Review;
end;

procedure TRasterTransformer.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   B: TIDBod;
begin
  if aMessage.Button = mbRight then
  begin
     Form.Show;
     exit;
  end;
  if TrStatus.Status <> rtsIDPoint then exit;
  if aMessage.Sender = Self then // local point
  begin
      if not TrStatus.RasDone then
      begin
         TrStatus.RasX := GetModelX(aMessage.X);
         TrStatus.RasY := GetModelY(aMessage.Y);
         TrStatus.RasDone := true;
         ShowTrafoRas;
      end
      else
      begin
         Text := 'Na MAP ! Ne v rastru proboha!';
         exit;
      end;
  end
  else
  begin                         // Map point ... maybe
      if not TrStatus.MapDone then
      begin
         TrStatus.MapX := Device.Projection.GetModelX(aMessage.X);
         TrStatus.MapY := Device.Projection.GetModelY(aMessage.Y);
         TrStatus.MapDone := true;
         ShowTrafoMap;
      end
      else
      begin
         Text := 'Na RASTRU ! to je to nad tmhle textem!';
         exit;
      end;
  end;
  TextByStatus;
  if TrStatus.RasDone and TrStatus.MapDone then // We will make nice idpoint to yourself
  begin
      B := TIDBod.Create;
      B.Cislo := TransformationTable.Count + 1;
      B.cX := TrStatus.MapX;
      B.cY := TrStatus.MapY;
      B.zX := TrStatus.RasX;
      B.zY := TrStatus.RasY;
      TransformationTable.AddElement(B);
      ClearStatusPoint;
      if TransformationTable.ActivePointsCount > 3 then
      begin
         Transformation.TransformationTable := TransformationTable;
         Transformation.EvaluateDifferences;
         Form.b_Transform.Visible := true;
      end;
      ShowTrafoMap;
      ShowTrafoRas;
      ViewTable;
      TextByStatus;
      //DrawCross(LastX,LastY);
  end;
end;

procedure TRasterTransformer.On_tm_MouseWheelDown( var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if aMessage.Sender <> Self then exit;
   if Handled then exit;
   Handled := true;
   if not Ready then exit;
   zoomOut;
end;

procedure TRasterTransformer.On_tm_MouseWheelUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if aMessage.Sender <> Self then exit;
   if Handled then exit;
   Handled := true;
   if not Ready then exit;
   ZoomIn;
end;

procedure TRasterTransformer.On_tm_Resize(var Handled: boolean);
begin
  if not Ready then exit;
  ChangeProjection(CenterX,CenterY,Scale);
   Clear;
  Review;
end;

procedure TRasterTransformer.On_tm_Review(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   //CursorVisible := false;
   if TrStatus.Status = rtsWaitForFile then exit;
   ShowTrafoMap;
end;

procedure TRasterTransformer.On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
  //Form.Caption := 'Start ' + IntToStr(aMessage.X) + 'x' + IntToStr(aMessage.Y);
  if aMessage.Sender <> Self then exit;
  LocalCursor := 2;
end;

procedure TRasterTransformer.RegisterCursors;
begin
   NewCursorName('NOSELECT');     //0
   NewCursorName('RAS');     //0
   NewCursorName('HANDFLAT');
   NewCursorName('HANDGRAB');
end;

procedure TRasterTransformer.Review;
var
   R: TRect;
begin
   R := Rect(GetPageX(0),GetPageY(0),GetPageX(Form.Picture.Graphic.Width),GetPageY(Form.Picture.Graphic.Height));
   Form.Image.Picture.Bitmap.Canvas.StretchDraw(R,Form.Picture.Graphic);
   ShowTrafoRas;
end;

procedure TRasterTransformer.SetDoOpen(const Value: boolean);
var
   aDirectory: string;
   iniName: string;
begin
   // Dialog
   if Value then
   begin
      iniName := ChangeFileExt(TMainTool(World.MainTool).Options.ProjectFile,'.ini');
      aDirectory := ExtractFilePath(iniName);
      aDirectory := ReadStringFromIni(iniName,'TRANSFORMER','SourceDirectory',aDirectory);
      Form.OpenDialog1.InitialDir := aDirectory;
      if Form.OpenDialog1.Execute then
      begin
         if Ready then
         begin
            PostMsgToOwners(tm_Review,nil,false);
            Application.ProcessMessages;
         end;
         if Form.Picture <> nil then
            Form.Picture.Free;
         Form.Picture := TPicture.Create;
         FileName := Form.OpenDialog1.FileName;
         aDirectory := ExtractFilePath(FileName);
         WriteStringToIni(iniName,'TRANSFORMER','SourceDirectory',aDirectory);

         Form.Picture.LoadFromFile(FileName);
         Form.Caption := 'Transformace souboru ' + ExtractFileName(FileName);
         Ready := true;
         Transformation := nil;
         TransformationTable := nil;
         ViewTable;
         ClearStatusPoint;
         ViewAll;
         TrStatus.Status := rtsPosition;
         Form.b_Transform.Visible := false;
         Form.b_Hand.Down := true;
         SetupCursor;
         TextByStatus;
      end;
   end;
end;

procedure TRasterTransformer.SetDoTransform(const Value: boolean);
var
   x,y,xs,ys: integer;
   xr,yr: double;
   BM,BMD: TBitmap;
   P,PS: PByteArray;
   j,Js: integer;
   //BM: TGraphic;
   x1,y1,x2,y2,x3,y3,x4,y4: extended;   // transformovane rohy rastru
   R: TDoubleRect;                    // extenze ... cesky opsany obdelnik
   ResX: double;
   D: TTopRaster_Device;
   aFileName: string;
   aDirectory: string;
   iniName: string;
begin
   // Dialog
   iniName := ChangeFileExt(TMainTool(World.MainTool).Options.ProjectFile,'.ini');
   aDirectory := ExtractFilePath(FileName);
   aDirectory := ReadStringFromIni(iniName,'TRANSFORMER','SaveDirectory',aDirectory);
   aFileName := ChangeFileExt(ExtractFileName(FileName),'.tif');
   Form.SaveDialog1.InitialDir := aDirectory;
   Form.SaveDialog1.FileName := aFileName;
   if not Form.SaveDialog1.Execute then exit;
   aFileName := Form.SaveDialog1.FileName;
   aDirectory := ExtractFilePath(aFileName);
   WriteStringToIni(iniName,'TRANSFORMER','SaveDirectory',aDirectory);
   // konec dialogu

   TrStatus.Status := rtsTransformation;
   TextByStatus;
   //1 zjisteni skutecne velikosti rastru ...

   // Kopirovani zdrojoveho rastru do konzistentni bitmapy
   BM := TBitmap.Create;
   BM.PixelFormat := pf24bit;
   BM.Width := Form.Picture.Graphic.Width;
   BM.Height := Form.Picture.Graphic.Height;
   BM.Canvas.Draw(0,0,Form.Picture.Graphic);

   // transformace rohu rastru do mapy
   Transformation.TransformationTable := TransformationTable;
   Transformation.TransformXY(0,BM.Height,X1,Y1);
   Transformation.TransformXY(BM.Width,BM.Height,X2,Y2);
   Transformation.TransformXY(BM.Width,0,X3,Y3);
   Transformation.TransformXY(0,0,X4,Y4);

   // nalezeni realnych rohu
   R.MinX := X1;
   if X2 < R.MinX then R.MinX := X2;
   if X3 < R.MinX then R.MinX := X3;
   if X4 < R.MinX then R.MinX := X4;
   R.MaxX := X1;
   if X2 > R.MaxX then R.MaxX := X2;
   if X3 > R.MaxX then R.MaxX := X3;
   if X4 > R.MaxX then R.MaxX := X4;
   R.MinY := Y1;
   if Y2 < R.MinY then R.MinY := Y2;
   if Y3 < R.MinY then R.MinY := Y3;
   if Y4 < R.MinY then R.MinY := Y4;
   R.MaxY := Y1;
   if Y2 > R.MaxY then R.MaxY := Y2;
   if Y3 > R.MaxY then R.MaxY := Y3;
   if Y4 > R.MaxY then R.MaxY := Y4;

   // zjisteni realneho rozliseni
   ResX := Distance(X1,Y1,X2,Y2)/BM.Width;

   // Tvorba rastroveho device
   D := TTopRaster_Device.Create;
   D.RAS_FileName := aFileName;
   D.RAS_X1 := -R.MaxY;
   D.RAS_Y1 := -R.MaxX;
   D.RAS_dX := R.MaxY - R.MinY;
   D.RAS_dY := R.MaxX - R.MinX;
   D.RAS_PixelSizeX := ResX;
   D.RAS_PixelSizeY := ResX;
   D.RAS_TYP := 5;
   //if MaxBlack = 0 then
   //   MaxBlack := $EF;
   D.MaxBlack := $EF;
   D.MaxBlackForAll := false;
   D.RAS_Compress := 4;
   D.OldProjection := Device.Projection;
   D.ModelView := R;
    { TODO -cZkontrolovat -oPPK : OTESTOVAT sakra.... !!!!!!!!! }
   //D.Projection.VirtualScale := D.OldProjection.VirtualScale;
   D.Reset;

   // vytvoreni Bitmapy pro raster BMD
   BMD := TBitmap.Create;
   BMD.PixelFormat := pf24bit;
   BMD.Width := D.RAS_Width;
   BMD.Height := D.RAS_Height;
   // premazani bilou
   BMD.Canvas.Brush.Style := bsSolid;
   BMD.Canvas.Brush.Color := $FFFFFF;
   BMD.Canvas.FillRect(Rect(0,0,BMD.Width,BMD.Height));

   // vlastni zapis bodu
   Form.Gauge.MinValue := 0;
   Form.Gauge.MaxValue := BMD.Width - 1;
   Form.Gauge.Progress := 0;

   for x := 0 to BMD.Width - 1 do
   begin
      for y := 0 to BMD.Height - 1 do
      begin
         Xr := D.Projection.GetModelX(x);
         Yr := D.Projection.GetModelY(y);
         Transformation.InvTransformXY(Xr,Yr,X1,Y1);
         Xs := Round(x1);
         Ys := Round(y1);
         if (Xs > 0) and (Xs < BM.Width) and (Ys > 0) and (Ys < BM.Height) then
         begin
           P := BMD.ScanLine[y];
           j := x * 3;
           PS := BM.ScanLine[Ys];
           js := xs * 3;
           P[j]   := PS[js];
           P[j+1] := PS[js+1];
           P[j+2] := PS[js+2];
         //Text := IntToStr(P[j])+','+IntToStr(P[j+1])+','+IntToStr(P[j+2]);
         j := BM.Canvas.Pixels[x,y];
         end;
      end;
      Form.Gauge.Progress := x;
   end;
   D.DrawDevice(0,0,BMD);
   D.Close;
   BMD.Free;
   BM.Free;
   D.Free;
   // inicializace
   Ready := false;
   ClearStatusPoint;
   Transformation := nil;
   TransformationTable := nil;
   Form.Picture.Free;
   Form.Picture := nil;
   Clear;
   ViewTable;
   PostMsgToOwners(tm_Review,nil,false);
   TrStatus.Status := rtsWaitForFile;
   Form.b_Hand.Down := true;
   Form.b_Transform.Visible := false;
   TextByStatus;
   Application.ProcessMessages;
end;

procedure TRasterTransformer.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure TRasterTransformer.SetIDPointCheck(const Value: boolean);
var
   i: integer;
begin
   if Transformation = nil then exit;
   i := Form.LB.ItemIndex;
   TransformationTable.IDBod[i].Active := not TransformationTable.IDBod[i].Active;
   Transformation.Initialize;
   Transformation.EvaluateDifferences;
   ViewTable;
   Form.LB.ItemIndex := i;
   Form.b_Transform.Visible := TransformationTable.ActivePointsCount > 3;
   Review;
   ShowTrafoRas;
   PostMsgToOwners(tm_Review,nil,false);
   Application.ProcessMessages;
   ShowTrafoMap;
end;

procedure TRasterTransformer.SetLocalCursor(const Value: integer);
var
  i: longInt;
  C: integer;
begin
   if Value <> FLocalCursor then
   begin
      if NoCursors or (Value > Curs.LastIndex) or (Value < 0)then
      begin
         FLocalCursor := -1;
         C := 0;
      end
      else
      begin
         FLocalCursor := Value;
         C := Cursors[Value];
      end;
      if Form.Image.Dragging then Form.Image.DragCursor := C
                             else Form.Image.Cursor := C;
      Application.ProcessMessages;
   end;
end;

procedure TRasterTransformer.SetText(const Value: string);
begin
   if Form = nil then exit;
   Form.StatusBar.SimpleText := Value;
end;

procedure TRasterTransformer.SetTransformation(const Value: TTransformation_Afinni2D);
begin
  ExchangeFieldElement(TElement(FTransformation), Value);
end;

procedure TRasterTransformer.SetTransformationTable(const Value: TTransformationTable);
begin
   ExchangeFieldElement(TElement(FTransformationTable),Value);
end;

procedure TRasterTransformer.SetupCursor;
begin
   case TrStatus.Status of
      rtsWaitForFile:
      begin
         LocalCursor := crNoSelect;
         CursorIndex := 5;
         CursorIndex := crNoSelect;
      end;
      rtsPosition:
      begin
         LocalCursor := crHandFlat;
         CursorIndex := 2;
         CursorIndex := -1;
      end;
      rtsIDPoint:
      begin
         LocalCursor := crRas;
         CursorIndex := 5;
         CursorIndex := crRas;
      end;
      else
      begin
         LocalCursor := crNoSelect;
         CursorIndex := 5;
         CursorIndex := crNoSelect;
      end;
   end;
end;

procedure TRasterTransformer.SetWaitForCoords(const Value: boolean);
begin
   if TrStatus.Status in [rtsWaitForFile,rtsTransformation] then exit;
   if Value Then
   begin
      TrStatus.Status := rtsIDPoint;
      Form.b_Point.Down := true;
   end
   else
   begin
      TrStatus.Status := rtsPosition;
      Form.b_Hand.Down := true;
   end;
   SetupCursor;
   TextByStatus;
end;

procedure TRasterTransformer.SetZoom(const Value: string);
begin
   if Value ='IN' then
      ZoomIn
   else
   if Value = 'ALL' then
      ViewAll
   else
   if Value = '1' then
   begin
      ChangeProjection(CenterX,CenterY,1);
      Review;
   end
   else
      ZoomOut;
end;

procedure TRasterTransformer.ShowTrafoMap;
var
   i: integer;
   B: TIDBod;
   aX,aY: integer;
begin
   if TrStatus.MapDone then
   begin
      aX := Device.Projection.GetPageX(TrStatus.MapX);
      aY := Device.Projection.GetPageY(TrStatus.MapY);
      DrawCross(aX,aY);
      DrawR(aX,aY,IntToStr(TransformationTable.Count + 1));
   end;
   if FTransformationTable = nil then exit;
   for i := 0 to TransformationTable.Lastindex do
   begin
      B := TransformationTable.IDBod[i];
      if not B.Active then continue;
      aX := Device.Projection.GetPageX(B.cX);
      aY := Device.Projection.GetPageY(B.cY);
      DrawCross(aX,aY);
      DrawR(aX,aY,B.Name);
   end;
end;

procedure TRasterTransformer.ShowTrafoRas;
var
   i: integer;
   B: TIDBod;
   aX,aY: integer;
begin
   if TrStatus.RasDone then
   begin
      aX := GetPageX(Round(TrStatus.RasX));
      aY := GetPageY(Round(TrStatus.RasY));
      DrawCrossRas(aX,aY);
      DrawRRas(aX,aY,IntToStr(TransformationTable.Count + 1));
   end;
   if FTransformationTable = nil then exit;
   for i := 0 to TransformationTable.Lastindex do
   begin
      B := TransformationTable.IDBod[i];
      if not B.Active then continue;
      aX := GetPageX(Round(B.zX));
      aY := GetPageY(Round(B.zY));
      DrawCrossRas(aX,aY);
      DrawRRas(aX,aY,B.Name);
   end;
end;

procedure TRasterTransformer.SmartClear;
var
   C: TCanvas;
   BM: TBitmap;
   X1,Y1,X2,Y2: integer;
begin
   if not Ready then exit;
   BM := Form.Image.Picture.Bitmap;
   C := BM.Canvas;
   X1 := GetPageX(0);
   Y1 := GetPageY(0);
   X2 := GetPageX(Form.Picture.Graphic.Width);
   Y2 := GetPageY(Form.Picture.Graphic.Height);
   if (X1 >= BM.Width) or (X2 <= 0) or (Y1 >= BM.Height) or (Y2 <= 0) then // jen cerno
   begin
      Clear;
      exit;
   end;
   C.Brush.Color:= 0;
   C.Brush.Style:= bsSolid;
   if X1 > 0then // left
   begin
      C.FillRect(Rect(0,0,X1,BM.Height));
   end;
   if Y1 > 0 then // top
   begin
      C.FillRect(Rect(0,0,BM.Width,Y1));
   end;
   if X2 < BM.Width then // Right
   begin
      C.FillRect(Rect(X2,0,BM.Width,BM.Height));
   end;
   if Y2 < BM.Height then // Bottom
   begin
      C.FillRect(Rect(0,Y2,BM.Width,BM.Height));
   end;
end;

procedure TRasterTransformer.TextByStatus;
begin
   case TrStatus.Status of
      rtsWaitForFile:
         Text := 'Vyberte soubor rastru pro transformaci';
      rtsPosition:
      begin
         if not (TrStatus.MapDone or TrStatus.RasDone) then
            Text := 'Vyberte shodn msta na map a v rastru  pro Identick body'
         else
         if not TrStatus.MapDone then
            Text := 'Naleznte njak pkn msto na map pro Identick bod'
         else
            Text := 'Zkuste najt v rastru podobn msto jako na map';
      end;
      rtsIDPoint:
      begin
         if not (TrStatus.MapDone or TrStatus.RasDone) then
            Text := 'Vytvote identick bod na map nebo v rastru'
         else
         if not TrStatus.MapDone then
            Text := 'Vytvote odpovdajc identick bod na map'
         else
            Text := 'Vytvote odpovdajc identick bod v rastru';
      end;
      else
         Text := 'Probh transformace ... nehbejte se, nedchejte, asnte';
   end
end;

procedure TRasterTransformer.ViewAll;
var
   Bmd: TBitmap;
   Bms: TGraphic;
   R: TRect;
   aX,aY: integer;
   rs,rd: Double;
   ScaleX: double;
   ScaleY: double;
   aScale: double;
begin
   Clear;
   R.Left := 0;
   R.Top  := 0;
   Bms := Form.Picture.Graphic;
   Bmd := Form.Image.Picture.Bitmap;
   ScaleX := Bms.Width  / Bmd.Width;
   ScaleY := Bms.Height / Bmd.Height;
   if ScaleX > ScaleY then aScale := ScaleX
                      else aScale := ScaleY;
   aX :=  Bms.Width div 2;
   aY :=  Bms.Height div 2;
   ChangeProjection(aX,aY, aScale);
   Review;
end;

procedure TRasterTransformer.ViewTable;
var
   i: integer;
   S: string;
   B: TIDBod;
begin
   Form.LB.Clear;
   if TransformationTable = nil then exit;
   for i := 0 to TransformationTable.LastIndex do
   begin
      B := TransformationTable.IDBod[i];
      S := '        '+IntToStr(B.Cislo)+'.  dx:'+FormatFloat('"+"0.000;"-"0.000',B.OdchX)+'  dy:'+FormatFloat('"+"0.000;"-"0.000',B.OdchY);
      Form.LB.Items.Add(S);
      Form.LB.Checked[i] := B.Active;
   end;
end;

procedure TRasterTransformer.ZoomIn;
begin
   Scale := Scale * 0.75;
   ChangeProjection(CenterX,Centery,Scale);
   Clear;
   Review;
end;

procedure TRasterTransformer.ZoomOut;
begin
   Scale := Scale * 1.5;
   ChangeProjection(CenterX,Centery,Scale);
   Clear;
   Review;
end;

initialization
   RegisterOrbisClass(TRasterTransformer);
end.
