unit t_GraphicForm;

interface
uses
   Controls,
   Graphics,
   Menus,
   // KERNEL
   Elements,
   Tools,
   // DEVICE
   e_Proje,
   // INTERPRET
   e_IScrpt,
   // GUI
   m_Intera,
   // APP
   f_GraphicForm,
   t_ExtendedFormTool;

type

{ TGraphicForm }

TGraphicForm = class(TExtendedFormTool)
 private
   function    GetActiveToolName: string;
   function    GetActiveToolClassName: string;
   procedure   SetActiveToolName(aValue: string);
   procedure   SetActiveToolClassName(aValue: string);
   function    GetPageColor: TColor;
   procedure   SetPageColor(aValue: TColor);
   function    GetProjection: TProjection;
   function    GetInterpretScript: TInterpretScript;
   procedure   SetInterpretScript(aValue: TInterpretScript);
    function GetForm: TGraphic_Form;
    procedure SetViewAll(const Value: boolean);
 protected
   procedure   ShowScale(aVal: string); virtual;

   procedure   SetInput(aInput: TElement); override;
   procedure   CreateDevice; virtual;
   procedure   LinkDeviceToForm; virtual;
   procedure   CreateInterpret; virtual;
   procedure   LinkInterpretToDevice; virtual;
   procedure   CreateTools; virtual;
   procedure   SetActiveToolIndex(aIndex: longInt); override;
   procedure   SetNextTool(aTool: TTool); virtual;
   function    GetVisible: boolean; virtual;
   procedure   SetVisible(aVisible: boolean);  virtual;
   function    FindMenuItem(const aText: string): TMenuItem;
   function    FindMenuItemIn(const aMenu, aItem: string): TMenuItem;
   procedure   CheckMenuItem(const aItem: string);
   function    GetMainMenu: TMainMenu; virtual;
   function    GetCaption: string; virtual;
   procedure   SetCaption(const aValue: string); virtual;
   procedure   SetBreakReView(aValue: boolean);
   procedure   SetElementForReView(aValue: TElement);
   procedure   SetElementForKillReView(aValue: TElement);
   procedure   SetFreeForm(aValue: boolean); virtual;

   property    MainMenu: TMainMenu read GetmainMenu;
 public
   constructor Create; override;
   procedure   DoIt; override;
   procedure   CreateForm; override;

   function    GetCursor: TCursor; override;
   procedure   SetCursor(const Value: TCursor); override;
   function    AddTool(aTool: TTool): TTool; override;
   function    InsertTool(aIndex: longInt; aTool: TTool): TTool; override;
   procedure   RemoveTool(aTool: TTool); override;
   procedure   FreeTool(aTool: TTool); override;
   procedure   FreeTools;

   procedure   Show;
   procedure   ShowAll;
   procedure   on_tm_Open(var Handled: boolean); virtual;
   procedure   on_tm_SaveAs(var Handled: boolean); virtual;
   procedure   on_tm_QuitTask; override;
   procedure   on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_Print(var Handled: boolean); virtual;
   procedure   On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   On_tm_Resize(var Handled: boolean); override;
   procedure   On_tm_RePaint(var Handled: boolean); virtual;
   procedure   On_tm_NewScale(var Handled: boolean); virtual;
   procedure   on_tm_ShowAll(var Handled: boolean); virtual;
   procedure   On_tm_EndOfPrint(var Handled: boolean); virtual;
   procedure   On_tm_Cursor(var aMessage: TOrbisCursorEvent; var Handled: boolean); override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;

   property    Form: TGraphic_Form read GetForm;
published
   property    NextTool: TTool write SetNextTool;
   property    ActiveToolName: string read GetActiveToolName write SetActiveToolName;
   property    ActiveToolClassName: string read GetActiveToolClassName write SetActiveToolClassName;
   property    PageColor: TColor read GetPageColor write SetPageColor;
   property    Projection: TProjection read GetProjection;
   property    InterpretScript: TInterpretScript read GetInterpretScript write SetInterpretScript;
   property    BreakReView: boolean write SetBreakReView stored false;
   property    ElementForReView: TElement write SetElementForReView;
   property    ElementForKillReView: TElement write SetElementForKillReView;
   property    ViewAll: boolean write SetViewAll;
   property    FreeForm: boolean write SetFreeForm;
end;


implementation
uses
   Forms,
   SysUtils,
   Windows,
   // KERNEL
   e_Class,
   m_ORBIS,
   // GLIB
   G_Elems,
   // DEVICE
   d_StdImg,
   // INTERPRET
   i_Uni;

{ TGraphicForm }

constructor TGraphicForm.Create;
begin
   inherited Create;
   CreateForm;
   CreateDevice;
   LinkDeviceToForm;
   CreateInterpret;
   LinkInterpretToDevice;
   CreateTools;
end;


procedure   TGraphicForm.SetBreakReView(aValue: boolean);
begin
   if Interpret.Busy then Interpret.BreakWork := true;
end;


procedure   TGraphicForm.SetElementForReView(aValue: TElement);
begin
   PostMsg(tm_ReView, Self, aValue, DontKill);
end;


procedure   TGraphicForm.SetElementForKillReView(aValue: TElement);
begin
   PostMsg(tm_ReView, Self, aValue, KillIt);
end;


procedure   TGraphicForm.SetFreeForm(aValue: boolean);
begin
  if Form = nil then exit;
   if aValue then Form.Free;
end;






function    TGraphicForm.GetInterpretScript: TInterpretScript;
begin
   result := nil;
   if Interpret is TUniversalInterpret then result := TUniversalInterpret(Interpret).Script;
end;


procedure   TGraphicForm.SetInterpretScript(aValue: TInterpretScript);
var
   //aTool: TUniversalTopolScriptCreator;
   aScript: TInterpretScript;
begin
   (*
   if aValue = nil then
   begin
   {$IFNDEF NOTIS}
      aTool := TUniversalTopolScriptCreator.Create;
      aTool.DoIt;
      aScript:= TInterpretScript(aTool.Output);
      TUniversalInterpret(FInterpret).Script := aScript;
      aTool.Free;
   {$ENDIF}
   end
   else
   *)
   begin
     if ((TElement(aValue) is TInterpretScript) and (Interpret is TUniversalInterpret))
        then TUniversalInterpret(Interpret).Script := aValue;
   end;
end;


function    TGraphicForm.GetPageColor: TColor;
begin
   try
      result := Device.Page.Color;
   except
      result := clBlack;
   end;
end;

procedure   TGraphicForm.SetPageColor(aValue: TColor);
begin
   try
      Device.Page.Color := aValue;
   except
   end;
end;

function    TGraphicForm.GetProjection: TProjection;
begin
   if Device <> nil then result := Device.Projection
                    else result := nil;
end;


function    TGraphicForm.GetActiveToolName: string;
begin
   result := ActiveTool.Name;
end;

function    TGraphicForm.GetActiveToolClassName: string;
begin
   result := ActiveTool.ClassName;
end;

procedure   TGraphicForm.SetActiveToolName(aValue: string);
var
   T: TTool;
begin
   T := TTool(Tools.FindElementName(aValue));
   if T = nil then exit;
   T.PostMsgToOwners(tm_ToolSelected, T, DontKill);
end;

procedure   TGraphicForm.SetActiveToolClassName(aValue: string);
var
   T: TTool;
begin
   T := nil;
   try
      T := TTool(Tools.FindElementClass(FindOrbisClass(aValue)));
   except
   end;
   if T = nil then exit;
   T.PostMsgToOwners(tm_ToolSelected, T, DontKill);
end;


procedure   TGraphicForm.DoIt;
begin
   Show;
   PostMsg(tm_ReView, Self, nil, DontKill);
end;

procedure   TGraphicForm.SetInput(aInput: TElement);
var
   Envelope: TRectEntity;
begin
   if FInput <> nil then
   begin
      FreeElement(FInput);
      FInput := nil;
   end;
   if aInput = nil then exit;
   Envelope := TRectEntity.Create;
   Envelope.Name := 'Oblka';      // p.k.17.1.99
   Envelope.AddAndUpdateRect(aInput{.CreateEnvelope});
   AddElement(Envelope);
   try
      if aInput is TRectEntity then
      begin
         Device.Projection.ChangeProjectionByRectangle(TRectEntity(aInput).Rect, Device.Page);
      end
      else
      if aInput is TCoordinate then
      begin
         Device.Projection.ChangeProjectionByRect(TCoordinate(aInput).GetRegion, Device.Page);
      end;
   except
   end;
   inherited SetInput(Envelope);
end;


procedure   TGraphicForm.CreateForm;
begin
   aForm := TGraphic_Form.Create(Application);
   Form.Tool := Self;
   Form.IsPossibleToClose := false;
end;

procedure   TGraphicForm.CreateDevice;
begin
   Device := TImageCanvasDevice.Create;
end;

procedure   TGraphicForm.LinkDeviceToForm;
begin
  if Form = nil then exit;
   with TImageCanvasDevice(Device) do
   begin
      Canvas := Form.Image.Canvas;
      Bitmap := Form.Image.Picture.Bitmap;
   end;
end;

procedure   TGraphicForm.CreateInterpret;
begin
   Interpret := TUniversalInterpret.Create;
end;



procedure   TGraphicForm.LinkInterpretToDevice;
begin
   Interpret.Device := Device;
end;

procedure   TGraphicForm.CreateTools;
begin
end;

procedure   TGraphicForm.SetActiveToolIndex(aIndex: longInt);
begin
   if aIndex = FActiveToolIndex then exit;
   inherited SetActiveToolIndex(aIndex);
end;

procedure   TGraphicForm.SetNextTool(aTool: TTool);
begin
   {
   aTool.Device := Device;
   aTool.Manager:= Self;
   AddTool(aTool);
   ActiveTool := aTool;
   }
   //CheckMenuItem(aTool.ClassName);
end;

function    TGraphicForm.GetVisible: boolean;
begin
   result := false;
   if Form = nil then exit;
    try
       result := Form.Visible;
    except
       result := false;
    end;
end;

procedure   TGraphicForm.SetVisible(aVisible: boolean);
begin
  if Form = nil then exit;
   try
      Form.Visible := aVisible;
      Device.SetupPage(nil);
   except
   end;
end;


function    TGraphicForm.GetMainMenu: TMainMenu;
begin
  result:= nil;
  if Form = nil then exit;
   result := Form.MainMenu;
end;

function    TGraphicForm.GetCaption: string;
begin
  result:= '';
  if Form = nil then exit;

   result := Form.Caption;
end;

procedure   TGraphicForm.SetCaption(const aValue: string);
begin
  if Form = nil then exit;
   Form.Caption := aValue;
end;



function    TGraphicForm.FindMenuItem(const aText: string): TMenuItem;
var
   i: longInt;
   Item: TMenuItem;
begin
   result := nil;
   for i := 0 to MainMenu.Items.Count -1 do
   begin
      Item := MainMenu.Items.Items[i];
      if UpperCase(Item.Name) = UpperCase(aText) then
      begin
         result := Item;
         exit;
      end;
   end;
end;


function    TGraphicForm.FindMenuItemIn(const aMenu, aItem: string): TMenuItem;
var
   ToolsIndex: longInt;
   i: longInt;
   Item: TMenuItem;
   SubItem: TMenuItem;
begin
   result := nil;
   Item := FindMenuItem(aMenu);
   for i := 0 to Item.Count -1 do
   begin
      SubItem := Item.Items[i];
      if UpperCase(SubItem.Name) = UpperCase(aItem) then
      begin
         result := SubItem;
         exit;
      end;
   end;
end;


procedure   TGraphicForm.CheckMenuItem(const aItem: string);
var
   ToolsIndex: longInt;
   i: longInt;
   Item: TMenuItem;
   SubItem: TMenuItem;
begin
   Item := FindMenuItem('TOOLS');
   for i := 0 to Item.Count -1 do
   begin
      SubItem := Item.Items[i];
      if UpperCase(SubItem.Name) = UpperCase(aItem) then SubItem.Checked := true
                                                    else SubItem.Checked := false;
   end;
end;


function    TGraphicForm.AddTool(aTool: TTool): TTool;
var
   ToolsIndex: longInt;
   i: longInt;
   aMenu, Item: TMenuItem;
begin
   inherited AddTool(aTool);
   {
   result:= aTool;
   aMenu := FindMenuItem('Tools');
   Item  := TMenuItem.Create(aMenu);
   Item.Name := aTool.ClassName;
   Item.Caption := aTool.Name;
   Item.OnClick := TInterActiveTool(aTool).Select;
   aMenu.Add(Item);
   }
end;


function    TGraphicForm.InsertTool(aIndex: longInt; aTool: TTool): TTool;
begin
   result :=  inherited InsertTool(aIndex, aTool);
end;

procedure   TGraphicForm.RemoveTool(aTool: TTool);
var
   MI: TMenuItem;
begin
   MI := FindMenuItemIn('TOOLS',aTool.ClassName);
   MI.Free;
   inherited RemoveTool(aTool);
end;

procedure   TGraphicForm.FreeTool(aTool: TTool);
var
   MI: TMenuItem;
begin
   MI := FindMenuItemIn('TOOLS',aTool.ClassName);
   MI.Free;
   inherited FreeTool(aTool);
end;

procedure   TGraphicForm.FreeTools;
var
   i: longInt;
begin
   for i := 0 to Tools.Count do
   begin
      FreeTool(TTool(Tools.Elements[i]));
   end;
end;

procedure   TGraphicForm.Show;
begin
   Visible := true;
end;


procedure   TGraphicForm.ShowAll;
begin
  if FInput = nil then exit;
  try
     if FInput is TRectEntity then
     begin
        Device.Projection.ChangeProjectionByRectangle(TRectEntity(FInput).Rect, Device.Page);
     end
     else
     if FInput is TCoordinate then
     begin
        Device.Projection.ChangeProjectionByRect(TCoordinate(FInput).GetRegion, Device.Page);
     end;
  except
  end;
end;



procedure   TGraphicForm.on_tm_Open(var Handled: boolean);
begin
end;
(*
var
   Loader: TLoader;
   SelEl: TElement;
   SwapG: boolean;
begin
   if Form.OpenDialog.Execute then
   begin
      SwapG := true;
      Loader := TLoader.CreateAll(Form.OpenDialog.FileName);
      try
         Form.OpenDialog.InitialDir := ExtractFilePath(Form.OpenDialog.FileName);
         Loader.DoIt;
         Input := Loader.Output;
         Loader.Output.Free;
         PostMsg(tm_Review, Self, nil, DontKill);
      except
      end;
      Loader.Free;
   end
end;
*)

procedure   TGraphicForm.on_tm_SaveAs(var Handled: boolean);
begin
end;
(*
var
   Saver: TSaver;
begin
   Form.SaveDialog.InitialDir := Form.OpenDialog.InitialDir;
   Form.SaveDialog.FileName   := Form.OpenDialog.FileName;
   if Form.SaveDialog.Execute then
   begin
      Form.OpenDialog.InitialDir := ExtractFilePath(Form.SaveDialog.FileName);
      Form.OpenDialog.FileName := Form.SaveDialog.FileName;
      if Input = nil then exit;
      Input.ResetID;
      Saver := TSaver.CreateAll(Form.SaveDialog.FileName);
      Saver.Input := Input;
      Saver.DoIt;
      Saver.Free
   end;
end;
*)



procedure   TGraphicForm.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   Coord: TDoubleCoord;

begin
  {
  if Form = nil then exit;
    Handled := true;
    begin
       Coord := Device.Projection.GetModelPoint(aMessage.x, aMessage.y);
       Form.l_x.Caption := FloatToStrF(Coord.x, ffFixed, 12, 2);
       Form.l_y.Caption := FloatToStrF(Coord.y, ffFixed, 12, 2);
    end;
   }
end;

procedure TGraphicForm.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
{
var
   P: Wintypes.TPoint;
}
begin
  (*
  if Form = nil then exit;
   Handled := true;
   if aMessage.Button = mbRight then
   begin
      {
      if Interpret.Busy then
      begin
         PostMouseEvent(tm_MouseDown, Self, aMessage.Button,
                        aMessage.Shift, aMessage.X, aMessage.Y, aMessage.ActiveDevice);
         exit;
      end;
      }
      P := Point(aMessage.x, aMessage.y);
//      P := Form.PaintBox.ClientToScreen(P);
      P := Form.Image.ClientToScreen(P);
      Form.PopupMenu1.PopUp(P.x,P.y);
   end;
   *)
end;



procedure   TGraphicForm.on_tm_QuitTask;
begin
   if Interpret.Busy then
   begin
      Status := Status + [st_DIE];
      Interpret.BreakWork := true;
   end
   else
   begin
     if Form <> nil then
        Form.Free;
      inherited on_tm_QuitTask;
   end;
end;

procedure   TGraphicForm.on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := true;
   if aMessage.Sender is TUniversalInterpret then
   begin
      if st_DIE in Status then
      begin
        if Form <> nil then
           Form.Free;
         inherited on_tm_QuitTask;
      end
      else
      if st_BREAK_AND_DOIT in Status then
      begin
         Status := Status - [st_BREAK_AND_DOIT];
         DoIt;
      end;
   end;
end;


procedure   TGraphicForm.on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   inherited;
   (*
   if not (aMessage.Data is TInterActiveTool) then exit;

   if ActiveTool is TInterActiveTool then
      TInterActiveTool(ActiveTool).On_DeActivate;

   ActiveTool := TInterActiveTool(aMessage.Data);
   TInterActiveTool(ActiveTool).Device := Device;
   ActiveTool.Input  := Self.Input;
   TInterActiveTool(ActiveTool).On_Activate;
   Handled := true;
   CheckMenuItem(ActiveTool.ClassName);
   FormCursor := TInterActiveTool(ActiveTool).Cursor;
   *)
end;

procedure   TGraphicForm.on_tm_Print(var Handled: boolean);
{
var
   P: TPrint;
}
begin
{
   P := TPrint.Create;
   P.Input := Interpret;
   P.DoIt;
   P.Free;
}
end;


procedure TGraphicForm.ShowScale(aVal: string);
begin
{
   try
       if (Form <> nil) and (Form.l_m <> nil) then
          Form.l_M.Caption := aVal;
   except
   end;
}
end;



procedure   TGraphicForm.On_tm_ReView(var aMessage: TOrbisMessage; var Handled: boolean);
var
   s: string;
begin
   if Interpret.Busy then
   begin
      Status := Status + [st_BREAK_AND_DOIT];
      Interpret.BreakWork := true;
      exit;
   end;
   Device.Pen.Mode:= pmCopy; { novinka 10.6.97 }

   inherited On_tm_ReView(aMessage, Handled);
{
   if Scale < 10 then
   begin
      Str(Device.Projection.Scale:0:3,S);
   end
   else S := IntToStr(round(Device.Projection.Scale));

   ShowScale(S);
   if CLoseAfterPrinting then PostMsg(tm_Print, Self, nil, DontKill);
}
   if ActiveTool <> nil then PostMsg(tm_Review,ActiveTool,nil,dontkill);
   Application.ProcessMessages;
   PostMsg(tm_Done, nil, self, dontKill);
end;


procedure TGraphicForm.On_tm_Resize(var Handled: boolean);
var
   i: longInt;
begin
   Handled := true;
   {
   PostMsg(tm_BreakInterpret, Self, nil, DontKill);
   }
   //BreakReView := true;
   //Application.ProcessMessages;
   Device.SetupPage(nil);
   PostMsg(tm_BreakAndReView, Self, nil, DontKill);
end;

procedure TGraphicForm.On_tm_RePaint(var Handled: boolean);
var
   i: longInt;
begin
   Handled := true;
   {
   PostMsg(tm_BreakInterpret, Self, nil, DontKill);
   }
   if assigned(Form) then
      Form.Image.Refresh;
end;



procedure TGraphicForm.On_tm_EndOfPrint(var Handled: boolean);
begin
   Handled := true;
   {
   if CloseAfterPrinting then
   begin
      PostMsgToOwners(tm_EndOfPrint, nil, DontKill);
      on_tm_QuitTask;
   end;
   }
end;



procedure TGraphicForm.On_tm_NewScale(var Handled: boolean);
(*
var
   aForm: TF_SetScale;
   D: double;
*)
begin
(*
  try
    Application.CreateForm(Tf_SetScale, aForm);
    try
      aForm.Combo.Text := IntToStr(round(Scale));
      case aForm.ShowModal of
         mrOK:
         begin
            D:= StrToExtended(aForm.Combo.Text);
            if D > 0 then
            begin
              Scale := D;
              PostMsg(tm_BreakAndReView, self, nil, DontKill);
              Handled:= true
            end
            else
              ShowAll
         end;
      end;
    finally
      aForm.Free;
    end
  except
  end
*)
end;


procedure   TGraphicForm.On_tm_ShowAll(var Handled: boolean);
begin
  ShowAll;
  PostMsg(tm_BreakAndReView, self, nil, DontKill);
  Handled:= true
end;



procedure TGraphicForm.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
begin
   case aMessage.Key of
      VK_F5:
      begin
         Scale:= Scale / 2;
         PostMsg(tm_BreakAndReView, Self, nil, DontKill);
      end;
      VK_F6:
      begin
         Scale:= Scale * 2;
         PostMsg(tm_BreakAndReView, Self, nil, DontKill);
      end;
      VK_ESCAPE:
      begin
         PostMsg(tm_BreakInterpret, Self, nil, DontKill);
      end;
   end;
end;



procedure TGraphicForm.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
  // if aMessage.Message = tm_Resize then
  //    PostTextMsg(tm_AddToProtocol,Name + '.on_Resize GF',nil);
  (*
   try
     if (ActiveTool <> nil) and (aMessage.Message <> tm_Review) then
        ActiveTool.HandleMessage(aMessage, Handled);
   except
   end;
   if Handled then exit;
   *)
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      //tm_ToolSelected: on_tm_ToolSelected(aMessage, Handled);
      tm_Print:        on_tm_Print(Handled);
      tm_EndOfPrint:   on_tm_EndOfPrint(Handled);
      tm_Open:         on_tm_Open(Handled);
      tm_SaveAs:       on_tm_SaveAs(Handled);
      tm_NewScale:     on_tm_NewScale(Handled);
      tm_ShowAll:      on_tm_ShowAll(Handled);
      tm_RePaint:      on_tm_RePaint(Handled);
   end;
end;




function TGraphicForm.GetForm: TGraphic_Form;
begin
   result := TGraphic_Form(aForm);
end;

procedure TGraphicForm.SetViewAll(const Value: boolean);
begin
   if not Value then exit;
   ShowAll;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
end;

procedure TGraphicForm.On_tm_Cursor(var aMessage: TOrbisCursorEvent;
  var Handled: boolean);
begin
   Cursor := aMessage.Cursor;
end;

function TGraphicForm.GetCursor: TCursor;
begin
  result:= 0;
  if Form = nil then exit;
  result := Form.Image.Cursor;
end;

procedure TGraphicForm.SetCursor(const Value: TCursor);
begin
   if Form = nil then exit;
   if Form.Image.Dragging then Form.Image.DragCursor := Value
                          else Form.Image.Cursor := Value;
end;

BEGIN
   RegisterOrbisClass(TGraphicForm);
END.
