unit t_ConfirmDialog;

interface
uses
   f_ConfirmDialog,
   t_OrbisDialog;

type

TOrbisConfirmDialog = class(TOrbisDialog)
private
    function GetForm: TOrbisConfirmDialog_Form;
    function GetText: string;
    procedure SetText(const Value: string);
public
   procedure CreateForm; override;
   property  Form: TOrbisConfirmDialog_Form read GetForm;
published
   property  Text: string read GetText write SetText stored false;
end;

function Confirm(const aText: string): boolean; overload;
function Confirm(const aCaption, aText: string): integer; overload;

implementation
uses
   Forms,
   e_Class;

function Confirm(const aText: string): boolean;
var
   D: TOrbisConfirmDialog;
begin
   D := TOrbisConfirmDialog.Create;
   D.Text := aText;
   D.Caption := '?';
   D.DoIt;
   result := D.Result = 1;
   D.Free;
end;

function Confirm(const aCaption, aText: string): integer;
var
   D: TOrbisConfirmDialog;
begin
   D := TOrbisConfirmDialog.Create;
   D.Text := aText;
   D.Caption := aCaption;
   D.DoIt;
   result := D.Result;
   D.Free;
end;

{ TOrbisConfirmDialog }

procedure TOrbisConfirmDialog.CreateForm;
begin
   aForm := TOrbisConfirmDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;


function TOrbisConfirmDialog.GetForm: TOrbisConfirmDialog_Form;
begin
   result := TOrbisConfirmDialog_Form(aForm);
end;

function TOrbisConfirmDialog.GetText: string;
begin
   result := '';
   if Form = nil then exit;
   result := Form.l_Message.Caption
end;

procedure TOrbisConfirmDialog.SetText(const Value: string);
begin
   if Form = nil then exit;
   Form.l_Message.Caption := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TOrbisConfirmDialog);
END.
