unit t_CoordMonitor;

interface
uses
   m_InterA,
   r_CoordMonitor,
   t_OrbisFrame;
type

TCoordMonitorFrame = class(TOrbisFrame)
  private
    FrY: double;
    FrX: double;
    FiX: integer;
    FiY: integer;
    function GetCoordFrame: TCoordMonitor_Frame;
    procedure SetiX(const Value: integer);
    procedure SetiY(const Value: integer);
    procedure SetrX(const Value: double);
    procedure SetrY(const Value: double);
public
   procedure CreateFrame; override;
   procedure On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure Process(var aMessage: TOrbisMouseEvent);
   property  CoordFrame: TCoordMonitor_Frame read GetCoordFrame;
published
   property iX: integer read FiX write SetiX;
   property iY: integer read FiY write SetiY;
   property rX: double read FrX write SetrX;
   property rY: double read FrY write SetrY;
end;

implementation
uses
   SysUtils,
   R_WGS84,
   T_Coord,
   e_Class;

{ TCoordMonitorFrame }

procedure TCoordMonitorFrame.CreateFrame;
begin
   aFrame := TCoordMonitor_Frame.Create(nil);
end;

function TCoordMonitorFrame.GetCoordFrame: TCoordMonitor_Frame;
begin
   result := TCoordMonitor_Frame(aFrame);
end;


procedure TCoordMonitorFrame.On_tm_DragOver(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
   Process(aMessage);
end;

procedure TCoordMonitorFrame.On_tm_MouseMove(
  var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   Process(aMessage);
end;

procedure TCoordMonitorFrame.Process(var aMessage: TOrbisMouseEvent);
var
   AMap: TMapCoord;
   Lat,Long: double;
   Frm: TWgs84Format;
begin
   if Device = nil then exit;
   iX := aMessage.X;
   iY := aMessage.Y;
   rX:= Device.Projection.GetModelX(iX);
   rY:= Device.Projection.GetModelY(iY);
   if CoordFrame.WGS84 then
   begin
      AMap.Y := - rX;
      AMap.X := - rY;
      JTSKtoWGS84(AMap, Lat, Long);
      if CoordFrame.Mod2 then
         Frm := afSeconds
      else
         Frm := afMinutes;
      CoordFrame.l_X.Caption := WGS84ToStr(Lat, Frm);
      CoordFrame.l_Y.Caption := WGS84ToStr(Long, Frm);
   end
   else
   begin
      if CoordFrame.Mod2 then
      begin
         rX := -rX;
         rY := -rY;
      end;
      CoordFrame.l_X.Caption := FormatFloat('0.##',rX);
      CoordFrame.l_Y.Caption := FormatFloat('0.##',rY);
   end;
end;

procedure TCoordMonitorFrame.SetiX(const Value: integer);
begin
  FiX := Value;
end;

procedure TCoordMonitorFrame.SetiY(const Value: integer);
begin
  FiY := Value;
end;

procedure TCoordMonitorFrame.SetrX(const Value: double);
begin
  FrX := Value;
end;

procedure TCoordMonitorFrame.SetrY(const Value: double);
begin
  FrY := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCoordMonitorFrame);
END.
