unit t_ExtendedFormTool;

interface
uses
   ActnList,
   Classes,
   Comctrls,
   Controls,
   Menus,
   Menubar,
   Windows,
   // KERNEL
   e_ToolModel,
   m_Intera,
   Elements,
   Tools,
   // APP
   f_ExtendedOrbisForm,
   t_ControlTool,
   t_FormTool,
   t_InterfaceTool;

type

TOrbisShortCutScript = class(TTool)
  private
    FScript: string;
    FShortCut: string;
    FComment: string;
    FShift: TShiftState;
    FKey: Word;
    FShortCut_: TShortCut;
    procedure SetComment(const Value: string);
    procedure SetKey(const Value: Word);
    procedure SetScript(const Value: string);
    procedure SetShift(const Value: TShiftState);
    procedure SetShortCut(const Value: string);
    function GetRealShortCut: string;
    procedure SetShortCut_(const Value: TShortCut);
protected
   function  GetName: string; override;
   procedure SetName(aName: string); override;
public
   procedure DoIt; override;
   property  Key: Word read FKey write SetKey;
   property  Shift: TShiftState read FShift write SetShift;
   property  ShortCut_: TShortCut read FShortCut_ write SetShortCut_;
published
   property ShortCut: string read FShortCut write SetShortCut;
   property RealShortCut: string read GetRealShortCut;
   property Script: string read FScript write SetScript;
   property Comment: string read FComment write SetComment;
end;

TOrbisShortCutScripts = class(TInterfaceTool)
  private
    FActualShortCutScript: TOrbisShortCutScript;
    function GetActualComment: string;
    function GetActualScript: string;
    function GetActualShortCut: string;
    procedure SetActualComment(const Value: string);
    procedure SetActualScript(const Value: string);
    procedure SetActualShortCut(const Value: string);
    procedure SetActualShortCutScript(const Value: TOrbisShortCutScript);
    procedure SetNewShortScriptCutName(const Value: string);
public
   procedure On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   function  FindByShortCut(aValue: TShortCut): TOrbisShortCutScript;
published
   property  NewShortCutScriptName: string write SetNewShortScriptCutName stored false;
   property  ActualShortCutScript: TOrbisShortCutScript read FActualShortCutScript write SetActualShortCutScript;
   property  ActualShortCut: string read GetActualShortCut write SetActualShortCut;
   property  ActualScript: string read GetActualScript write SetActualScript;
   property  ActualComment: string read GetActualComment write SetActualComment;
end;


TExtendedToolGroup = class(TControlTool)
private
   FActualAction: TAction;
   FActionList: TActionList;
   FexForm: TExtendedOrbis_Form;
   FImageList: TImagelist;
   FAlign: TAlign;
   procedure SetActionList(const Value: TActionList);
   procedure SetexForm(const Value: TExtendedOrbis_Form); virtual;
   procedure SetImageList(const Value: TImagelist);
    function GetActualCaption: string;
    function GetActualHint: string;
    function GetActualImageIndex: integer;
    function GetActualScript: string;
    function GetActualTool: TTool;
    function GetActualVisible: boolean;
    function GetButtonHeight: integer;
    function GetButtonWidth: integer;
    function GetFlat: boolean;
    function GetImageHeight: integer;
    function GetImageWidth: integer;
    function GetShowCaptions: boolean;
    procedure SetActualCaption(const Value: string);
    procedure setActualHint(const Value: string);
    procedure SetActualImageIndex(const Value: integer);
    procedure SetActualScript(const Value: string);
    procedure SetActualTool(const Value: TTool);
    procedure SetActualVisible(const Value: boolean);
    procedure SetButtonHeight(const Value: integer);
    procedure SetButtonWidth(const Value: integer);
    procedure SetFindActionName(const Value: string);
    procedure SetFlat(const Value: boolean);
    procedure SetImageHeight(const Value: integer);
    procedure SetImageWidth(const Value: integer);
    procedure SetNewImage(const Value: string);
    procedure SetNewScriptAction(const Value: string);
    procedure SetNewToolAction(const Value: string);
    procedure SetShowCaptions(const Value: boolean);
    procedure SetToolBar(const Value: TToolbar);
    procedure SetEmpty(const Value: boolean);
    procedure SetNewSeparator(const Value: string);
    function GetWrapable: boolean;
    procedure SetWrapable(const Value: boolean);
    function GetToolBar: TToolbar;
    function GetAutoDock: boolean;
    procedure SetAutoDock(const Value: boolean);
    function GetEmpty: boolean;
    function GetActualShortCut: string;
    procedure SetActualShortCut(const Value: string);
protected
    procedure SetAlign(const Value: TAlign); override;
    function GetLeft: integer; override;
    function GetTop: integer;  override;
public
   procedure CreateSelfByModel; override;
   function  CreateToolByModel(aModel: TToolModel): TTool; override;
   procedure AddToolsByModel(aTool: TTool); virtual;
   procedure AddToolByModel(aTool: TTool); virtual;
   procedure ActionExecute(Sender: TObject);
   procedure ActionPopup(Sender: TObject; MousePos: TPoint; var Handled: Boolean);
   function  FindActionByTool(aTool: TTool): TAction;
   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   property  ToolBar: TToolbar read GetToolBar write SetToolBar;
   property  ImageList: TImagelist read FImageList write SetImageList;
   property  ActionList: TActionList read FActionList write SetActionList;
   property  exForm: TExtendedOrbis_Form read FexForm write SetexForm;
   property  Empty: boolean read GetEmpty write SetEmpty;
published
   property   Flat: boolean read GetFlat write SetFlat;
   property   ButtonHeight: integer read GetButtonHeight write SetButtonHeight;
   property   ButtonWidth: integer read GetButtonWidth write SetButtonWidth;
   property   ImageHeight: integer read GetImageHeight write SetImageHeight;
   property   ImageWidth: integer read GetImageWidth write SetImageWidth;
   property   Wrapable: boolean read GetWrapable write SetWrapable;
   // For manual setting only:
   property   NewImage: string write SetNewImage stored false;
   property   NewScriptAction: string write SetNewScriptAction stored false;
   property   NewToolAction: string write SetNewToolAction stored false;
   property   NewSeparator: string write SetNewSeparator stored false;
   property   FindActionName: string write SetFindActionName stored false;
   property   ActualVisible: boolean read GetActualVisible write SetActualVisible stored false;
   property   ActualScript: string read GetActualScript write SetActualScript stored false;
   property   ActualTool: TTool read GetActualTool write SetActualTool stored false;
   property   ActualImageIndex: integer read GetActualImageIndex write SetActualImageIndex stored false;
   property   ActualCaption: string read GetActualCaption write SetActualCaption stored false;
   property   ActualHint: string read GetActualHint write setActualHint stored false;
   property   ActualShortCut: string read GetActualShortCut write SetActualShortCut stored false;
   property   ShowCaptions: boolean read GetShowCaptions write SetShowCaptions;
   property   AutoDock: boolean read GetAutoDock write SetAutoDock;
end;

TMenuGroup = class(TExtendedToolGroup)
private
   procedure SetexForm(const Value: TExtendedOrbis_Form); override;
    function GetMenuBar: TMenuBar;
public
   property  MenuBar: TMenuBar read GetMenuBar;
end;


TExtendedFormTool = class(TFormTool)
private
   FMenu: TMenuGroup;
   FTasks: TExtendedToolGroup;
   FTaskTools: TExtendedToolGroup;
   FMonitors: TTechnology;
   FFrameSelectors: TTechnology;
   procedure  SetNewGroupName(const Value: string);
   function   GetGroups: TTools;
   procedure  SetGroups(const Value: TTools);
   function   GetMonitors: TTechnology;
   function   GetTasks: TExtendedToolGroup;
   function   GetTaskTools: TExtendedToolGroup;
   function   GetActiveGroup: TExtendedToolGroup;
   procedure  SetActiveGroup(const Value: TExtendedToolGroup);
   function   GetActiveTask: TTool;
   procedure  SetActiveTask(const Value: TTool);
   function   GetFrameSelectors: TTechnology;
   function   GetActiveFrameSelector: TTool;
   procedure  SetActiveFrameSelector(const Value: TTool);
   function GetMenu: TMenuGroup;
    function GetExtForm: TExtendedOrbis_Form;
    function GetAutoDock: boolean;
    procedure SetAutoDock(const Value: boolean);
    procedure SetToolToSelect(const Value: TTool);
protected
   function    GetActiveTool: TTool; override;
   procedure   SetActiveTool(aTool: TTool); override;
   function    FindMenuItem(const aText: string): TMenuItem;
   function    FindMenuItemIn(const aMenu, aItem: string): TMenuItem;
   procedure   CheckMenuItem(const aItem: string);
   function    GetMainMenu: TMainMenu; virtual;
   property    MainMenu: TMainMenu read GetmainMenu;
public
   procedure   CreateForm; override;
   procedure   CreateDynamicPart; override;
   function    AddTool(aTool: TTool): TTool; override;
   function    GetText: string; virtual;
   procedure   SetText(const Value: string); virtual;
   function    GetText1: string; virtual;
   procedure   SetText1(const Value: string); virtual;
   procedure   RemoveTaskTools; virtual;
   procedure   RemoveTasks; virtual;
   procedure   AddTasks(const Task: TTool); virtual;
   procedure   AddTaskTools(const Task: TTool); virtual;
   procedure   on_tm_ExternalTaskSelected(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   on_tm_TaskSelected(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   on_tm_ShowText(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   BeginUpdate; virtual;
   procedure   EndUpdate; virtual;
   property    ActiveGroup: TExtendedToolGroup read GetActiveGroup write SetActiveGroup;
   property    ActiveTask: TTool read GetActiveTask write SetActiveTask;
   property    ActiveFrameSelector: TTool read GetActiveFrameSelector write SetActiveFrameSelector;
   property    ExtForm: TExtendedOrbis_Form read GetExtForm;
published
   property    ToolToSelect: TTool write SetToolToSelect stored false;
   property    Text: string read GetText write SetText;
   property    Text1: string read GetText1 write SetText1;
   property    Groups: TTools read GetGroups write SetGroups stored false;
   property    NewGroupName: string write SetNewGroupName stored false;
   property    Menu: TMenuGroup read GetMenu stored false;
   property    Tasks: TExtendedToolGroup read GetTasks stored false;
   property    TaskTools: TExtendedToolGroup read GetTaskTools stored false;
   property    Monitors: TTechnology read GetMonitors stored false;
   property    FrameSelectors: TTechnology read GetFrameSelectors stored false;
   property    AutoDock: boolean read GetAutoDock write SetAutoDock;
end;

implementation
uses
   {!!!}Dialogs,
   Forms,
   Graphics,
   SysUtils,
   //ToolWin,
   // KERNEL
   f_Dock_Host,
   e_Class,
   e_Type,
   m_Orbis,
   m_Proj,
   u_String,
   // APP
   c_Actions,
   t_Device,
   t_FrameSelect,
   t_OrbisFrame;

{ TExtendedFormTool }

function    TExtendedFormTool.FindMenuItem(const aText: string): TMenuItem;
var
   i: longInt;
   Item: TMenuItem;
begin
   result := nil;
   for i := 0 to MainMenu.Items.Count -1 do
   begin
      Item := MainMenu.Items.Items[i];
      if UpperCase(Item.Name) = UpperCase(aText) then
      begin
         result := Item;
         exit;
      end;
   end;
end;


function    TExtendedFormTool.FindMenuItemIn(const aMenu, aItem: string): TMenuItem;
var
   i: longInt;
   Item: TMenuItem;
   SubItem: TMenuItem;
begin
   result := nil;
   Item := FindMenuItem(aMenu);
   for i := 0 to Item.Count -1 do
   begin
      SubItem := Item.Items[i];
      if UpperCase(SubItem.Name) = UpperCase(aItem) then
      begin
         result := SubItem;
         exit;
      end;
   end;
end;


procedure   TExtendedFormTool.CheckMenuItem(const aItem: string);
begin
   {
   Item := FindMenuItem('TOOLS');
   for i := 0 to Item.Count -1 do
   begin
      SubItem := Item.Items[i];
      if UpperCase(SubItem.Name) = UpperCase(aItem) then SubItem.Checked := true
                                                    else SubItem.Checked := false;
   end;
   }
end;

function    TExtendedFormTool.GetMainMenu: TMainMenu;
begin
  result:= nil;
  if ExtForm = nil then exit;
   result := ExtForm.MainMenu;
end;


procedure TExtendedFormTool.AddTasks(const Task: TTool);
var
   M,MM: TMenuItem;
   i: integer;
begin
   if Tasks = nil then exit;
   Tasks.AddToolsByModel(Task);
   if ExtForm <> nil then
   begin
      M := ExtForm.Tasks;
      if Tasks = nil then exit;
      if Tasks.ActionList = nil then exit;
      for i := Tasks.ActionList.ActionCount - 1 downto 0 do
      begin
         MM := TMenuItem.Create(ExtForm);
         MM.Action := Tasks.ActionList.Actions[i];
         M.Add(MM);
      end;
      M.Visible := true;
   end;
end;

procedure TExtendedFormTool.AddTaskTools(const Task: TTool);
var
   M,MM: TMenuItem;
   i: integer;
begin
   if TaskTools = nil then exit;
   TaskTools.AddToolsByModel(Task);
   if ExtForm <> nil then
   begin
      M := ExtForm.TaskTools;
      M.Caption := Tasks.FActualAction.Caption;
      if TaskTools = nil then exit;
      if TaskTools.ActionList = nil then exit;
      for i := TaskTools.ActionList.ActionCount - 1 downto 0 do
      begin
         MM := TMenuItem.Create(ExtForm);
         MM.Action := TaskTools.ActionList.Actions[i];
         M.Add(MM);
      end;
      M.Visible := true;
   end;
end;

function TExtendedFormTool.AddTool(aTool: TTool): TTool;
begin
   result := inherited AddTool(aTool);
   if result is TExtendedToolGroup then
   begin
      TExtendedToolGroup(result).exForm := TExtendedOrbis_Form(aForm);
      if result is TMenuGroup then FMenu := TMenuGroup(result) else
      if UpperCase(result.Name) = 'TASKS' then FTasks := TExtendedToolGroup(result) else
      if UpperCase(result.Name) = 'TASKTOOLS' then FTaskTools := TExtendedToolGroup(result)else
   end
   else
   if result is TTechnology then
   begin
      if UpperCase(result.Name) = 'FRAMESELECTORS' then
         FFrameSelectors := TTechnology(result)
      else
      if UpperCase(result.Name) = 'MONITORS' then
         FMonitors := TTechnology(result);
   end;
end;

procedure TExtendedFormTool.CreateForm;
begin
   aForm := TExtendedOrbis_Form.Create(Application);
   ExtForm.Tool := Self;
   ExtForm.IsPossibleToClose := false;
end;



function TExtendedFormTool.GetActiveFrameSelector: TTool;
begin
   result := nil;
   if FrameSelectors = nil then exit;
   result := FrameSelectors.ActiveTool;
end;

function TExtendedFormTool.GetActiveGroup: TExtendedToolGroup;
var
   T: TTool;
begin
   result := nil;
   T := inherited GetActiveTool;
   if not (T is TExtendedToolGroup) then exit;
   result := TExtendedToolGroup(T);
end;

function TExtendedFormTool.GetActiveTask: TTool;
begin
   result := nil;
   if Tasks = nil then exit;
   result := Tasks.ActiveTool;
end;

function TExtendedFormTool.GetActiveTool: TTool;
var
   G: TExtendedToolGroup;
begin
   result := nil;
   G := ActiveGroup;
   if G = nil then exit;
   result := G.ActiveTool;
end;

function TExtendedFormTool.GetFrameSelectors: TTechnology;
begin
   result := FFrameSelectors;
end;

function TExtendedFormTool.GetGroups: TTools;
begin
   result := Tools;
end;

function TExtendedFormTool.GetMenu: TMenuGroup;
begin
   result := FMenu;
end;

function TExtendedFormTool.GetMonitors: TTechnology;
begin
   result := FMonitors;
end;

function TExtendedFormTool.GetTasks: TExtendedToolGroup;
begin
   result := FTasks;
end;

function TExtendedFormTool.GetTaskTools: TExtendedToolGroup;
begin
   result := FTaskTools;
end;

function TExtendedFormTool.GetText: string;
begin
   result := '';
end;

procedure TExtendedFormTool.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
var
   i: integer;
begin
   //if aMessage.Message = tm_Resize then
   //   PostTextMsg(tm_AddToProtocol,Name + '.on_Resize',nil);
   Handled := false;
   try
     if (Monitors <> nil) and not(Monitors.NoTools) then
     begin
        for i := 0 to Monitors.Tools.LastIndex do
        begin
           Monitors.Tool[i].HandleMessage(aMessage, Handled);
        end;
     end;
     if (ActiveTool <> nil) then
     begin
        case aMessage.Message  of
           tm_Review, tm_Cursor: ;// udlosti, kter se nepedvaj aktivnmu nstroji
        else
           ActiveTool.HandleMessage(aMessage, Handled);
        end;
     end;
   except
   end;
   if Handled then exit;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_ExternalTaskSelected: on_tm_ExternalTaskSelected(aMessage, Handled);
      tm_TaskSelected:         on_tm_TaskSelected(aMessage, Handled);
      tm_ToolSelected:         on_tm_ToolSelected(aMessage, Handled);
      tm_ShowText:             on_tm_ShowText(aMessage, Handled);
   end;
end;


procedure TExtendedFormTool.on_tm_ExternalTaskSelected(var aMessage: TOrbisMessage; var Handled: boolean);
var
   T: TTool;
   S: string;
begin
   BeginUpdate;
   if (ActiveTool <> nil) and (ActiveTool is TInterfaceTool) then
   begin
      if not TInterfaceTool(ActiveTool).IsPossibleToDeactivate then exit;
   end;
   T := TTool(aMessage.Data);
   S := 'No Tool';
   if T <> nil then S := T.Name;
   //PostTextMsg(tm_AddToProtocol, Name+'.on_ExternalTask: '+S,nil);
   //ShowMessage('Remove TaskTools');
   RemoveTaskTools;
   //ShowMessage('Remove Tasks');
   RemoveTasks;
   AddTasks(T);
   ActiveTask := nil;
   if ExtForm = nil then exit;
   ExtForm.MenuBar.Menu := nil;
   ExtForm.MenuBar.Menu := MainMenu;
   EndUpdate;
end;

procedure TExtendedFormTool.on_tm_ShowText(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   if not (aMessage is TOrbisTextMessage) then exit;
   Text := TOrbisTextMessage(aMessage).Text;
end;

procedure TExtendedFormTool.on_tm_TaskSelected(var aMessage: TOrbisMessage;var Handled: boolean);
var
   T: TTool;
begin
   BeginUpdate;
   RemoveTaskTools;
   T := TTool(aMessage.Data);
   if T = nil then
   begin
      if ActiveTask <> nil then
      begin
         if ActiveTask.DynamicTool then ActiveTask.DestroyDynamicPart;
         Tasks.FActualAction.Checked := true;
         Tasks.FActualAction.Checked := false;
         ActiveTask := nil;
      end;
      exit;
   end;
   try
     if not T.Idle = true then
        T.DoIt;
   except
   end;
   if ActiveTask <> nil then
   begin
      if ActiveTask.DynamicTool then ActiveTask.DestroyDynamicPart;
      ActiveTask := nil;
   end;
   if T.DynamicTool then T.CreateDynamicPart;
   AddTaskTools(TTool(aMessage.Data));
   ActiveTask := T;
   Tasks.FActualAction.Checked := true;
   if ExtForm = nil then exit;
   ExtForm.MenuBar.Menu := nil;
   ExtForm.MenuBar.Menu := MainMenu;
   EndUpdate;
end;

procedure TExtendedFormTool.on_tm_ToolSelected(var aMessage: TOrbisMessage;var Handled: boolean);
var
   G: TExtendedToolGroup;  // skupina noveho nastroje
   T: TTool;               // novy nastroj
   IT: TInterfaceTool;     // novy nastroj jako IT (Interface tool)
   A: TAction;             // akce noveho nastroje
   IsAT: boolean;          // Aktivni nastroj je nenilovy
   IsIAT: boolean;         // Aktivni nastroj je nenilovy a je to IT
   AT: TTool;              // Aktivni nastroj
   IAT: TInterfaceTool;    // Aktivni nastroj jako IT
   Flash: boolean;
   FlashReturn: boolean;
   FlashChange: boolean;
   FlashAbort: boolean;
   NoAction: boolean;
   NoActualAction: boolean;
begin
   Flash := false;
   FlashReturn := false;
   FlashChange := false;
   FlashAbort := false;
   NoAction := false;
   NoActualAction := true;
   T := TTool(aMessage.Data);
   // deaktivace
   if T = nil then
   begin
      if ActiveTool = nil then exit;
      if ActiveGroup.FActualAction <> nil then
         ActiveGroup.FActualAction.Checked := false;
      ActiveTool := nil;
      Cursor := crDefault;
      exit;
   end;
   // naplneni promennych pro pristup k Aktivnimu nastroji
   AT := ActiveTool;
   IsAT := AT <> nil;
   IsIAT := IsAt and (AT is TInterfaceTool);
   if IsIAT then IAT := TInterfaceTool(AT) else IAT := nil;

   G := TExtendedToolGroup(T.FindOwnerClass(TExtendedToolGroup));
   if G = nil then
   begin
      PostTextMsg(tm_AddToprotocol,'Tool: '+ T.Name + ' is not component of '+ Name,nil);
      exit;
   end;
   A := G.FindActionByTool(T);
   NoAction := A = nil;
   if G = Tasks then // Tool T je TASK
   begin
      Tasks.FActualAction.Checked := false;
      if (T = ActiveTask) then
      begin
         Tasks.FActualAction.Checked := true;
         exit;
      end;
      if IsIAT and not IAT.IsPossibleToDeactivate then
      begin
         if not NoAction then
         begin
            A.Checked := true;
            A.Checked := false;
         end;
         Tasks.FActualAction.Checked := true;
         exit;
      end;
      Tasks.FActualAction := A;
      PostMsg(tm_TaskSelected,Self,T,false);
      exit;
   end;
   if IsAt then NoActualAction := ActiveGroup.FActualAction = nil;

   if not (T is TInterfaceTool) then
   begin
      if not NoAction then A.Checked := true;
      T.DoIt;
      if not NoAction then A.Checked := false;
      if not NoActualAction then
         if IsAT then ActiveGroup.FActualAction.Checked := true;
      exit;
   end
   else
   if isAT then
   begin
      if not NoActualAction then
         ActiveGroup.FActualAction.Checked := false;
      if T = AT then
      begin
         if not NoActualAction then
            ActiveGroup.FActualAction.Checked := true;
         exit;
      end;
      // een flash voln
      IT := TInterfaceTool(T);
      if IAT.AllowFlash and IT.CanBeFlash then
         Flash := true // bude flash tool
      else
      if IAT.CanBeFlash and (IAT.FlashTool <> nil) and IT.AllowFlash then
         if (IAT.FlashTool = IT) then
            FlashReturn := true // flash return
         else
            FlashAbort := true // flashAbort
      else
      if IAT.CanBeFlash and (IAT.FlashTool <> nil) and IT.CanBeFlash then
         FlashChange := true;
      // Zeptame se Aktivniho nastroje, jestli se mu chce koncit
      if not Flash then
      if (FlashAbort and not IAT.FlashTool.IsPossibleToDeactivate)
         or not IAT.IsPossibleToDeActivate then
      begin
         // ...asi nechce
         if not NoAction then
         begin
           A.Checked := true;
           A.Checked := false;
         end;
         if not NoActualAction then
            ActiveGroup.FActualAction.Checked := true;
         exit;
      end;
   end;
   if Flash then
   begin
      IAT.FlashTool := IT;
      IT.FlashTool := IAT;
      IAT.StartFlashTool;
   end;
   if FlashChange then
   begin
      IT.FlashTool := IAT.FlashTool;
      IT.FlashTool.FlashTool := IT;
   end;
   if FlashReturn then
   begin
      IAT.FlashTool := nil;
      IT.FlashTool := nil;
   end;
   if not NoAction then
   begin
      A.Checked := true;
      if (Flash or FlashChange) then
         Text := A.Hint + ' > F'
      else
         Text := A.Hint;
      Text1 := '';
   end;

   G.FActualAction := A;
   IT := TInterfaceTool(T);
   if FlashAbort then
      IAT.FlashTool.On_DeActivate; // deaktivace zkl. nstroje
   if not Flash and IsIAT then              // pokud je Aktivni nastroj Interface Tool...
      IAT.On_DeActivate;      // ... tak ho deaktivujeme

   Cursor := crDefault;
   ActiveTool := IT;
   //Cursor := IT.Cursor;
   if not FlashReturn then
   begin
      IT.Device := Device;
      IT.Input  := Self.Input;
      if IT is TFrameTool then
      begin
         if TFrameTool(IT).FrameName <> '' then
         begin
            if FrameSelectors <> nil then
               ActiveFrameSelector := FrameSelectors.GetToolByName(TFrameTool(IT).FrameName);
         end;
         TFrameTool(IT).FrameSelector := TFrameSelector(ActiveFrameSelector);
      end;
      IT.On_Activate;             // a noveho chlapce aktivujeme
   end
   else
      IT.EndFlashTool;
   Handled := true;
   //CheckMenuItem(ActiveTool.ClassName);
   //FormCursor := TInterActiveTool(ActiveTool).Cursor;
end;

procedure TExtendedFormTool.RemoveTasks;
var
   M: TMenuItem;
begin
   if ExtForm <> nil then
   begin
      M := ExtForm.Tasks;
      While M.Count > 0 do M.Items[0].Free;
      M.Visible := false;
   end;
   if Tasks = nil then exit;
   Tasks.Empty := true;
end;

procedure TExtendedFormTool.RemoveTaskTools;
var
   M: TMenuItem;
begin
   if ExtForm <> nil then
   begin
      M := ExtForm.TaskTools;
      While M.Count > 0 do M.Items[0].Free;
      M.Caption := '-';
      M.Visible := false;
   end;
   if TaskTools = nil then exit;
   TaskTools.Empty := true;
end;

procedure TExtendedFormTool.SetActiveFrameSelector(const Value: TTool);
begin
   if FrameSelectors = nil then exit;
   FrameSelectors.ActiveTool := Value;
end;

procedure TExtendedFormTool.SetActiveGroup(const Value: TExtendedToolGroup);
begin
   inherited SetActiveTool(Value);
end;

procedure TExtendedFormTool.SetActiveTask(const Value: TTool);
begin
   if Tasks = nil then exit;
   Tasks.ActiveTool := Value;
end;

procedure TExtendedFormTool.SetActiveTool(aTool: TTool);
var
   G: TExtendedToolGroup;
begin
   if aTool = nil then
   begin
      ActiveGroup := nil;
      exit;
   end;
   G := TExtendedToolGroup(aTool.FindOwnerClass(TExtendedToolGroup));
   if G = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Tool '+aTool.Name+' is not component of '+ Name,nil);
      exit;
   end;
   ActiveGroup := G;
   G.ActiveTool := aTool;
end;

procedure TExtendedFormTool.SetGroups(const Value: TTools);
begin
   Tools := Value;
end;

procedure TExtendedFormTool.SetNewGroupName(const Value: string);
var
   G: TExtendedToolGroup;
begin
   if GetToolByName(Value) <> nil then
   begin
      PostTextMsg(tm_AddToProtocol, 'Group named '+ Value +' already exists !',nil);
      exit;
   end;
   G := TExtendedToolGroup.Create;
   G.Name := Value;
   AddTool(G);
end;


procedure TExtendedFormTool.SetText(const Value: string);
begin

end;

function TExtendedFormTool.GetExtForm: TExtendedOrbis_Form;
begin
   result := TExtendedOrbis_Form(aForm);
end;

procedure TExtendedFormTool.CreateDynamicPart;
begin
   ExtForm.TopBar.DisableAlign;
   inherited;
   ExtForm.TopBar.EnableAlign;
//   PostMsg(tm_resize, self, nil, false)
end;

function TExtendedFormTool.GetAutoDock: boolean;
begin
   result := ExtForm.TopBar.AutoDock;
end;

procedure TExtendedFormTool.SetAutoDock(const Value: boolean);
begin
   if ExtForm = nil then exit;
   ExtForm.TopBar.AutoDock := Value;
   ExtForm.BottomBar.AutoDock := Value;
end;

procedure TExtendedFormTool.BeginUpdate;
begin

end;

procedure TExtendedFormTool.EndUpdate;
begin

end;

procedure TExtendedFormTool.SetToolToSelect(const Value: TTool);
begin
   PostMsg(tm_ToolSelected,Self,Value,false);
   Application.ProcessMessages;
end;

function TExtendedFormTool.GetText1: string;
begin
   result := '';
end;

procedure TExtendedFormTool.SetText1(const Value: string);
begin

end;

{ TExtendedToolGroup }

procedure TExtendedToolGroup.ActionExecute(Sender: TObject);
begin
   PostMsgToOwners(tm_ToolSelected,TToolAction(Sender).Tool,false);
   //PostTextMsg(tm_AddToProtocol,'SelectedTool: '+TToolAction(Sender).Name,nil);
end;


procedure TExtendedToolGroup.AddToolByModel(aTool: TTool);
var
   aModel: TToolModel;
begin
   if aTool = nil then exit;
   AddTool(aTool);
   aModel := aTool.SelfModel;
   if aModel = nil then exit;
   NewToolAction := aTool.Name;
   ActualTool := aTool;
   ActualCaption := aModel.Alias;
   ActualHint := aModel.Comment;
   ActualImageIndex := aModel.ImageIndex;
   ActualShortCut := aModel.ShortCut;
end;

procedure TExtendedToolGroup.AddToolsByModel(aTool: TTool);
var
   M: TToolModel;
   S: TStringListElement;
   i: integer;
   T: TTechnology;
   TB: TToolBar;
   DH: TDock_Host;
begin
   if aTool = nil then exit;
   if not (aTool is TTechnology) then exit;
   T := TTechnology(aTool);
   if T.Tools.NoElements then exit;
   M := T.SelfModel;
   if M = nil then exit;
   if not M.NoImages then
   begin
      try
         S := TStringListElement(M.Images);
         for i := 0 to S.LastIndex do
         begin
            NewImage := S.Strings[i];
         end;
      except
      end;
   end;
   for i := 0 to T.Tools.LastIndex do
   begin
      AddToolByModel(T.Tool[i]);
   end;
   TB := ToolBar;
   if TB <> nil then
   if TB.Floating then
   if (TB.HostDockSite <> nil) and (TB.HostDockSite is TDock_Host) then
   begin
      DH := TDock_Host(TB.HostDockSite);
//      DH.AutoSize := false;
      DH.Height := 1000;
      DH.Height := T.Tools.Count * (TB.ButtonHeight + TB.Indent + 2) + 40;
      TB.HostDockSite.Width  := TB.ButtonWidth + TB.Indent * 2 + 15;
//      TB.HostDockSite.Height := TB.Height + 10;
//      DH.Width  := TB.Width + 10;
//      DH.AutoSize := true;
   end;

end;


procedure TExtendedToolGroup.CreateSelfByModel;
var
   M: TToolModel;
   S: TStringListElement;
   i: integer;
begin
   M := SelfModel;
   if M = nil then exit;
   inherited;
   if not M.NoImages then
   begin
      try
         S := TStringListElement(M.Images);
         for i := 0 to S.LastIndex do
         begin
            NewImage := S.Strings[i];
         end;
      except
      end;
   end;
end;

function TExtendedToolGroup.CreateToolByModel(aModel: TToolModel): TTool;
var
   ET: TExtendedFormTool;
begin
   result := nil;
   if aModel = nil then exit;
   if (aModel.Name = '.') or (aModel.Name = '|') then
   begin
      NewSeparator := aModel.Name;
      exit;
   end;
   result := inherited CreateToolByModel(aModel);
   if result = nil then exit;
   if result is TControlTool then exit;
   if result is TOrbisFrame then
   begin
      ET := TExtendedFormTool(FindClassInOwners(TExtendedFormTool));
      if ET <> nil then
         TOrbisFrame(result).Device := ET.Device;
      exit;
   end;
   NewToolAction := result.Name;
   ActualTool := result;
   ActualCaption := aModel.Alias;
   ActualHint := aModel.Comment;
   ActualImageIndex := aModel.ImageIndex;
end;

function TExtendedToolGroup.FindActionByTool(aTool: TTool): TAction;
var
   i: integer;
   A: TAction;
begin
   result := nil;
   for i := 0 to ActionList.ActionCount - 1 do
   begin
      A := TAction(ActionList.Actions[i]);
      if not (A is TToolAction) then continue;
      if TToolAction(A).Tool = aTool then
      begin
         result := A;
         exit;
      end;
   end;
end;

function TExtendedToolGroup.GetActualCaption: string;
begin
   result := '';
   if FactualAction <> nil then result := FActualAction.Name;
end;

function TExtendedToolGroup.GetActualHint: string;
begin
   result := '';
   if FactualAction <> nil then result := FActualAction.Hint;
end;

function TExtendedToolGroup.GetActualImageIndex: integer;
begin
   result := -1;
   if FactualAction <> nil then result := FActualAction.ImageIndex;
end;

function TExtendedToolGroup.GetActualScript: string;
begin
   result := '';
   if FactualAction = nil then exit;
   if not (FActualAction is TScriptAction) then exit;
   result := TScriptAction(FActualAction).Script;
end;

function TExtendedToolGroup.GetActualTool: TTool;
begin
   result := nil;
   if FactualAction = nil then exit;
   if not (FActualAction is TToolAction) then exit;
   result := TToolAction(FActualAction).Tool;
end;

function TExtendedToolGroup.GetActualVisible: boolean;
begin
   result := false;
   if FactualAction <> nil then result := FActualAction.Visible;
end;

function TExtendedToolGroup.GetButtonHeight: integer;
begin
   result := 0;
   if ToolBar <> nil then result := ToolBar.ButtonHeight;
end;

function TExtendedToolGroup.GetButtonWidth: integer;
begin
   result := 0;
   if ToolBar <> nil then result := ToolBar.ButtonWidth;
end;

function TExtendedToolGroup.GetFlat: boolean;
begin
   result := false;
   if ToolBar <> nil then result := ToolBar.Flat;
end;


function TExtendedToolGroup.GetImageHeight: integer;
begin
   result := 0;
   if ImageList <> nil then result := ImageList.Height;
end;

function TExtendedToolGroup.GetImageWidth: integer;
begin
   result := 0;
   if ImageList <> nil then result := ImageList.Width;
end;


function TExtendedToolGroup.GetShowCaptions: boolean;
begin
   result := false;
   if ToolBar <> nil then result := ToolBar.ShowCaptions;
end;


function TExtendedToolGroup.GetWrapable: boolean;
begin
   result := false;
   if ToolBar <> nil then result := ToolBar.Wrapable;
end;

procedure TExtendedToolGroup.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   OwnersHandleMessage(aMessage,Handled);
end;

procedure TExtendedToolGroup.SetActionList(const Value: TActionList);
begin
   FActionList := Value;
end;

procedure TExtendedToolGroup.SetActualCaption(const Value: string);
begin
   if FactualAction <> nil then FActualAction.Caption := Value;
end;

procedure TExtendedToolGroup.setActualHint(const Value: string);
begin
   if FactualAction <> nil then FActualAction.Hint := Value;
end;

procedure TExtendedToolGroup.SetActualImageIndex(const Value: integer);
begin
   if FactualAction <> nil then FActualAction.ImageIndex := Value;
end;

procedure TExtendedToolGroup.SetActualScript(const Value: string);
begin
   if FActualAction = nil then exit;
   if not(FActualAction is TScriptAction) then exit;
   TScriptAction(FActualAction).Script := Value;
end;

procedure TExtendedToolGroup.SetActualTool(const Value: TTool);
begin
   if FActualAction = nil then exit;
   if not(FActualAction is TToolAction) then exit;
   TToolAction(FActualAction).Tool := Value;
end;

procedure TExtendedToolGroup.SetActualVisible(const Value: boolean);
begin
   if FactualAction <> nil then FActualAction.Visible := Value;
end;

procedure TExtendedToolGroup.SetAlign(const Value: TAlign);
begin
   if Value = FAlign then exit;
   case Value of
      alTop:    ToolBar.Parent := FExForm.TopBar;
      alBottom: ToolBar.Parent := FExForm.BottomBar;
      alLeft:   ToolBar.Parent := FExForm.LeftBar;
      alRight:  ToolBar.Parent := FExForm.RightBar;
   end;
   ToolBar.Align := Value;
   FAlign := Value;
end;

procedure TExtendedToolGroup.SetButtonHeight(const Value: integer);
begin
   if ToolBar <> nil then ToolBar.ButtonHeight := Value;
end;

procedure TExtendedToolGroup.SetButtonWidth(const Value: integer);
begin
   if ToolBar <> nil then ToolBar.ButtonWidth := Value;
end;

procedure TExtendedToolGroup.SetEmpty(const Value: boolean);
var
   V: boolean;
begin
   //ShowMessage(Name + ': Destroy Comps');
   V := ToolBar.ShowCaptions;
   ToolBar.ShowCaptions := false;
   //Application.ProcessMessages;
   ToolBar.DestroyComponents;
   ToolBar.ShowCaptions := V;
   //ShowMessage('Destroy Actions');
   ActionList.DestroyComponents;
   //ShowMessage('Clearing Imgs');
   ImageList.Clear;
   //ShowMessage('Free Tools');
   Tools.FreeAll;
   //ShowMessage('After Free Tools');
   FActualAction:= nil;
   ActiveTool:= nil
end;

procedure TExtendedToolGroup.SetexForm(const Value: TExtendedOrbis_Form);
begin
   FexForm := Value;
   if Value = nil then exit;
   ActionList := TActionList.Create(FExForm);
   ActionList.Name := 'al_' + Name;
   ImageList  := TImageList.Create(FExForm);
   ImageList.Name := 'img_' + Name;
   ActionList.Images := ImageList;
   ToolBar := TToolBar.Create(FExForm);
   ToolBar.Indent := 2;
   ToolBar.Name := 'tb_' + Name;
   ToolBar.Cursor := crDrag;
   ToolBar.AutoSize := true;
   ToolBar.ShowHint := true;
   ToolBar.AutoSize := true;
//   ToolBar.
   //ToolBar.Transparent := true;
   //ToolBar.EdgeBorders := [ebLeft,ebRight,ebTop,ebBottom];
   ToolBar.EdgeBorders := [];
   ToolBar.DragKind := dkDock;
   //ToolBar.DragMode := dmAutomatic;
   ToolBar.DragMode := dmManual;
   FExForm.TopBar.InsertControl(ToolBar);
   ToolBar.Images := ImageList;
end;

procedure TExtendedToolGroup.SetFindActionName(const Value: string);
var
   i: integer;
   A: TAction;
begin
   FActualAction := nil;
   for i := 0 to ActionList.ActionCount - 1 do
   begin
      A := TAction(ActionList.Actions[i]);
      if CompareText(Value, A.Name) = 0 then
      begin
         FActualAction := A;
         break;
      end;
   end;
end;

procedure TExtendedToolGroup.SetFlat(const Value: boolean);
begin
   if ToolBar <> nil then ToolBar.Flat := Value;
end;


procedure TExtendedToolGroup.SetImageHeight(const Value: integer);
begin
   if ImageList = nil then exit;
   ImageList.Height := Value;
end;

procedure TExtendedToolGroup.SetImageList(const Value: TImagelist);
begin
   FImageList := Value;
end;

procedure TExtendedToolGroup.SetImageWidth(const Value: integer);
begin
   if ImageList = nil then exit;
   ImageList.Width := Value;
end;


procedure TExtendedToolGroup.SetNewImage(const Value: string);
var
   BM,BM1: Graphics.TBitmap;
   S: string;
   BColor: TColor;
begin
   BM := Graphics.TBitmap.Create;
   S := '';
   S := GetAbsolutePath(Value);
   try
      BM.LoadFromFile(S);
      BColor := BM.Canvas.Pixels[0,0];
   except
      BColor := clGray;
   end;
   if (BM.Height <> ImageHeight) or (BM.Width <> ImageWidth) then
   begin
      BM1 := Graphics.TBitmap.Create;
      BM1.Height := ImageHeight;
      BM1.Width := ImageWidth;
      BM1.Canvas.StretchDraw(Rect(0,0,ImageHeight,ImageWidth),BM);
      ImageList.AddMasked(BM1,BColor);
   end
   else
      ImageList.AddMasked(BM,BColor);
end;

procedure TExtendedToolGroup.SetNewScriptAction(const Value: string);
var
   A: TAction;
   B: TToolButton;
begin
   A := TScriptAction.Create(ActionList);
   A.ActionList := ActionList;
   FActualAction := A;
   B := TToolButton.Create(ToolBar);
   ToolBar.InsertControl(B);
   //ToolBar.EdgeInner :=
   //ToolBar.EdgeOuter :=
   B.Action := A;
   B.Style := tbsCheck;
   A.DisableIfNoHandler := false;
end;

procedure TExtendedToolGroup.SetNewSeparator(const Value: string);
var
   B: TToolButton;
begin
   B := TToolButton.Create(ToolBar);
   ToolBar.InsertControl(B);
   if Value = '|' then B.Style := tbsDivider
                  else B.Style := tbsSeparator;
   B.Width := 10;
end;

procedure TExtendedToolGroup.SetNewToolAction(const Value: string);
var
   A: TAction;
   B: TToolButton;
begin
   A := TToolAction.Create(ActionList);
   A.ActionList := ActionList;
   FActualAction := A;
   A.OnExecute := ActionExecute;
   B := TToolButton.Create(ToolBar);
   ToolBar.InsertControl(B);
   B.Action := A;
   B.Style := tbsCheck;
   B.Cursor := crHandPoint;
   B.OnContextPopup := ActionPopUp;
   A.DisableIfNoHandler := false;
end;



procedure TExtendedToolGroup.SetShowCaptions(const Value: boolean);
begin
   if ToolBar <> nil then ToolBar.ShowCaptions := Value;
end;

procedure TExtendedToolGroup.SetToolBar(const Value: TToolbar);
begin
  Control := Value;
end;


procedure TExtendedToolGroup.SetWrapable(const Value: boolean);
begin
   if ToolBar <> nil then ToolBar.Wrapable := Value;
end;



function TExtendedToolGroup.GetToolBar: TToolbar;
begin
   result := TToolBar(Control);
end;

function TExtendedToolGroup.GetLeft: integer;
begin
   result := inherited GetLeft;
   //if result < 12 then result := 0;
end;

function TExtendedToolGroup.GetTop: integer;
begin
   result := inherited GetTop;
   //if result < 3 then result := 0;
end;

function TExtendedToolGroup.GetAutoDock: boolean;
begin
   result := false;
   if ToolBar <> nil then
      result := ToolBar.DragMode = dmAutomatic;
end;

procedure TExtendedToolGroup.SetAutoDock(const Value: boolean);
begin
   if ToolBar = nil then exit;
   if Value then ToolBar.DragMode := dmAutomatic
            else ToolBar.DragMode := dmManual;
end;

function TExtendedToolGroup.GetEmpty: boolean;
begin
   result := NoTools;
end;

function TExtendedToolGroup.GetActualShortCut: string;
begin
   result := '';
   if FactualAction <> nil then
   begin
      result := ShortCutToText(FActualAction.ShortCut);
   end;
end;

procedure TExtendedToolGroup.SetActualShortCut(const Value: string);
begin
   if FactualAction <> nil then FActualAction.ShortCut := TextToShortCut(Value);
end;

procedure TExtendedToolGroup.ActionPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
begin
   ShowMessage('Hint: '+TToolAction(TToolButton(Sender).Action).Hint);
end;

{ TMenuGroup }

function TMenuGroup.GetMenuBar: TMenuBar;
begin
   result := TMenuBar(ToolBar);
end;

procedure TMenuGroup.SetexForm(const Value: TExtendedOrbis_Form);
begin
   FexForm := Value;
   if Value = nil then exit;
   ToolBar := Value.MenuBar;
end;

{ TOrbisShortCutScript }

procedure TOrbisShortCutScript.DoIt;
begin
   PostTextMsg(tm_Run,Script,nil);
end;

function TOrbisShortCutScript.GetName: string;
begin
   result := ShortCut + ' ' + Comment;
end;

function TOrbisShortCutScript.GetRealShortCut: string;
begin
   result := ShortCutToText(ShortCut_);
end;

procedure TOrbisShortCutScript.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TOrbisShortCutScript.SetKey(const Value: Word);
begin
  FKey := Value;
end;

procedure TOrbisShortCutScript.SetName(aName: string);
var
   S1,S2: string;
begin
   if StringAsTwoParams(aName,'|',S1,S2) then
   begin
      ShortCut := S1;
      COmment := S2;
   end
   else
   begin
      Comment := S1;
   end;
end;

procedure TOrbisShortCutScript.SetScript(const Value: string);
begin
  FScript := Value;
end;

procedure TOrbisShortCutScript.SetShift(const Value: TShiftState);
begin
  FShift := Value;
end;

procedure TOrbisShortCutScript.SetShortCut(const Value: string);
begin
   FShortCut := Value;
   ShortCut_ := TextToShortCut(Value);
   ShortCutToKey(ShortCut_, FKey, FShift);
end;

procedure TOrbisShortCutScript.SetShortCut_(const Value: TShortCut);
begin
  FShortCut_ := Value;
end;

{ TOrbisShortCutScripts }

function TOrbisShortCutScripts.FindByShortCut(aValue: TShortCut): TOrbisShortCutScript;
var
   i: integer;
   e: TElement;
   OS: TOrbisShortCutScript;
begin
   result := nil;
   for i := 0 to LastIndex do
   begin
      e := Elements[i];
      if not (e is TOrbisShortCutScript) then continue;
      OS := TOrbisShortCutScript(e);
      if OS.ShortCut_ = aValue then
      begin
         result := OS;
         break;
      end;
   end;
end;

function TOrbisShortCutScripts.GetActualComment: string;
var
   OS: TOrbisShortCutScript;
begin
   result := '';
   OS := ActualShortCutScript;
   if OS = nil then exit;
   result := OS.Comment;
end;

function TOrbisShortCutScripts.GetActualScript: string;
var
   OS: TOrbisShortCutScript;
begin
   result := '';
   OS := ActualShortCutScript;
   if OS = nil then exit;
   result := OS.Script;
end;

function TOrbisShortCutScripts.GetActualShortCut: string;
var
   OS: TOrbisShortCutScript;
begin
   result := '';
   OS := ActualShortCutScript;
   if OS = nil then exit;
   result := OS.ShortCut;
end;

procedure TOrbisShortCutScripts.On_tm_Keydown(var aMessage: TOrbisKeyEvent;  var Handled: boolean);
var
   ShCut: TShortCut;
   OS: TOrbisShortCutScript;
begin
   ShCut := Menus.ShortCut(aMessage.Key, aMessage.Shift);
   OS := FindByShortCut(ShCut);
   if OS <> nil then
      OS.DoIt;
end;

procedure TOrbisShortCutScripts.SetActualComment(const Value: string);
var
   OS: TOrbisShortCutScript;
begin
   OS := ActualShortCutScript;
   if OS = nil then exit;
   OS.Comment := Value;
end;

procedure TOrbisShortCutScripts.SetActualScript(const Value: string);
var
   OS: TOrbisShortCutScript;
begin
   OS := ActualShortCutScript;
   if OS = nil then exit;
   OS.Script := Value;
end;

procedure TOrbisShortCutScripts.SetActualShortCut(const Value: string);
var
   OS: TOrbisShortCutScript;
begin
   OS := ActualShortCutScript;
   if OS = nil then exit;
   OS.ShortCut := Value;
end;

procedure TOrbisShortCutScripts.SetActualShortCutScript(const Value: TOrbisShortCutScript);
begin
  FActualShortCutScript := Value;
end;

procedure TOrbisShortCutScripts.SetNewShortScriptCutName(const Value: string);
var
   OS: TOrbisShortCutScript;
begin
   OS := TOrbisShortCutScript.Create;
   OS.Name := Value;
   AddElement(OS);
   ActualShortCutScript := OS;
end;

initialization
   RegisterOrbisClass(TExtendedToolGroup);
   RegisterOrbisClass(TMenuGroup);
   RegisterOrbisClass(TExtendedFormTool);
   RegisterOrbisClass(TOrbisShortCutScript);
   RegisterOrbisClass(TOrbisShortCutScripts);
end.

