unit Orbis01;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  shlObj,
  ExtCtrls,
  ActnList,
  ComCtrls,
  Elements,
  t_GraphicEditors,
  e_LHCModel,
  e_KatastrModel,
  e_LHCProject,
  e_Orbis01Projects,
  e_ProjectionInfo,
  e_ThematicMap,
  i_Basic,
  g_Elems,
  c_Actions,
  Orbis01PubUnits,
  t_LHCPositionMonitor,
  t_UserLHCAreaDialog,
  r_OrbisGraphicFrame,
  Tools;

const
//Error codes

   err_Orbis                = 1; // systmov chyba ORBISu
   err_LHCNotConected       = 2; // Grafika poadovanho LHC nen pipojena
   err_InCorrectLHCNumber   = 3; // blb slo LHC (<=0)
   err_Projects             = 4; // chyb seznam projekt
   err_InCorrectProjectPath = 5; // chybn cesta zdrojovch dat (neexistujc XML,nejsou bloky DET ap.)
   err_Cont_LHCNotConected  = 6; // LHC neni pripojeno pro souvisle zobrazeni
   err_Cont_Other           = 7; // chyba v souborech souvisleho zobrazeni
   err_SelectNotFound       = 8; // Vyber daneho nazvu nebyl nalezen
   err_LHCChangeError       = 9; // Zmena LHC nebo roku na zaznam ktery uz existuje

type

TOrbis01 = class;

TViewerAction = class(TToolAction)
public
   procedure CreateTool; override;
end;

TShiftImageAction = class(TToolAction)
public
   procedure CreateTool; override;
end;

Tgis_SkupinaSelectAction = class(TToolAction)
public
   procedure CreateTool; override;
end;

Tgis_ParcelaSelectAction = class(TToolAction)
public
   procedure CreateTool; override;
end;


TMeasureLengthAction = class(TToolAction)
public
   procedure CreateTool; override;
end;

TMeasureAreaAction = class(TToolAction)
  private
    FUsePattern: boolean;
    FUserScript: string;
    procedure SetUsePattern(const Value: boolean);
    procedure SetUserScript(const Value: string);
public
   procedure CreateTool; override;
published
   property UsePattern: boolean read FUsePattern write SetUsePattern;
   property UserScript: string read FUserScript write SetUserScript;
end;

TRasterManipulatorAction = class(TToolAction)
  private
    Fed_RasterLayerName: string;
    Fed_RasterDirectory: string;
    function Geted_RasterLayerName: string;
    procedure Seted_RasterLayerName(const Value: string);
    procedure Seted_RasterDirectory(const Value: string);
    function Geted_RasterDirectory: string;
public
   procedure CreateTool; override;
   procedure AddRasterFile(const aRasterFile: string);
   function  FreeRaster(const aRasterFile: string): boolean;
   procedure AddRastersForView;
   procedure FreeAllRasters;
   procedure FillRastersList(aList: TStrings);
published
   property ed_RasterLayerName: string read Geted_RasterLayerName write Seted_RasterLayerName;
   property ed_RasterDirectory: string read Geted_RasterDirectory write Seted_RasterDirectory;
end;

TOnUserLineDialog_Event = procedure (Karto: boolean; Length: double; CoordCount: integer; var Snap,SnapToLine,SnapToLinePoint,SnapToLineBody: boolean; var SnapDistance: double; DruhyList: string; var Druh,Barva,Znacka,Presnost,aresult: integer) of object;


TPolyLineEditorAction = class(TToolAction)
  private
    FSnapDistance: double;
    FSnapToLinePoint: boolean;
    FSnapToLineBody: boolean;
    Fed_CanCreateNewObject: boolean;
    Fed_SnapToLine: boolean;
    Fed_SnapLevels: string;
    Fed_LevelName: string;
    Fed_DruhyList: string;
    Fed_Presnost: integer;
    Fed_Karto: boolean;
    Fed_Topology: boolean;
    FOnUserLineDialog: TOnUserLineDialog_Event;
    procedure SetSnapDistance(const Value: double);
    procedure SetSnapToLineBody(const Value: boolean);
    procedure SetSnapToLinePoint(const Value: boolean);
    procedure Seted_CanCreateNewObject(const Value: boolean);
    procedure Seted_LevelName(const Value: string);
    procedure Seted_SnapLevels(const Value: string);
    procedure Seted_SnapToLine(const Value: boolean);
    procedure Seted_DruhyList(const Value: string);
    procedure Seted_Karto(const Value: boolean);
    procedure Seted_Presnost(const Value: integer);
    procedure Seted_Topology(const Value: boolean);
    procedure SetOnUserLineDialog(const Value: TOnUserLineDialog_Event);
public
   procedure CreateTool; override;
published
   property  ed_CanCreateNewObject: boolean read Fed_CanCreateNewObject write Seted_CanCreateNewObject;
   property  ed_LevelName: string read Fed_LevelName write Seted_LevelName;
   property  ed_SnapLevels: string read Fed_SnapLevels write Seted_SnapLevels;
   property  ed_SnapToLine: boolean read Fed_SnapToLine write Seted_SnapToLine;
   property  ed_SnapToLinePoint: boolean read FSnapToLinePoint write SetSnapToLinePoint;
   property  ed_SnapToLineBody: boolean read FSnapToLineBody write SetSnapToLineBody;
   property  ed_SnapDistance: double read FSnapDistance write SetSnapDistance;
   property  ed_Karto: boolean read Fed_Karto write Seted_Karto;
   property  ed_DruhyList: string read Fed_DruhyList write Seted_DruhyList;
   property  ed_Presnost: integer read Fed_Presnost write Seted_Presnost;
   property  ed_Topology: boolean read Fed_Topology write Seted_Topology;
   property  OnUserLineDialog: TOnUserLineDialog_Event read FOnUserLineDialog write SetOnUserLineDialog;
end;

TAreaEditorAction= class (TToolAction)
  private
    Fed_CanCreateNewObject: boolean;
    Fed_ConstructLevelName: string;
    Fed_FrameLevelName: string;
    Fed_TargetLevelName: string;
    Fed_SLULevelName: string;
    Fed_ConstructLevelCopy: boolean;
    procedure Seted_CanCreateNewObject(const Value: boolean);
    procedure Seted_ConstructLevelName(const Value: string);
    procedure Seted_FrameLevelName(const Value: string);
    procedure Seted_SLULevelName(const Value: string);
    procedure Seted_TargetLevelName(const Value: string);
    procedure Seted_ConstructLevelCopy(const Value: boolean);
public
   procedure CreateTool; override;
published
   property ed_CanCreateNewObject: boolean read Fed_CanCreateNewObject write Seted_CanCreateNewObject;
   property ed_FrameLevelName: string read Fed_FrameLevelName write Seted_FrameLevelName;
   property ed_TargetLevelName: string read Fed_TargetLevelName write Seted_TargetLevelName;
   property ed_ConstructLevelName: string read Fed_ConstructLevelName write Seted_ConstructLevelName;
   property ed_ConstructLevelCopy: boolean read Fed_ConstructLevelCopy write Seted_ConstructLevelCopy;
   property ed_SLULevelName: string read Fed_SLULevelName write Seted_SLULevelName;
end;

TTextEditorAction= class (TToolAction)
  private
    Fed_LevelName: string;
    procedure Seted_LevelName(const Value: string);
public
   procedure CreateTool; override;
published
   property ed_LevelName: string read Fed_LevelName write Seted_LevelName;
end;

TPointEditorAction= class (TToolAction)
  private
    Fed_LevelName: string;
    Fed_DruhyList: string;
    Fed_DruhyListAngle: string;
    procedure Seted_LevelName(const Value: string);
    procedure Seted_DruhyListAngle(const Value: string);
    procedure Seted_DruhyList(const Value: string);
public
   procedure CreateTool; override;
published
   property ed_LevelName: string read Fed_LevelName write Seted_LevelName;
   property ed_DruhyList: string read Fed_DruhyList write Seted_DruhyList;
   property ed_DruhyListAngle: string read Fed_DruhyListAngle write Seted_DruhyListAngle;
end;

TNodeEditorAction = class(TToolAction)
  private
    FSnapToLinePoint: boolean;
    Fed_SnapToLine: boolean;
    Fed_SnapLevelName: string;
    FSnapToLineBody: boolean;
    Fed_LevelName: string;
    FSnapDistance: double;
    procedure Seted_LevelName(const Value: string);
    procedure Seted_SnapLevelName(const Value: string);
    procedure Seted_SnapToLine(const Value: boolean);
    procedure SetSnapDistance(const Value: double);
    procedure SetSnapToLineBody(const Value: boolean);
    procedure SetSnapToLinePoint(const Value: boolean);
 public
   procedure CreateTool; override;
 published
   property ed_LevelName: string read Fed_LevelName write Seted_LevelName;
   property ed_SnapLevelName: string read Fed_SnapLevelName write Seted_SnapLevelName;
   property ed_SnapToLine: boolean read Fed_SnapToLine write Seted_SnapToLine;
   property ed_SnapToLinePoint: boolean read FSnapToLinePoint write SetSnapToLinePoint;
   property ed_SnapToLineBody: boolean read FSnapToLineBody write SetSnapToLineBody;
   property ed_SnapDistance: double read FSnapDistance write SetSnapDistance;
end;

TSimpleEditorAction = class(TToolAction)
  private
    FOnMouseDownScript: string;
    FOnMouseUpScript: string;
    FOnMouseDown: TOn_CoordsClick;
    FOnMouseUp: TOn_CoordsClick;
    FOnMouseMove: TOn_CoordsClick;
    procedure SetOnMouseDownScript(const Value: string);
    procedure SetOnMouseUpScript(const Value: string);
    procedure SetOnMouseUp(const Value: TOn_CoordsClick);
    procedure SetOnMouseDown(const Value: TOn_CoordsClick);
    procedure SetOnMouseMove(const Value: TOn_CoordsClick);
  published
 public
   procedure CreateTool; override;
 published
   property OnMouseUpScript: string read FOnMouseUpScript write SetOnMouseUpScript;
   property OnMouseDownScript: string read FOnMouseDownScript write SetOnMouseDownScript;
   property OnMouseDown: TOn_CoordsClick read FOnMouseDown write SetOnMouseDown;
   property OnMouseUp: TOn_CoordsClick read FonMouseUp write SetOnMouseUp;
   property OnMouseMove: TOn_CoordsClick read FOnMouseMove write SetOnMouseMove;
end;

TSimpleRectAction = class(TToolAction)
  private
    FOnRectChange: TOn_RectChange;
    procedure SetOnRectChange(const Value: TOn_RectChange);
 public
   procedure CreateTool; override;
 published
   property  OnRectChange: TOn_RectChange read FOnRectChange write SetOnRectChange;
end;

TOrbisActionList = class(TActionList)
  private
    FOrbis01: TOrbis01;
    FGroupName: string;
    FCLient: boolean;
    procedure SetOrbis01(const Value: TOrbis01);
    procedure SetGroupName(const Value: string);
    procedure SetCLient(const Value: boolean);
public
   destructor destroy; override;
   property GroupName: string read FGroupName write SetGroupName;
published
   property Orbis01: TOrbis01 read FOrbis01 write SetOrbis01;
   property CLient: boolean read FCLient write SetCLient;
end;

TOnGeneral_Event = procedure(_ID, Value: string) of object;
TOnInterpretDone = procedure of object;
TOnSelectEvent = procedure of object;
TOnNewModel_Event = procedure of object;
TOnSaveUserData_Event = procedure(var ToSave: boolean) of object;
TOnProjectionChangedEvent = procedure(aX,aY,aScale,aVirtualScale: double) of object;
TOnMeasureEvent= procedure(Value,Value1: double) of object;
TOnUserLHCAreaDialogEvent = procedure (NewObject: boolean; Area: double; var UserID,Group,Color,Symbol,aresult: integer; var Props: string) of object;
TOnContinualChange_Event = procedure (aContinual: boolean) of object;
TOnProgress_Event     = procedure(_ID, _Event: integer; _Name, _Text: string; _MinValue, _MaxValue, _Progress: integer) of object;
TOnProgressStartEvent = procedure(_ID: integer; ProgressName: string; MinProgress, MaxProgress, Progress: integer) of object;
TOnProgressEvent      = procedure(_ID: integer; Value: integer) of object;
TOnProgressTextEvent  = procedure(_ID: integer; Value: string) of object;
TOnSelectDirectory_Event = function(const Caption: string; const Root: WideString; var Directory: string): boolean of object;
TOnSelectFile_Event = function(const Caption: string; const Directory: String; const Filter: string; var FileName: string): boolean of object;
{$IFDEF FILEEVENT}
TFile_Event = procedure(const aFile: string) of object;
{$ENDIF}
TOnDownloadURL_Event = function(const anURL: string; const aPath: String): boolean of object;


  TOrbis01 = class(TPanel)
  private
    FActive: boolean;
    FError: integer;
    FTool: TTool;
    FDirectory_Service: string;
    FDirectory_Script: string;
    FOrbisActionList: TOrbisActionList;
    FGF: TOrbisGraphic_Frame;
    FDirectory_Store: string;
    FDirectory_Project: string;
    FActualMap: string;
    FPOROST: string;
    FSKUPINA: string;
    FDILEC: string;
    FODDELENI: integer;
    FLHC: integer;
    FOnSelect: TOnSelectEvent;
    FOnProjectionCHanged: TOnProjectionChangedEvent;
    FDirectory_Root: string;
    FOnAreaChanged: TOnMeasureEvent;
    FOnLengthChanged: TOnMeasureEvent;
    Fpar_PODLOMENI: integer;
    Fpar_CISLO: integer;
    Fpar_KATASTR: integer;
    FNoProtocol: boolean;
    FOnNewModel: TOnNewModel_Event;
    FOnUserLHCAreaDialog: TOnUserLHCAreaDialogEvent;
    FOnSaveUserData: TOnSaveUserData_Event;
    FParcelaSelected: boolean;
    FSkupinaSelected: boolean;
    FLHCMonitor: TLHCPositionMonitor;
    FMonitorCoords: boolean;
    FMonitorParcely: boolean;
    FMonitorLHC: boolean;
    FMonitorActive: boolean;
    FOnMonitorParcelyCHange: TOn_MonitorParcelyChange;
    FOnMonitorCoordsChange: TOn_MonitorCoordsChange;
    FOnMonitorLHCCHange: TOn_MonitorLHCChange;
    FOnNewDATA: TOnNewModel_Event;
    FOnContinualChanged: TOnContinualChange_Event;
    FOnContinualLoadStart: TOnSelectEvent;
    FOnContinualLoadEnd: TOnSelectEvent;
    FActualSelectName: string;
    FActualSelect: TEntity;
    FClearIfLHCNotFound: boolean;
    FManualProjectionChange: boolean;
    FOnRastersChange: TOnNewModel_Event;
    FOnVrsChange: TOnNewModel_Event;
    FOnMapsChange: TOnNewModel_Event;
    FGrayscale: boolean;
    FJPGPixelFormat: integer;
    FJPGCompressionQuality: integer;
    FNo_BDE: boolean;
    FOnTypChange: TOnNewModel_Event;
    FOn_FileBeforeLoad: TFile_Event;
    FOn_FileAfterSave: TFile_Event;
    FOn_FileExists: TFile_Event;
    FOn_FileDelete: TFile_Event;
    FOn_ProgressStart: TOnProgressStartEvent;
    FOn_ProgressText: TOnProgressTextEvent;
    FOn_ProgressEnd: TOnProgressEvent;
    FOn_Progress: TOnProgressEvent;
    FOn_ProgressMax: TOnProgressEvent;
    FOn_SelectDirectory: TOnSelectDirectory_Event;
    FOn_SelectFile: TOnSelectFile_Event;
    FOnInterpretDone: TOnInterpretDone;
    FClient: boolean;
    FSelX: double;
    FSelY: double;
    FROK: integer;
    FOnGeneralEvent: TOnGeneral_Event;
    FOn_DownloadURL: TOnDownloadURL_Event;
    FNewUserDataSplitted: boolean;
    FUID: string;
    procedure SetActive(const Value: boolean);
    procedure SetError(const Value: integer);
    procedure SetTool(const Value: TTool);
    procedure SetDirectory_Script(const Value: string);
    procedure SetDirectory_Service(const Value: string);
    procedure SetOrbisActionList(const Value: TOrbisActionList);
    procedure SetGF(const Value: TOrbisGraphic_Frame);
    procedure SetDirectory_Store(const Value: string);
    procedure SetDirectory_Project(const Value: string);
    function GetMaps: TThematicMaps;
    procedure SetMaps(const Value: TThematicMaps);
    procedure SetActualMap(const Value: string);
    procedure SetDILEC(const Value: string);
    procedure SetLHC(const Value: integer);
    procedure SetROK(const Value: integer);
    procedure SetODDELENI(const Value: integer);
    procedure SetPOROST(const Value: string);
    procedure SetSKUPINA(const Value: string);
    function GetForestModel: Tgis_ForestModel;
    procedure SetOnSelect(const Value: TOnSelectEvent);
    function GetProjects: TOrbis01Projects;
    function GetProject: TORBISLHCProject;
    procedure SetOnProjectionCHanged(
      const Value: TOnProjectionChangedEvent);
    function GetCenterX: double;
    function GetCenterY: double;
    function GetScale: double;
    procedure SetCenterX(const Value: double);
    procedure SetCenterY(const Value: double);
    procedure SetScale(const Value: double);
    procedure SetDirectory_Root(const Value: string);
    procedure SetActiveToolName(const Value: string);
    function GetActiveToolName: string;
    { Private declarations }
//    procedure AppIdle(Sender: TObject; var Done: Boolean);
    procedure SetOnAreaChanged(const Value: TOnMeasureEvent);
    procedure SetOnLengthChanged(const Value: TOnMeasureEvent);
    function GetParcelModel: Tgis_ParcelModel;
    procedure Setpar_CISLO(const Value: integer);
    procedure Setpar_KATASTR(const Value: integer);
    procedure Setpar_PODLOMENI(const Value: integer);
    procedure SetNoProtocol(const Value: boolean);
    procedure SetOnNewModel(const Value: TOnNewModel_Event);
    procedure SetOnUserLHCAreaDialog(const Value: TOnUserLHCAreaDialogEvent);
    procedure SetOnSaveUserData(const Value: TOnSaveUserData_Event);
    procedure SetParcelaSelected(const Value: boolean);
    procedure SetSkupinaSelected(const Value: boolean);
    procedure SetLHCMonitor(const Value: TLHCPositionMonitor);
    procedure SetMonitorActive(const Value: boolean);
    procedure SetMonitorCoords(const Value: boolean);
    procedure SetMonitorLHC(const Value: boolean);
    procedure SetMonitorParcely(const Value: boolean);
    procedure SetOnMonitorCoordsChange(const Value: TOn_MonitorCoordsChange);
    procedure SetOnMonitorLHCCHange(const Value: TOn_MonitorLHCChange);
    procedure SetOnMonitorParcelyCHange(const Value: TOn_MonitorParcelyChange);
    procedure SetOnNewDATA(const Value: TOnNewModel_Event);
    procedure SetContinual(const Value: boolean);
    function GetContinual: boolean;
    procedure SetOnContinualChanged(const Value: TOnContinualChange_Event);
    function GetCont: TRectEntity;
    function GetLayers: TEntity;
    procedure SetOnContinualLoadEnd(const Value: TOnSelectEvent);
    procedure SetOnContinualLoadStart(const Value: TOnSelectEvent);
    procedure SetActualSelect(const Value: TEntity);
    procedure SetActualSelectName(const Value: string);
    procedure SetShiftView(const Value: string);
    procedure SetClearIfLHCNotFound(const Value: boolean);
    procedure SetManualProjectionChange(const Value: boolean);
    procedure SetOnRastersChange(const Value: TOnNewModel_Event);
    function GetInterpret: TInterpret;
    procedure SetOnVrsChange(const Value: TOnNewModel_Event);
    function GetEditLayers: TEntity;
    procedure SetActualEditLayer(const Value: string);
    function GetActualEditLayer: string;
    procedure SetOnMapsChange(const Value: TOnNewModel_Event);
    function GetActualMap: string;
    function GetNo_BDE: boolean;
    procedure SetNo_BDE(const Value: boolean);
    procedure SetOnTypChange(const Value: TOnNewModel_Event);
    procedure SetOn_FileAfterSave(const Value: TFile_Event);
    procedure SetOn_FileBeforeLoad(const Value: TFile_Event);
    procedure SetOn_FileExists(const Value: TFile_Event);
    procedure SetOn_Progress(const Value: TOnProgressEvent);
    procedure SetOn_ProgressEnd(const Value: TOnProgressEvent);
    procedure SetOn_ProgressMax(const Value: TOnProgressEvent);
    procedure SetOn_ProgressStart(const Value: TOnProgressStartEvent);
    procedure SetOn_ProgressText(const Value: TOnProgressTextEvent);
    procedure SetOn_SelectDirectory(const Value: TOnSelectDirectory_Event);
    procedure SetOn_SelectFile(const Value: TOnSelectFile_Event);
    procedure SetOnInterpretDone(const Value: TOnInterpretDone);
    function GetVirtualScale: double;
    procedure SetVirtualScale(const Value: double);
    function GetProjections: TProjectionInfoModel;
    procedure SetClient(const Value: boolean);
    procedure SetSelX(const Value: double);
    procedure SetSelY(const Value: double);
    procedure SetLHC_ROK(const Value: string);
    procedure SetOnGeneralEvent(const Value: TOnGeneral_Event);
    procedure SetOn_DownloadURL(const Value: TOnDownloadURL_Event);
    procedure SetOn_FileDelete(const Value: TFile_Event);
    function GetContinual_Rok: integer;
    procedure SetContinual_Rok(const Value: integer);
    procedure SetNewUserDataSplitted(const Value: boolean);
    procedure SetUID(const Value: string);
  protected
    property  GF: TOrbisGraphic_Frame read FGF write SetGF;
    property  LHCMonitor: TLHCPositionMonitor read FLHCMonitor write SetLHCMonitor;
    property Tool: TTool read FTool write SetTool;
    property Maps: TThematicMaps read GetMaps write SetMaps;
    property Projects: TOrbis01Projects read GetProjects;
    property Cont: TRectEntity read GetCont;
    property Layers: TEntity read GetLayers;
    property EditLayers: TEntity read GetEditLayers;
    property ForestModel: Tgis_ForestModel read GetForestModel;
    property ParcelModel: Tgis_ParcelModel read GetParcelModel;
    procedure SelectEvent(const aElement: TElement);
    procedure ProcessUserLHCAreaDialog(NewObject: boolean; Area: double; _Skupina: TElement; var UserID,Group,Color,Symbol,aresult: integer; var Props: string);
    procedure SaveUserDataEvent(var ToSave: boolean);
    procedure ProjectionChangedEvent(aX,aY,aScale,aVirtualScale: double);
    procedure ContinualChangeEvent(aContinual: boolean);
    procedure NewModelEvent;
    procedure NewDATAEvent;
    procedure RastersChangeEvent;
    procedure MapsChangeEvent;
    procedure GeneralEvent(_ID,Value: string);
    procedure VrsChangeEvent;
    procedure TypChangeEvent;
    procedure ProgressEvent(_ID, _Event: integer; _Name, _Text: string; _MinValue, _MaxValue, _Progress: integer);
{$IFDEF FILEEVENT}
    procedure FileBeforeLoadEvent(const aFile: string);
    procedure FileAfterSaveEvent(const aFile: string);
    procedure FileExistsEvent(const aFile: string);
    procedure FileDeleteEvent(const aFile: string);
{$ENDIF}
    procedure DelFil(aFileName: string);
    procedure RenFil(aFileName, aFileNameN: string);
    procedure CheckLHCYear(LHC_KOD,LHC_Year: integer);
    procedure CHeckTheForest;
    procedure CHeckTheKatastr;
    procedure CHeckContinualSpace;
    procedure Cont_FreeOdd(aOdd: TEntity);
    procedure Cont_FreeKart(aKart: TEntity);
    procedure Cont_FreeUser(aUser: TEntity);
    procedure Cont_FreeG(aG: TEntity);
    procedure Cont_LoadOddeleni(aLHC: TOrbis01LHCInfo; aOdd: integer);
    procedure Cont_LoadOdd(iLHC: TOrbis01LHCInfo; aODD: TEntity);
    procedure Cont_LoadKart(aKart: TEntity);
    procedure Cont_LoadG(aG: TEntity; var RealLoad: boolean);
  public
    constructor Create(AOwner: TComponent); override;
    destructor  destroy; override;
    procedure DoResize(Sender: TObject);
    {$IFDEF OCX}
    procedure Activate;
    {$ENDIF}
    procedure ExploreWorld;
    procedure OILEditor;
    procedure NewProjectDialog;
    procedure NewKatastrDialog;
    procedure NewProject(const aPath,DataProfile: string);
    procedure NewKatastr(const aPath: string; aID: integer);
    procedure Import;
    function  GetRasterInfo: TOrbis01RasterInfo;
    procedure CheckRasters;
    procedure SetupRasters;
    function  GetVrsInfo: TOrbis01LayerInfo;
    function  GetTypInfo: TOrbis01LayerInfo;
    procedure CheckVrs;
    procedure CheckTyp;
    procedure SetupVrs;
    procedure SetupTyp;
    function  LoadProjectDataFromORB(const aFileName, DataType: string): integer;
    procedure LoadProject(aProjectName: string);
    procedure LoadKatastr(aFileName: string);
    procedure LoadKatastrID(aID: integer);
    procedure SaveProjectAsContinual;
    function  JoinProjects(aSourcePath: string): integer;
    function  RenumberLHC(LHC_OLD, LHC_NEW : integer): boolean;
    function  DeleteLHC(LHC_KOD,LHC_Year: integer): boolean;
    function  DeleteLHC_Continual(LHC_KOD,LHC_Year: integer): boolean;
    function  ChangeLHCYear(LHC_KOD,LHC_Year,NewYear: integer): boolean;
    function  LHC_UserDataSplitted(LHC_KOD,LHC_Year: integer): boolean;
    function  LHC_SplitUserData(LHC_KOD,LHC_Year: integer): boolean;
    function  LHC_UserDataUUIDIsSet(LHC_KOD,LHC_Year: integer): boolean;
    function  LHC_SetUserDataUUID(LHC_KOD,LHC_Year: integer; aUserID: string; SetUserID: boolean): boolean;
    function  Synchronize_with_mobile_user_data_layer(aLayer,MobileFile,DesktopFile, MediaFile: string): integer;
//   results:
//   00      ani jeden soubor neexistuje
//   10      mobilni soubor existuje, Desktop vytvoren
//   01      tedy 1 Desktop existuje, Mobil vytvoren
//   11      Oba existuji, probehla synchronizace
//   02      Nesouhlasi nazvy vrstev
//   30      Mobilni soubor chyba ... vadna struktura a tak
//   03      Desktop             --"--

    function  DeleteUserLayer(LHC_KOD, LHC_Year: integer; aLayerName: string): boolean;
    function  DeleteUserPhysicalLayer(LHC_KOD, LHC_Year: integer; aLayerName, aLayerType: string): boolean;
    function  QueryUserLayerCounts(LHC_KOD, LHC_Year: integer; aLayerName: string; var aCountR: TOrbis01UserLayerCounts): boolean;
    procedure FillMapsList(aList: TStrings);
    procedure FillEditLayersList(aList: TStrings);
    procedure FillAreSymbolsList(aList: TStrings);
    procedure FillAreSymbolsComboBoxEx(aCombo: TComboBoxEx; aList: TStrings;ImageWidth,ImageHeight: integer);
    procedure FillKartoTextsList(aList: TStrings);
    function  FillLHCsString: string;
    function  FillMapsString: string;
    function  OrbisIndexToColor(aIndex: integer): integer;
    procedure ShowMap(aMap: string);
    procedure SetupMaps;
    procedure NewMap;
    procedure MapDialog;
    function  FindTheForest: Tgis_ForestUnit;
    function  FindPSKByXY(aX,aY: double): Tgis_Porostniskupina;
    function  ShowKartoText(aText: string): boolean;
    procedure AddToSelect(aUnit: Tgis_ForestUnit);
    procedure ClearTheSelect;
    procedure ShowTheForest(Pattern: boolean);
    procedure ShowTheKatastr(Pattern: boolean);
    procedure ShowTheSelect(Pattern: boolean);
    procedure HighlightTheForest;
    procedure HighlightTheKatastr;
    procedure HighlightTheSelect;
    procedure ZoomIn;
    procedure ZoomOut;
    procedure ViewAll;
    procedure ReView;
    procedure Print;
    procedure PrintMap;
    procedure ClearData;
    procedure NewEditLayer;
    procedure ActualEditLayerToActualMap;
    procedure CheckUserDataForSave;
    procedure LoadUserDataFrom(const aFileName: string);
    procedure SaveUserDataTo(const aFileName: string; ActualLayerOnly: boolean);
    procedure ExecuteScript(const aScript: string);
    procedure SetParam(const aName,aValue: string);
    function  GetParam(aName: string): string;
    procedure OnKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure OnWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
    procedure OnWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
    procedure StartSizeMove;
    procedure EndSizeMove;
    procedure ExportToJPGFile(aVal: string);
    procedure ExportToBMPFile(aVal: string);
    procedure PrintToCanvas(aCanvas: TCanvas;var aScale: double; aR: TRect; aDPI: integer; UseRectAndDPI: boolean);
    function  PrintSymbLegToCanvas(aCanvas: TCanvas; aScale: double; aR: TRect; aDPI: integer; UseRectAndDPI: boolean; Test: boolean; var Rows: integer; OneSymbol: boolean; aKind: char; Kod: integer): boolean;
    function  PrintSymbolToCanvas(aCanvas: TCanvas; aScale: double; aR: TRect; aDPI: integer; UseRectAndDPI: boolean; aKind: char; Kod: integer): boolean;
    function  PrintLegendToCanvas(aCanvas: TCanvas; aScale: double; aR: TRect; aDPI: integer; UseRectAndDPI: boolean; Test: boolean; var Rows: integer): boolean;
    function  Projection_Move(aNext: boolean): boolean;
    function  Projection_Status(var NoNext,NoPrevious: boolean): boolean;
    property  JPGCompressionQuality: integer read  FJPGCompressionQuality write FJPGCompressionQuality;
    { kvalita 1..100 }
    property  JPGGrayscale: boolean read FGrayscale write FGrayscale;
    property  JPGPixelFormat: integer read FJPGPixelFormat write FJPGPixelFormat;
    property  ShiftView: string write SetShiftView;
    property  ActualMap: string read GetActualMap write SetActualMap;
    property  ActualEditLayer: string read GetActualEditLayer write SetActualEditLayer;
    property  Project: TORBISLHCProject read GetProject;
    property  ActiveToolName: string read GetActiveToolName write SetActiveToolName;
    property  ActualSelectName: string read FActualSelectName write SetActualSelectName;
    property  ActualSelect: TEntity read FActualSelect write SetActualSelect;
    property  ManualProjectionChange: boolean read FManualProjectionChange write SetManualProjectionChange;
    property  Interpret: TInterpret read GetInterpret;
    property  Projections: TProjectionInfoModel read GetProjections;
    property  Client: boolean read FClient write SetClient;
  published
    {$IFNDEF OCX}
    property  Active: boolean read FActive write SetActive;
    {$ENDIF}
    property Error: integer read FError write SetError;
    property Directory_Root: string read FDirectory_Root write SetDirectory_Root;
    property Directory_Script: string read FDirectory_Script write SetDirectory_Script;
    property Directory_Service: string read FDirectory_Service write SetDirectory_Service;
    property Directory_Store: string read FDirectory_Store write SetDirectory_Store;
    property Directory_Project: string read FDirectory_Project write SetDirectory_Project;
    property OrbisActionList: TOrbisActionList read FOrbisActionList write SetOrbisActionList;
    property LHC: integer read FLHC write SetLHC;
    property ROK: integer read FROK write SetROK;
    property CONTINUAL_ROK: integer read GetContinual_Rok write SetContinual_Rok;
    property LHC_ROK: string write SetLHC_ROK;
    property ODDELENI: integer read FODDELENI write SetODDELENI;
    property DILEC: string read FDILEC write SetDILEC;
    property POROST: string read FPOROST write SetPOROST;
    property SKUPINA: string read FSKUPINA write SetSKUPINA;
    property UID: string read FUID write SetUID;
    property par_KATASTR: integer read Fpar_KATASTR write Setpar_KATASTR;
    property par_CISLO: integer read Fpar_CISLO write Setpar_CISLO;
    property par_PODLOMENI: integer read Fpar_PODLOMENI write Setpar_PODLOMENI;
    property SkupinaSelected: boolean read FSkupinaSelected write SetSkupinaSelected;
    property ParcelaSelected: boolean read FParcelaSelected write SetParcelaSelected;
    property OnGeneralEvent: TOnGeneral_Event read FOnGeneralEvent write SetOnGeneralEvent;
    property OnInterpretDone: TOnInterpretDone read FOnInterpretDone write SetOnInterpretDone;
    property OnSelect: TOnSelectEvent read FOnSelect write SetOnSelect;
    property OnNewModel: TOnNewModel_Event read FOnNewModel write SetOnNewModel;
    property OnNewDATA: TOnNewModel_Event read FOnNewDATA write SetOnNewDATA;
    property OnRastersChange: TOnNewModel_Event read FOnRastersChange write SetOnRastersChange;
    property OnMapsChange: TOnNewModel_Event read FOnMapsChange write SetOnMapsChange;
    property OnVrsChange: TOnNewModel_Event read FOnVrsChange write SetOnVrsChange;
    property OnTypChange: TOnNewModel_Event read FOnTypChange write SetOnTypChange;
    property OnContinualChanged: TOnContinualChange_Event read FOnContinualChanged write SetOnContinualChanged;
    property OnContinualLoadStart: TOnSelectEvent read FOnContinualLoadStart write SetOnContinualLoadStart;
    property OnContinualLoadEnd: TOnSelectEvent read FOnContinualLoadEnd write SetOnContinualLoadEnd;
    property OnProjectionCHanged: TOnProjectionChangedEvent read FOnProjectionCHanged write SetOnProjectionCHanged;
    property OnLengthChanged: TOnMeasureEvent read FOnLengthChanged write SetOnLengthChanged;
    property OnAreaChanged: TOnMeasureEvent read FOnAreaChanged write SetOnAreaChanged;
    property OnUserLHCAreaDialog: TOnUserLHCAreaDialogEvent read FOnUserLHCAreaDialog write SetOnUserLHCAreaDialog;
    property OnSaveUserData: TOnSaveUserData_Event read FOnSaveUserData write SetOnSaveUserData;
    property OnMonitorCoordsChange: TOn_MonitorCoordsChange read FOnMonitorCoordsChange write SetOnMonitorCoordsChange;
    property OnMonitorLHCCHange: TOn_MonitorLHCChange read FOnMonitorLHCCHange write SetOnMonitorLHCCHange;
    property OnMonitorParcelyCHange: TOn_MonitorParcelyChange read FOnMonitorParcelyCHange write SetOnMonitorParcelyCHange;
    property On_ProgressStart: TOnProgressStartEvent read FOn_ProgressStart write SetOn_ProgressStart;
    property On_ProgressEnd: TOnProgressEvent read FOn_ProgressEnd write SetOn_ProgressEnd;
    property On_Progress: TOnProgressEvent read FOn_Progress write SetOn_Progress;
    property On_ProgressMax: TOnProgressEvent read FOn_ProgressMax write SetOn_ProgressMax;
    property On_ProgressText: TOnProgressTextEvent read FOn_ProgressText write SetOn_ProgressText;
    property On_SelectDirectory: TOnSelectDirectory_Event read FOn_SelectDirectory write SetOn_SelectDirectory;
    property On_SelectFile     : TOnSelectFile_Event read FOn_SelectFile write SetOn_SelectFile;
{$IFDEF FILEEVENT}
    property On_FileBeforeLoad: TFile_Event read FOn_FileBeforeLoad write SetOn_FileBeforeLoad;
    property On_FileAfterSave: TFile_Event read FOn_FileAfterSave write SetOn_FileAfterSave;
    property On_FileExists: TFile_Event read FOn_FileExists write SetOn_FileExists;
    property On_FileDelete: TFile_Event read FOn_FileDelete write SetOn_FileDelete;
{$ENDIF}
    property On_DownloadURL: TOnDownloadURL_Event read FOn_DownloadURL write SetOn_DownloadURL;
    property Scale: double read GetScale write SetScale;
    property VirtualScale: double read GetVirtualScale write SetVirtualScale;
    property CenterX: double read GetCenterX write SetCenterX;
    property CenterY: double read GetCenterY write SetCenterY;
    property SelX: double read FSelX write SetSelX;
    property SelY: double read FSelY write SetSelY;
    property NoProtocol: boolean read FNoProtocol write SetNoProtocol;
    property MonitorCoords: boolean read FMonitorCoords write SetMonitorCoords;
    property MonitorLHC: boolean read FMonitorLHC write SetMonitorLHC;
    property MonitorParcely: boolean read FMonitorParcely write SetMonitorParcely;
    property MonitorActive: boolean read FMonitorActive write SetMonitorActive;
    property Continual: boolean read GetContinual write SetContinual;
    property ClearIfLHCNotFound: boolean read FClearIfLHCNotFound write SetClearIfLHCNotFound;
    property No_BDE: boolean read GetNo_BDE write SetNo_BDE;
    property NewUserDataSplitted: boolean read FNewUserDataSplitted write SetNewUserDataSplitted;
  end;


procedure Register;

implementation
uses
   StrUtils,
   System.IOUtils,
   System.Types,
   JPEG,
 {$IF RTLVersion >= 27}
   System.Actions,
 {$ENDIF}
   e_Class,
   e_Color,
   e_Device,
   e_Type,
   e_TopolELements,
   m_Orbis,
   m_Proj,
   t_Device,
   t_gis_PorostniSkupinaSelector,
   t_gis_ParcelaCastSelector,
   t_ForestModelSelector,
   t_GraphicLinkTool,
   t_LineDialog,
   t_TopolLineDialog,
   t_Loader,
   t_Saver,
   t_Main,
   t_MeasureTools,
   t_PrepareSymbols,
   t_RasterDialog,
   t_RasterManipulator,
   t_ShImg,
   t_Snap,
   t_SRFEditor,
   t_TezPDialog,
   t_TextEditor,
   t_TopolPointDialog,
   t_SimpleTextDialog,
   t_ThematicMapsEditor,
   t_NodeEditor,
   t_ViewTools,
   u_Global,
   u_String,
   Worlds, t_FormTool, t_InterfaceTool, E_proje;


procedure Register;
begin
  RegisterComponents('ORBIS', [TOrbis01,TOrbisActionList]);
  RegisterActions('ORBIS',[TViewerAction,TShiftImageAction,Tgis_SkupinaSelectAction,Tgis_ParcelaSelectAction,TRasterManipulatorAction,TMeasureLengthAction,TMeasureAreaAction,TPolyLineEditorAction,TAreaEditorAction,TTextEditorAction,TPointEditorAction,TNodeEditorAction,TSimpleEditorAction,TSimpleRectAction],nil);
end;

{ TOrbis01 }

constructor TOrbis01.Create(AOwner: TComponent);
begin
  inherited;
  Error := 0;
  GF := TOrbisGraphic_Frame.Create(Self);
  InsertControl(GF);
  GF.Align := alClient;
  Application.ProcessMessages;
  //onResize := DoResize;
  //DoResize(nil);
  //Application.ProcessMessages;
end;

function TOrbis01.DeleteLHC(LHC_KOD, LHC_Year: integer): boolean;
var
   LI: TOrbis01LHCInfo;
   S: string;
   LHC_ID: string;
   aFile: string;
   PrjS: string;
begin
   result := false;
   Error := 0;
   if not FActive then exit;
   if Projects = nil then  // Projekty chyb ... nco je blb
   begin
      Error := Err_Projects;
      exit;
   end;
   LI := Projects.GetLHCInfo(LHC_KOD, LHC_Year, true);
   if LI = nil then // LHC nen pipojeno
   begin
      Error := err_LHCNotConected;
      exit;
   end;
   // Deleting Continual
   if LI.Continual then
      DeleteLHC_Continual(LHC_KOD,LHC_Year);
   LHC_ID := IntToStr(LHC_KOD) + '_' + IntToStr(LHC_Year);
   // Deleting UserData
   Prjs := TMainTool(World.MainTool).Options.ProjectDirectory;
   CheckBackSlash(Prjs);
   Prjs := GetAbsolutePath(Prjs);
   aFile := Prjs+ 'USERDATA_' + LHC_ID + '.orb';
   DelFil(aFile);
   S := LI.ProjectName;
   // Deleting DataFile
   aFile := Prjs + S + '.opr';
   DelFil(aFile);
   // Deleting ProjectFile
   aFile := Prjs + S + '.orb';
   DelFil(aFile);
   // Delete LHC Record  and Save projects
   Projects.LHCs.FreeElement(LI);
   TMainTool(World.MainTool).SaveProjects;
   result := true;
end;

function TOrbis01.DeleteLHC_Continual(LHC_KOD, LHC_Year: integer): boolean;
var
   LI: TOrbis01LHCInfo;
   S: string;
   LHC_ID: string;
   aFile: string;
   Files: TStringDynArray;
begin
   result := false;
   Error := 0;
   if not FActive then exit;
   if Projects = nil then  // Projekty chyb ... nco je blb
   begin
      Error := Err_Projects;
      exit;
   end;
   LI := Projects.GetLHCInfo(LHC_KOD, LHC_Year, true);
   if LI = nil then // LHC nen pipojeno
   begin
      Error := err_LHCNotConected;
      exit;
   end;
   // Deleting Continual
   if LI.Continual then
   begin
      LHC_ID := IntToStr(LHC_KOD);
      if LHC_Year > 0 then
         LHC_ID := LHC_ID + '_' + IntToStr(LHC_Year);
      S := TMainTool(World.MainTool).Options.ProjectDirectory;
      CHeckBackSlash(S);
      S := S + 'CONTINUAL\';
      S := GetAbsolutePath(S);
      aFile := S+LHC_ID+'.orb';
      DelFil(aFile);
      // Erase lhc cont directory
       S := S + LHC_ID;
      {$IFDEF FILEEVENT}
         // nacteni souboru z DB aby fungovalo prochazeni
         World.MainTool.PostTextMsg(tm_FileExists,S+'\*.*',nil);
         Application.ProcessMessages;
      {$ENDIF}
       Files := TDirectory.GetFiles(S);
       for aFile in Files do
       begin
          DelFil(aFile);
       end;
       LI.Continual := false;
       TMainTool(World.MainTool).SaveProjects;
       result := true;
   end;
end;

function TOrbis01.DeleteUserLayer(LHC_KOD, LHC_Year: integer; aLayerName: string): boolean;
var
   aSplit: boolean;
begin
   result := false;
   aSplit := LHC_UserDataSplitted(LHC_KOD, LHC_Year);
   if Error > 0 then exit;
   result := TMainTool(World.MainTool).DeleteUserLayer(LHC_KOD, LHC_Year, aLayerName, aSplit);
end;

function TOrbis01.DeleteUserPhysicalLayer(LHC_KOD, LHC_Year: integer; aLayerName, aLayerType: string): boolean;
var
   aSplit: boolean;
begin
   result := false;
   aSplit := LHC_UserDataSplitted(LHC_KOD, LHC_Year);
   if Error > 0 then exit;
   result := TMainTool(World.MainTool).DeleteUserPhysicalLayer(LHC_KOD, LHC_Year, aLayerName, aLayerType, aSplit);
end;

procedure TOrbis01.DelFil(aFileName: string);
begin
   TMainTool(World.MainTool).DelFil(aFileName);
end;

procedure TOrbis01.SetActive(const Value: boolean);
var
   G: TGraphicLinkTool;
   ClearActivation: boolean;
begin
//   ClearActivation := true;
   if Value = FActive then exit;
   //if not Value then exit;
   if csDesigning in ComponentState then
   begin
      ShowMessage('Set Active in runtime only!');
      exit;
   end;
   if not Value then
   begin
      //Application.OnMessage:= nil;
      //Application.OnException := nil;
      //World.RemoveElement(OrbisClasses);
      //World.Free;
      GF.Tool := nil;
      GF.Enabled := false;
      //SystemColorPalette := nil;
      //World := nil;
      TMainTool(World.MainTool).On_FileBeforeLoadEvent := nil;
      TMainTool(World.MainTool).On_FileAfterSaveEvent := nil;
      TMainTool(World.MainTool).On_FileExistsEvent := nil;
      TMainTool(World.MainTool).On_FileDeleteEvent := nil;

      TMainTool(World.MainTool).OnNewModelEvent := nil;
      TMainTool(World.MainTool).OnNewDATAEvent := nil;
      TMainTool(World.MainTool).OnRastersChangeEvent := nil;
      TMainTool(World.MainTool).OnGeneralEvent := nil;
      TMainTool(World.MainTool).OnMapsChangeEvent := nil;
      TMainTool(World.MainTool).OnVrsChangeEvent := nil;
      TMainTool(World.MainTool).OnTypChangeEvent := nil;
      TMainTool(World.MainTool).OnUserLHCAreaDialog := nil;
      TMainTool(World.MainTool).OnSaveUserDataEvent := nil;
      TMainTool(World.MainTool).OnContinualChangeEvent := nil;
      TMainTool(World.MainTool).On_Progress_Event := nil;
      TMainTool(World.MainTool).Project := nil;
      LHCMonitor := nil;
      G := TGraphicLinkTool(World.MainTool.Tools.FindElementClass(TGraphicLinkTool));
      if G <> nil then
      begin
         Tool := nil;
         G.Input := nil;
         G.LinkTools.DisconnectActionList;
         G.Free();
      end;
      World.ObjectModel.FreeAll;
      FActive := Value;
      exit;
   end;
   {$IFDEF OCX}
      Application.Handle := HInstance;
      Application.CreateHandle;
   {$ENDIF}
   ClearActivation := World = nil;
   if ClearActivation then
   begin
      TMainTool.CreateWorld;
      if World.MainTool = nil then
      begin
         FActive := false;
         Error := 1;
         exit;
      end;
   end;
   World.UID := UID;   // User ID
   // bylo v not clear
      if Directory_Root <> '' then RootDirectory := Directory_Root;
      TMainTool(World.MainTool).ReadIniFile;
      if Directory_Script <> '' then
         TMainTool(World.MainTool).Options.ScriptDirectory := Directory_Script;
      if Directory_Service <> '' then
         TMainTool(World.MainTool).Options.ServiceDirectory := Directory_Service;
      if Directory_Store <> '' then
         TMainTool(World.MainTool).Options.StoreDirectory := Directory_Store;
      if Directory_Project <> '' then
         TMainTool(World.MainTool).Options.ProjectDirectory := Directory_Project;
   // bylo v not clear
   TMainTool(World.MainTool).On_FileBeforeLoadEvent := FileBeforeLoadEvent;
   TMainTool(World.MainTool).On_FileAfterSaveEvent := FileAfterSaveEvent;
   TMainTool(World.MainTool).On_FileExistsEvent := FileExistsEvent;
   TMainTool(World.MainTool).On_FileDeleteEvent := FileDeleteEvent;
   TMainTool(World.MainTool).On_SelectDirectory_Event := On_SelectDirectory;
   TMainTool(World.MainTool).On_DownloadURL_Event := On_DownloadURL;
   TMainTool(World.MainTool).On_SelectFile_Event := On_SelectFile;
   if Assigned(On_ProgressStart) or Assigned(On_ProgressEnd) or
      Assigned(On_Progress) or Assigned(On_ProgressMax) or
      Assigned(On_ProgressText) then
         TMainTool(World.MainTool).On_Progress_Event := ProgressEvent;

   // zmena aby se new model volal ... jako vubec...
   TMainTool(World.MainTool).OnNewModelEvent := NewModelEvent;
   if ClearActivation then
      World.MainTool.DoIt
   else
      TMainTool(WOrld.MainTool).DoItReactivate;
   TMainTool(World.MainTool).OnNewDATAEvent := NewDATAEvent;
   TMainTool(World.MainTool).OnRastersChangeEvent := RastersChangeEvent;
   TMainTool(World.MainTool).OnGeneralEvent := GeneralEvent;
   TMainTool(World.MainTool).OnMapsChangeEvent := MapsChangeEvent;
   TMainTool(World.MainTool).OnVrsChangeEvent := VrsChangeEvent;
   TMainTool(World.MainTool).OnTypChangeEvent := TypChangeEvent;
   TMainTool(World.MainTool).OnUserLHCAreaDialog := ProcessUserLHCAreaDialog;
   TMainTool(World.MainTool).OnSaveUserDataEvent := SaveUserDataEvent;
   TMainTool(World.MainTool).OnContinualChangeEvent := ContinualChangeEvent;
   TMainTool(World.MainTool).NewUserDataSplitted := NewUserDataSplitted;
   G := TGraphicLinkTool.Create;
   World.MainTool.AddTool(G);
   TMainTool(World.MainTool).NoProtocol := NoProtocol;
   Tool := G;
   G.GFrame := GF;
   G.DoIt;
   GF.Enabled := true;
   {$IFDEF OCX}
      OrbisActionList := TOrbisActionList.Create(Self);
      OrbisActionList.AddAction(TViewerAction.Create(Self));
      OrbisActionList.AddAction(TShiftImageAction.Create(Self));
      OrbisActionList.AddAction(Tgis_SkupinaSelectAction.Create(Self));
      OrbisActionList.AddAction(Tgis_ParcelaSelectAction.Create(Self));
      OrbisActionList.AddAction(TMeasureLengthAction.Create(Self));
      OrbisActionList.AddAction(TMeasureAreaAction.Create(Self));
      OrbisActionList.AddAction(TPolyLineEditorAction.Create(Self));
      OrbisActionList.AddAction(TTextEditorAction.Create(Self));
      OrbisActionList.AddAction(TPointEditorAction.Create(Self));
      //Application.OnIdle := AppIdle;
   {$ENDIF}
   G.LinkTools.ActionList := OrbisActionList;
   G.OnSelect := SelectEvent;
   //G.OnProjectionChanged := OnProjectionCHanged;
   G.OnInterpretDone := OnInterpretDone;
   G.OnProjectionChanged := ProjectionChangedEvent;
   G.OnLengthChanged := OnLengthChanged;
   G.OnAreaChanged := OnAreaChanged;
   LHCMonitor := TLHCPositionMonitor.Create;
   LHCMonitor.Device := G.Device;
   G.Monitors.AddTool(LHCMonitor);
   LHCMonitor.MonitorCoords := MonitorCoords;
   LHCMonitor.MonitorLHC := MonitorLHC;
   LHCMonitor.MonitorParcely := MonitorParcely;
   LHCMonitor.OnCoordsChange := OnMonitorCoordsChange;
   LHCMonitor.OnLHCChange := OnMonitorLHCCHange;
   LHCMonitor.OnParcelyChange := OnMonitorParcelyChange;
   FActive := Value;
   NoBDE := FNo_BDE;
end;

procedure TOrbis01.SetError(const Value: integer);
begin
  FError := Value;
end;

procedure TOrbis01.DoResize(Sender: TObject);
begin
   inherited;
   try
      {
      if not Visible then exit;
      if GF = nil then exit;
      GF.Image.Picture.Bitmap.Height:= GF.Image.Height;
      GF.Image.Picture.Bitmap.Width:= GF.Image.Width;
      if GF.Tool = nil then exit;
      if GF.Tool.Idle then exit;
      GF.Tool.PostMsg(tm_ReSize, GF.Tool, nil, false);
      }
      {
      Tool.Device.SetupPage(nil);
      Tool.PostMsg(tm_ReView, Tool, nil, DontKill);
      }
   except
   end;
end;

procedure TOrbis01.SetTool(const Value: TTool);
begin
   FTool := Value;
end;

procedure TOrbis01.SetUID(const Value: string);
begin
  FUID := Value;
  if FActive then
     World.UID := Value;
end;

procedure TOrbis01.ExploreWorld;
begin
   if not FActive then exit;
   World.MainTool.PostMsg(tm_Explore,nil,World,false);
   Application.ProcessMessages;
end;

procedure TOrbis01.SetDirectory_Script(const Value: string);
begin
   FDirectory_Script := Value;
   if FActive then
      TMainTool(World.MainTool).Options.ScriptDirectory := Value;
end;

procedure TOrbis01.SetDirectory_Service(const Value: string);
begin
   FDirectory_Service := Value;
   if FActive then
      TMainTool(World.MainTool).Options.ServiceDirectory := Value;
end;


procedure TOrbis01.SetOrbisActionList(const Value: TOrbisActionList);
begin
   if Value = FOrbisActionList then exit;
   FOrbisActionList := Value;
   if Value = nil then exit;
   Value.Orbis01 := Self;
end;

destructor TOrbis01.destroy;
begin
   if OrbisActionList <> nil then
      OrbisActionList.Orbis01 := nil;
   FActive := false;
   inherited;
end;

procedure TOrbis01.SetGF(const Value: TOrbisGraphic_Frame);
begin
   FGF := Value;
end;

procedure TOrbis01.Import;
begin
   if not FActive then exit;
   World.MainTool.PostMsg(tm_Import,nil,nil,false);
   Application.ProcessMessages;
end;

procedure TOrbis01.SetDirectory_Store(const Value: string);
begin
   FDirectory_Store := Value;
   if FActive then
      TMainTool(World.MainTool).Options.StoreDirectory := Value;
   Application.ProcessMessages;
end;

procedure TOrbis01.SetDirectory_Project(const Value: string);
begin
   FDirectory_Project := Value;
   if FActive then
      TMainTool(World.MainTool).Options.ProjectDirectory := Value;
end;

function TOrbis01.GetMaps: TThematicMaps;
begin
   result := nil;
   if not FActive then exit;
   Result := TThematicMaps(World.ObjectModel.FindElementClass(TThematicMaps));
end;

function TOrbis01.GetNo_BDE: boolean;
begin
   if FActive then
      if FNo_BDE <> NoBDE then
         FNo_BDE := NoBDE;
    result := FNo_BDE
end;

procedure TOrbis01.SetMaps(const Value: TThematicMaps);
begin

end;

procedure TOrbis01.FillMapsList(aList: TStrings);
var
   i: integer;
begin
   if not FActive then exit;
   if aList = nil then exit;
   aList.Clear;
   if Maps = nil then exit;
   for i := 0 to Maps.LastIndex do aList.Add(Maps.Elements[i].Name);
end;

procedure TOrbis01.SetActualMap(const Value: string);
begin
   FActualMap := Value;
   if not FActive then exit;
   if Maps = nil then exit;
   Maps.MapName := Value;
end;

procedure TOrbis01.ShowMap(aMap: string);
begin
    if not FActive then exit;
    if Maps = nil then exit;
    if Tool = nil then exit;
    Tool.PostMsg(tm_ChangeThematic,Tool,Maps.Element[aMap],false);
    Tool.PostMsg(tm_BreakAndReview,Tool,nil,false);
    Application.ProcessMessages;
    ActualMap := aMap;
    //Maps.MapName := aMap;
end;

procedure TOrbis01.SetupMaps;
var
   ME: TTHematicMapsEditor;
begin
   if not FActive then exit;
   if Maps = nil then exit;
   ME := TTHematicMapsEditor.Create;
   ME.Input := Maps;
   ME.PostTool := Tool;
   ME.DirectPost := true;
   ME.DoIt;
   ME.Free;
end;

procedure TOrbis01.SetDILEC(const Value: string);
begin
  FDILEC := Value;
end;

procedure TOrbis01.SetLHC(const Value: integer);
begin
  FLHC := Value;
end;

procedure TOrbis01.SetODDELENI(const Value: integer);
begin
  FODDELENI := Value;
end;

procedure TOrbis01.SetPOROST(const Value: string);
begin
  FPOROST := Value;
end;

procedure TOrbis01.SetROK(const Value: integer);
begin
   FROK := Value;
   CONTINUAL_ROK := Value;
end;

procedure TOrbis01.SetSKUPINA(const Value: string);
begin
  FSKUPINA := Value;
end;

procedure TOrbis01.ShowTheForest(Pattern: boolean);
var
   FM: Tgis_ForestModel;
   e: TElement;
   LI: TOrbis01LHCInfo;
begin
   Error := 0;
   if not FActive then exit;
   CheckTheForest;
   FM := ForestModel;
   if FM = nil then exit;
   FM.LHC_Cislo := LHC;
   FM.ActualLS := nil; // aby to nasel i v jinym reviru
   FM.Oddeleni_Cislo := Oddeleni;
   if Continual then
   begin
      if FM.ActualOddeleni = nil then
      begin
         LI := Projects.GetLHCInfo(LHC,Rok);
         if LI <> nil then
         begin
            Cont_LoadOddeleni(LI,Oddeleni);
            FM.Oddeleni_Cislo := Oddeleni;
         end;
      end;
   end;
   FM.Dilec_Name := Dilec;
   FM.Porost_Name := Porost;
   FM.Skupina_Name := Skupina;
   e := FM.ActualSkupina;
   if e = nil then
      e := FM.ActualPorost;
   if e = nil then
      e := FM.ActualDilec;
   if e = nil then
      e := FM.ActualOddeleni;
   if e = nil then
      e := FM.ActualLHC;
   TGraphicLinkTool(Tool).ElementForProjection := e;
   LHC := FM.LHC_Cislo;
   ROK := FM.LHC_ROK;
   ODDELENI := FM.Oddeleni_Cislo;
   Dilec := FM.Dilec_Name;
   Porost := FM.Porost_Name;
   Skupina := FM.Skupina_Name;
   Application.ProcessMessages;
   if Pattern then
      TGraphicLinkTool(Tool).ElementForPattern := e;
end;

function TOrbis01.GetForestModel: Tgis_ForestModel;
var
   e: TElement;
begin
   result := nil;
   if not FActive then exit;
   e := World.ObjectModel.FindElementName('DATA');
   if e = nil then exit;
   if not (e is TEntity) then exit;
   result := Tgis_ForestModel(TEntity(e).FindElementClass(tgis_ForestModel));
end;

procedure TOrbis01.SelectEvent(const aElement: TElement);
var
   SK: Tgis_PorostniSkupina;
   PC: Tgis_ParcelaCast;
begin
   SkupinaSelected := false;
   ParcelaSelected := false;
   if aElement =  nil then exit;
   if aElement is Tgis_PorostniSkupina then
   begin
      SkupinaSelected := true;
      SK := Tgis_PorostniSkupina(aElement);
      LHC := SK.LHC;
      ODDELENI := SK.Oddeleni;
      DILEC := SK.Dilec;
      POROST := SK.Porost;
      SKUPINA := SK.Name;
   end
   else
   if aElement is Tgis_ParcelaCast then
   begin
      ParcelaSelected := true;
      PC := Tgis_ParcelaCast(aElement);
      par_KATASTR := PC.KU_KOD;
      par_CISLO := PC.PARCIS;
      par_PODLOMENI := PC.PARPOD;
   end
   else
   if Client then
   begin
      if aElement is Tgis_PorostniSkupinaSelector then
      begin
         SelX := Tgis_PorostniSkupinaSelector(aElement).SelX;
         SelY := Tgis_PorostniSkupinaSelector(aElement).SelY;
         SkupinaSelected := true;
      end
      else
      if aElement is Tgis_ParcelaCastSelector then
      begin
         ParcelaSelected := true;
         SelX := Tgis_ParcelaCastSelector(aElement).SelX;
         SelY := Tgis_ParcelaCastSelector(aElement).SelY;
      end;
   end
   else
      exit;
   if Assigned(OnSelect) then
      OnSelect;
end;

procedure TOrbis01.SetOnSelect(const Value: TOnSelectEvent);
begin
  FOnSelect := Value;
end;

function TOrbis01.GetProjects: TOrbis01Projects;
begin
   result := nil;
   if not FActive then exit;
   result := TOrbis01Projects(World.ObjectModel.FindElementClass(TOrbis01Projects));
end;

procedure TOrbis01.CHeckTheForest;
var
   LI: TOrbis01LHCInfo;
   LHC_KOD,LHC_Year: integer;
   DataProfN: string;
   S: string;
begin
   if not FActive then exit;
   if LHC <= 0 then // Nesmysln slo LHC
   begin
      Error := Err_IncorrectLHCNumber;
      if ClearIfLHCNotFound then
         ClearData;
      exit;
   end;
   if Projects = nil then  // Projekty chyb ... nco je blb
   begin
      Error := Err_Projects;
      if ClearIfLHCNotFound then
         ClearData;
      exit;
   end;
   LI := Projects.GetLHCInfo(LHC,ROK);
   LHC_KOD := LHC;
   if LI = nil then // LHC nen pipojeno
   begin
      Error := err_LHCNotConected;
      if ClearIfLHCNotFound and not Continual then
         ClearData;
      exit;
   end;
   LHC_Year := LI.LHCYear;
   DataProfN := LI.DataProfileName;
   if Continual then
   begin
      if not LI.Continual then
         Error := err_Cont_LHCNotConected;
      //ForestModel
      exit;
   end;
   // LHC je pipojeno, kontrolujeme, zda je projekt naten
   if Project = nil then // zjevn nen
   begin
      S := Directory_Project;
      CHeckBackSlash(S);
      S := S + Li.ProjectName+'.opr';
      TMainTool(World.MainTool).CheckLHCYear(LHC_KOD, LHC_Year, true);
      TMainTool(World.MainTool).NextUserDataSplitted := LI.UserDataSplitted;
      LoadProject(S);
      TMainTool(World.MainTool).DataProfileName := DataProfN;
      TMainTool(World.MainTool).CheckDataProfile;

//      CHeckLHCYear(LHC_KOD, LHC_Year);
   end
   else
   if Project.Name <> LI.ProjectName then // zase ne
   begin
      S := Directory_Project;
      CHeckBackSlash(S);
      S := S + Li.ProjectName+'.opr';
      TMainTool(World.MainTool).CheckLHCYear(LHC_KOD, LHC_Year, true);
      TMainTool(World.MainTool).NextUserDataSplitted := LI.UserDataSplitted;
      LoadProject(S);
      TMainTool(World.MainTool).DataProfileName := DataProfN;
      TMainTool(World.MainTool).CheckDataProfile;
//      CHeckLHCYear(LHC_KOD, LHC_Year);
   end;
end;

function TOrbis01.GetProject: TORBISLHCProject;
begin
   result := nil;
   if not FActive then exit;
   result := TMainTool(World.MainTool).Project;
end;

function TOrbis01.GetProjections: TProjectionInfoModel;
begin
   result := nil;
   if not FActive then exit;
   result := TProjectionInfoModel(World.ObjectModel.FindElementClass(TProjectionInfoModel));
end;

procedure TOrbis01.LoadProject(aProjectName: string);
begin
   if not FActive then exit;
   TMainTool(World.MainTool).LoadProjectFromFile(aProjectName);
   LHCMonitor.Reactivate;
end;

function TOrbis01.LoadProjectDataFromORB(const aFileName, DataType: string): integer;
begin
   result := TMainTool(World.MainTool).LoadProjectDataFormORB(aFileName, DataType);
end;

procedure TOrbis01.NewProjectDialog;
begin
   if not FActive then exit;
   World.MainTool.PostMsg(tm_NewProject,nil,nil,false);
   Application.ProcessMessages;
   LHCMonitor.Reactivate;
end;

procedure TOrbis01.ViewAll;
begin
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   TGraphicLinkTool(Tool).ViewAll := true;
   Application.ProcessMessages;
end;

procedure TOrbis01.ZoomIn;
begin
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   TGraphicLinkTool(Tool).Zoom := 'IN';
   Application.ProcessMessages;
end;

procedure TOrbis01.ZoomOut;
begin
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   TGraphicLinkTool(Tool).Zoom := 'OUT';
   Application.ProcessMessages;
end;

procedure TOrbis01.SetOnProjectionCHanged(
  const Value: TOnProjectionChangedEvent);
begin
  FOnProjectionCHanged := Value;
end;

function TOrbis01.GetCenterX: double;
begin
   result := 0;
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   result := TGraphicLinkTool(Tool).Projection.Center_X;
end;

function TOrbis01.GetCenterY: double;
begin
   result := 0;
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   result := TGraphicLinkTool(Tool).Projection.Center_Y;
end;

function TOrbis01.GetScale: double;
begin
   result := 0;
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   result := TGraphicLinkTool(Tool).Projection.Scale;
end;

procedure TOrbis01.RenFil(aFileName, aFileNameN: string);
begin
   TMainTool(World.MainTool).RenFil(aFileName, aFileNameN);
end;

function TOrbis01.RenumberLHC(LHC_OLD, LHC_NEW: integer): boolean;
var
   SLE: TStringListElement;
begin
   SLE := TStringListElement.Create;
   World.ObjectModel.NextElement := SLE;
   SLE.Name := 'LHC_RENUM';
   SLE.SetPropertyLongIntByName('LHC_OLD',LHC_OLD);
   SLE.SetPropertyLongIntByName('LHC_NEW',LHC_NEW);
   SLE.SetPropertyBooleanByName('Result',false);
   ExecuteScript('GORBIS\LHCRenumber.oil');
   Result := SLE.GetBooleanByName('Result');
   SLE.Free;
end;

procedure TOrbis01.ReView;
begin
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   Error := 0;
   Tool.PostMsg(tm_BreakAndReview,Tool,nil,false);
   Application.ProcessMessages;
   if ManualProjectionChange then
   begin
      ManualProjectionChange := false;
      ProjectionChangedEvent(CenterX, CenterY, Scale, VirtualScale);
   end;
end;

procedure TOrbis01.SetCenterX(const Value: double);
begin
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   TGraphicLinkTool(Tool).Projection.Center_X := Value;
   ManualProjectionChange := true;
end;

procedure TOrbis01.SetCenterY(const Value: double);
begin
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   TGraphicLinkTool(Tool).Projection.Center_Y := Value;
   ManualProjectionChange := true;
end;

procedure TOrbis01.SetScale(const Value: double);
begin
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   TGraphicLinkTool(Tool).Projection.Scale := Value;
   ManualProjectionChange := true;
end;

procedure TOrbis01.SetSelX(const Value: double);
begin
  FSelX := Value;
end;

procedure TOrbis01.SetSelY(const Value: double);
begin
  FSelY := Value;
end;

procedure TOrbis01.Print;
begin
   if not FActive then exit;
   Tool.PostMsg(tm_Print,Tool,nil,false);
   Application.ProcessMessages;
end;


procedure TOrbis01.SetDirectory_Root(const Value: string);
begin
  FDirectory_Root := Value;
end;

procedure TOrbis01.NewProject(const aPath,DataProfile: string);
begin
   Error := 0;
   if not FActive then exit;
   TMainTool(World.MainTool).DataProfileName := DataProfile;
   World.MainTool.PostTextMsg(tm_NewProject,aPath,nil);
   Application.ProcessMessages;
   if World.MainTool.Error > 0 then
      Error := err_InCorrectProjectPath;
   LHCMonitor.Reactivate;
end;

procedure TOrbis01.PrintMap;
begin
   if not FActive then exit;
   TMainTool(World.MainTool).PrintMap;
end;

function TOrbis01.PrintSymbLegToCanvas(aCanvas: TCanvas; aScale: double;
  aR: TRect; aDPI: integer; UseRectAndDPI, Test: boolean; var Rows: integer;
  OneSymbol: boolean; aKind: char; Kod: integer): boolean;
var
   CD: TCanvasDevice;
//   IP: TInterpret;
   aRes: double;
   SL: TStringListElement;
   En: TOwnerEntity;
   aScript: string;
begin
   Result := false;
   if not FActive then exit;
   CD := TCanvasDevice.Create;
   CD.Canvas := aCanvas;
   if UseRectAndDPI then
   begin
      CD.Page.Rect := aR;
      CD.Page.ClipRegion := aR;
      CD.Page.Clipping := true;
      CD.Page.CenterToClipRegion := true;
      if aDPI > 0  then
      begin
         aRes := 1000 * aDPI / 25.4;
         CD.Page.SetResolution(aRes, aRes);
      end;
   end;
   CD.Projection.ChangeProjectionSimple(CD.Page);
   // Struktura pro script
   En := TOwnerEntity.Create;
   En.Name := 'LEGCANVAS';
   SL := TStringListElement.Create;
   SL.Name := 'Params';
   En.NextElement := SL;
   World.ObjectModel.NextElement := EN;
   SL.AddNameString('Scale',FLoatToStr(aScale));
   SL.AddNameString('Minx',FLoatToStr(CD.Projection.ModelMinX));
   SL.AddNameString('Miny',FLoatToStr(CD.Projection.ModelMinY));
   SL.AddNameString('Maxx',FLoatToStr(CD.Projection.ModelMaxX));
   SL.AddNameString('Maxy',FLoatToStr(CD.Projection.ModelMaxY));
   SL.AddNameString('Maxy',FLoatToStr(CD.Projection.ModelMaxY));
   SL.AddNameString('Result','True');
   if OneSymbol then
   begin
      SL.AddNameString('Kind',aKind);
      SL.AddNameString('Kod',IntToStr(Kod));
      aScript := 'GORBIS\PrintSymbolToCanvas.oil';
   end
   else
   begin
      if Test then
         SL.AddNameString('Test','True')
      else
         SL.AddNameString('Test','False');
      SL.AddNameString('Rows','0');
      aScript := 'GORBIS\PrintLegendToCanvas.oil';
   end;
   En.NextElement := CD;
   ExecuteScript(aScript);
   if not OneSymbol then
      Rows := SL.GetIntegerByName('Rows');
   Result := SL.GetBooleanByName('Result');
   En.Free;
   //CD.Free;
end;

function TOrbis01.PrintSymbolToCanvas(aCanvas: TCanvas; aScale: double;
  aR: TRect; aDPI: integer; UseRectAndDPI: boolean; aKind: char;
  Kod: integer): boolean;
var
   arows: integer;
begin
    result := PrintSymbLegToCanvas(aCanvas, aScale, aR, aDPI, UseRectAndDPI, false, aRows, true, aKind, Kod);
end;

function TOrbis01.PrintLegendToCanvas(aCanvas: TCanvas; aScale: double;
  aR: TRect; aDPI: integer; UseRectAndDPI: boolean; Test: boolean; var Rows: integer): boolean;
begin
    result := PrintSymbLegToCanvas(aCanvas, aScale, aR, aDPI, UseRectAndDPI, Test, Rows, false,'L',0);
end;

procedure TOrbis01.PrintToCanvas(aCanvas: TCanvas;var  aScale: double; aR: TRect; aDPI: integer; UseRectAndDPI: boolean);
var
   CD: TCanvasDevice;
   G: TGraphicLinkTool;
   BackupDevice: TDevice;
   IP: TInterpret;
   aRes: double;
begin
   if not FActive then exit;
   G := TGraphicLinkTool(Tool);
   if G = nil then exit;
   IP := G.Interpret;
   BackupDevice := IP.Device;
   CD := TCanvasDevice.Create;
   CD.Canvas := aCanvas;
   CD.Page.Color := BackupDevice.Page.Color;
   if UseRectAndDPI then
   begin
      CD.Page.Rect := aR;
      CD.Page.ClipRegion := aR;
      CD.Page.Clipping := true;
      CD.Page.CenterToClipRegion := true;
      if aDPI > 0  then
      begin
        aRes := 1000 * aDPI / 25.4;
        CD.Page.SetResolution(aRes, aRes);
      end;
   end;
   IP.Device := CD;
   if aScale = 0 then
   begin
      CD.Projection.ChangeProjectionByRect(BackupDevice.Projection.ModelView,CD.Page);
      aScale := CD.Projection.Scale;
   end
   else
      CD.Projection.ChangeProjection(BackupDevice.Projection.CenterPoint,aScale,CD.Page);
   IP.DoIt;
   IP.Device := BackupDevice;
   CD.Free;
end;

procedure TOrbis01.SetActiveToolName(const Value: string);
begin
   if not FActive then exit;
   TGraphicLinkTool(Tool).ActiveToolName := Value;
end;

procedure TOrbis01.GeneralEvent(_ID, Value: string);
begin
   if Assigned(OnGeneralEvent) then
      OnGeneralEvent(_ID,Value);
end;

function TOrbis01.GetActiveToolName: string;
begin
   result := '';
   if not FActive then exit;
   result := TGraphicLinkTool(Tool).ActiveToolName;
end;

//procedure TOrbis01.AppIdle(Sender: TObject; var Done: Boolean);
//begin
//end;

function TOrbis01.FillMapsString: string;
var
   i: integer;
begin
   if not FActive then exit;
   result := '';
   if Maps = nil then exit;
   for i := 0 to Maps.LastIndex do
   begin
      if i > 0 then result := result + ',';
      result :=  result + '"'+Maps.Elements[i].Name+'"';
   end;
end;

{$IFDEF OCX}
procedure TOrbis01.Activate;
begin
   SetActive(true);
end;
{$ENDIF}

procedure TOrbis01.SetOnAreaChanged(const Value: TOnMeasureEvent);
begin
  FOnAreaChanged := Value;
end;

procedure TOrbis01.SetOnLengthChanged(const Value: TOnMeasureEvent);
begin
  FOnLengthChanged := Value;
end;

procedure TOrbis01.OILEditor;
begin
   if not FActive  then exit;
   Tool.PostMsg(tm_OIL,nil,nil,false);
end;

procedure TOrbis01.OnKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   if not FActive then exit;
   if Tool = nil then exit;
   TDeviceTool(Tool).PostKeyEvent(tm_KeyDown, Tool, Key, Shift);
   //Tool.PostTextMsg(tm_AddToProtocol,'Key: '+IntToStr(Key)+ ' '+CHar(Key),nil);
end;

procedure TOrbis01.OnWheelDown(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
var
   P: TPoint;
begin
    Handled := true;
    if Tool = nil then exit;
    if GF = nil  then exit;

    P := GF.Image.ScreenToClient(MousePos);
    TDeviceTool(Tool).PostMouseEvent(tm_MouseWheelDown, Tool, mbMiddle, Shift, P.X, P.Y, FActive);
end;

procedure TOrbis01.OnWheelUp(Sender: TObject; Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
var
   P: TPoint;
begin
    Handled := true;
    if Tool = nil then exit;
    if GF = nil  then exit;

    P := GF.Image.ScreenToClient(MousePos);
    TDeviceTool(Tool).PostMouseEvent(tm_MouseWheelUp, Tool, mbMiddle, Shift, P.X, P.Y, FActive);
end;

procedure TOrbis01.NewKatastr(const aPath: string; aID: integer);
begin
   Error := 0;
   if not FActive then exit;
   TMainTool(World.MainTool).NewKatastr(aPath,aID);
   if World.MainTool.Error > 0 then
      Error := err_InCorrectProjectPath;
   LHCMonitor.Reactivate;
end;

procedure TOrbis01.NewKatastrDialog;
begin
   {
   if not FActive then exit;
   World.MainTool.PostMsg(tm_NewKatastr,nil,nil,false);
   Application.ProcessMessages;
   }
end;

function TOrbis01.GetParam(aName: string): string;
begin
   result := '';
   if not FActive then exit;
   result := TMainTool(World.MainTool).GetParam(aName);
end;

function TOrbis01.GetParcelModel: Tgis_ParcelModel;
var
   e: TElement;
begin
   result := nil;
   if not FActive then exit;
   e := World.ObjectModel.FindElementName('DATA');
   if e = nil then exit;
   if not (e is TEntity) then exit;
   result := Tgis_ParcelModel(TEntity(e).FindElementClass(tgis_ParcelModel));
end;

function TOrbis01.LHC_SetUserDataUUID(LHC_KOD, LHC_Year: integer; aUserID: string; SetUserID: boolean): boolean;
begin
   result := false;
   if LHC_UserDataUUIDIsSet(LHC_KOD, LHC_Year) then exit;
   if Error > 0 then exit;
   if not FActive then exit;
   result := TMainTool(World.MainTool).SetUserDataUUID(LHC_KOD, LHC_Year, aUserID, SetUserID);
end;

function TOrbis01.LHC_SplitUserData(LHC_KOD, LHC_Year: integer): boolean;
begin
   result := false;
   if LHC_UserDataSplitted(LHC_KOD, LHC_Year) then exit;
   if Error > 0 then exit;
   if not FActive then exit;
   result := TMainTool(World.MainTool).SplitUserData(LHC_KOD, LHC_Year);
end;

function TOrbis01.LHC_UserDataSplitted(LHC_KOD, LHC_Year: integer): boolean;
var
   LI: TOrbis01LHCInfo;
//   S: string;
//   LHC_ID: string;
//   aFile: string;
//   PrjS: string;
begin
   result := false;
   Error := 0;
   if not FActive then exit;
   if Projects = nil then  // Projekty chyb ... nco je blb
   begin
      Error := Err_Projects;
      exit;
   end;
   LI := Projects.GetLHCInfo(LHC_KOD, LHC_Year, true);
   if LI = nil then // LHC nen pipojeno
   begin
      Error := err_LHCNotConected;
      exit;
   end;
   result := LI.UserDataSplitted;
end;

function TOrbis01.LHC_UserDataUUIDIsSet(LHC_KOD, LHC_Year: integer): boolean;
var
   LI: TOrbis01LHCInfo;
//   S: string;
//   LHC_ID: string;
//   aFile: string;
//   PrjS: string;
begin
   result := false;
   Error := 0;
   if not FActive then exit;
   if Projects = nil then  // Projekty chyb ... nco je blb
   begin
      Error := Err_Projects;
      exit;
   end;
   LI := Projects.GetLHCInfo(LHC_KOD, LHC_Year, true);
   if LI = nil then // LHC nen pipojeno
   begin
      Error := err_LHCNotConected;
      exit;
   end;
   result := LI.UserDataUUIDExists;
end;

procedure TOrbis01.LoadKatastr(aFileName: string);
begin
   TMainTool(World.MainTool).LoadKatastrData(aFileName);
   LHCMonitor.Reactivate;
end;

procedure TOrbis01.LoadKatastrID(aID: integer);
begin
   TMainTool(World.MainTool).LoadKatastr(aId);
   Error := TMainTool(World.MainTool).Error;
   LHCMonitor.Reactivate;
end;

procedure TOrbis01.CHeckTheKatastr;
var
   LK: TOrbis01KatastrInfo;
//   S: string;
begin
   if not FActive then exit;
   if par_KATASTR <= 0 then // Nesmysln slo Katastru
   begin
      Error := Err_IncorrectLHCNumber;
      exit;
   end;
   if ParcelModel.GetKatastrCislo(par_Katastr) <> nil then
      exit; //Katastr existuje a je nacten
   if Projects = nil then  // Projekty chyb ... nco je blb
   begin
      Error := Err_Projects;
      exit;
   end;
   LK := Projects.GetKatastrInfo(par_Katastr);
   if LK = nil then // LHC nen pipojeno
   begin
      Error := err_LHCNotConected;
      exit;
   end;
   LoadKatastrID(par_Katastr);
   {
   S := Directory_Project;
   CHeckBackSlash(S);
   S := S + LK.ProjectName;
   LoadKatastr(S);
   }
end;

procedure TOrbis01.Setpar_CISLO(const Value: integer);
begin
  Fpar_CISLO := Value;
end;

procedure TOrbis01.Setpar_KATASTR(const Value: integer);
begin
  Fpar_KATASTR := Value;
end;

procedure TOrbis01.Setpar_PODLOMENI(const Value: integer);
begin
  Fpar_PODLOMENI := Value;
end;

procedure TOrbis01.ShowTheKatastr(Pattern: boolean);
var
   PM: Tgis_ParcelModel;
   e: TElement;
begin
   Error := 0;
   if not FActive then exit;
   CheckTheKatastr;
   PM := ParcelModel;
   if PM = nil then exit;
   PM.Katastr_Cislo := par_KATASTR;
   PM.GetParcelaCislo(par_CISLO,par_PODLOMENI,0);
   e := PM.ActualParcela;
   if e = nil then
      e := PM.ActualKatastr;
   if e = nil then
   begin
      par_Katastr := 0;
      par_Cislo := 0;
      par_Podlomeni := 0;
      exit;
   end;
   TGraphicLinkTool(Tool).ElementForProjection := e;
   par_Katastr := PM.Katastr_Cislo;
   if PM.ActualParcela <> nil then
   begin
      par_Cislo := PM.ActualParcela.PARCIS;
      par_Podlomeni := PM.ActualParcela.PARPOD;
   end
   else
   begin
      par_Cislo := 0;
      par_Podlomeni := 0;
   end;
   Application.ProcessMessages;
   if Pattern then
      TGraphicLinkTool(Tool).ElementForPattern := e;
end;

procedure TOrbis01.SetNewUserDataSplitted(const Value: boolean);
begin
    FNewUserDataSplitted := Value;
    if FActive then
       TMainTool(World.MainTool).NewUserDataSplitted := Value;
end;

procedure TOrbis01.SetNoProtocol(const Value: boolean);
begin
   FNoProtocol := Value;
   if not FActive then exit;
   TMainTool(World.MainTool).NoProtocol := Value;
end;

procedure TOrbis01.SetNo_BDE(const Value: boolean);
begin
   FNo_BDE := Value;
   if FActive then
      NoBDE := Value;
end;

procedure TOrbis01.SetOnNewModel(const Value: TOnNewModel_Event);
begin
  FOnNewModel := Value;
end;

procedure TOrbis01.NewModelEvent;
begin
   ActualSelect := nil;
   if Assigned(OnNewModel) then
      OnNewModel;
end;

procedure TOrbis01.SetOnUserLHCAreaDialog(const Value: TOnUserLHCAreaDialogEvent);
begin
  FOnUserLHCAreaDialog := Value;
end;

procedure TOrbis01.ProcessUserLHCAreaDialog(NewObject: boolean;Area: double; _Skupina: TElement; var UserID, Group, Color, Symbol,
  aresult: integer; var Props: string);
var
   SK: Tgis_PorostniSkupina;
begin
   if (_Skupina =  nil) or not(_Skupina is Tgis_PorostniSkupina) then
   begin
   end
   else
   begin
      SK := Tgis_PorostniSkupina(_Skupina);
      LHC := SK.LHC;
      ODDELENI := SK.Oddeleni;
      DILEC := SK.Dilec;
      POROST := SK.Porost;
      SKUPINA := SK.Name;
   end;
   if Assigned(OnUserLHCAreaDialog) then
      OnUserLHCAreaDialog(NewObject,Area,UserID,Group,Color,Symbol,aresult,Props);
end;

procedure TOrbis01.ProgressEvent(_ID, _Event: integer; _Name, _Text: string;
  _MinValue, _MaxValue, _Progress: integer);
begin
   case _Event  of
      tm_StartProgress:
         if Assigned(On_ProgressStart) then
            On_ProgressStart(_ID,_Name,_MinValue,_MaxValue,_MinValue);
      tm_EndProgress:
         if Assigned(On_ProgressEnd) then
            On_ProgressEnd(_ID,0);
      tm_Progress:
         if Assigned(On_Progress) then
            On_Progress(_ID,_Progress);
      tm_MaxProgress:
         if Assigned(On_ProgressMax) then
            On_ProgressMax(_ID,_MaxValue);
      tm_ProgressText: if Assigned(On_ProgressText) then
         On_ProgressText(_ID, _Text);
   end;
end;

procedure TOrbis01.SaveUserDataEvent(var ToSave: boolean);
begin
   if Assigned(OnSaveUserData) then
      OnSaveUserData(ToSave);
end;

procedure TOrbis01.SetOnSaveUserData(const Value: TOnSaveUserData_Event);
begin
  FOnSaveUserData := Value;
end;

procedure TOrbis01.CheckUserDataForSave;
begin
   if FActive then
      TMainTool(World.MainTool).CheckUserDataForSave;
end;

procedure TOrbis01.FillAreSymbolsList(aList: TStrings);
var
   PS: TPrepareAreSymbols;
begin
   PS := TPrepareAreSymbols.Create;
   PS.FillARESymbolsList(aList);
   PS.Free;
end;
{$IFDEF FILEEVENT}
procedure TOrbis01.FileAfterSaveEvent(const aFile: string);
begin
   if Assigned(On_FileAfterSave) then
      On_FileAfterSave(aFile);

end;

procedure TOrbis01.FileBeforeLoadEvent(const aFile: string);
begin
   if Assigned(On_FileBeforeLoad) then
      On_FileBeforeLoad(aFile);
end;

procedure TOrbis01.FileDeleteEvent(const aFile: string);
begin
   if Assigned(On_FileDelete) then
      On_FileDelete(aFile);
end;

procedure TOrbis01.FileExistsEvent(const aFile: string);
begin
   if Assigned(On_FileExists) then
      On_FileExists(aFile);
end;
{$ENDIF}
procedure TOrbis01.FillAreSymbolsComboBoxEx(aCombo: TComboBoxEx;
  aList: TStrings; ImageWidth, ImageHeight: integer);
var
   PS: TPrepareAreSymbols;
begin
   PS := TPrepareAreSymbols.Create;
   PS.FillComboBoxEx(aCombo,aList,ImageWidth,ImageHeight);
   PS.Free;
end;

function TOrbis01.OrbisIndexToColor(aIndex: integer): integer;
begin
   result := DejBarvicku(aIndex);
end;

procedure TOrbis01.HighlightTheForest;
var
   FM: Tgis_ForestModel;
   e: TElement;
begin
   Error := 0;
   if not FActive then exit;
   CheckTheForest;
   FM := ForestModel;
   if FM = nil then exit;
   FM.LHC_Cislo := LHC;
   FM.Oddeleni_Cislo := Oddeleni;
   FM.Dilec_Name := Dilec;
   FM.Porost_Name := Porost;
   FM.Skupina_Name := Skupina;
   e := FM.ActualSkupina;
   if e = nil then
      e := FM.ActualPorost;
   if e = nil then
      e := FM.ActualDilec;
   if e = nil then
      e := FM.ActualOddeleni;
   if e = nil then
      e := FM.ActualLHC;
   //TGraphicLinkTool(Tool).ElementForProjection := e;
   LHC := FM.LHC_Cislo;
   ODDELENI := FM.Oddeleni_Cislo;
   Dilec := FM.Dilec_Name;
   Porost := FM.Porost_Name;
   Skupina := FM.Skupina_Name;
   Application.ProcessMessages;
   TGraphicLinkTool(Tool).ElementForPattern := e;
end;

procedure TOrbis01.HighlightTheKatastr;
var
   PM: Tgis_ParcelModel;
   e: TElement;
begin
   Error := 0;
   if not FActive then exit;
   CheckTheKatastr;
   PM := ParcelModel;
   if PM = nil then exit;
   PM.Katastr_Cislo := par_KATASTR;
   PM.GetParcelaCislo(par_CISLO,par_PODLOMENI,0);
   e := PM.ActualParcela;
   if e = nil then
      e := PM.ActualKatastr;
   if e = nil then
   begin
      par_Katastr := 0;
      par_Cislo := 0;
      par_Podlomeni := 0;
      exit;
   end;
   //TGraphicLinkTool(Tool).ElementForProjection := e;
   par_Katastr := PM.Katastr_Cislo;
   if PM.ActualParcela <> nil then
   begin
      par_Cislo := PM.ActualParcela.PARCIS;
      par_Podlomeni := PM.ActualParcela.PARPOD;
   end
   else
   begin
      par_Cislo := 0;
      par_Podlomeni := 0;
   end;
   Application.ProcessMessages;
   TGraphicLinkTool(Tool).ElementForPattern := e;
end;

procedure TOrbis01.SetParam(const aName, aValue: string);
begin
   if not FActive then exit;
   TMainTool(World.MainTool).SetParam(aName, aValue);
end;

procedure TOrbis01.SetParcelaSelected(const Value: boolean);
begin
  FParcelaSelected := Value;
end;

procedure TOrbis01.SetSkupinaSelected(const Value: boolean);
begin
  FSkupinaSelected := Value;
end;

procedure TOrbis01.SetLHCMonitor(const Value: TLHCPositionMonitor);
begin
   FLHCMonitor := Value;
end;

procedure TOrbis01.SetLHC_ROK(const Value: string);
var
   S1,S2: string;
begin
   StringAsTwoParamsWS(Value,'_',S1,S2);
   LHC := StrToIntDef(S1,0);
   ROK := StrToIntDef(S2,0);
end;

procedure TOrbis01.SetMonitorActive(const Value: boolean);
begin
   FMonitorActive := Value;
   if not FActive then exit;
   LHCMonitor.Active := Value;
end;

procedure TOrbis01.SetMonitorCoords(const Value: boolean);
begin
  FMonitorCoords := Value;
   if not FActive then exit;
   LHCMonitor.MonitorCoords := Value;
end;

procedure TOrbis01.SetMonitorLHC(const Value: boolean);
begin
  FMonitorLHC := Value;
   if not FActive then exit;
   LHCMonitor.MonitorLHC := Value;
end;

procedure TOrbis01.SetMonitorParcely(const Value: boolean);
begin
  FMonitorParcely := Value;
   if not FActive then exit;
   LHCMonitor.MonitorParcely := Value;
end;

procedure TOrbis01.SetOnMonitorCoordsChange(
  const Value: TOn_MonitorCoordsChange);
begin
  FOnMonitorCoordsChange := Value;
end;

procedure TOrbis01.SetOnMonitorLHCCHange(
  const Value: TOn_MonitorLHCChange);
begin
  FOnMonitorLHCCHange := Value;
end;

procedure TOrbis01.SetOnMonitorParcelyCHange(
  const Value: TOn_MonitorParcelyChange);
begin
  FOnMonitorParcelyCHange := Value;
end;

function TOrbis01.JoinProjects(aSourcePath: string): integer;
begin
   result := 1;
   if not FActive then exit;
   result := TMainTool(World.MainTool).JoinProjects(aSourcePath);
end;

procedure TOrbis01.LoadUserDataFrom(const aFileName: string);
begin
   if not FActive then exit;
   TMainTool(World.MainTool).LoadUserData(aFileName);
end;

procedure TOrbis01.SaveUserDataTo(const aFileName: string; ActualLayerOnly: boolean);
begin
   if not FActive then exit;
   TMainTool(World.MainTool).SaveUserData(aFileName,ActualLayerOnly);
end;

procedure TOrbis01.ExecuteScript(const aScript: string);
begin
   if not FActive then exit;
   TMainTool(World.MainTool).ExecuteScript := aScript;
end;

function TOrbis01.ShowKartoText(aText: string): boolean;
var
   i: integer;
   el: TEntity;
   e: TELement;
   aLayers: TEntity;
   TE: TTopolTextElement;
begin
   result := false;
   if not FActive then exit;
   aLayers := Layers;
   if aLayers = nil then exit;
   el := TEntity(aLayers.FindElementName('TXT_TXT'));
   if (el = nil) or (el.Count = 0)  then exit;
   for i := 0 to el.LastIndex do
   begin
      e := el.Elements[i];
      if not (e is TTopolTextElement) then continue;
      TE := TTopolTextElement(e);
      if TE.Text = aText then
      begin
         result := true;
         CenterX := TE.X;
         CenterY := TE.Y;
         ReView;
         Application.ProcessMessages;
         break;
      end;
   end;
end;

function TOrbis01.FindPSKByXY(aX, aY: double): Tgis_Porostniskupina;
var
   Sel: TForestModelSelector;
begin
//   result := nil;
   Sel := TForestModelSelector.Create;
   Sel.FindInput;
   Sel.FrameClassName := 'Tgis_PorostniSkupina';
   Sel.X := aX;
   Sel.Y := aY;
   Sel.DoIt;
   result := Tgis_PorostniSkupina(Sel.Output);
   if result <> nil then
   begin
      LHC := result.LHC;
      ODDELENI := result.Oddeleni;
      Dilec := result.Dilec;
      Porost := result.Porost;
      Skupina := result.Name;
      SkupinaSelected := true;
   end;
   Sel.Free;
end;

function TOrbis01.FindTheForest: Tgis_ForestUnit;
var
   FM: Tgis_ForestModel;
   e: TGis_ForestUnit;
   LI: TOrbis01LHCInfo;
begin
   Error := 0;
   result := nil;
   if not FActive then exit;
   CheckTheForest;
   FM := ForestModel;
   if FM = nil then exit;
   FM.LHC_Cislo := LHC;
   FM.ActualLS := nil; // aby to nasel i v jinym reviru
   FM.Oddeleni_Cislo := Oddeleni;
   if Continual then
   begin
      if FM.ActualOddeleni = nil then
      begin
         LI := Projects.GetLHCInfo(LHC,ROK);
         Cont_LoadOddeleni(LI,Oddeleni);
         FM.Oddeleni_Cislo := Oddeleni;
      end;
   end;
   FM.Dilec_Name := Dilec;
   FM.Porost_Name := Porost;
   FM.Skupina_Name := Skupina;
   e := FM.ActualSkupina;
   if e = nil then
      e := FM.ActualPorost;
   if e = nil then
      e := FM.ActualDilec;
   if e = nil then
      e := FM.ActualOddeleni;
   if e = nil then
      e := FM.ActualLHC;
   result := e;
   LHC := FM.LHC_Cislo;
   ROK := FM.LHC_ROK;
   ODDELENI := FM.Oddeleni_Cislo;
   Dilec := FM.Dilec_Name;
   Porost := FM.Porost_Name;
   Skupina := FM.Skupina_Name;
   Application.ProcessMessages;
end;

procedure TOrbis01.SetOnNewDATA(const Value: TOnNewModel_Event);
begin
  FOnNewDATA := Value;
end;

procedure TOrbis01.NewDATAEvent;
begin
   ActualSelect := nil;
   if Assigned(OnNewDATA) then
      OnNewDATA;
end;

procedure TOrbis01.SetContinual(const Value: boolean);
var
   MT: TMainTool;
//   Act: boolean;
begin
   if not FActive then exit;
   MT := TMainTool(World.MainTool);
   if Value = MT.Continual then exit;
   MT.Continual := Value;
end;

procedure TOrbis01.SetContinual_Rok(const Value: integer);
begin
   if not FActive then exit;
   TMainTool(World.MainTool).ContinualYear := Value;
end;

procedure TOrbis01.ProjectionChangedEvent(aX, aY, aScale, aVirtualScale: double);
var
   P: TProjectionInfoModel;
begin
   try
   P := Projections;
   if P <> nil  then
      P.Projections.ProjectionChange(aX,aY,aScale,aVirtualScale);
   if Assigned(OnProjectionCHanged) then
      OnProjectionCHanged(aX, aY, aScale, aVirtualScale);
   if not Continual then exit;
   CHeckContinualSpace;
   except

   end;
end;

function TOrbis01.Projection_Move(aNext: boolean): boolean;
var
   P: TProjectionInfoModel;
   Ppi: TProjectionInfo;
begin
   result := false;
   if not FActive then exit;
   P := Projections;
   if P = nil then exit;
   if aNext then
      P.Projections.Next
   else
      P.Projections.Previous;
   Ppi := P.Projections.ActualProjectionInfo;
   Scale := Ppi.Scale;
   VirtualScale := Ppi.VirtualScale;
   CenterX := Ppi.X;
   CenterY := Ppi.Y;
   Tool.PostMsg(tm_BreakAndReview,Tool,nil,false);
   Application.ProcessMessages;
   if ManualProjectionChange then
      ManualProjectionChange := false;
   result := true;
end;


function TOrbis01.Projection_Status(var NoNext, NoPrevious: boolean): boolean;
var
   P: TProjectionInfoModel;
begin
   result := false;
   if not FActive then exit;
   P := Projections;
   if P = nil then exit;
   NoNext := P.Projections.NoNext;
   NoPrevious := P.Projections.NoPrevious;
   result := true;
end;

function TOrbis01.QueryUserLayerCounts(LHC_KOD, LHC_Year: integer; aLayerName: string; var aCountR: TOrbis01UserLayerCounts): boolean;
var
   aSplit: boolean;
begin
   result := false;
   aSplit := LHC_UserDataSplitted(LHC_KOD, LHC_Year);
   if Error > 0 then exit;
   result := TMainTool(World.MainTool).QueryUserLayerCounts(LHC_KOD, LHC_Year, aLayerName, aCountR, aSplit);
end;

function TOrbis01.GetContinual: boolean;
begin
   result := false;
   if not FActive then exit;
   Result := TMainTool(World.MainTool).Continual;
end;

function TOrbis01.GetContinual_Rok: integer;
begin
   result := 0;
   if not FActive then exit;
   result := TMainTool(World.MainTool).ContinualYear;
end;

procedure TOrbis01.SetOnContinualChanged(const Value: TOnContinualChange_Event);
begin
   FOnContinualChanged := Value;
end;

procedure TOrbis01.ContinualChangeEvent(aContinual: boolean);
begin
   if Assigned(OnContinualChanged) then
      OnContinualChanged(aContinual);
   if Continual then
      CHeckContinualSpace;
   LHCMonitor.Reactivate;
end;

procedure TOrbis01.SaveProjectAsContinual;
begin
   if not FActive then exit;
   TMainTool(World.MainTool).SaveProjectAsContinual;
end;

function TOrbis01.ChangeLHCYear(LHC_KOD, LHC_Year, NewYear: integer): boolean;
var
   LI, LI1: TOrbis01LHCInfo;
   S,SN: string;
   LHC_ID,LHC_ID_New: string;
   aFile, aFileN: string;
   PrjS,PrjN: string;
   Load: TLoad;
   Save: TSave;
   Prj: TOrbisLHCProject;
begin
   result := false;
   Error := 0;
   if not FActive then exit;
   if Projects = nil then  // Projekty chyb ... nco je blb
   begin
      Error := Err_Projects;
      exit;
   end;
   LI := Projects.GetLHCInfo(LHC_KOD, LHC_Year, true);
   if LI = nil then // LHC nen pipojeno
   begin
      Error := err_LHCNotConected;
      exit;
   end;
   LI1 := Projects.GetLHCInfo(LHC_KOD, NewYear, true);
   if LI1 <> nil then // LHC uz existuje
   begin
      Error := err_LHCChangeError;
      exit;
   end;
   // Deleting Continual
   if LI.Continual then
      DeleteLHC_Continual(LHC_KOD,LHC_Year);
   LHC_ID := IntToStr(LHC_KOD) + '_' + IntToStr(LHC_Year);
   LHC_ID_New := IntToStr(LHC_KOD) + '_' + IntToStr(NewYear);

   // Renaming UserData
   Prjs := TMainTool(World.MainTool).Options.ProjectDirectory;
   CheckBackSlash(Prjs);
   Prjs := GetAbsolutePath(Prjs);
   aFile  := Prjs+ 'USERDATA_' + LHC_ID + '.orb';
   aFileN := Prjs+ 'USERDATA_' + LHC_ID_New + '.orb';
   RenFil(aFile, aFileN);
   S  := LI.ProjectName;
   SN := LHC_ID_New;
   PrjN := 'LHP_'+SN;
   // Renaming ProjectFile
   aFile  := Prjs + S  + '.opr';
   aFileN := Prjs + PrjN + '.opr';
   RenFil(aFile,aFileN);
   Load := TLoad.CreateAll(aFileN);
   Load.DoIt;
   Prj := TOrbisLHCProject(Load.Output);
   Load.Free;
   Prj.Name := PrjN;
   Save := TSave.CreateAll(aFileN);
   Save.Input := Prj;
   Save.DoIt;
   Save.Free;
   Prj.Free;
   // Renaming DataFile
   aFile  := Prjs + S  + '.orb';
   aFileN := Prjs + PrjN + '.orb';
   RenFil(aFile,aFileN);
   // Edit LHC Record  and Save projects
   LI.ProjectName := PrjN;
   LI.Name := SN;
   LI.LHCYear := NewYear;
   TMainTool(World.MainTool).SaveProjects;
   result := true;
end;

procedure TOrbis01.CHeckContinualSpace;
var
   G: TGraphicLinkTool;
   R: TDoubleRect;
   C: TRectEntity;
   En,En1,L: TEntity;
   Load,LoadLock,Lock,LID: TEntity;
   RE: TRectEntity;
   i,j: integer;
//   iLHC: integer;
//   IsLoad: boolean;
   RR: TRectEntity;
   Be: TBooleanElement;
   e: TElement;
   aScale,GScale,SD: double;
   Generalize: boolean;
   CSetup: TStringListElement;
   ToReview: boolean;
   LI: TOrbis01LHCInfo;

   function TimeToExit: boolean;
   begin
      result := BE.Value;
      if result then
      begin
         if RR <> nil then
            RR.Free;
         RR := nil;
         BE.Free;
         BE := nil;
      end;
   end;
begin
   G := TGraphicLinkTool(Tool);
   G.NoResetReviewByElement := true;
   R := G.Projection.CLipModelView;
   aScale := G.Projection.Scale;
   C := Cont;
   BE := nil;
   RR := nil;
   if C = nil then exit;
// uvolnovani zbytecnych objektu
   LoadLock := TEntity(C.FindElementName('LOADLOCK'));
   LID := TEntity(C.FindElementName('LID'));
   Lock := TEntity(C.FindElementName('LOCK'));;
   Load := TEntity(C.FindElementName('LOAD'));
   if Load = nil then exit;
   GScale := 50000;
   CSetup := TStringListElement(C.FindElementName('Setup'));
   if CSetup <> nil then
   begin
      SD := CSetup.GetDoubleByName('GScale');
      if SD > 0 then
         GScale := SD;
   end;
   Generalize := aScale >= GScale;
   if Assigned(OnContinualLoadStart) then
      OnContinualLoadStart;
   // preruseni predchozich nacitani
   for i := 0 to LID.LastIndex do
   begin
      e := LID.Elements[i];
      if not (e is TBooleanElement) then continue;
      TBooleanElement(e).Value := true;
   end;
   // kontrolni element do LID (dokud bude false, pokracujeme)
   Be := TBooleanElement.CreateValue(false);
   LID.AddElement(Be);
   // ruseni nactenych ODD mimo obdelnik
   En := TEntity(Load.FindElementName('ODD'));
   for i := EN.LastIndex downto 0 do
   begin
      RE := TRectEntity(En.Elements[i]);
      if Generalize or not RE.IsOverRegion(R.MinX,R.MinY,R.MaxX,R.MaxY) then
      if not RE.IsMyOwner(Lock) then
         Cont_FreeOdd(RE);
      Application.ProcessMessages;
      if TimeToExit then
         exit;
   end;
   // ruseni nactenych USERDATA mimo obdelnik
   En := TEntity(Load.FindElementName('USER'));
   for i := EN.LastIndex downto 0 do
   begin
      RE := TRectEntity(En.Elements[i]);
      if Generalize or not RE.IsOverRegion(R.MinX,R.MinY,R.MaxX,R.MaxY) then
      if not RE.IsMyOwner(Lock) then
         Cont_FreeUser(RE);
      Application.ProcessMessages;
      if TimeToExit then
         exit;
   end;
   // ruseni nactenych karto mimo obdelnik
   En := TEntity(Load.FindElementName('KART'));
   for i := EN.LastIndex downto 0 do
   begin
      RE := TRectEntity(En.Elements[i]);
      if Generalize or not RE.IsOverRegion(R.MinX,R.MinY,R.MaxX,R.MaxY) then
      if not RE.IsMyOwner(Lock) then
         Cont_FreeKart(RE);
      Application.ProcessMessages;
      if TimeToExit then
         exit;
   end;
   // Generalizovana data
   En := TEntity(Load.FindElementName('G'));
   if not Generalize then
   for i := EN.LastIndex downto 0 do
   begin
      RE := TRectEntity(En.Elements[i]);
      Cont_FreeG(RE);
      Application.ProcessMessages;
      if TimeToExit then
         exit;
   end;
   // nacteni LHC
   En := TEntity(C.FindElementName('LHC'));
   for i := EN.LastIndex downto 0 do
   begin
      //RR := TRectEntity.Create;
//      IsLoad := false;
      RE := TRectEntity(En.Elements[i]);            // LHC
      if not RE.IsOverRegion(R.MinX,R.MinY,R.MaxX,R.MaxY) then continue;
      LI := TOrbis01LHCInfo(Projects.LHCs.FindElementName(RE.Name));
      EN1 := TEntity(C.FindElementName('ActiveLHC'));
      if EN1 <> nil  then
      begin
         EN1.FreeAll;
         EN1.AddElement(LI); // tohle vsechno kvuli nacteni uzivatelskych dat
      end;
//      iLHC := StrToIntDef(RE.Name, 1);
      L := TEntity(RE.FindElementName('R'));        // R
      if Generalize then
      begin
         EN1 := TEntity(L.FindElementName('G'));     // KART
         if EN1 <> nil then
         for j := 0 to EN1.LastIndex do
         begin
            RE := TRectEntity(En1.Elements[j]);
            if RE.IsMyOwner(LoadLock) then continue;
            if not RE.IsOverRegion(R.MinX,R.MinY,R.MaxX,R.MaxY) then continue;
            LoadLock.AddElement(RE);
            Cont_LoadG(RE, ToReview);
            LoadLock.RemoveElement(RE);
            //RR.UpdateRect(RE);
            //IsLoad := true;
            if ToReview then
               G.PostMsg(tm_ReViewByElement, G, RE, false);
            Application.ProcessMessages;
            if TimeToExit then
               exit;
         end;
      end
      else
      begin
         EN1 := TEntity(L.FindElementName('ODD'));     // ODD
         for j := 0 to EN1.LastIndex do
         begin
            RE := TRectEntity(En1.Elements[j]);
            if RE.IsMyOwner(LoadLock) then continue;
            if not RE.NoElements then Continue;
            if not RE.IsOverRegion(R.MinX,R.MinY,R.MaxX,R.MaxY) then continue;
            LoadLock.AddElement(RE);
            Cont_LoadOdd(LI,RE);
            LoadLock.RemoveElement(RE);
            //RR.UpdateRect(RE);
            //isLoad := true;
            G.PostMsg(tm_ReViewByElement, G, RE, false);
            Application.ProcessMessages;
            if TimeToExit then
               exit;
         end;
         EN1 := TEntity(L.FindElementName('USERDATA'));     // USERDATA
         if EN1 = nil then      // jeste nejsou nactena
         begin
            TMainTool(World.MainTool).UserDataSplitted := LI.UserDataSplitted;
            TMainTool(World.MainTool).LoadUserDataCheckSplit;
            EN1 := TEntity(L.FindElementName('USERDATA'));
            if EN1 = nil then
            begin
               L.CreateElement('TRectEntity','USERDATA'); // aby se to uz neopakovalo
            end;
            if TimeToExit then
               exit;
         end;
         EN1 := TEntity(L.FindElementName('KART'));     // KART
         for j := 0 to EN1.LastIndex do
         begin
            RE := TRectEntity(En1.Elements[j]);
            if RE.IsMyOwner(LoadLock) then continue;
            if not RE.NoElements then Continue;
            if not RE.IsOverRegion(R.MinX,R.MinY,R.MaxX,R.MaxY) then continue;
            LoadLock.AddElement(RE);
            Cont_LoadKart(RE);
            LoadLock.RemoveElement(RE);
            //RR.UpdateRect(RE);
            //IsLoad := true;
            G.PostMsg(tm_ReViewByElement, G, RE, false);
            Application.ProcessMessages;
            if TimeToExit then
               exit;
         end;
      end;
      {
      if IsLoad then
      begin
         G.PostMsg(tm_ReViewByElement, G, RR, true);
         Application.ProcessMessages;
      end
      else
         RR.Free;
      }
   end;
   if BE <> nil then
      BE.Free;
   if Assigned(OnContinualLoadEnd) then
      OnContinualLoadEnd;
end;

function TOrbis01.GetCont: TRectEntity;
begin
   result := TRectEntity(World.ObjectModel.FindElementName('CONT'));
end;

procedure TOrbis01.Cont_FreeKart(aKart: TEntity);
var
   En: TEntity;
begin
   if aKart = nil then exit;
   En := TEntity(aKart.FindElementClass(TElement));
   if En <> nil then
      En.HardFreeAll;
   aKart.Free;
end;

procedure TOrbis01.Cont_FreeOdd(aOdd: TEntity);
var
   En: TEntity;
begin
   if aOdd = nil then exit;
   En := TEntity(aOdd.FindElementName('OD'));
   if En <> nil then
      En.HardFreeAll;
   En := TEntity(aOdd.FindElementName('LAYERS'));
   if En <> nil then
      En.HardFreeAll;
   aOdd.Free;
end;

procedure TOrbis01.Cont_FreeUser(aUser: TEntity);
begin
   if aUser = nil then exit;
   aUser.HardFreeAll;
   aUser.Free;
end;

procedure TOrbis01.Cont_LoadOddeleni(aLHC: TOrbis01LHCInfo; aOdd: integer);
var
   SLHC,SODD: string;
   C: TEntity;
   ELHC: TEntity;
   eOD: TEntity;
   eODD: TEntity;
   En: TEntity;
begin
   if aLHC = nil then exit;

   SLHC := aLHC.Name;   //IntToStr(aLHC);
   SODD := IntToStr(aODD);
   C := Cont;
   if Cont = nil then exit;                         //CONT
   ELHC := TEntity(C.FindElementName('LHC'));       // |_LHC
   if ELHC = nil then exit;                         //    |
   ELHC := TEntity(ELHC.FindElementName(SLHC));     //    |_1234{_rok}
   if ELHC = nil then exit;                         //       |
   eOD := TEntity(ELHC.FindElementName('ODD'));     //       |_ODD
   if eOD = nil then exit;                          //          |
   eODD := TEntity(eOD.FindElementName(SODD));      //          |_15
   if eODD = nil then exit;                         //             |
   En := TEntity(eODD.FindElementClass(TElement));  //             |_1234\blabla.orb
   if En = nil then exit;                           //                    |_ODD
   if not En.NoElements then exit; // je to uz nacteny
   Cont_LoadOdd(aLHC, En);
end;

procedure TOrbis01.Cont_LoadKart(aKart: TEntity);
var
   aCont: TRectEntity;
   aLayers: TEntity;
   L,LKART: TEntity;
   eKART: TEntity;
   T: TLoad;
   S: string;
   En: TEntity;
   E: TElement;
   i: integer;
   aLay: TEntity;
begin
   if aKart = nil then exit;
   if not aKart.NoElements then exit;
   aLayers := Layers;
   if aLayers = nil then exit;
   aCont := Cont;
   if aCont = nil then exit;
   L := TEntity(aCont.FindElementName('LOAD'));
   if L = nil then exit;
   LKART := TEntity(L.FindElementName('KART'));
   S := TMainTool(World.MainTool).Options.ProjectDirectory;
   CHeckBackSlash(S);
   S := S + 'CONTINUAL\' + aKart.Name;
   S := GetAbsolutePath(S);
   eKART := nil;
   try
   T := TLoad.CreateAll(S);
   T.NoProgress := true;
   T.DoIt;
   eKART := TEntity(T.Output);
   aKart.AddElement(eKART);
   LKART.AddElement(EKART);
   T.Free;
   except
   end;
   if EKART = nil then exit;
   En := TEntity(EKART.FindElementClass(TElement));
   if En = nil then exit;
   for i := 0 to En.LastIndex do
   begin
      e := En.Elements[i];
      aLay := TEntity(aLayers.FindElementName(e.Name));
      if aLay <> nil then
         aLay.AddElement(e);
   end;
end;

procedure TOrbis01.Cont_LoadOdd(iLHC: TOrbis01LHCInfo; aODD: TEntity);
var
   aFM: Tgis_ForestModel;
   aCont: TRectEntity;
   aLayers: TEntity;
   L,LODD: TEntity;
   eODD: TEntity;
   T: TLoad;
   S: string;
   En: TEntity;
   E: TElement;
   i: integer;
   aLay: TEntity;
   aOddeleni: Tgis_Oddeleni;
   aLS: TEntity;
   iLS: integer;
//   ie: TIntegerElement;
   eLS: Tgis_LS;
begin
   if iLHC = nil then exit;
   if aODD = nil then exit;
   if not aODD.NoElements then exit;
   aFM := ForestModel;
   if aFM = nil then exit;
   aLayers := Layers;
   if aLayers = nil then exit;
   aCont := Cont;
   if aCont = nil then exit;
   L := TEntity(aCont.FindElementName('LOAD'));
   if L = nil then exit;
   LODD := TEntity(L.FindElementName('ODD'));
   S := TMainTool(World.MainTool).Options.ProjectDirectory;
   CHeckBackSlash(S);
   S := S + 'CONTINUAL\' + aODD.Name;
   S := GetAbsolutePath(S);
   eODD := nil;
   try
   T := TLoad.CreateAll(S);
   T.NoProgress := true;
   T.DoIt;
   eODD := TEntity(T.Output);
   aODD.AddElement(eODD);
   LODD.AddElement(EODD);
   T.Free;
   except
   end;
   if EODD = nil then exit;
   // Kartograficke vrstvy
   En := TEntity(EODD.FindElementname('LAYERS'));
   if En = nil then exit;
   for i := 0 to En.LastIndex do
   begin
      e := En.Elements[i];
      aLay := TEntity(aLayers.FindElementName(e.Name));
      if aLay <> nil then
         aLay.AddElement(e);
   end;
   // oddeleni do ForestModelu
   En := TEntity(EODD.FindElementname('OD'));
   if En = nil then exit;
   aLS := TEntity(EODD.FindElementname('LS'));
   if aLS = nil then exit;
   for i := 0 to EN.LastIndex do
   begin
      e := En.Elements[i];
      if not (e is Tgis_Oddeleni) then Continue;
      aOddeleni := Tgis_Oddeleni(e);
      e := aLS.Elements[i];
      if not (e is TIntegerElement) then
         iLS := 1
      else
         iLS := TIntegerElement(e).Value;
      aFM.CreateLHCCislo(iLHC.LHCNumber);
      aFM.CreateLSCislo(iLS);
      eLS := aFM.ActualLS;
      eLS.CheckItems;
      eLS.Items.AddElement(aOddeleni);
      eLS.UpdateRect(aOddeleni);
   end;
end;

function TOrbis01.GetLayers: TEntity;
var
   En: TEntity;
begin
   result := nil;
   En := TEntity(World.ObjectModel.FindElementName('DATA'));
   if En = nil then exit;
   result := TEntity(En.FindElementName('LAYERS'));
end;

procedure TOrbis01.SetOnContinualLoadEnd(const Value: TOnSelectEvent);
begin
  FOnContinualLoadEnd := Value;
end;

procedure TOrbis01.SetOnContinualLoadStart(const Value: TOnSelectEvent);
begin
  FOnContinualLoadStart := Value;
end;

procedure TOrbis01.SetOnGeneralEvent(const Value: TOnGeneral_Event);
begin
  FOnGeneralEvent := Value;
end;

procedure TOrbis01.SetOnInterpretDone(const Value: TOnInterpretDone);
begin
  FOnInterpretDone := Value;
end;

procedure TOrbis01.Cont_FreeG(aG: TEntity);
var
   aCont: TRectEntity;
   aLayers: TEntity;
   L,LG: TEntity;
//   eG: TEntity;
   En: TEntity;
   E: TElement;
   i: integer;
   aLay: TEntity;
begin
   if aG = nil then exit;
   aLayers := Layers;
   if aLayers = nil then exit;
   aCont := Cont;
   if aCont = nil then exit;
   L := TEntity(aCont.FindElementName('LOAD'));
   if L = nil then exit;
   LG := TEntity(L.FindElementName('G'));
   LG.RemoveElement(aG);
   En := aG;
   for i := 0 to En.LastIndex do
   begin
      e := En.Elements[i];
      aLay := TEntity(aLayers.FindElementName(e.Name));
      if aLay <> nil then
         aLay.RemoveElement(e);
   end;
end;

procedure TOrbis01.Cont_LoadG(aG: TEntity; var RealLoad: boolean);
var
   aCont: TRectEntity;
   aLayers: TEntity;
   L,LG: TEntity;
   eG: TEntity;
   T: TLoad;
   S: string;
   En: TEntity;
   E: TElement;
   i: integer;
   aLay: TEntity;
begin
   RealLoad := false;
   if aG = nil then exit;
   aLayers := Layers;
   if aLayers = nil then exit;
   aCont := Cont;
   if aCont = nil then exit;
   L := TEntity(aCont.FindElementName('LOAD'));
   if L = nil then exit;
   LG := TEntity(L.FindElementName('G'));
   eG := nil;
   if aG.NoElements then
   begin
      S := TMainTool(World.MainTool).Options.ProjectDirectory;
      CHeckBackSlash(S);
      S := S + 'CONTINUAL\' + aG.Name;
      S := GetAbsolutePath(S);
      try
      T := TLoad.CreateAll(S);
      T.NoProgress := true;
      T.DoIt;
      eG := TEntity(T.Output);
      aG.AddElement(eG);
      T.Free;
      except
      end;
   end
   else
      EG := TEntity(aG.FindElementClass(TElement));
   if EG = nil then exit;
   if eG.IsMyOwner(LG) then exit; // uz tam je
   RealLoad := true;
   LG.AddElement(EG);
   En := EG;
   for i := 0 to En.LastIndex do
   begin
      e := En.Elements[i];
      aLay := TEntity(aLayers.FindElementName(e.Name));
      if aLay <> nil then
         aLay.AddElement(e);
   end;
end;

procedure TOrbis01.SetActualSelect(const Value: TEntity);
begin
  FActualSelect := Value;
end;

procedure TOrbis01.SetActualSelectName(const Value: string);
var
   e: TElement;
//   en: TEntity;
begin
   FActualSelectName := Value;
   ActualSelect := nil;
   if not FActive then exit;
   Error := err_SelectNotFound;
   e := World.ObjectModel.FindElementName('DATA');
   if e = nil then exit;
   if not (e is TEntity) then exit;
   e := TEntity(e).FindElementName('Layers');
   if e = nil then exit;
   if not (e is TEntity) then exit;
   e := TEntity(e).FindElementName(Value);
   if e = nil then exit;
   if not (e is TEntity) then exit;
   Error := 0;
   ActualSelect := TEntity(e);
end;

procedure TOrbis01.ActualEditLayerToActualMap;
begin
   ExecuteScript('GORBIS\UserLayer_ActualLayerToActualMap.oil');
end;

procedure TOrbis01.AddToSelect(aUnit: Tgis_ForestUnit);
begin
   if ActualSelect = nil then
      exit;
   if ActualSelect is TRectEntity then
      TRectEntity(ActualSelect).AddAndUpdateRect(aUnit)
   else
      ActualSelect.AddElement(aUnit);
end;

procedure TOrbis01.ClearTheSelect;
begin
   if ActualSelect = nil then
      exit;
   ActualSelect.FreeAll;
   if ActualSelect is TRectEntity then
      TRectEntity(ActualSelect).ResetAndUpdate;
end;

procedure TOrbis01.HighlightTheSelect;
begin
   TGraphicLinkTool(Tool).ElementForPattern := ActualSelect;
end;

procedure TOrbis01.ShowTheSelect(Pattern: boolean);
begin
   if ActualSelect = nil then
      exit;
   TGraphicLinkTool(Tool).ElementForProjection := ActualSelect;
   Application.ProcessMessages;
   if Pattern then
      TGraphicLinkTool(Tool).ElementForPattern := ActualSelect;
end;

procedure TOrbis01.SetShiftView(const Value: string);
begin
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   TGraphicLinkTool(Tool).ShiftView := Value;
   Application.ProcessMessages;
end;

procedure TOrbis01.SetClearIfLHCNotFound(const Value: boolean);
begin
  FClearIfLHCNotFound := Value;
end;

procedure TOrbis01.SetClient(const Value: boolean);
begin
  FClient := Value;
  if OrbisActionList <> nil then
     OrbisActionList.CLient := Value;
end;

procedure TOrbis01.ClearData;
begin
   if FActive then
      TMainTool(World.MainTool).CreateNewDATA;
end;

procedure TOrbis01.SetManualProjectionChange(const Value: boolean);
begin
  FManualProjectionChange := Value;
end;

procedure TOrbis01.RastersChangeEvent;
begin
   if Assigned(OnRastersChange) then
      OnRastersChange;
end;

procedure TOrbis01.SetOnRastersChange(const Value: TOnNewModel_Event);
begin
  FOnRastersChange := Value;
end;

function TOrbis01.GetRasterInfo: TOrbis01RasterInfo;
//var
//   RI: TOrbis01RasterInfo;
begin
   result := TOrbis01RasterInfo(World.ObjectModel.FindElementClass(TOrbis01RasterInfo));
end;

procedure TOrbis01.CheckLHCYear(LHC_KOD,LHC_Year: integer);
var
   FM: Tgis_ForestModel;
begin
   if LHC_Year = 0 then  exit;
   FM := ForestModel;
   FM.LHC_Cislo := LHC_KOD;
   if FM.ActualLHC <> nil then
   if FM.ActualLHC.ROK <> LHC_Year then
      FM.ActualLHC.ROK := LHC_Year;
end;

procedure TOrbis01.CheckRasters;
begin
   TMainTool(World.MainTool).CHeckProjectRasters;
   TMainTool(World.MainTool).SaveProjectRasters;
end;

procedure TOrbis01.SetupRasters;
begin
   TMainTool(World.MainTool).SetUpProjectRasters;
end;


procedure TOrbis01.EndSizeMove;
begin
   GF.EndSizeMove;
end;

procedure TOrbis01.StartSizeMove;
begin
  GF.StartSizeMove;
end;

function TOrbis01.Synchronize_with_mobile_user_data_layer(aLayer,MobileFile, DesktopFile, MediaFile: string): integer;
//   results:
//   00      ani jeden soubor neexistuje
//   10      mobilni soubor existuje, Desktop vytvoren
//   01      tedy 1 Desktop existuje, Mobil vytvoren
//   11      Oba existuji, probehla synchronizace
//   02      Nesouhlasi nazvy vrstev
//   30      Mobilni soubor chyba ... vadna struktura a tak
//   03      Desktop             --"--
begin
   result := 0;
   if not FActive then exit;
   result := TMainTool(World.MainTool).Synchronize_with_mobile_user_data_layer(aLayer,MobileFile,DesktopFile,MediaFile);
end;

function TOrbis01.GetInterpret: TInterpret;
begin
   result :=  TGraphicLinkTool(Tool).Interpret;
end;

procedure TOrbis01.SetupVrs;
begin
   TMainTool(World.MainTool).SetupProjectVRS;
end;

procedure TOrbis01.SetVirtualScale(const Value: double);
begin
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   TGraphicLinkTool(Tool).Projection.VirtualScale := Value;
   ManualProjectionChange := true;
end;

procedure TOrbis01.SetupTyp;
begin
   TMainTool(World.MainTool).SetupProjectTYP;
end;

function TOrbis01.GetVirtualScale: double;
begin
   result := 5000;
   Error := 1;
   if not FActive then exit;
   if Tool = nil then exit;
   if not (Tool is TGraphicLinkTool) then exit;
   Error := 0;
   result := TGraphicLinkTool(Tool).Projection.VirtualScale;
end;

function TOrbis01.GetVrsInfo: TOrbis01LayerInfo;
begin
   result := TOrbis01LayerInfo(World.ObjectModel.FindElementName('VRS'));
end;

function TOrbis01.GetTypInfo: TOrbis01LayerInfo;
begin
   result := TOrbis01LayerInfo(World.ObjectModel.FindElementName('TYP'));
end;

procedure TOrbis01.CheckVrs;
begin
   TMainTool(World.MainTool).CheckProjectVRS;
   TMainTool(World.MainTool).SaveProjectVRS;
end;

procedure TOrbis01.CheckTyp;
begin
   TMainTool(World.MainTool).CheckProjectTYP;
   TMainTool(World.MainTool).SaveProjectTYP;
end;

procedure TOrbis01.SetOnVrsChange(const Value: TOnNewModel_Event);
begin
  FOnVrsChange := Value;
end;

procedure TOrbis01.SetOn_DownloadURL(const Value: TOnDownloadURL_Event);
begin
  FOn_DownloadURL := Value;
end;

{$IFDEF FILEEVENT}

procedure TOrbis01.SetOn_FileAfterSave(const Value: TFile_Event);
begin
  FOn_FileAfterSave := Value;
end;

procedure TOrbis01.SetOn_FileBeforeLoad(const Value: TFile_Event);
begin
  FOn_FileBeforeLoad := Value;
end;

procedure TOrbis01.SetOn_FileDelete(const Value: TFile_Event);
begin
  FOn_FileDelete := Value;
end;

procedure TOrbis01.SetOn_FileExists(const Value: TFile_Event);
begin
  FOn_FileExists := Value;
end;
procedure TOrbis01.SetOn_Progress(const Value: TOnProgressEvent);
begin
  FOn_Progress := Value;
end;

procedure TOrbis01.SetOn_ProgressEnd(const Value: TOnProgressEvent);
begin
  FOn_ProgressEnd := Value;
end;

procedure TOrbis01.SetOn_ProgressMax(const Value: TOnProgressEvent);
begin
  FOn_ProgressMax := Value;
end;

procedure TOrbis01.SetOn_ProgressStart(const Value: TOnProgressStartEvent);
begin
  FOn_ProgressStart := Value;
end;

procedure TOrbis01.SetOn_ProgressText(const Value: TOnProgressTextEvent);
begin
  FOn_ProgressText := Value;
end;

procedure TOrbis01.SetOn_SelectDirectory(const Value: TOnSelectDirectory_Event);
begin
  FOn_SelectDirectory := Value;
end;

procedure TOrbis01.SetOn_SelectFile(const Value: TOnSelectFile_Event);
begin
  FOn_SelectFile := Value;
end;

{$ENDIF}
procedure TOrbis01.SetOnTypChange(const Value: TOnNewModel_Event);
begin
  FOnTypChange := Value;
end;

procedure TOrbis01.VrsChangeEvent;
begin
   if Assigned(OnVrsChange) then
      OnVrsChange;
end;

procedure TOrbis01.TypChangeEvent;
begin
   if Assigned(OnTypChange) then
      OnTypChange;
end;

procedure TOrbis01.FillEditLayersList(aList: TStrings);
var
   i: integer;
   el: TEntity;
begin
   if not FActive then exit;
   if aList = nil then exit;
   aList.Clear;
   el := EditLayers;
   if el = nil then exit;
   for i := 0 to el.LastIndex do
      if TOrbis01UserLayerInfo(el.Elements[i]).Restriction < 2 then
         aList.Add(el.Elements[i].Name);
end;


procedure TOrbis01.FillKartoTextsList(aList: TStrings);
var
   i: integer;
   el: TEntity;
   e: TELement;
   aLayers: TEntity;
   TE: TTopolTextElement;
   SL: TStringList;
   S: string;

   function TestText(aText: string): boolean;
   var
      ii: integer;
   begin
      result := true;
      // oddeleni, dilec atp.
      if length(aText) = 1 then
      begin
         result := false;
         exit;
      end;
      // cislo ...
      ii := StrToIntDef(aText,0);
      if ii > 0 then
      begin
         result := false;
         exit;
      end;
   end;

begin
   if not FActive then exit;
   if aList = nil then exit;
   aList.Clear;
   aLayers := Layers;
   if aLayers = nil then exit;
   el := TEntity(aLayers.FindElementName('TXT_TXT'));
   if (el = nil) or (el.Count = 0)  then exit;
   SL := TStringList.Create;
   SL.Sorted := true;
   SL.Duplicates := dupIgnore;
   for i := 0 to el.LastIndex do
   begin
      e := el.Elements[i];
      if not (e is TTopolTextElement) then continue;
      TE := TTopolTextElement(e);
      //if TE.Druh in [1,2,12,21] then continue;
      S := TE.Text;
      if TestText(S) then
         SL.Add(TE.Text);
   end;
   aList.Assign(SL);
   SL.Free;
end;

function TOrbis01.FillLHCsString: string;
var
   i: integer;
   eLHC: TOrbis01LHCEntity;
begin
   if not FActive then exit;
   result := '';
   if Projects = nil then exit;
   eLHC := Projects.LHCs;
   if eLHC = nil  then exit;
   if eLHC.NoElements then exit;
   for i := 0 to eLHC.LastIndex do
   begin
      if i > 0 then result := result + ',';
      result :=  result + '"'+eLHC.Elements[i].Name+'"';
   end;
end;

function TOrbis01.GetEditLayers: TEntity;
begin
   result := nil;
   if not FActive then exit;
   Result := TEntity(World.ObjectModel.FindElementName('UserLayers'));
end;


procedure TOrbis01.SetActualEditLayer(const Value: string);
begin
   TMainTool(World.MainTool).ActualEditLayer := Value;
end;

function TOrbis01.GetActualEditLayer: string;
begin
   result := 'USR';
   if not FActive then exit;
   result := TMainTool(World.MainTool).ActualEditLayer;
end;

procedure TOrbis01.NewEditLayer;
begin
   ExecuteScript('GORBIS\UserLayer_New.oil');
end;

procedure TOrbis01.MapDialog;
begin
   ExecuteScript('GORBIS\Map_Setup.oil');
end;

procedure TOrbis01.NewMap;
begin
   ExecuteScript('GORBIS\Map_New.oil');
end;

procedure TOrbis01.SetOnMapsChange(const Value: TOnNewModel_Event);
begin
  FOnMapsChange := Value;
end;

procedure TOrbis01.MapsChangeEvent;
begin
   if Assigned(OnMapsChange) then
      OnMapsChange;
end;

function TOrbis01.GetActualMap: string;
begin
   result := '';
   if not FActive or (Maps = nil) then
      result := FActualMap
   else
   begin
      result := Maps.MapName;
      if FActualMap <> result then
         FActualMap := result;
   end;
end;

procedure TOrbis01.ExportToBMPFile(aVal: string);
begin
   if not FActive then exit;
   TGraphicLinkTool(Tool).GFrame.Image.Picture.Bitmap.SaveToFile(aVal);
end;

procedure TOrbis01.ExportToJPGFile(aVal: string);
var
  aJPG: TJPEGImage;
begin
   if not FActive then exit;
   aJPG:= TJPEGImage.Create;
   if JPGCompressionQuality > 0 then
   begin
     if JPGCompressionQuality > 100 then
        JPGCompressionQuality:= 100;
     aJPG.CompressionQuality:= JPGCompressionQuality;
   end;
   aJPG.GrayScale:= JPGGrayscale;
   if JPGPixelFormat > 1 then
      JPGPixelFormat:= 1;

   aJPG.PixelFormat:= TJPEGPixelFormat(JPGPixelFormat);
   aJPG.Assign(TGraphicLinkTool(Tool).GFrame.Image.Picture.Bitmap);
   aJPG.SaveToFile(aVal);
   aJPG.Free
end;

{ TOrbisActionList }

destructor TOrbisActionList.destroy;
begin
   if Orbis01 <> nil then
      Orbis01.OrbisActionList := nil;
   inherited;
end;

procedure TOrbisActionList.SetCLient(const Value: boolean);
var
   i: integer;
   CA: TContainedAction;
begin
   FCLient := Value;
   for i := 0 to ActionCount - 1 do
   begin
      CA := Actions[i];
      if CA is TOrbisAction then
         TOrbisAction(CA).CLient := Value;
   end;
end;

procedure TOrbisActionList.SetGroupName(const Value: string);
begin
  FGroupName := Value;
end;

procedure TOrbisActionList.SetOrbis01(const Value: TOrbis01);
begin
   if Value = FOrbis01 then exit;
   FOrbis01 := Value;
   if Value = nil then exit;
   Value.OrbisActionList := Self;
end;

{ TViewerAction }

procedure TViewerAction.CreateTool;
begin
   if Tool <> nil then exit;
   Tool := TStandardViewer.Create;
end;

{ TShiftImageAction }

procedure TShiftImageAction.CreateTool;
begin
   if Tool <> nil then exit;
   Tool := TShiftImage.Create;
   TShiftImage(Tool).ShareInput := true;
end;

{ Tgis_SkupinaSelectAction }

procedure Tgis_SkupinaSelectAction.CreateTool;
begin
   if Tool <> nil then exit;
   Tool := Tgis_PorostniSkupinaSelector.Create;
end;

{ TPolyLineEditorAction }

procedure TPolyLineEditorAction.CreateTool;
var
   ed: TPolyLineEditorNoFrame;
   td: TTopolLineDialog;
begin
   if Tool <> nil then exit;
   ed := TPolyLineEditorNoFrame.Create;
   Tool := ed;
   ed.ConfirmText := 'Linie byla zmnna, pejete si zmny uloit?   Storno-pokraovn editace';
   //ed.CanCreateNewObject := True;
   ed.CanCreateNewObject := ed_CanCreateNewObject;
   ed.PixelTolerance := 10;
   ed.AllowFlash := True;
   //ed.LevelName := 'ObjectModel.DATA.LAYERS.CONSTRUCT_SRF';
   //ed.SnapLevels := 'ObjectModel.DATA.LAYERS.LIN_DET,ObjectModel.DATA.LAYERS.LIN_SLU,ObjectModel.DATA.LAYERS.ARE_SRF';
   ed.LevelName := ed_LevelName;
   ed.SnapLevels := ed_SnapLevels;
   ed.Topology := ed_Topology;
   if ed_Karto then
   begin
      td := TTopolLineDialog.Create;
      ed.Dialog := td;
      td.DruhyList := ed_DruhyList;
      td.Presnost := ed_Presnost;
      td.OnUserLineDialog := OnUserLineDialog;
   end
   else
      ed.Dialog := TLineDialog.Create;
   ed.SnapTool := TPoinTLineSnapTool.Create;
   ed.SnapTool.SnapToLine := ed_SnapToLine;
   ed.SnapTool.SnapToLinePoint := ed_SnapToLinePoint;
   ed.SnapTool.SnapToLineBody := ed_SnapToLineBody;
   ed.SnapTool.SnapDistance := ed_SnapDistance;
end;

procedure TPolyLineEditorAction.Seted_CanCreateNewObject(
  const Value: boolean);
begin
  Fed_CanCreateNewObject := Value;
end;

procedure TPolyLineEditorAction.Seted_DruhyList(const Value: string);
begin
  Fed_DruhyList := Value;
end;

procedure TPolyLineEditorAction.Seted_Karto(const Value: boolean);
begin
  Fed_Karto := Value;
end;

procedure TPolyLineEditorAction.Seted_LevelName(const Value: string);
begin
  Fed_LevelName := Value;
end;

procedure TPolyLineEditorAction.Seted_Presnost(const Value: integer);
begin
  Fed_Presnost := Value;
end;

procedure TPolyLineEditorAction.Seted_SnapLevels(const Value: string);
begin
  Fed_SnapLevels := Value;
end;

procedure TPolyLineEditorAction.Seted_SnapToLine(const Value: boolean);
begin
  Fed_SnapToLine := Value;
end;

procedure TPolyLineEditorAction.Seted_Topology(const Value: boolean);
begin
  Fed_Topology := Value;
end;

procedure TPolyLineEditorAction.SetOnUserLineDialog(
  const Value: TOnUserLineDialog_Event);
begin
     FOnUserLineDialog := Value;
end;

procedure TPolyLineEditorAction.SetSnapDistance(const Value: double);
begin
  FSnapDistance := Value;
end;

procedure TPolyLineEditorAction.SetSnapToLineBody(const Value: boolean);
begin
  FSnapToLineBody := Value;
end;

procedure TPolyLineEditorAction.SetSnapToLinePoint(const Value: boolean);
begin
  FSnapToLinePoint := Value;
end;

{ TAreaEditorAction }

procedure TAreaEditorAction.CreateTool;
var
   ed: TEditorSRFPloch;
   d: TUserLHCAreaDialog;
begin
   if Tool <> nil then exit;
   ed := TEditorSRFPloch.Create;
   Tool := ed;
   ed.CanCreateNewObject := True;
   ed.FrameLevelName := ed_FrameLevelName;
   ed.TargetLevelName := ed_TargetLevelName;
   ed.ConstructLevelName := ed_ConstructLevelName;
   ed.ConstructLevelCopy := ed_ConstructLevelCopy;
   ed.SLULevelName := ed_SLULevelName;

   //ed.Dialog := TTezPDialog.Create;
   //TTezpDialog(ed.Dialog).SRFOnly := True;
   d := TUserLHCAreaDialog.Create;
   ed.Dialog := d;
   d.OnUserLHCAreaDialog := TMainTool(World.MainTool).OnUserLHCAreaDialog;
end;

procedure TAreaEditorAction.Seted_CanCreateNewObject(const Value: boolean);
begin
  Fed_CanCreateNewObject := Value;
end;

procedure TAreaEditorAction.Seted_ConstructLevelCopy(const Value: boolean);
begin
  Fed_ConstructLevelCopy := Value;
end;

procedure TAreaEditorAction.Seted_ConstructLevelName(const Value: string);
begin
  Fed_ConstructLevelName := Value;
end;

procedure TAreaEditorAction.Seted_FrameLevelName(const Value: string);
begin
  Fed_FrameLevelName := Value;
end;

procedure TAreaEditorAction.Seted_SLULevelName(const Value: string);
begin
  Fed_SLULevelName := Value;
end;

procedure TAreaEditorAction.Seted_TargetLevelName(const Value: string);
begin
  Fed_TargetLevelName := Value;
end;

{ TMeasureLengthAction }

procedure TMeasureLengthAction.CreateTool;
begin
   if Tool <> nil then exit;
   Tool := TMeasureLengthTool.Create;
end;

{ TMeasureAreaAction }

procedure TMeasureAreaAction.CreateTool;
begin
   if Tool <> nil then exit;
   Tool := TMeasureAreaTool.Create;
   TMeasureAreaTool(Tool).UsePattern := UsePattern;
   TMeasureAreaTool(Tool).UserScript := UserScript;
end;

procedure TMeasureAreaAction.SetUsePattern(const Value: boolean);
begin
   FUsePattern := Value;
   if Tool <> nil then
      TMeasureAreaTool(Tool).UsePattern := Value;
end;

procedure TMeasureAreaAction.SetUserScript(const Value: string);
begin
   FUserScript := Value;
   if Tool <> nil then
      TMeasureAreaTool(Tool).UserScript := Value;
end;

{ TRasterManipulatorAction }

procedure TRasterManipulatorAction.AddRasterFile(const aRasterFile: string);
begin
   if Tool = nil then exit;
   TRasterManipulator(Tool).AddRasterFile(aRasterFile);
end;

procedure TRasterManipulatorAction.AddRastersForView;
begin
   if Tool = nil then exit;
   TRasterManipulator(Tool).ReadRastersForDevice;
end;

procedure TRasterManipulatorAction.CreateTool;
var
   rm: TRasterManipulator;
begin
   if Tool <> nil then exit;
   rm := TRasterManipulator.Create;
   Tool := rm;
   rm.RasterLayerName := ed_RasterlayerName;
   rm.Directory := ed_RasterDirectory;
   rm.Dialog := TRasterDialog.Create;
end;

procedure TRasterManipulatorAction.FillRastersList(aList: TStrings);
var
   rm: TRasterManipulator;
   en: TEntity;
   i: integer;
begin
   if aList = nil then exit;
   aList.Clear;
   if Tool = nil then exit;
   rm := TRasterManipulator(Tool);
   en := rm.FindLevel(rm.RasterLayerName);
   if en = nil then exit;
   for i := 0 to en.LastIndex do aList.Add(en.Elements[i].Name);
end;

procedure TRasterManipulatorAction.FreeAllRasters;
begin
   if Tool = nil then exit;
   TRasterManipulator(Tool).FreeAllRasters;
end;

function TRasterManipulatorAction.FreeRaster(const aRasterFile: string): boolean;
begin
   result := true;
   if Tool = nil then exit;
   result := TRasterManipulator(Tool).FreeRaster(aRasterFile);
end;

function TRasterManipulatorAction.Geted_RasterDirectory: string;
begin
   if Tool = nil then
      result := Fed_RasterDirectory
   else
      result := TRasterManipulator(Tool).Directory;
end;

function TRasterManipulatorAction.Geted_RasterLayerName: string;
begin
   result := Fed_RasterLayername;
end;

procedure TRasterManipulatorAction.Seted_RasterDirectory(const Value: string);
begin
   Fed_RasterDirectory := Value;
   if Tool = nil then exit;
   TRasterManipulator(Tool).Directory := Value;

end;

procedure TRasterManipulatorAction.Seted_RasterLayerName(const Value: string);
begin
   Fed_RasterLayerName := Value;
   if Tool = nil then exit;
   TRasterManipulator(Tool).RasterLayerName := Value;
end;

{ Tgis_ParcelaSelectAction }

procedure Tgis_ParcelaSelectAction.CreateTool;
begin
   if Tool <> nil then exit;
   Tool := Tgis_ParcelaCastSelector.Create;
end;

{ TTextEditorAction }

procedure TTextEditorAction.CreateTool;
var
   ed: TSimpleTextEditor;
begin
   if Tool <> nil then exit;
   ed := TSimpleTextEditor.Create;
   Tool := ed;
   ed.PixelTolerance := 0;
   ed.AllowFlash := True;
   //ed.LevelName := 'ObjectModel.DATA.LAYERS.CONSTRUCT_SRF';
   //ed.SnapLevels := 'ObjectModel.DATA.LAYERS.LIN_DET,ObjectModel.DATA.LAYERS.LIN_SLU,ObjectModel.DATA.LAYERS.ARE_SRF';
   ed.LevelName := ed_LevelName;
   ed.Dialog := TSimpleTextDialog.Create;
end;

procedure TTextEditorAction.Seted_LevelName(const Value: string);
begin
  Fed_LevelName := Value;
end;

{ TPointEditorAction }

procedure TPointEditorAction.CreateTool;
var
   ed: TPointEditor;
   d: TTopolPointDialog;
begin
   if Tool <> nil then exit;
   ed := TPointEditor.Create;
   Tool := ed;
   ed.PixelTolerance := 10;
   ed.AllowFlash := True;
   ed.InterpretName := 'TInterpretTopBodu';
   //ed.LevelName := 'ObjectModel.DATA.LAYERS.CONSTRUCT_SRF';
   //ed.SnapLevels := 'ObjectModel.DATA.LAYERS.LIN_DET,ObjectModel.DATA.LAYERS.LIN_SLU,ObjectModel.DATA.LAYERS.ARE_SRF';
   ed.LevelName := ed_LevelName;
   d := TTopolPointDialog.Create;
   ed.Dialog := d;
   d.DruhyList := ed_DruhyList;
   d.DruhyListAngle := ed_DruhyListAngle;
   d.AllowAngleChange := true;
end;


procedure TPointEditorAction.Seted_DruhyList(const Value: string);
begin
  Fed_DruhyList := Value;
end;

procedure TPointEditorAction.Seted_DruhyListAngle(const Value: string);
begin
  Fed_DruhyListAngle := Value;
end;

procedure TPointEditorAction.Seted_LevelName(const Value: string);
begin
  Fed_LevelName := Value;
end;

{ TNodeEditorAction }

procedure TNodeEditorAction.CreateTool;
var
   ed: TNodeEditor;
begin
   if Tool <> nil then exit;
   ed := TNodeEditor.Create;
   Tool := ed;
   ed.PixelTolerance := 10;
   ed.AllowFlash := True;
   //ed.LevelName := 'ObjectModel.DATA.LAYERS.CONSTRUCT_SRF';
   //ed.SnapLevels := 'ObjectModel.DATA.LAYERS.LIN_DET,ObjectModel.DATA.LAYERS.LIN_SLU,ObjectModel.DATA.LAYERS.ARE_SRF';
   ed.LevelName := ed_LevelName;
   ed.SnapLevelName := ed_SnapLevelName;
   ed.SnapTool := TPoinTLineSnapTool.Create;
   ed.SnapTool.SnapToLine := ed_SnapToLine;
   ed.SnapTool.SnapToLinePoint := ed_SnapToLinePoint;
   ed.SnapTool.SnapToLineBody := ed_SnapToLineBody;
   ed.SnapTool.SnapDistance := ed_SnapDistance;
end;

procedure TNodeEditorAction.Seted_LevelName(const Value: string);
begin
  Fed_LevelName := Value;
end;

procedure TNodeEditorAction.Seted_SnapLevelName(const Value: string);
begin
  Fed_SnapLevelName := Value;
end;

procedure TNodeEditorAction.Seted_SnapToLine(const Value: boolean);
begin
  Fed_SnapToLine := Value;
end;

procedure TNodeEditorAction.SetSnapDistance(const Value: double);
begin
  FSnapDistance := Value;
end;

procedure TNodeEditorAction.SetSnapToLineBody(const Value: boolean);
begin
  FSnapToLineBody := Value;
end;

procedure TNodeEditorAction.SetSnapToLinePoint(const Value: boolean);
begin
  FSnapToLinePoint := Value;
end;

{ TSimpleEditorAction }

procedure TSimpleEditorAction.CreateTool;
begin
   if Tool <> nil then exit;
   Tool := TSimpleGraphicEditor.Create;
   TSimpleGraphicEditor(Tool).OnMouseUpScript := OnMouseUpScript;
   TSimpleGraphicEditor(Tool).OnMouseDownScript := OnMouseDownScript;
   TSimpleGraphicEditor(Tool).On_MouseDown := OnMouseDown;
   TSimpleGraphicEditor(Tool).On_MouseUp := OnMouseUp;
   TSimpleGraphicEditor(Tool).On_MouseMove := OnMouseMove;
   TSimpleGraphicEditor(Tool).NextCursorName := 'SELECTPOINT'
end;

procedure TSimpleEditorAction.SetOnMouseUp(const Value: TOn_CoordsClick);
begin
  FOnMouseUp := Value;
  if Tool <> nil then
      TSimpleGraphicEditor(Tool).On_MouseUp := Value;
end;

procedure TSimpleEditorAction.SetOnMouseDown(const Value: TOn_CoordsClick);
begin
  FOnMouseDown := Value;
  if Tool <> nil then
      TSimpleGraphicEditor(Tool).On_MouseDown := Value;
end;

procedure TSimpleEditorAction.SetOnMouseDownScript(const Value: string);
begin
  FOnMouseDownScript := Value;
  if Tool <> nil then
      TSimpleGraphicEditor(Tool).OnMouseDownScript := Value;
end;

procedure TSimpleEditorAction.SetOnMouseMove(const Value: TOn_CoordsClick);
begin
  FOnMouseMove := Value;
  if Tool <> nil then
      TSimpleGraphicEditor(Tool).On_MouseMove := Value;
end;

procedure TSimpleEditorAction.SetOnMouseUpScript(const Value: string);
begin
  FOnMouseUpScript := Value;
  if Tool <> nil then
      TSimpleGraphicEditor(Tool).OnMouseUpScript := Value;
end;

{ TSimpleRectAction }

procedure TSimpleRectAction.CreateTool;
begin
   if Tool <> nil then exit;
   Tool := TSimpleRectEditor.Create;
   TSimpleRectEditor(Tool).On_RectChange := OnRectChange;
end;

procedure TSimpleRectAction.SetOnRectChange(const Value: TOn_RectChange);
begin
  FOnRectChange := Value;
  if Tool <> nil then
      TSimpleRectEditor(Tool).On_RectChange := Value;
end;

end.
