unit U_DriversBDE;

interface


uses BDE, Classes, u_LanguagesBDE, Elements, SysUtils, forms;


Type
   TBlobSize = 32..1000;
   TBlobsToCache = 64..65536;
   TBlockSize = 1024..4096;
   TCollChar = 0..2;
   TDateMode = 0..2;
   TDateSeparator = (ZpetLom, Poml, Tecka);
   TDBNLS = 0..2;
   TFillFactor = 1..100;
   TLevel = 3..7;
   TListSynonyms = (tALL, tNONE, tPRIVATE);
   TLockMode = -1..32766;
   TNetProtocol = (t3270, tAPPC, tASYNC, tDECNET, tNAMED_PIPES, tNETBIOS, tSPX_IPX, tTCP_IP, tVINES, tTNS);
   TOpenMode = (R, RW);
   TSchemaCacheSize = 0..32;
   TSchemaCacheTime = -1..2147483647;
   TSQLPassThruMode = (tNOT_SHARED, tSHARED_AUTOCOM, tSHARED_NOAUTOCOM);
   TSQLQryMode = (tNULL, tLOCAL, tSERVER);
   TTimeOut = 120..65536;
   TaType = (tFILE, tSERVER);


   TDriverBDE = class (TOwnerElement)
   private
      FApplicationName: String;
      FBatchCount: SmallInt;        {max 32K}
      FBlobEditLoggin: Boolean;
      FBlobSize: TBlobSize;         {>=32, <= 1000, default 32}
      FBlobsToCache: TBlobsToCache; {>=64, <= 65536, default 64}
      FBlockSize: TBlockSize;       {1024, 2048, 4096, default 2048}
      FCollChar: TCollChar;         {0,1,2}
      FConnectTimeOut: Word;        {0 a , default 60}
      FDatabaseName: String;
      FDateMode: TDateMode;         {0 (MDY), 1 (DMY), 2 (YMD)}
      FDateSeparator: TDateSeparator;  {'/', '-', '.'}
      FDBNLS: TDBNLS;               {0..2}
      FDLL: TStringList;            {SQLD_IB.DLL}
      FDLL32: TStringList;          {SQLINT32.DLL}
      FDriversFlag: SmallInt;       {Default NULL, kompabilita s dvj
                                    verz SQL link 512}
      FEnableBCD: Boolean;          {default FALSE}
      FEnableIntegers: Boolean;      {default FALSE}
      FEnableSchemaCache: Boolean;  {default FALSE}
      FFillFactor: TFillFactor;     {1 a 100, default 95}
      FHostName: String;
      FLangDriver: TLangDriver;
      FLevel: TLevel;             {3, 4, 5, 7, default 5}
      FListSynonyms: TListSynonyms; {ALL, NONE, PRIVATE}
      FLockMode: TLockMode;         {-1..32766}
      FMaxDbProcesses: Word;        {default 31, neomezeno}
      FMaxQueryTime: Word;          {default 3600, neomezeno}
      FMaxRows: LongInt;            {default -1, neomezeno}
      FMdxBlockSize: Integer;       {512, 1024, Integer MOD 512, defaulr 1024}
      FMemoFileBlockSize: Integer;  {512, 1024, Integer MOD 512, defaulr 1024}
      FNationalLangName: String;   {jazyk ve kterm dv server hlky}
      FNetDir: String;              {path }
      FNetProtocol: TNetProtocol;   {3270, APPC, ASYNC, DECNET, NAMED PIPES, NETBIOS,
                                     SPX/IPX, TCP/IP, VINES, TNS}
      FOpenMode: TOpenMode;         {READ ONLY, READ/WRITE}
      FRowSetSize: LongInt;         {default 20, neomezeno}
      FServerName: String;          {Default IB_SERVER:/PATH/DATABASE.GDB}
      FSchemaCacheDir: String;      {Path}
      FSchemaCacheSize: TSchemaCacheSize;    {0  a 32, default 8}
      FSchemaCacheTime: TSchemaCacheTime;    {default -1, -1,0, 1 a 2147483647}
      FSQLPassThruMode: TSQLPassThruMode; {default NOT SHARED,
                                           SHARED AUTOCOMMIT,
                                           SHARED NOAUTOCOMMIT}
      FSQLQryMode: TSQLQryMode;     {default NULL, LOCAL, SERVER}
      FStrictIntegrity: Boolean;
      FSystemDatabase: String;      {Path\*.mdw}
      FTDSPacketSize: Word;               {0 a 65535, optiml 4096 a 8192}
      FTimeOut: TTimeOut;           {120..65535}
      FTraceMode: Word;
        { Default 0 jinak
                                     Bit Settings	Logged Information
                                     0x0001	prepared query statement
                                     0x0002	executed query statements
                                     0x0004	vendor errors
                                     0x0008	statement options (that is: allocate, free)
                                     0x0010	connect / disconnect
                                     0x0020	transaction
                                     0x0040	BLOB I/O
                                     0x0080	miscellaneous
                                     0x0100	vendor calls
                                     }
      FaType: TaType;                 { FILE, SERVER, IntrBase je SERVER}
      FVendorInit: TStringList;
      FVersion: Single;            {1.0}
      FUserName: String;

   protected
      function CheckBlockSize(aVal: TBlockSize): Boolean; virtual;
      function CheckBlockSize2(aVal: TBlockSize): Boolean; virtual;
      function CheckLevel(aVal: TLevel): Boolean; virtual; abstract;
      function GetCollChar: TCollChar; virtual; abstract;
      function GetDLL: TStringList; virtual; abstract;
      function GetDLL32: TStringList; virtual; abstract;
      function GetLangDriver: TLangDriver; virtual; abstract;
      function GetListSynonyms: TListSynonyms; virtual; abstract;
      function GetNetProtocol: TNetProtocol; virtual; abstract;
      function GetOpenMode: TOpenMode; virtual; abstract;
      function GetSQLPassThruMode: TSQLPassThruMode; virtual; abstract;
      function GetSQLQryMode: TSQLQryMode; virtual; abstract;
      function GetTraceMode: Word; virtual; abstract;
      function GetaType: TaType; virtual; abstract;
      function GetVendorInit: TStringList; virtual; abstract;

      procedure SetApplicationName(aVal: String); virtual; abstract;
      procedure SetBatchCount(aVal: SmallInt); virtual; abstract;
      procedure SetBlobEditLoggin(aVal: Boolean); virtual; abstract;
      procedure SetBlobSize(aVal: TBlobSize); virtual; abstract;
      procedure SetBlobsToCache(aVal: TBlobsToCache); virtual; abstract;
      procedure SetBlockSize(aVal: TBlockSize); virtual; abstract;
      procedure SetCollChar(aVal: TCollChar); virtual; abstract;
      procedure SetConnectTimeOut(aVal: Word); virtual; abstract;
      procedure SetDatabaseName(aVal: String); virtual; abstract;
      procedure SetDateMode(aVal: TDateMode); virtual; abstract;
      procedure SetDateSeparator(aVal: TDateSeparator); virtual; abstract;
      procedure SetDBNLS(aVal: TDBNLS); virtual; abstract;
      procedure SetDLL(aVal: TStringList); virtual; abstract;
      procedure SetDLL32(aVal: TStringList); virtual; abstract;
      procedure SetDriversFlag(aVal: SmallInt); virtual; abstract;
      procedure SetEnableBCD(aVal: Boolean); virtual; abstract;
      procedure SetEnableIntegers(aVal: Boolean); virtual; abstract;
      procedure SetEnableSchemaCache(aVal: Boolean); virtual; abstract;
      procedure SetFillFactor(aVal: TFillFactor); virtual; abstract;
      procedure SetHostName(aVal: String); virtual; abstract;
      procedure SetLangDriver(aVal: TLangDriver); virtual; abstract;
      procedure SetLevel(aVal: TLevel); virtual; abstract;
      procedure SetListSynonyms(aVal: TListSynonyms); virtual; abstract;
      procedure SetLockMode(aVal: TLockMode); virtual; abstract;
      procedure SetMaxDbProcesses(aVal: Word); virtual; abstract;
      procedure SetMaxQueryTime(aVal: Word); virtual; abstract;
      procedure SetMaxRows(aVal: LongInt); virtual; abstract;
      procedure SetMdxBlockSize(aVal: Integer); virtual; abstract;
      procedure SetMemoFileBlockSize(aVal: Integer); virtual; abstract;
      procedure SetNationalLangName(aVal: String); virtual; abstract;
      procedure SetNetDir(aVal: String); virtual; abstract;
      procedure SetNetProtocol(aVal: TNetProtocol); virtual; abstract;
      procedure SetOpenMode(aVal: TOpenMode); virtual; abstract;
      procedure SetRowSetSize(aVal: LongInt); virtual; abstract;
      procedure SetServerName(aVal: String); virtual; abstract;
      procedure SetSchemaCacheDir(aVal: String); virtual; abstract;
      procedure SetSchemaCacheSize(aVal: TSchemaCacheSize); virtual; abstract;
      procedure SetSchemaCacheTime(aVal: TSchemaCacheTime); virtual; abstract;
      procedure SetSQLPassThruMode(aVal: TSQLPassThruMode); virtual; abstract;
      procedure SetSQLQryMode(aVal: TSQLQryMode); virtual; abstract;
      procedure SetStrictIntegrity(aVal: Boolean); virtual; abstract;
      procedure SetSystemDatabase(aVal: String); virtual; abstract;
      procedure SetTDSPacketSize(aVal: Word); virtual; abstract;
      procedure SetTimeOut(aVal: TTimeOut); virtual; abstract;
      procedure SetTraceMode(aVal: Word); virtual; abstract;
      procedure SetaType(aVal: TaType); virtual; abstract;
      procedure SetVendorInit(aVal: TStringList); virtual; abstract;
      procedure SetVersion(aVal: Single); virtual; abstract;
      procedure SetUserName(aVal: String); virtual; abstract;
   public
      property ApplicationName: String read FApplicationName write SetApplicationName;
      property BatchCount: SmallInt read FBatchCount write SetBatchCount;
      property BlobEditLoggin: Boolean read FBlobEditLoggin write SetBlobEditLoggin;
      property BlobSize: TBlobSize read FBlobSize write SetBlobSize;
      property BlobsToCache: TBlobsToCache read FBlobsToCache write SetBlobsToCache;
      property BlockSize: TBlockSize read FBlockSize write SetBlockSize;
      property CollChar: TCollChar read GetCollChar write SetCollChar;
      property ConnectTimeOut: Word read FConnectTimeOut write SetConnectTimeOut;
      property DatabaseName: String read FDatabaseName write SetDatabaseName;
      property DateMode: TDateMode read FDateMode write SetDateMode;
      property DateSeparator: TDateSeparator read FDateSeparator write SetDateSeparator;
      property DBNLS: TDBNLS read FDBNLS write SetDBNLS;
      property DLL: TStringList read GetDLL write SetDLL;
      property DLL32: TStringList read GetDLL32 write SetDLL32;
      property DriversFlag: SmallInt read FDriversFlag write SetDriversFlag;
      property EnableBCD: Boolean read FEnableBCD write SetEnableBCD;
      property EnableIntegers: Boolean read FEnableIntegers write SetEnableIntegers;
      property EnableSchemaCache: Boolean read FEnableSchemaCache write SetEnableSchemaCache;
      property FillFactor: TFillFactor read FFillFactor write SetFillFactor;
      property HostName: String read FHostName write SetHostName;
      property LangDriver: TLangDriver read GetLangDriver write SetLangDriver;
      property Level: TLevel read FLevel write SetLevel;
      property ListSynonyms: TListSynonyms read GetListSynonyms write SetListSynonyms;
      property LockMode: TLockMode read FLockMode write SetLockMode;
      property MaxDbProcesses: Word read FMaxDbProcesses write SetMaxDbProcesses;
      property MaxQueryTime: Word read FMaxQueryTime write SetMaxQueryTime;
      property MaxRows: LongInt read FMaxRows write SetMaxRows;
      property MdxBlockSize: Integer read FMdxBlockSize write SetMdxBlockSize;
      property MemoFileBlockSize: Integer read FMemoFileBlockSize write SetMemoFileBlockSize;
      property NationalLangName: String read FNationalLangName write SetNationalLangName;
      property NetDir: String read FNetDir write SetNetDir;
      property NetProtocol: TNetProtocol read GetNetProtocol write SetNetProtocol;
      property OpenMode: TOpenMode read GetOpenMode write SetOpenMode;
      property RowSetSize: LongInt read FRowSetSize write SetRowSetSize;
      property ServerName: String read FServerName write SetServerName;
      property SchemaCacheDir: String read FSchemaCacheDir write SetSchemaCacheDir;
      property SchemaCacheSize: TSchemaCacheSize read FSchemaCacheSize write SetSchemaCacheSize;
      property SchemaCacheTime: TSchemaCacheTime read FSchemaCacheTime write SetSchemaCacheTime;
      property SQLSQLPassThruMode: TSQLPassThruMode read GetSQLPassThruMode write SetSQLPassThruMode;
      property SQLQryMode: TSQLQryMode read GetSQLQryMode write SetSQLQryMode;
      property StrictIntegrity: Boolean read FStrictIntegrity write SetStrictIntegrity;
      property SystemDatabase: String read FSystemDatabase write SetSystemDatabase;
      property TDSPacketSize: Word read FTDSPacketSize write SetTDSPacketSize;
      property TimeOut: TTimeOut read FTimeOut write SetTimeOut;
      property TraceMode: Word read GetTraceMode write SetTraceMode;
      property aType: TaType read GetaType write SetaType;
      property VendorInit: TStringList read GetVendorInit write SetVendorInit;
      property Version: Single read FVersion write SetVersion;
      property UserName: String read FUserName write SetUserName;
   published
      constructor Create; override;
      destructor  Destroy; override;

   end;



   TdBaseDriver = class(TDriverBDE)
   private
      {zde prepsat metody}
      function CheckLevel(aVal: TLevel): Boolean; override;
      function GetLangDriver: TLangDriver; override;
      function GetaType: TaType; override;

      procedure SetLangDriver(aVal: TLangDriver); override;
      procedure SetLevel(aVal: TLevel); override;
      procedure SetMdxBlockSize(aVal: Integer); override;
      procedure SetMemoFileBlockSize(aVal: Integer); override;
      procedure SetaType(aVal: TaType); override;
      procedure SetVersion(aVal: Single); override;
   public
      constructor Create; override;
      destructor  Destroy; override;

   published
      property LangDriver;
      property Level;
      property MdxBlockSize;
      property MemoFileBlockSize;
      property aType;
      property Version;
   end;




   TDriversNativeBDE = class (TOwnerElement)
      FdBaseDriver: TdBaseDriver;
{
      FFoxProDriver: TFoxProDriver;
      FInformixDriver: TInformixDriver;
      FIntrBaseDriver: TIntrBaseDriver;
      FMsAccessDriver: TAccessDriver;
      FMsSQLDriver: TMsSQLDriver;
      FOracleDriver: TOracleDriver;
      FParadoxDriver: TParadoxDriver;
      FSybaseDriver: TSybaseDriver;
 }
   private
      function  GetdBaseDriver: TdBaseDriver;

      procedure SetdBaseDriver(aVal: TdBaseDriver);
      procedure SetNewDriverName(aValue: string);
   public
      constructor Create; override;
      destructor  Destroy; override;
      procedure   CreateAllDrivers; virtual;
   published
      property ActualDriver: TLangDriver read GetActualDriver;
      property dBaseDriver: TdBaseDriver read GetdBaseDriver write SetdBaseDriver;
      property NewDriveName: string write SetNewDriverName;

   end;

 *)


implementation

uses dialogs, u_dbtyps;



constructor TDriverBDE.Create;
begin
   inherited create;
end;



destructor TDriverBDE.Destroy;
begin
   inherited destroy;
end;



function TDriverBDE.CheckBlockSize(aVal: TBlockSize): Boolean;
var
   pom: Integer;
begin
   pom := aVal mod 512;

   if pom = 0 then
      Result := True
   else
      Result := False;
end;



function TDriverBDE.CheckBlockSize2(aVal: TBlockSize): Boolean;
begin

   if ((aVal = 1024) or (aVal = 2048) or (aVal = 4096)) then
      Result := True
   else
      Result := False;
end;



// ------------------------------
// dBASE drive
// ------------------------------

constructor TdBaseDriver.Create;
begin
   inherited create;
end;



destructor TdBaseDriver.Destroy;
begin
   inherited destroy;
end;



function TdBaseDriver.CheckLevel(aVal: TLevel): Boolean;
type
   Tset1 = set of TLevel;

var
   set1: TSet1;
begin
   {Verze dBase, default je 5}
   set1 := [3,4,5,7];

   if aVal in set1 then
      Result := True
   else
      Result := False;
end;



function TdBaseDriver.GetLangDriver: TLangDriver;
begin
   if FLangDriver = nil then FLangDriver := TLangDriver.Create;

   Result := FLangDriver;
end;



procedure TdBaseDriver.SetLangDriver(aVal: TLangDriver);
begin
   FLangDriver := nil;
   FLangDriver := aVal;
end;



procedure TdBaseDriver.SetLevel(aVal: TLevel);
begin
   if CheckLevel(aVal) then
      FLevel := aVal
   else
   begin
//      screen.cursor := crDefault;
      MessageDlg('Nelze zmnit propertu LEVEL na hodnotu ' + IntToStr(Integer(aVal))
      + cNewRow + 'Rozsah paltnch hodnot: '
      + cNewRow + '5 pro dBASE 5.0'
      + cNewRow + '4 pro dBASE 4.0'
      + cNewRow + '3 pro dBASE III a dBASE III PLUS tabulkov formt.'
      + cNewRow + cNewRow + 'Hodnota bude zmnna na defaultn nastaven LEVEL=5',
       mtError, [mbOK], 0);

     {piadm default hodnotu}
      Flevel := 5;
   end;
end;



procedure TdBaseDriver.SetMdxBlockSize(aVal: Integer);
begin
   if CheckBlockSize(aVal) then
      FMdxBlockSize := aVal
   else
   begin
//      screen.cursor := crDefault;
      MessageDlg('Nelze zmnit propertu MDX BLOCK SIZE na hodnotu '
      + IntToStr(aVal) + cNewRow + 'Rozsah platnch hodnot: '
      + cNewRow + '512, 1024, 1536, 2048, ...'
      + cNewRow + cNewRow
      + 'Hodnota bude zmnna na defaultn nastaven MDX BLOCK SIZE = 1024',
       mtError, [mbOK], 0);

     {piadm default hodnotu}
      FMdxBlockSize := 1024;
   end;
end;



procedure TdBaseDriver.SetMemoFileBlockSize(aVal: Integer);
begin
   if CheckBlockSize(aVal) then
      FMemoFileBlockSize := aVal
   else
   begin
//      screen.cursor := crDefault;
      MessageDlg('Nelze zmnit propertu MEMO FILE BLOCK SIZE na hodnotu '
      + IntToStr(aVal) + cNewRow + 'Rozsah platnch hodnot: '
      + cNewRow + '512, 1024, 1536, 2048, ...'
      + cNewRow + cNewRow
      + 'Hodnota bude zmnna na defaultn nastaven MEMO FILE BLOCK SIZE = 1024',
       mtError, [mbOK], 0);

     {piadm default hodnotu}
      FMemoFileBlockSize := 1024;
   end;
end;



procedure TdBaseDriver.SetVersion(aVal: Single);
begin
   FVersion := aVal;
end;



function TdBaseDriver.GetaType: TaType;
begin
   Result := FaType;
end;



procedure TdBaseDriver.SetaType(aVal: TaType);
begin
   FaType := aVal;
end;



procedure TDriversNativeBDE.SetdBaseDriver(aVal: TBaseDriver);
begin
   if aVal = nil then exit;

   FdBaseDriver := aVal;
end;



function TDriversNativeBDE.GetdBaseDriver: TBaseDriver;
var
   D: TdBaseDriver;
begin

  if FdBaseDriver = nil then FdBaseDriver := TdBaseDriver.Create;

   Result := FdBaseDriver;
end;


(*
procedure TDriverBDE.SetBlobEditLoggin(aVal: Boolean);
begin
   FBlobEditLoggin := aVal;
end;



procedure TDriverBDE.SetBlobSize(aVal: TBlobSize);
begin
   FBlobSize := aVal;
end;



procedure TDriverBDE.SetBlobsToCache(aVal: TBlobsToCache);
begin
   FBlobsToCache := aVal;
end;



procedure TDriverBDE.SetBlockSize(aVal: TBlockSize);
begin
   if CheckBlockSize2(aVal) then
      FBlockSize := aVal
   else
   begin
//      screen.cursor := crDefault;
      MessageDlg('Nelze zmnit propertu BLOCK SIZE na hodnotu '
      + IntToStr(aVal) + cNewRow + 'Rozsah platnch hodnot: '
      + cNewRow + '1024, 2048, 4096.'
      + cNewRow + cNewRow
      + 'Hodnota bude zmnna na defaultn nastaven BLOCK SIZE = 2048',
       mtError, [mbOK], 0);

     {piadm default hodnotu}
      FBlockSize := 2048;
   end;
end;



procedure TDriverBDE.SetEnableBCD(aVal: Boolean);
begin
   FEnableBCD := aVal;
end;



procedure TDriverBDE.SetEnableSchemaCache(aVal: Boolean);
begin
   FEnableSchemaCache := aVal;
end;
*)



end.
