unit PrevTab;
INTERFACE

uses

  CtiPar,
  Dos;


type
  PPrevodBunky = ^TPrevodBunky;
  TPrevodBunky = record
     Nazev: string[6];
     Kod  : integer;
     Dalsi: PPrevodBunky;
  end;

  { Objekt pro nacitani Prevodni tabulky z textoveho souboru }
  { ************************************************************ }

  PCteniPrevodniTabulky = ^TCteniPrevodniTabulky;
  TCteniPrevodniTabulky = object(TCteniParametru)
    procedure CtiTabulku;
    function  Cti(Kod: longInt): PLiniovaZnacka;
    function  CtiRetez(Paklic: PPaklicNaPetrklic): PRetezovaZnacka;
    function  CtiClanek: PLinZnacka;
    function  CtiPetrklic(Paklic: PPaklicNaPetrklic): PPetrklicZn;
    function  DejBoolParametr: boolean;
    function  DejIntParametr: Integer;
    function  DejVlevo: boolean;
  end;


IMPLEMENTATION

uses

  CtiRadek,
  StrProc;

const

  Ano = True;
  Ne  = False;

  lzBarva          = 'BARVA';
  lzClanek         = 'CLANEK';
  lzEnd            = 'END';
  lzKod            = 'KOD';
  lzLiniovaZnacka  = 'LINIOVAZNACKA';
  lzNahradniZnacka = 'NAHRADNIZNACKA';
  lzNazev          = 'NAZEV';
  lzPetrklic       = 'PETRKLIC';
  lzRetez          = 'RETEZ';
  lzSila           = 'SILA';
  lzSoubor         = 'SOUBOR';
  lzStrana         = 'STRANA';
  lzTyp            = 'TYP';
  lzUhel           = 'UHEL';
  lzVelikost       = 'VELIKOST';
  lzVzdalenost     = 'VZDALENOST';
  lzZakladniZnacka = 'ZAKLADNIZNACKA';

  lzCentrovat      = 'CENTROVAT';
  lzKreslitPosledni= 'KRESLITPOSLEDNI';
  lzKreslitPrvni   = 'KRESLITPRVNI';
  lzLamatMezeru    = 'LAMATMEZERU';
  lzLamatZnacku    = 'LAMATZNACKU';
  lzNatacet        = 'NATACET';
  lzOchrannaZona   = 'OCHRANNAZONA';

{
  **********************************************************
  *                                                        *
  *             Tela metod TCteniLinioveZnacek             *
  *                                                        *
  **********************************************************
}

function  TCteniLiniovychZnacek.CtiLiniovouZnacku(Kod: longInt): PLiniovaZnacka;
var
  LiniovaZnacka: PLiniovaZnacka;
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Paklic: PPaklicNaPetrklic;

begin
  Paklic := new(PPaklicNaPetrklic, Init);
  LiniovaZnacka := new(PLiniovaZnacka,Init);

  { Nacitame Znacku s konkretnim kodem - ta se musi ovsem nejdriv najit }
  { ------------------------------------------------------------------- }
  if longBool(Kod) and (Kod > 999) then
  begin
     LiniovaZnacka^.Kod := -1;
     repeat
        { Hledam Liniovou Znacku }
        { ---------------------- }
        repeat
           DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                      NazevParametru, HodnotaParametru);
        until JeToInfo and (HodnotaParametru = lzLiniovaZnacka);
        { Kontrola kodu }
         DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                    NazevParametru, HodnotaParametru);
         if JeToParametr and (NazevParametru = lzKod) then
            LiniovaZnacka^.Kod :=  DejLongIntParametr else
         if JeToParametr and (NazevParametru = lzNazev) then
            LiniovaZnacka^.Nazev :=  HodnotaParametru else
     until (LiniovaZnacka^.Kod = Kod) or PosledniRadek ;
     if PosledniRadek then
     begin
        { Zadna takova znacka tady nikdy nebyla }
        { ------------------------------------- }
        LiniovaZnacka^.Free;
        CtiLiniovouZnacku := nil;
        exit;
     end;
  end;
  { Vlastni nacitani znacky }
  { ----------------------- }
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Retez }
     { ----- }
     if JeToInfo and (HodnotaParametru = lzRetez) then
        LiniovaZnacka^.PridejZnacku(CtiRetez(Paklic))
     else
     { Kod }
     { --- }
     if JeToParametr and (NazevParametru = lzKod) then
        LiniovaZnacka^.Kod :=  DejLongIntParametr
     else
     { Nazev }
     { ----- }
     if JeToParametr and (NazevParametru = lzNazev) then
        LiniovaZnacka^.Nazev :=  HodnotaParametru
     else
     { NahradniZnacka }
     { -------------- }
     if JeToParametr and (NazevParametru = lzNahradniZnacka) then
        LiniovaZnacka^.NahradniZnacka :=  DejIntParametr;
  until JeToInfo and (HodnotaParametru = lzEnd);

  if (LiniovaZnacka^.Kod > -1) and Assigned(LiniovaZnacka^.Znacky) then
     CtiLiniovouZnacku := LiniovaZnacka
  else
  begin
     CtiLiniovouZnacku := nil;
     LiniovaZnacka^.Free;
  end;
  dispose(Paklic, Done);
end;

{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiRetez(Paklic: PPaklicNaPetrklic): PRetezovaZnacka;
var
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Retez: PRetezovaZnacka;
begin
  Retez := New(PRetezovaZnacka,Init);
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Clanek }
     { ------ }
     if JeToInfo and (HodnotaParametru = lzClanek) then
        Retez^.PridejZnacku(CtiClanek)
     else
     { Petrklic }
     { -------- }
     if JeToInfo and (HodnotaParametru = lzPetrklic) then
        Retez^.PridejZnacku(CtiPetrklic(Paklic))
     else
     { ZakladniZnacka }
     { -------------- }
     if JeToParametr and (NazevParametru = lzZakladniZnacka) then
        Retez^.ZakladniZnacka := DejIntParametr
     else
     { Vzdalenost }
     { ---------- }
     if JeToParametr and (NazevParametru = lzVzdalenost) then
        Retez^.NastavVzdalenost(DejRealParametr)
     else
     { Strana }
     { ------ }
     if JeToParametr and (NazevParametru = lzStrana) then
        Retez^.Vlevo := DejVlevo
     else
     { Barva }
     { ------ }
     if JeToParametr and (NazevParametru = lzBarva) then
        Retez^.Barva :=  DejIntParametr
     else
     { Status - KreslitPrvni }
     { --------------------- }
     if JeToParametr and (NazevParametru = lzKreslitPrvni) then
     begin
        if DejBoolParametr then
           Retez^.Status := Retez^.Status or znKreslitPrvni
        else
           if Retez^.Status and znKreslitPrvni <> 0 then
              Retez^.Status := Retez^.Status xor znKreslitPrvni;
     end
     else
     { Sila }
     { ---- }
     if JeToParametr and (NazevParametru = lzSila) then
        Retez^.Sila :=  DejIntParametr;
  until JeToInfo and (HodnotaParametru = lzEnd);
  if (Retez^.ZakladniZnacka > -1) or Assigned(Retez^.Znacky) then
     CtiRetez := Retez
  else
  begin
     CtiRetez := nil;
     Retez^.Free;
  end;
end;

{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiClanek: PLinZnacka;
var
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Clanek: PLinZnacka;
  PomBool: boolean;
begin
  Clanek := New(PLinZnacka,Init);
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Velikost }
     { -------- }
     if JeToParametr and (NazevParametru = lzVelikost) then
        Clanek^.NastavVelikostVmm(DejRealParametr)
     else
     { Uhel }
     { ---- }
     if JeToParametr and (NazevParametru = lzUhel) then
        Clanek^.NastavUhel(DejRealParametr)
     else
     { Vzdalenost }
     { ---------- }
     if JeToParametr and (NazevParametru = lzVzdalenost) then
        Clanek^.NastavVzdalenost(DejRealParametr)
     else
     { Sila }
     { ---- }
     if JeToParametr and (NazevParametru = lzSila) then
        Clanek^.Sila :=  DejIntParametr
     else
     { Status - Centrovat }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzCentrovat) then
     begin
        if DejBoolParametr then
           Clanek^.Status := Clanek^.Status or znCentrovat
        else
           if Clanek^.Status and znCentrovat <> 0 then
              Clanek^.Status := Clanek^.Status xor znCentrovat;
     end
     else
     { Status - Natacet }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzNatacet) then
     begin
        if DejBoolParametr then
           Clanek^.Status := Clanek^.Status or znNatacetDleLinie
        else
           if Clanek^.Status and znNatacetDleLinie <> 0 then
              Clanek^.Status := Clanek^.Status xor znNatacetDleLinie;
     end
     else
     { Status - LamatZnacku }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzLamatZnacku) then
     begin
        if DejBoolParametr then
           Clanek^.Status := Clanek^.Status or znLamatZnacku
        else
           if Clanek^.Status and znLamatZnacku <> 0 then
              Clanek^.Status := Clanek^.Status xor znLamatZnacku;
     end
     else
     { Status - LamatMezeru }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzLamatMezeru) then
     begin
        if DejBoolParametr then
           Clanek^.Status := Clanek^.Status or znLamatMezeru
        else
           if Clanek^.Status and znLamatMezeru <> 0 then
              Clanek^.Status := Clanek^.Status xor znLamatMezeru;
     end
     else
     { Status - KreslitPosledni }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzKreslitPosledni) then
     begin
        if DejBoolParametr then
           Clanek^.Status := Clanek^.Status or znKreslitPosledni
        else
           if Clanek^.Status and znKreslitPosledni <> 0 then
              Clanek^.Status := Clanek^.Status xor znKreslitPosledni;
     end;
  until JeToInfo and (HodnotaParametru = lzEnd);

  if (Clanek^.Velikost > 0) then
  begin
     Clanek^.VyrobDefinici;
     CtiClanek := Clanek;
  end
  else
  begin
     CtiClanek := nil;
     Clanek^.Free;
  end;
end;

{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.CtiPetrklic(Paklic: PPaklicNaPetrklic): PPetrklicZn;
var
  JeToParametr,
  JeToInfo,
  PosledniRadek: boolean;
  NazevParametru,
  HodnotaParametru: string;
  Petrklic: PPetrklicZn;
  PomInt: longInt;
begin
  Petrklic := New(PPetrklicZn,Init);
  repeat
     DejRadek ( JeToParametr, JeToInfo, PosledniRadek,
                NazevParametru, HodnotaParametru);
     { Soubor }
     { ------ }
     if JeToParametr and (NazevParametru = lzSoubor) then
        Petrklic^.Soubor := BezVnejsichMezer (HodnotaParametru)
     else
     { Kod }
     { --- }
     if JeToParametr and (NazevParametru = lzKod) then
     begin
        PomInt := DejLongIntParametr;
        if PomInt in [1..255] then Petrklic^.Kod := PomInt
                              else Petrklic^.Kod := 0;
     end
     else
     { Velikost }
     { -------- }
     if JeToParametr and (NazevParametru = lzVelikost) then
        Petrklic^.NastavVelikostVmm(DejRealParametr)
     else
     { Uhel }
     { ---- }
     if JeToParametr and (NazevParametru = lzUhel) then
        Petrklic^.NastavUhel(DejRealParametr)
     else
     { Vzdalenost }
     { ---------- }
     if JeToParametr and (NazevParametru = lzVzdalenost) then
        Petrklic^.NastavVzdalenost(DejRealParametr)
     else
     { Sila }
     { ---- }
     if JeToParametr and (NazevParametru = lzSila) then
        Petrklic^.Sila :=  DejIntParametr
     else
     { Status - Centrovat }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzCentrovat) then
     begin
        if DejBoolParametr then
           Petrklic^.Status := Petrklic^.Status or znCentrovat
        else
           if Petrklic^.Status and znCentrovat <> 0 then
              Petrklic^.Status := Petrklic^.Status xor znCentrovat;
     end
     else
     { Status - Natacet }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzNatacet) then
     begin
        if DejBoolParametr then
           Petrklic^.Status := Petrklic^.Status or znNatacetDleLinie
        else
           if Petrklic^.Status and znNatacetDleLinie <> 0 then
              Petrklic^.Status := Petrklic^.Status xor znNatacetDleLinie;
     end
     else
     { Status - LamatZnacku }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzLamatZnacku) then
     begin
        if DejBoolParametr then
           Petrklic^.Status := Petrklic^.Status or znLamatZnacku
        else
           if Petrklic^.Status and znLamatZnacku <> 0 then
              Petrklic^.Status := Petrklic^.Status xor znLamatZnacku;
     end
     else
     { Status - KreslitPosledni }
     { ------------------ }
     if JeToParametr and (NazevParametru = lzKreslitPosledni) then
     begin
        if DejBoolParametr then
           Petrklic^.Status := Petrklic^.Status or znKreslitPosledni
        else
           if Petrklic^.Status and znKreslitPosledni <> 0 then
              Petrklic^.Status := Petrklic^.Status xor znKreslitPosledni;
     end;

  until JeToInfo and (HodnotaParametru = lzEnd);

  if (Petrklic^.Kod in [1..255]) and (Petrklic^.Velikost > 0) then
  with Petrklic^ do
  begin
     Paklic^.NovyFontJmenem(Soubor);
     Definice    := Paklic^.VratRealnouDefiniciZnaku(Kod, Velikost, Uhel);
     CtiPetrklic := Petrklic;
  end
  else
  begin
     CtiPetrklic := nil;
     Petrklic^.Free;
  end;
end;


{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.DejBoolParametr: boolean;
var
  CtiR: TCtiRadek;

begin
  CtiR.Init(Hodnota);
  DejBoolParametr:= CtiR.DejBoolean;
  CtiR.Done
end;

{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.DejIntParametr: Integer;
var
  CtiR: TCtiRadek;

begin
  CtiR.Init(Hodnota);
  DejIntParametr:= CtiR.DejInteger;
  CtiR.Done
end;

{-----------------------------------------------------------}
function  TCteniLiniovychZnacek.DejVlevo: boolean;
var
  Znak: char;
  i: integer;

begin
  for i := 1 to Length(Hodnota) do
  begin
     Znak := UpCase(Hodnota[i]);
     if Znak in ['L','P','R'] then
     begin
        DejVlevo := Znak = 'L';
        exit;
     end;
  end;
  DejVlevo := Ne;
end;

{-----------------------------------------------------------}
{
  **********************************************************
  *                                                        *
  *             Tela metod TPaklicNaPetrklic               *
  *                                                        *
  **********************************************************
}


constructor TPaklicNaPetrklic.Init;
begin
   inherited init;
   JmenoSouboru := '';
end;

{-----------------------------------------------------------}
function    TPaklicNaPetrklic.NovyFontJmenem(NazevFontu: PathStr): Boolean;
const
{ Overovaci znacka BGI fontu }
  BGICode = $08084B50;
var
  FontF: file;
  Magic: LongInt;
begin
  NovyFontJmenem := Ano;
  OuKej    := Ano;
  if NazevFontu <> JmenoSouboru then begin
     if PFont <> NIL then begin
        FreeMem(PFont,DelkaFontu);
        AktualniFont := 0;
        PFont := NIL;
     end;
     {$I-}
     JmenoSouboru := '';
     NazevFontu:= fSearch (NazevFontu, CestaKSpoustecimuProgramu+';'+
                           GetEnv('PATH'));
     Assign (FontF, NazevFontu);
     Reset  (FontF, 1);
     if IOResult <> 0 then Begin
        Hlaska (Hl_BGIFONTS1+NazevFontu+' !' );
        Oukej          := Ne;
        NovyFontJmenem := Ne;
        exit;
     end
     {$I+}
     else begin
        DelkaFontu := FileSize (FontF)-$80;
        GetMem (PFont,DelkaFontu);
        if PFont = NIL then begin
           Hlaska(Hl_BGIFONTS2 );
           OuKej          := Ne;
           NovyFontJmenem := Ne;
        end
        else begin
           { Je to vubec BGI format? }
           { ----------------------- }
           Seek (FontF,0);
           BlockRead (FontF, Magic, SizeOf(longInt));
           if Magic <> BGICode then
           { Prdlajs! }
           { -------- }
           begin
              OuKej          := Ne;
              NovyFontJmenem := Ne;
           end
           else
           begin
              Seek (FontF,$80);
              BlockRead (FontF, PFont^,DelkaFontu);
              Udaje        := PFont;
              JmenoSouboru := NazevFontu;
           end;
        end;
     end;
     Close (FontF);
  end;
end;

{-----------------------------------------------------------}
function    TPaklicNaPetrklic.VratRealnouDefiniciZnaku ( KodZnaku: byte;
                                                         Velikost,
                                                         ASmer: real): PBodZnacky;
type
   PDelky = ^TDelky;
   TDelky = Array[0..65520] of byte;
var
  Vektor       : PVektory;
  Index        : Word;
  i            : Byte;
  xB,yB        : Byte;
  x,y,
  PomKoefVysky : Real;
  PomkoefSirky : Real;
  Kladna,
  Spoj         : Boolean;
  PosledniZnak : Byte;
  BZ, Pbz      : PBodZnacky;
  Delky        : PDelky;
  Delka        : byte;
  SinSM,CosSm  : real;

  procedure Uprav;
  var
    hX : Real;
    begin
       hX := 0 - (x*CosSm - y*SinSm);
       x  := 0 - (x*SinSm + y*CosSm);
       y  := hX;
    end;

begin
  if OuKej then
  begin
    Vektor := VratVektory;
    Delky := Pointer(NativeInt(@Udaje^.Adresy)+(Udaje^.PocetZnaku shl 1));
    Delka := Delky^[KodZnaku-Udaje^.PrvniZnak];

    SinSm := Sin(ASmer);
    CosSm := Cos(ASmer);

    if Delka > 0 then
       PomKoefVysky  := Velikost/Delka
    else PomKoefVysky:= 1;
    PomKoefSirky := PomKoefVysky;
    PosledniZnak := Udaje^.PrvniZnak + Udaje^.PocetZnaku - 1;
    if not (KodZnaku in [Udaje^.PrvniZnak..PosledniZnak]) then exit;
    Index := KodZnaku - Udaje^.PrvniZnak;
    Index := Udaje^.Adresy[Index];
    BZ    := nil;
    While (Vektor^[Index] OR Vektor^[Index+1]) <> 0 do begin
      if BZ = nil then
      begin
         New(BZ);
         BZ^.Dalsi := nil;
         Pbz := BZ;
      end
      else
      begin
         New(Pbz^.Dalsi);
         Pbz := Pbz^.Dalsi;
         Pbz^.Dalsi := nil;
      end;
      Kladna := True;
      xB := Vektor^[Index];
      yB := Vektor^[index+1];
      asm
         mov    al,xB         {Kopiruji 6.bit (znamenko)}
         mov    ah,al
         and    ah,01111111B
         and    al,01000000B  {na pozici 7. ... pak se  }
         shl    al,1          {s cislem zachazi jako s  }
         or     ah,al         {normalnim ShortIntem.    }
         mov    xB,ah
      end;
      x := PomKoefSirky * ShortInt(xB);
      asm
         mov    al,yB
         mov    ah,al
         and    ah,10000000B
         mov    Spoj,ah

         mov    ah,al
         and    ah,01111111B
         and    al,01000000B
         shl    al,1
         or     ah,al
         mov    yB,ah
      end;

      y := PomKoefVysky * ShortInt(yB);
      Uprav;

      Pbz^.Spoj := Spoj;
      Pbz^.Sour.y := y;
      Pbz^.Sour.x := x;

      Inc(Index,2);
    end;
    { Nastaveni mezery }
    { ---------------- }
    if Pbz <> nil then
    begin
       Pbz^.Sour.y :=  0-Velikost;
       Pbz^.Sour.x :=  0;
    end;
    VratRealnouDefiniciZnaku := BZ;
  end
  else VratRealnouDefiniciZnaku := nil;
end;

{-----------------------------------------------------------}
{
              **********************
              *   private Methods  *
              **********************
}
{-----------------------------------------------------------}


End.