unit i_SelectedGeometry;

interface
uses
   Graphics,
   Elements,
   e_Device,
   i_Basic;
type

TSelectedGeometryInterpret = class(TRectInterpret)
private
    FColor: TColor;
    FBigR: double;
    FSmallR: double;
    procedure SetColor(const Value: TColor);
    procedure SetBigR(const Value: double);
    procedure SetSmallR(const Value: double);
protected
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   function  IsItsClass(AObject: TObject): boolean; override;
   procedure DrawRect(X, Y, HalfSize: double);
   property  SmallR: double read FSmallR write SetSmallR;
   property  BigR: double read FBigR write SetBigR;
public
   procedure Reset(aDevice: TDevice); override;
   function  CanUniGoInside(Element: TElement): boolean; override;
published
   property Color: TColor read FColor write SetColor;
end;


implementation
uses
   e_Class,
   e_Geometry,
   e_GeometryElement;

const
  PixelR: integer = 4;

{ TSelectedGeometryInterpret }

function TSelectedGeometryInterpret.CanUniGoInside(
  Element: TElement): boolean;
begin
   result := false;
end;

procedure TSelectedGeometryInterpret.DrawRect(X, Y, HalfSize: double);
begin
   {
   Device.Rectangle(X-HalfSize,Y-HalfSize,X+HalfSize,Y+HalfSize);
   }
   Device.Pen.Style:= psSolid;
   Device.Pen.Width:= 1;
   Device.MoveTo(X+HalfSize, Y+HalfSize);
   Device.LineTo(X-HalfSize, Y+HalfSize);
   Device.LineTo(X-HalfSize, Y-HalfSize);
   Device.LineTo(X+HalfSize, Y-HalfSize);
   Device.LineTo(X+HalfSize, Y+HalfSize);
   Device.MoveTo(X, Y);

end;

function TSelectedGeometryInterpret.IsItsClass(
  AObject: TObject): boolean;
begin
   result := aObject is TGeometry;
end;

procedure TSelectedGeometryInterpret.Process(Element: TElement);
var
   L: TPolyLineElement;
   A,Are: TAreaElement;
   P: TPointElement;
   T: TTextElement;
   aR: double;
   KillAre: boolean;
   i: integer;
   G: TElement;
   GE: TGeometryElement;
   procedure ProcessArea(aElement: TElement);
   begin
      KillAre := false;
      A := TAreaElement(aElement);
      if A is TPolyLineAreaElement then
      begin
         Are := TPolyLineAreaElement(A).CreateAreaElement;
         KillAre :=  true;
      end
      else
      begin
         Are := A;
      end;
      Device.Brush.Color:= Color;
      Device.Brush.Style:= bsFDiagonal;
      Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 3;
      Device.Pen.Color:= Color;
      Device.Pen.Style:= psSolid;
      Device.PolyPolygon(Are);
      if KillAre then
         Are.Free;
   end;
begin
   if Color = 0 then Color := clSilver;
   G := Element;
   if G is TGeometryElement then
   begin
      GE := TGeometryElement(G);
      G := GE.Geometry;
   end;
   if G is TPolyLineElement then
   begin
      L := TPolyLineElement(G);
      Device.Pen.Color:= Color;
      Device.Pen.Style:= psSolid;
      Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 1;
      Device.Brush.Style:= bsSolid;
      Device.Brush.Color := clWhite;
      if L.CoordsCount > 1 then
      begin
         DrawRect(L.Xi[0], L.Yi[0],BigR);
         Device.MoveTo(L.Xi[0], L.Yi[0]);
         for i := 1 to L.LastIndex do
         begin
             Device.Pen.Style:= psDot;
             Device.Pen.Width:= 3;
             Device.LineTo(L.Xi[i], L.Yi[i]);
             DrawRect(L.Xi[i], L.Yi[i],SmallR)
         end;
      end;
   end
   else
   if G is TAreaElement then
   begin
      if G is TMultiAreaElement then
      begin
      for i := 0 to TMultiAreaElement(G).LastIndex do
      begin
         ProcessArea(TMultiAreaElement(G).Elements[i])
      end;
      end
      else
         ProcessArea(G);
   end
   else
   if G is TPointElement then
   begin
      P := TPointElement(G);
      Device.Pen.Color:= Color;
      Device.Pen.Style:= psSolid;
      Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 3;
      Device.Brush.Style:= bsSolid;
      Device.Brush.Color := clWhite;
      Device.MoveTo(P.X - BigR*2, P.Y);
      Device.LineTo(P.X + BigR*2, P.Y);
      Device.MoveTo(P.X, P.Y - BigR*2);
      Device.LineTo(P.X, P.Y + BigR*2);
      DrawRect(P.X, P.Y,BigR);
      DrawRect(P.X, P.Y,BigR*2);
   end
   else
   if G is TTextElement then
   begin
      T := TTextElement(G);
      Device.Pen.Color:= Color;
      Device.Pen.Style:= psSolid;
      Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 3;
      Device.Brush.Style:= bsSolid;
      Device.Brush.Color := clWhite;
      Device.MoveTo(T.MinX, T.MinY);
      Device.LineTo(T.MaxX, T.MinY);
      Device.LineTo(T.MaxX, T.MaxY);
      Device.LineTo(T.MinX, T.MaxY);
      Device.LineTo(T.MinX, T.MinY);
      DrawRect(T.MinX, T.MinY,SmallR);
      DrawRect(T.MaxX, T.MinY,SmallR);
      DrawRect(T.MaxX, T.MaxY,SmallR);
      DrawRect(T.MinX, T.MaxY,SmallR);
   end
end;

function TSelectedGeometryInterpret.QGoInside(Element: TElement): boolean;
var
   G: TElement;
   GE: TGeometryElement;
   GEmpty: boolean;
begin
   GEmpty := true;
   G := Element;
   if Element is TGeometryElement then
   begin
      GE := TGeometryELement(Element);
      if GE.Geometry <> nil then
      begin
         G := GE.Geometry;
         GEmpty := false;
      end;
   end;
   result := GEmpty and (Element is TEntity) and not((G is TPolyLineElement) or (G is TAreaElement));
end;

function TSelectedGeometryInterpret.QProcessBefore(Element: TElement): boolean;
var
   G: TElement;
   GE: TGeometryElement;
   GFull: boolean;
begin
   GFull := true;
   G := Element;
   if Element is TGeometryElement then
   begin
      GE := TGeometryELement(Element);
      G := nil;
      G := GE.Geometry;
      GFull := G <> nil;
   end;
   result := GFull and ((G is TPolyLineElement) or (G is TAreaElement) or (G is TPointElement) or (G is TTextElement));
end;

procedure TSelectedGeometryInterpret.Reset(aDevice: TDevice);
begin
   inherited Reset(aDevice);
   SmallR := Device.Projection.GetModelLength(PixelR);
   BigR := Device.Projection.GetModelLength(PixelR+2);
end;

procedure TSelectedGeometryInterpret.SetBigR(const Value: double);
begin
  FBigR := Value;
end;

procedure TSelectedGeometryInterpret.SetColor(const Value: TColor);
begin
   FColor := Value;
end;

procedure TSelectedGeometryInterpret.SetSmallR(const Value: double);
begin
  FSmallR := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSelectedGeometryInterpret);
END.
