unit t_FreeDuplicateGeometryElements;

interface
uses
   Tools;
// Both tools: Geometry elements entity is in INPUT property

type

TFreeDuplicateGeometryLines = class(TTool)
  private
    FWithoutPrompt: boolean;
    procedure SetWithoutPrompt(const Value: boolean);
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string; override;
published
   property WithoutPrompt: boolean read FWithoutPrompt write SetWithoutPrompt;
end;

TFreeDuplicateGeometryAreas = class(TTool)
  private
    Fdown: boolean;
    FWithoutPrompt: boolean;
    FParams: string;
    procedure Setdown(const Value: boolean);
    procedure SetWithoutPrompt(const Value: boolean);
    procedure SetParams(const Value: string);
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string; override;
published
   property down: boolean read Fdown write Setdown;
   property WithoutPrompt: boolean read FWithoutPrompt write SetWithoutPrompt;
   property Params: string read FParams write SetParams;
end;

TFreeDuplicateGeometryPoints = class(TTool)
private
    FWithoutPrompt: boolean;
    FParams: string;
    FTolerance: double;
    procedure SetParams(const Value: string);
    procedure SetWithoutPrompt(const Value: boolean);
    procedure SetTolerance(const Value: double);
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string; override;
published
   property WithoutPrompt: boolean read FWithoutPrompt write SetWithoutPrompt;
   property Params: string read FParams write SetParams;
   property Tolerance: double read FTolerance write SetTolerance;
end;


implementation
uses
   Classes,
   Dialogs,
   Forms,
   SysUtils,
   Elements,
   e_TopolElements,
   e_GeometryElement,
   e_Geometry,
   e_Class,
   e_QTree,
   MathLib,
   t_QTreeTools;

{ TFreeDuplicateGeometryLines }

procedure TFreeDuplicateGeometryLines.DoIt;
var
   i,j: integer;
   En,Ow: TEntity;
   El: TElement;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   DCount: integer;
   Oi: integer;
   OE: TOwnerEntity;
begin
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   En := TEntity(Input);
   if En.NoElements then exit;
   StartProgress('Odstrann duplicitnch lini' ,0,En.LastIndex);
   ProgressText := 'dn duplicity...';
   DCount := 0;
   for i := En.LastIndex downto 0 do
   begin
      progress := i;
      if i mod 50 = 0 then
         Application.ProcessMessages;
      El := En.Elements[i];
      if not (El is TTopolLineElement) then continue;
      TL := TTopolLineElement(El);
      if TL.NoGeometry then continue;
      L := TPolyLineElement(TL.Geometry);
      for j := 0 to L.OwnersCount - 1 do
      begin
         Ow := L.Owners[j];
         if Ow = TL then continue;
         if Ow.IsMyOwner(En) then // dal linie ze stejn entity
         begin
            TL.Free;
            DCount := DCount + 1;
            ProgressText := 'duplicity: '+ IntTostr(DCount);
            Break;
         end;
      end;
   end;
   OE := TOwnerEntity.Create;
   Oi := 0;
   for i := 0 to En.LastIndex do
   begin
      El := En.Elements[i];
      if El.IsMyOwner(OE) then
         Inc(Oi)
      else
         OE.AddElement(El);
   end;
   if Oi > 0 then
   begin
      En.FreeAll;
      En.AddAllFrom := OE;
   end;
   OE.Free;
   if not WithoutPrompt then
      ShowMessage(Input.Name+' Bylo odstranno '+IntToStr(DCount)+' duplicit, '+IntToStr(Oi)+' nsobnch vlastnictv');
end;


function TFreeDuplicateGeometryLines.NumberToPropertyName(aNumber: Integer): string;
begin
  case aNumber of
     0: result := 'WithoutPrompt';
  end;
end;

procedure TFreeDuplicateGeometryLines.SetWithoutPrompt(
  const Value: boolean);
begin
    FWithoutPrompt := Value;
end;

{ TFreeDuplicateGeometryAreas }

procedure TFreeDuplicateGeometryAreas.DoIt;
var
   ParamsL: TStringList;
   En: TEntity;
   E: TElement;
   GE,GE1: TGeometryElement;
   MA: TMultiAreaElement;
   A,A1: TAreaElement;
   G,G1: TGeometry;
   TQ: TAddEntityToQTree;
   LevelQT: TRectEntityQTree;
   W: TQTreeWalker;
   Err,ErrN: TOwnerEntity;
   MAErr: TOwnerEntity; // vnitrni prekryvy geometrie
   i,j,k: integer;
   iDCount,DCount: integer;
   BErr: boolean;
   S: string;
   function CheckParams: boolean;
   var
      pari: integer;
      S1,S2: string;
   begin
      result := true;
      if ParamsL.Count = 0 then exit;
      for pari := 0 to ParamsL.Count - 1 do
      begin
         S1 := GE.GetPropertyStringByName(ParamsL.Strings[pari]);
         S2 := GE1.GetPropertyStringByName(ParamsL.Strings[pari]);
         if not (S1 = S2) then
         begin
            result := false;
            exit;
         end;
      end;
   end;
begin
   if (Input = nil) or not(Input is TEntity) then
   begin
      if not WithoutPrompt then
         ShowMessage(Name + 'Input nen nastaven nebo nen entitou');
      exit;
   end;
   En := TEntity(Input);
   if En.NoElements  then
   begin
      exit;
   end;
   E := En.Elements[0];
   GE := nil;
   if E is TGeometryElement then
   begin
      GE := TGeometryElement(E);
      G := GE.Geometry;
   end;
   if (GE = nil) or (G = nil) or not (G is TAreaElement)then
   begin
      if not WithoutPrompt then
         ShowMessage(Name + 'Vrstva neobsahuje plochy');
      exit;
   end;
   //ShowMessage(Params);
   StartProgress('Odstrann duplicitnch ploch' ,0,En.LastIndex);
   ProgressText := 'pprava ...';
   Application.ProcessMessages;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   AddElement(LevelQT);
   TQ := TAddEntityToQTree.Create;
   TQ.Input := En;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   ParamsL := TStringList.Create;
   ParamsL.CommaText := Params;
   Err := TOwnerEntity.Create;
   Err.Name := 'ERR';
   AddElement(Err);
   ErrN := TOwnerEntity.Create;
   ErrN.Name := 'ERR_N';
   AddElement(ErrN);
   MAErr := TOwnerEntity.Create;
   MAErr.Name := 'Internal Err';
   AddElement(MAErr);
   W := TQTreeWalker.Create;
   DCount := 0;
   iDCount := 0;
   // Nejprve odstraneni internich prekryvu
   ProgressText := 'Odstrann internch pekryv...';
   for i := 0 to EN.LastIndex do
   begin
      Progress := i;
      ProgressText := 'internal: '+IntToStr(iDCount)+'/ external: '+IntToStr(DCount);
      Application.ProcessMessages;
      e := En.Elements[i];
      if not(e.IsMyOwner(Err) or e.IsMyOwner(ErrN))and (e is TGeometryElement) then
      begin
         GE := TGeometryElement(e);
         G := GE.Geometry;
         if G is TMultiAreaElement then  // prekryvy vlastni geometrie
         begin
            MA := TMultiAreaElement(G);
            MAErr.FreeAll;
            if MA.Count > 1 then
            for j := 0 to MA.LastIndex - 1 do // do predposledni plochy
            begin
               A := MA.AreaElement[j];
               BErr := false;
               if A.IsMyOwner(MAErr) then continue; // uz je prekryv
               for k := j + 1 to MA.LastIndex do
               begin
                  A1 := MA.AreaElement[k];
                  if A.IsHitByXY(A1.X,A1.Y,0) then
                  begin
                     MAErr.AddElement(A1);
                     BErr := true;
                     // kontrola velikosti ploch !!!?!?....
                     // tady nekde bude ruseni duplicitnich geometrii
                  end;
               end;
               if BErr then
               begin
                  Inc(iDCount);
               end;
            end;
            if not MAErr.NoElements then
            for j := MAErr.LastIndex downto 0 do
            begin
               MAErr.Elements[j].Free;
            end;
         end;
      end;
   end;
   // a externich
   if Down then
      i := En.LastIndex
   else
      i := 0;
   while (i >= 0) AND (i <= EN.LastIndex) do
   begin
      Progress := i;
      ProgressText := 'internal: '+IntToStr(iDCount)+'/ external: '+IntToStr(DCount);
      Application.ProcessMessages;
      e := En.Elements[i];
      if not (e.IsMyOwner(Err) or e.IsMyOwner(ErrN)) and (e is TGeometryElement) then
      begin
         GE := TGeometryElement(e);
         G := GE.Geometry;

         E := W.GetFirstElementForRectangle(LevelQT,GE.Minx,GE.MinY,GE.MaxX,GE.MaxY,1);
         while E <> nil do
         begin
            GE1 := TGeometryElement(E);
            if not (e.IsMyOwner(Err) or e.IsMyOwner(ErrN)) AND (GE <> GE1) then
            begin
               G1 := GE1.Geometry;
               if not (G1 is TMultiAreaElement) then
               begin
                  if G.IsHitByXY(G1.X,G1.Y,0) then
                  begin
                     // uplny prekryv, kontrola parametru
                     if CHeckParams then
                        Err.AddElement(GE1)
                     else
                        ErrN.AddElement(GE1);
                     Inc(DCount);
                  end;
               end
               else
               begin // multiarea
                  MA := TMultiAreaElement(G1);
                  bErr := true; // zatim kompletni prekryv
                  MAErr.FreeAll;
                  for j := 0 to MA.LastIndex do
                  begin
                     A := MA.AreaElement[j];
                     if G.IsHitByXY(A.X,A.Y,0) then
                        MAErr.AddElement(A)
                     else
                        bErr := false; // neuplny prekryv
                  end;
                  if bErr then
                  begin
                     // uplny prekryv, kontrola parametru
                     if CHeckParams then
                        Err.AddElement(GE1)
                     else
                        ErrN.AddElement(GE1);
                  end
                  else
                     if not MAErr.NoElements then
                        ErrN.AddElement(GE1);
               end;
            end;
            E := W.GetNextElement;
         end;
      end;
      if Down then
         Dec(i)
      else
         Inc(i);
   end;
   W.Free;
   Progress := 0;
   MaxProgress := Err.LastIndex;
   ProgressText := 'Ruen duplicit: '+IntToStr(DCount);
   Application.ProcessMessages;
   for i := Err.LastIndex downto 0 do
   begin
      Progress := i;
      Application.ProcessMessages;
      e := Err.Elements[i];
      e.Free;
   end;
   Err.Free;
   LevelQT.Free;
   ParamsL.Free;
   if not WithoutPrompt then
      ShowMessage(Input.Name+' Bylo odstranno '+IntToStr(iDCount)+ ' internch a '+IntToStr(DCount)+' externch pekryv. '+IntToStr(ErrN.Count)+' nelze eit automaticky.');
   ErrN.Free;
end;

function TFreeDuplicateGeometryAreas.NumberToPropertyName(aNumber: Integer): string;
begin
  case aNumber of
     0: result := 'Params';
     1: result := 'WithoutPrompt';
     2: result := 'Down';
  end;
end;

procedure TFreeDuplicateGeometryAreas.Setdown(const Value: boolean);
begin
   Fdown := Value;
end;

procedure TFreeDuplicateGeometryAreas.SetParams(const Value: string);
begin
  FParams := Value;
end;

procedure TFreeDuplicateGeometryAreas.SetWithoutPrompt(const Value: boolean);
begin
   FWithoutPrompt := Value;
end;


{ TFreeDuplicateGeometryPoints }

procedure TFreeDuplicateGeometryPoints.DoIt;
var
   ParamsL: TStringList;
   En: TEntity;
   E: TElement;
   GE,GE1: TGeometryElement;
   G,G1: TGeometry;
   TQ: TAddEntityToQTree;
   LevelQT: TRectEntityQTree;
   W: TQTreeWalker;
   Err,ErrN: TOwnerEntity;
   i,j,k: integer;
   iDCount,DCount: integer;
   BErr: boolean;
   S: string;
   PNTS: boolean;
   function CheckParams: boolean;
   var
      pari: integer;
      S1,S2: string;
   begin
      result := true;
      if ParamsL.Count = 0 then exit;
      for pari := 0 to ParamsL.Count - 1 do
      begin
         S1 := GE.GetPropertyStringByName(ParamsL.Strings[pari]);
         S2 := GE1.GetPropertyStringByName(ParamsL.Strings[pari]);
         if not (S1 = S2) then
         begin
            result := false;
            exit;
         end;
      end;
   end;
begin
   if (Input = nil) or not(Input is TEntity) then
   begin
      if not WithoutPrompt then
         ShowMessage(Name + 'Input nen nastaven nebo nen entitou');
      exit;
   end;
   En := TEntity(Input);
   if En.NoElements  then
   begin
      exit;
   end;
   E := En.Elements[0];
   GE := nil;
   if E is TGeometryElement then
   begin
      GE := TGeometryElement(E);
      G := GE.Geometry;
   end;
   if (GE = nil) or (G = nil) or not ((G is TPointElement)or(G is TTextElement)) then
   begin
      if not WithoutPrompt then
         ShowMessage(Name + 'Vrstva neobsahuje body nebo texty');
      exit;
   end;
   PNTS := G is TPointElement;
   //ShowMessage(Params);
   if PNTS then
      S := 'bod'
   else
      S := 'text';
   StartProgress('Odstrann duplicitnch '+S ,0,En.LastIndex);
   ProgressText := 'pprava ...';
   Application.ProcessMessages;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   AddElement(LevelQT);
   TQ := TAddEntityToQTree.Create;
   TQ.Input := En;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   ParamsL := TStringList.Create;
   ParamsL.CommaText := Params;
   Err := TOwnerEntity.Create;
   Err.Name := 'ERR';
   AddElement(Err);
   ErrN := TOwnerEntity.Create;
   ErrN.Name := 'ERR_N';
   AddElement(ErrN);
   W := TQTreeWalker.Create;
   DCount := 0;
   iDCount := 0;
   // Nejprve odstraneni internich prekryvu
   ProgressText := 'Odstrann internch pekryv...';
   if Tolerance = 0 then Tolerance := 1;
   for i := 0 to EN.LastIndex do
   begin
      Progress := i;
      ProgressText := 'nejde: '+IntToStr(iDCount)+'/ jde: '+IntToStr(DCount);
      Application.ProcessMessages;
      e := En.Elements[i];
      if not(e.IsMyOwner(Err) or e.IsMyOwner(ErrN))and (e is TGeometryElement) then
      begin
         GE := TGeometryElement(e);
         G := GE.Geometry;
         if not ((G is TPointElement) or (G is TTextElement)) then continue;
         E := W.GetFirstElementForRectangle(LevelQT,G.X,G.Y,G.X,G.Y,Tolerance);
         while E <> nil do
         begin
            GE1 := TGeometryElement(E);
            if not (e.IsMyOwner(Err) or e.IsMyOwner(ErrN)) AND (GE <> GE1) then
            begin
               G1 := GE1.Geometry;
               if Tolerance > Distance(G.X, G.Y, G1.X, G1.Y) then
               begin
                  if CHeckParams then
                  begin
                     Err.AddElement(GE1);
                     Inc(DCount);
                  end
                  else
                  begin
                     ErrN.AddElement(GE1);
                     Inc(iDCount);
                  end;
               end;
            end;
            E := W.GetNextElement;
         end;
      end;
   end;
   W.Free;
   Progress := 0;
   MaxProgress := Err.LastIndex;
   ProgressText := 'Ruen duplicit: '+IntToStr(DCount);
   Application.ProcessMessages;
   for i := Err.LastIndex downto 0 do
   begin
      Progress := i;
      Application.ProcessMessages;
      e := Err.Elements[i];
      e.Free;
   end;
   Err.Free;
   LevelQT.Free;
   ErrN.Free;
   ParamsL.Free;
   if not WithoutPrompt then
      ShowMessage(Input.Name+' Bylo odstranno '+IntToStr(DCount)+ ' duplicit - '+IntToStr(iDCount)+' nelze eit automaticky.');
end;

function TFreeDuplicateGeometryPoints.NumberToPropertyName(aNumber: Integer): string;
begin
  case aNumber of
     0: result := 'Params';
     1: result := 'Tolerance';
     2: result := 'WithOutPrompt';
  end;
end;

procedure TFreeDuplicateGeometryPoints.SetParams(const Value: string);
begin
  FParams := Value;
end;

procedure TFreeDuplicateGeometryPoints.SetTolerance(const Value: double);
begin
  FTolerance := Value;
end;

procedure TFreeDuplicateGeometryPoints.SetWithoutPrompt(const Value: boolean);
begin
  FWithoutPrompt := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TFreeDuplicateGeometryLines);
   RegisterOrbisClass(TFreeDuplicateGeometryAreas);
   RegisterOrbisClass(TFreeDuplicateGeometryPoints);
END.

