unit t_GeometryElementUtils;

interface
uses
   Elements,
   e_Device,
   e_Type,
   i_Basic,
   Tools;

type

TSelectAreasBorders = class(TTool)
  private
    FComment: string;
    FAreas: TEntity;
    FLines: TEntity;
    FOuter: boolean;
    procedure SetAreas(const Value: TEntity);
    procedure SetComment(const Value: string);
    procedure SetLines(const Value: TEntity);
    function GetResultLines: TEntity;
    procedure SetOuter(const Value: boolean);
protected
   procedure   SetOutput(aOutput: TElement); override;
public
   procedure DoIt; override;
   property  ResultLines: TEntity read GetResultLines;
published
   property Lines: TEntity read FLines write SetLines;
   property Areas: TEntity read FAreas write SetAreas;
   property Outer: boolean read FOuter write SetOuter;
   property Comment: string read FComment write SetComment;
end;

TLineFindAreaForLayer = class(TTool)
  private
    FLayer: TEntity;
    procedure SetLayer(const Value: TEntity);
public
   function NumberToPropertyName(aNumber: longInt): string;override;
   procedure DoIt; override;
published
   property Layer: TEntity read FLayer write SetLayer;
end;

TAreaLayerReconstruction = class(TTool)
  private
    FAreas: TEntity;
    FLines: TEntity;
    FComment: string;
    procedure SetAreas(const Value: TEntity);
    procedure SetLines(const Value: TEntity);
    procedure SetComment(const Value: string);
public
   function NumberToPropertyName(aNumber: longInt): string;override;
   procedure DoIt; override;
   function ReconstructLines: boolean;
   function ReconstructAreas: boolean;
published
   property Lines: TEntity read FLines write SetLines;
   property Areas: TEntity read FAreas write SetAreas;
   property Comment: string read FComment write SetComment;
end;

TFindLinesForLayerName = class(TTool)
  private
    FNotLayerName: string;
    FLayerName: string;
    procedure SetLayerName(const Value: string);
    procedure SetNotLayerName(const Value: string);
public
   function NumberToPropertyName(aNumber: longInt): string;override;
   procedure DoIt; override;
published
   property LayerName: string read FLayerName write SetLayerName;
   property NotLayerName: string read FNotLayerName write SetNotLayerName;
end;

TIntegrateAreaLayers = class(TTool)
  private
    FComment: string;
    FLines2: TEntity;
    FAreas1: TEntity;
    FAreas2: TEntity;
    FLines1: TEntity;
    FCHangeGeometryOnly: boolean;
    FNoLines: boolean;
    FLines: TEntity;
    FAreas: TEntity;
    procedure SetAreas1(const Value: TEntity);
    procedure SetAreas2(const Value: TEntity);
    procedure SetComment(const Value: string);
    procedure SetLines1(const Value: TEntity);
    procedure SetLines2(const Value: TEntity);
    procedure SetCHangeGeometryOnly(const Value: boolean);
    procedure SetNoLines(const Value: boolean);
    procedure SetLines(const Value: TEntity);
    procedure SetAreas(const Value: TEntity);
public
   procedure DoIt; override;
   property  NoLines: boolean read FNoLines write SetNoLines;
   procedure Process;
   procedure FindLines;
   procedure MakeAreas;
   property  Lines: TEntity read FLines write SetLines;
   property  Areas: TEntity read FAreas write SetAreas;
published
   property Lines1: TEntity read FLines1 write SetLines1;
   property Lines2: TEntity read FLines2 write SetLines2;
   property Areas1: TEntity read FAreas1 write SetAreas1;
   property Areas2: TEntity read FAreas2 write SetAreas2;
   property CHangeGeometryOnly: boolean read FCHangeGeometryOnly write SetCHangeGeometryOnly;
   property Comment: string read FComment write SetComment;
end;

TGeneralizeLines = class(TTool)
  private
    FTolerance: double;
    FComment: string;
    procedure SetTolerance(const Value: double);
    procedure SetComment(const Value: string);
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string;override;
published
   property Tolerance: double read FTolerance write SetTolerance;
   property Comment: string read FComment write SetComment;
end;

TCreateAreasFromLinesAndPoints = class(TTool)
private
    FAreaClassName: string;
    FAreaClass: TElementClass;
    FTransferModel: TStringListElement;
    FPointsEntity: TEntity;
    FAreas: TEntity;
    FLines: TEntity;
    FComment: string;
    FCommentAreas: string;
    FCommentModel: string;
    FNoLines: boolean;
    procedure SetAreaClassName(const Value: string);
    procedure SetAreaClass(const Value: TElementClass);
    function GetTransferModel: TStringListElement;
    procedure SetPointsEntity(const Value: TEntity);
    procedure SetAreas(const Value: TEntity);
    procedure SetLines(const Value: TEntity);
    function GetAreasEntity: TEntity;
    procedure SetAreasEntity(const Value: TEntity);
    procedure SetComment(const Value: string);
    procedure SetCommentAreas(const Value: string);
    procedure SetCommentModel(const Value: string);
    procedure SetNoLines(const Value: boolean);
protected
   procedure CreateAreas;
   procedure CreateModel;
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string;override;
   property AreaClass: TElementClass read FAreaClass write SetAreaClass;
   property Areas: TEntity read FAreas write SetAreas;
   property Lines: TEntity read FLines write SetLines;
   property AreasEntity: TEntity read GetAreasEntity write SetAreasEntity;
published
   property AreaClassName: string read FAreaClassName write SetAreaClassName;
   //property LinesEntity: TEntity;  Lines entity je input
   property PointsEntity: TEntity read FPointsEntity write SetPointsEntity;
   property TransferModel: TStringListElement read GetTransferModel;
   property Comment: string read FComment write SetComment;
   property CommentAreas: string read FCommentAreas write SetCommentAreas;
   property CommentModel: string read FCommentModel write SetCommentModel;
   property NoLines: boolean read FNoLines write SetNoLines;
end;

TCopyAsSimpleLines = class(TTool)
  private
    FComment: string;
    FDevice: TDevice;
    FInterpret: TInterpret;
    FInterpretName: string;
    procedure SetComment(const Value: string);
    procedure SetDevice(const Value: TDevice);
    procedure SetInterpret(const Value: TInterpret);
    procedure SetInterpretName(const Value: string);
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string;override;
   property Interpret: TInterpret read FInterpret write SetInterpret;
published
   property Device: TDevice read FDevice write SetDevice;
   property Comment: string read FComment write SetComment;
   property InterpretName: string read FInterpretName write SetInterpretName;
end;

implementation
uses
   Classes,
   SysUtils,
   Dialogs,
   Forms,
   d_Geometry,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   e_QTree,
   e_TopolElements,
   g_Elems,
   e_IScrpt,
   i_Uni,
   MathLib,
   m_Orbis,
   t_Aremak,
   t_FreeDuplicateGeometryElements,
   t_GeometryCHecks,
   t_LineExtractor,
   t_TisCre,
   t_QTreeTools;

{ TSelectAreasBorders }

procedure TSelectAreasBorders.DoIt;
var
   i,j: integer;
   founded: integer;
   Found: boolean;
   e: TElement;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   A,Are,Are1: TAreaElement;
   O,EE: TEntity;
   Si: string;
begin
   if (Lines = nil) or Lines.NoElements then
   begin
      ShowMessage(Comment+': Vrstva lini nen nastavena nebo je przdn!');
      exit;
   end;
   if (Areas = nil) or Areas.NoElements then
   begin
      ShowMessage(Comment+': Vrstva ploch nen nastavena nebo je przdn!');
      exit;
   end;
   StartProgress(Comment,0,Lines.LastIndex);
   founded := 0;
   Si := IntToStr(Lines.Count);
   for i := 0 to Lines.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Lines.Elements[i];
      if not (e is TTopolLineElement) then continue;
      TL := TTopolLIneElement(e);
      if TL.NoGeometry then continue;
      L := TPolyLineElement(TL.Geometry);
      Are := nil;
      Are1 := nil;
      found := false;
      for j := 0 to L.OwnersCount - 1 do
      begin
         // hledme plochy
         O := L.Owners[j];
         if not (O is TAreaElement) then continue; // neni to plocha tak co
         A := TAreaElement(O);
         if A.IsHole then
            A := TAreaElement(A.HoleOwnerArea); // u dry eme jejho vlastnka
         EE := nil;
         if A <> nil then
            EE := A.FindFirstClassWithOwnerInOwners(TGeometryElement,Areas);
         if EE = nil then continue; // neni hranic plochy ... teda asi = nen co eit kofola !
         if ARE = nil then
         begin
            ARE := A;
            if not Outer then  // nehledame vnejsi linie, muzeme koncit
               break;
         end
         else
         begin                 // asi hledame vnejsi linie
            if A <> ARE then
            begin
               ARE1 := A;
               break;
            end;
         end;
      end;
      if (ARE <> nil) then
      begin
         if not Outer then
            Found := true
         else
            Found := ARE1 = nil;
      end;
      if Found then
      begin
         ResultLines.AddElement(TL);
         ProgressText := IntToStr(Founded)+'/'+Si;
         Application.ProcessMessages;
         inc(Founded);
      end;
   end;
end;

function TSelectAreasBorders.GetResultLines: TEntity;
begin
   if Output = nil then
   begin
      Output := TOwnerEntity.Create;
   end;
   result := TEntity(Output);
end;

procedure TSelectAreasBorders.SetAreas(const Value: TEntity);
begin
  FAreas := Value;
end;

procedure TSelectAreasBorders.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TSelectAreasBorders.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

procedure TSelectAreasBorders.SetOuter(const Value: boolean);
begin
  FOuter := Value;
end;

procedure TSelectAreasBorders.SetOutput(aOutput: TElement);
begin
   ExchangeFieldElement(FOutput, aOutput);
end;

{ TLineFindAreaForLayer }

procedure TLineFindAreaForLayer.DoIt;
var
   O: TEntity;
   A: TAreaElement;
   i: integer;
begin
   Output := nil;
   if input = nil then exit;
   if Layer = nil then exit;
   if not (Input is TPolyLineElement) then exit;
   for i := 0 to Input.OwnersCount - 1 do
   begin
      O := Input.Owners[i];
      if not (O is TAreaElement) then continue;
      A := TAreaElement(O);
      if A.IsHole then
      begin
         A := TAreaElement(A.HoleOwnerArea);
         if A = nil then continue;
      end;
      if A.FindClassWithOwnerInOwners(TGeometryElement,Layer) = nil then continue;
      Output := A;
      exit;
   end;
end;

function TLineFindAreaForLayer.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Layer'
   else
      result := '';
   end;
end;


procedure TLineFindAreaForLayer.SetLayer(const Value: TEntity);
begin
  FLayer := Value;
end;

{ TAreaLayerReconstruction }

procedure TAreaLayerReconstruction.DoIt;
var
   S: string;
begin
   S := 'Rekonstrukce ploch vyaduje funkni vrstvu ';
   if (Lines = nil) or Lines.NoElements then
   begin
      ShowMessage(S +'lini');
      exit;
   end;
   if (Areas = nil) or Areas.NoElements then
   begin
      ShowMessage(S +'ploch');
      exit;
   end;
   StartProgress(Comment,0,0);
   ReconstructLines;
   ReconstructAreas;
   EndProgress;
end;

function TAreaLayerReconstruction.NumberToPropertyName(
  aNumber: Integer): string;
begin

end;

function TAreaLayerReconstruction.ReconstructAreas: boolean;
var
   i,j: integer;
   e: TElement;
   GE: TGeometryElement;
   G: TGeometry;
   A,NA: TAreaElement;
   MA: TMultiAreaElement;
   S: string;
   ACL: TAreasFromClearLines;
   NAreas: TEntity;
   TQ: TAddEntityToQTree;
   QTree: TRectEntityQTree;
   QTFinder: TFindRectEntityInQTreeByXY;

   procedure ProcessArea;
   begin
      QTFinder.X := A.X;
      QTFInder.Y := A.Y;
      QTFinder.Tolerance := 0;
      QTFinder.DoIt;
      NA := TAreaElement(QTFinder.Output);
      if NA = nil then
      begin
         PostTextMsg(tm_AddToProtocol,'NewArea lost: ' + GE.Name,nil);
         exit;
      end;
      NA.AbsRefX := A.X;
      NA.AbsRefY := A.Y;
      if MA <> nil then
         MA.FreeElement(A)
      else
         GE.Geometry := nil;
      if (MA <> nil) and MA.NoElements then
         MA.Free;
      if NA.FixedAreaIndex < 1 then
      begin
         NA.FixedAreaIndex := 1;
         GE.AddGeometry(NA);
      end;
   end;
begin
   result := true;
   S := 'Rekonstrukce ploch ';
   ProgressText := S + '...peplochovn';
   Application.ProcessMessages;
   ACL := TAreasFromClearLines.Create;
   ACL.Input := Lines;
   ACL.DoIt;
   NAreas := TEntity(ACL.Output);
   AddElement(NAreas);
   ACL.Free;
   ProgressText := S + '...tvorba QTree';
   Application.ProcessMessages;
   QTree :=TRectEntityQTree.Create;
   QTree.Name := 'LEVEL QT';
   AddElement(QTree);
   TQ := TAddEntityToQTree.Create;
   TQ.Input := NAreas;
   TQ.Output := QTree;
   TQ.DoIt;
   TQ.Free;
   QTFinder := TFindRectEntityInQTreeByXY.Create;
   QTFinder.Input := QTree;
   AddElement(QTFinder);
   ProgressText := S + '...Zmna ploch';
   MaxProgress := Areas.LastIndex;
   Application.ProcessMessages;
   for i := 0 to Areas.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      e := Areas.Elements[i];
      if not (e is TGeometryElement) then
      begin
         ShowMessage('Vrstva ploch neobsahuje Geometry elementy, konme...');
         exit;
      end;
      GE := TGeometryElement(e);
      if GE.NoGeometry then continue;
      G := GE.Geometry;
      if not (G is TAreaElement) then
      begin
         ShowMessage('Vrstva ploch neobsahuje plochy jako geometrii, konme...');
         exit;
      end;
      A := TAreaElement(G);
      MA := nil;
      if (A is TMultiAreaElement) then
      begin
         MA := TMultiAreaElement(A);
         for j := MA.LastIndex downto 0 do
         begin
            A := MA.AreaElement[j];
            ProcessArea;
         end;
      end
      else
         ProcessArea;
   end;
end;

function TAreaLayerReconstruction.ReconstructLines: boolean;
var
   S: string;
   CL: TClearLines;
   CNVL: TClearNoValidLines;
   FD: TFreeDuplicateGeometryLines;
begin
   result := true;
   S := 'Rekonstrukce lini ';
   ProgressText := S + '...itn';
   Application.ProcessMessages;
   CNVL := TClearNoValidLines.Create;
   CNVL.Input := Lines;
   CNVL.WithoutPrompt := true;
   CNVL.DoIt;
   CNVL.Free;
   CL := TClearLines.Create;
   CL.Input := Lines;
   CL.DoIt;
   CL.Free;
   ProgressText := S + '...odstrann duplicit';
   Application.ProcessMessages;
   FD := TFreeDuplicateGeometryLines.Create;
   FD.Input := Lines;
   FD.WithoutPrompt := true;
   FD.DoIt;
   FD.Free;
   CNVL := TClearNoValidLines.Create;
   CNVL.Input := Lines;
   CNVL.WithoutPrompt := true;
   CNVL.DoIt;
   CNVL.Free;
end;

procedure TAreaLayerReconstruction.SetAreas(const Value: TEntity);
begin
  FAreas := Value;
end;

procedure TAreaLayerReconstruction.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TAreaLayerReconstruction.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

{ TFindLinesForLayerName }

procedure TFindLinesForLayerName.DoIt;
var
   SLT: TStringList;
   SLF: TStringList;
   En: TEntity;
   E: TElement;
   OE: TEntity;
   i,oi,Listi: integer;
   GE: TGeometryElement;
   G: TGeometry;
   Ow: TEntity;
   OKplus,OKminus: boolean;
begin
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   En := TEntity(Input);
   if En.NoElements then exit;
   if Output = nil then
   begin
      OE := TPolyLineCollection.Create;
      Output := OE;
   end
   else
   begin
      if not (Output is TEntity) then exit;
      OE := TEntity(Output);
   end;
   // priprava seznamu
   SLT := TStringList.Create;
   SLT.CommaText := LayerName;
   SLF := TStringList.Create;
   SLF.CommaText := NotLayerName;
   // pridat progress
   for i := 0 to En.LastIndex do
   begin
      e := En.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      G := GE.Geometry;
      if G = nil then continue;
      if not (G is TPolyLineElement) then continue;
      OKplus := false;
      OKMinus := false;
      for  oi := 0 to G.OwnersCount -1 do
      begin
         Ow := G.Owners[oi];
         if not (Ow is TGeometryElement) then continue;
         if not OKPlus then
         if SLT.Count > 0 then
         for Listi := 0 to SLT.Count - 1 do
         begin
            if Ow.FindOwnerName(SLT.Strings[Listi])<> nil then
            begin
               OKplus := true;
               break;
            end;
         end;
         for Listi := 0 to SLF.Count - 1 do
         begin
            if Ow.FindOwnerName(SLF.Strings[Listi])<> nil then
            begin
               OKminus := true;
               break;
            end;
         end;
         if OKminus then  break;
      end;
      if not OKminus then
      if OKPlus then
         OE.AddElement(GE);

   end;
end;

function TFindLinesForLayerName.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'LayerName';
   else
      result := 'NotLayerName';
   end;
end;

procedure TFindLinesForLayerName.SetLayerName(const Value: string);
begin
  FLayerName := Value;
end;

procedure TFindLinesForLayerName.SetNotLayerName(const Value: string);
begin
  FNotLayerName := Value;
end;

{ TIntegrateAreaLayers }

procedure TIntegrateAreaLayers.DoIt;
var
   S: string;
begin
   if Output = nil then
   begin
      Output := TOwnerEntity.Create;
      AddElement(Output);
   end;
   if Comment = '' then
      Comment := 'Area layers integration';
   S := 'chyb nebo je przdn!';
   if (Areas1 = nil) or (Areas1.NoElements) then
   begin
      ShowMessage(Comment + ' Areas1 '+S);
      exit;
   end;
   if (Areas2 = nil) or (Areas2.NoElements) then
   begin
      ShowMessage(Comment + ' Areas2 '+S);
      exit;
   end;
   NoLines := (Lines1 = nil) and (Lines2 = nil);
   if not NoLines and ((Lines1 = nil) or (Lines2 = nil)) then
   begin
      ShowMessage(Comment + ' Lines '+S);
      exit;
   end;
   Process;
end;

procedure TIntegrateAreaLayers.FindLines;
var
   LEX: TLineExtractor;
begin
   ProgressText := 'Hledn lini..';
   Application.ProcessMessages;
   Lines := TPolyLineCollection.Create;
   Lines.Name := 'Lines';
   AddElement(Lines);
   //DET
   LEX := TLineExtractor.Create;
   LEX.CopyLines := true;
   LEX.Input := Areas1;
   LEX.Output := Lines;
   LEX.DoIt;
   LEX.Free;
   //SRF
   LEX := TLineExtractor.Create;
   LEX.CopyLines := true;
   LEX.Input := Areas2;
   LEX.Output := Lines;
   LEX.DoIt;
   LEX.Free;
   TPolyLineCollection(Lines).Update;
end;

procedure TIntegrateAreaLayers.MakeAreas;
var
   QT1,QT2: TRectEntityQTree;
   QTFinder: TFindRectEntityInQTreeByXY;
   TQ: TAddEntityToQTree;
   AM: TCreateAreasFromLinesTool;
   A: TAreaElement;
   E: TElement;
   i: integer;
   Znacka: integer;
   Barva: integer;
   GE: TGeometryElement;
   Layer: TEntity;
begin
   Layer := TEntity(Output);
   // Priprava QTrees
   ProgressText := 'Pprava QTree 1...';
   Application.ProcessMessages;
   QT1 :=TRectEntityQTree.Create;
   AddElement(QT1);
   QT1.Name := 'QT 1';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Areas1;
   TQ.Output := QT1;
   TQ.DoIt;
   TQ.Free;
   ProgressText := 'Pprava QTree 2...';
   Application.ProcessMessages;
   QT2 :=TRectEntityQTree.Create;
   AddElement(QT2);
   QT2.Name := 'QT 2';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Areas2;
   TQ.Output := QT2;
   TQ.DoIt;
   TQ.Free;
   // plochy
   QTFinder := TFindRectEntityInQTreeByXY.Create;
   AddElement(QTFinder);
   ProgressText := 'Sofistikovan tvorba ploch...';
   Application.ProcessMessages;
   AM := TCreateAreasFromLinesTool.Create;
   AM.Input := Lines;
   AM.DoIt;
   Areas := TEntity(AM.Output);

   ProgressText := 'Sofistikovan tvorba ploch ... FINLE';
   MaxProgress := Areas.LastIndex;
   Application.ProcessMessages;

   for i := 0 to Areas.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      E := Areas.Elements[i];
      if not (E is TAreaElement) then Continue;
      A := TAreaElement(E);
      // hledame A2 plochu
      E := nil;
      if QT2 <> nil then
      begin
         QTFinder.Input := QT2;
         QTFinder.X := A.X;
         QTFinder.Y := A.Y;
         QTFinder.DoIt;
         E := QTFinder.Output;
      end;
      if E <> nil then
      begin
         GE := TGeometryElement(CreateElementCopy(E));
         GE.Geometry := A;
         Layer.AddElement(GE);
         continue;
      end;
      // hledame A1 plochu
      QTFinder.Input := QT1;
      QTFinder.X := A.X;
      QTFinder.Y := A.Y;
      QTFinder.DoIt;
      E := QTFinder.Output;
      if E <> nil then
      begin
         GE := TGeometryElement(CreateElementCopy(E));
         GE.Geometry := A;
         Layer.AddElement(GE);
      end;
   end;
   //Areas := nil;
   AM.Free;

end;

procedure TIntegrateAreaLayers.Process;
begin
   StartProgress(Comment,0,0);
   if NoLines then
      FindLines;
   if Lines.NoElements then
   begin
      EndProgress;
      exit;
   end;
   MakeAreas;
   EndProgress;
end;

procedure TIntegrateAreaLayers.SetAreas(const Value: TEntity);
begin
  ExchangeFieldElement(TElement(FAreas), Value);
end;

procedure TIntegrateAreaLayers.SetAreas1(const Value: TEntity);
begin
  FAreas1 := Value;
end;

procedure TIntegrateAreaLayers.SetAreas2(const Value: TEntity);
begin
  FAreas2 := Value;
end;

procedure TIntegrateAreaLayers.SetCHangeGeometryOnly(const Value: boolean);
begin
  FCHangeGeometryOnly := Value;
end;

procedure TIntegrateAreaLayers.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TIntegrateAreaLayers.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

procedure TIntegrateAreaLayers.SetLines1(const Value: TEntity);
begin
  FLines1 := Value;
end;

procedure TIntegrateAreaLayers.SetLines2(const Value: TEntity);
begin
  FLines2 := Value;
end;

procedure TIntegrateAreaLayers.SetNoLines(const Value: boolean);
begin
  FNoLines := Value;
end;

{ TCreateAreasFromLinesAndPoints }

procedure TCreateAreasFromLinesAndPoints.CreateAreas;
var
   ACL: TAreasFromClearLines;
   //i: integer;
begin
   if CommentAreas = '' then
      CommentAreas := 'Areas';
   ProgressText := CommentAreas;
   Application.ProcessMessages;
   ACL := TAreasFromClearLines.Create;
   ACL.Input := Lines;
   ACL.DoIt;
   Areas := TEntity(ACL.Output);
   ACL.Free;
end;

procedure TCreateAreasFromLinesAndPoints.CreateModel;
var
   QTFinder: TFindRectEntityInQTreeByXY;
   TQ: TAddEntityToQTree;
   QT: TRectEntityQTree;
   S,S1,S2: string;
   i,j: integer;
   e,ne: TElement;
   RE: TRectEntity;
   ARE: TAreaElement;
   SolvedAreas: TOwnerEntity;
   GE: TGeometryElement;
   SE: TStringElement;
begin
   if CommentModel = '' then
      CommentModel := 'Model';
   ProgressText := CommentModel;
   Application.ProcessMessages;
   QTFinder := TFindRectEntityInQTreeByXY.Create;
   QT :=TRectEntityQTree.Create;
   QT.Name := 'Areas QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Areas;
   TQ.Output := QT;
   TQ.DoIt;
   MaxProgress := PointsEntity.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   QTFinder.Input := QT;
   SolvedAreas := TOwnerEntity.Create;
   SolvedAreas.Name := 'SA';
   AddElement(SolvedAreas);
   for i := 0 to PointsEntity.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := PointsEntity.Elements[i];
      if not (e is TRectEntity) then continue;
      RE := TRectEntity(e);
      //if T.Text='801' then
         //T.X := T.X+0.2;
      QTFinder.X := RE.X;
      QTFinder.Y := RE.Y;
      QTFinder.DoIt;
      e := QTFinder.Output;
      if e = nil then continue;
      if not (e is TAreaElement) then Continue;
      if e.IsMyOwner(SolvedAreas) then Continue;
      SolvedAreas.AddElement(e);
      ARE := TAreaElement(e);
      ARE.AbsRefX := RE.X;
      ARE.AbsRefY := RE.Y;
      ne := AreaClass.Create;
      GE := TGeometryElement(ne);
      if NoLines then
      if ARE is TPolyLineAreaElement then
         ARE := TPolyLineAreaElement(ARE).CreateAreaElement;
      GE.Geometry := ARE;
      AreasEntity.AddElement(GE);
      for j := 0 to TransferModel.LastIndex do
      begin
        SE := TransferModel.StringElements[j];
        GE.SetPropertyStringByName(SE.Value,RE.GetPropertyStringByName(SE.Name));
      end;
   end;
   if AreasENtity is TRectEntity then
      TRectEntity(AreasEntity).Update;
   TQ.Free;
   QT.Free;
   QTFinder.Free;
   SolvedAreas.Free;
end;

procedure TCreateAreasFromLinesAndPoints.DoIt;
begin
   Error := 1;
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   Lines := TEntity(Input);
   if Lines.NoElements then exit;
   Error := 2;
   if PointsEntity = nil then exit;
   if PointsEntity.NoElements then exit;
   Error := 3;
   AreaClass := GetOrbisClass(AreaClassName);
   if AreaClass = nil then exit;
   if not AreaClass.InheritsFrom(TGeometryElement) then exit;
   Error := 4;
   if Output = nil then
   begin
      Output := TGeometryElementCollection.Create;
      AddElement(Output);
   end
   else
   begin
      if not (Output is TEntity) then exit;
   end;
   Error := 0;
   if Comment = '' then Comment := Name;
   StartProgress(Comment,0,0);
   CreateAreas;
   CreateModel;
end;

function TCreateAreasFromLinesAndPoints.GetAreasEntity: TEntity;
begin
   result := TEntity(Output);
end;

function TCreateAreasFromLinesAndPoints.GetTransferModel: TStringListElement;
begin
   if FTransferModel = nil then
   begin
      FTransferModel := TStringListElement.Create;
   end;
   result := FTransferModel;
end;

function TCreateAreasFromLinesAndPoints.NumberToPropertyName(
  aNumber: Integer): string;
begin

end;

procedure TCreateAreasFromLinesAndPoints.SetAreaClass(const Value: TElementClass);
begin
  FAreaClass := Value;
end;

procedure TCreateAreasFromLinesAndPoints.SetAreaClassName(const Value: string);
begin
  FAreaClassName := Value;
end;


procedure TCreateAreasFromLinesAndPoints.SetAreas(const Value: TEntity);
begin
   ExchangeFieldElement(TElement(FAreas), Value);
end;

procedure TCreateAreasFromLinesAndPoints.SetAreasEntity(const Value: TEntity);
begin
   Output := Value;
end;

procedure TCreateAreasFromLinesAndPoints.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCreateAreasFromLinesAndPoints.SetCommentAreas(
  const Value: string);
begin
  FCommentAreas := Value;
end;

procedure TCreateAreasFromLinesAndPoints.SetCommentModel(const Value: string);
begin
  FCommentModel := Value;
end;

procedure TCreateAreasFromLinesAndPoints.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

procedure TCreateAreasFromLinesAndPoints.SetNoLines(const Value: boolean);
begin
  FNoLines := Value;
end;

procedure TCreateAreasFromLinesAndPoints.SetPointsEntity(const Value: TEntity);
begin
  FPointsEntity := Value;
end;

{ TGeneralizeLines }

procedure TGeneralizeLines.DoIt;
var
   i,j: integer;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   E: TElement;
   first,last: integer;
   LinesEntity: TEntity;

function Test(i1,i2: integer): boolean;
var
   i: integer;
   X1,Y1,X2,Y2,PX,PY,dP: double;
   PataIn: boolean;
begin
   result := true;
   if (i2 < 0)or((i1 - i2) < 2) then
   begin
      result := false;
      exit;
   end;
   X1 := L.Xi[i1];
   Y1 := L.Yi[i1];
   X2 := L.Xi[i2];
   Y2 := L.Yi[i2];
   for i := i1-1 downto i2+1 do
   begin
       DejVztahyBod_Usecka(L.Xi[i], L.Yi[i],X1,Y1,X2,Y2,PataIn,PX,PY,dP);
       if dP > Tolerance then
       begin
          result := false;
          exit;
       end;
   end;
end;

begin
   if Input = nil then exit;
   if not(Input is TEntity) then exit;
   LinesEntity := TEntity(Input);
   if Tolerance <= 0 then Tolerance := 0.001;
   if Comment = '' then Comment := Name;
   StartProgress(Comment,0,LinesEntity.LastIndex);
   //progressText := 'Generalizace...';
   for j := 0 to LinesEntity.LastIndex do
   begin
      Progress := j;
      Application.ProcessMessages;
      E := LinesEntity.Elements[j];
      if E is TTopolLineElement then
      begin
         TL := TTopolLineElement(E);
         if TL.NoGeometry then continue;
         L := TPolyLineElement(TL.Geometry);
      end
      else
      begin
         if e is TPolyLineElement then
            L := TPolyLineElement(e)
         else
            continue;
      end;
      if L.Count < 3 then continue;
      first := L.LastIndex;
      Last :=  first - 1;
      while last >= 0 do
      begin
         if Test(first,last-1) then
         begin
            dec(last);
         end
         else
         begin
            if (first - last)>= 2 then
            begin
               for i := first-1 downto last + 1 do
               begin
                  L.FreeElement(L.Elements[i]);
               end;
            end;
            first := last;
            dec(last);
         end;
      end;
   end;
end;

function TGeneralizeLines.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Tolerance';
   else
      result := 'Comment';
   end;
end;

procedure TGeneralizeLines.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TGeneralizeLines.SetTolerance(const Value: double);
begin
  FTolerance := Value;
end;

{ TCopyAsSimpleLines }

procedure TCopyAsSimpleLines.DoIt;
var
   Eni,Eno: TEntity;
   D: TPolyLine_Device;
   i: integer;
   TL: TTopolLineElement;
   L: TPolyLineElement;
   e: TElement;
   Prog: boolean;
   Progi: integer;
   LinE: TEntity;
   Tis: TUniversalTopolScriptCreator;
   OC: TElementClass;
begin
   Error := 1;
   if Device = nil then exit;
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   Eni := TEntity(Input);
   if Output = nil then
   begin
      Eno := TGeometryElementCollection.Create;
      Output := Eno;
      AddElement(Eno);
   end;
   if not (Output is TEntity) then exit;
   Eno := TEntity(Output);
   Error := 0;
   D := TPolyLine_Device.Create;
   AddElement(D);
   LinE := TGeometryElementCollection.Create;
   AddElement(LinE);
   D.Lines := LinE;
   D.Device := Device;
   if InterpretName = '' then
   begin
      Interpret := TUniversalInterpret.Create;
      Tis := TUniversalTopolScriptCreator.Create;
      Tis.NoRect := true;
      Tis.DoIt;
      TUniversalInterpret(Interpret).Script := TInterpretScript(Tis.Output);
      Tis.Free;
   end
   else
   begin
      OC := GetOrbisClass(InterpretName);
      e := nil;
      if OC <> nil then
         e := OC.Create;
      if (e = nil) or not (e is TInterpret) then
      begin
         e.Free;
         e := nil;
      end;
      Interpret := TInterpret(e);
   end;
   if Interpret = nil then exit;
   Interpret.NoRect := true;
   Interpret.Device := D;
   Progi := Eni.Count div 100;
   if progi < 1 then
      progi := 1;
   if progi > 50 then
      progi := 50;
   //Prog := Eni.Count > 50;
   Prog := true;
   if Prog then
      StartProgress(Comment,0,Eni.LastIndex);
   D.Reset;
   for i := 0 to Eni.LastIndex do
   begin
      //if Prog and (i mod 50 = 0) then
      if Prog and (i mod progi = 0) then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Eni.Elements[i];
      Interpret.Input := e;
      Interpret.DoIt;
   end;
   D.Close;
   D.Free;
   if Prog then
   begin
      Progress := 0;
      MaxProgress := LinE.LastIndex;
      progi := LinE.Count div 100;
      if progi < 1 then
         progi := 1;
      if progi > 50 then
         progi := 50;
   end;
   for i := LinE.LastIndex downTo 0 do
   begin
      if Prog and (i mod progi = 0) then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := LinE.Elements[i];
      if not (e is TPolyLineElement) then continue;
      L := TPolyLineElement(e);
      TL := TTopolLineElement.Create;
      TL.Druh := 1;
      TL.Barva := 13;
      TL.Znacka := 0;
      TL.Geometry := L;
      Eno.AddElement(TL);
   end;
   if Eno is TRectEntity then
      TRectEntity(Eno).Update;
   LinE.Free;
end;

function TCopyAsSimpleLines.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Device';
      1: result := 'Comment';
      2: result := 'Output';
      3: result := 'InterpretName';
   end;
end;

procedure TCopyAsSimpleLines.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCopyAsSimpleLines.SetDevice(const Value: TDevice);
begin
  FDevice := Value;
end;

procedure TCopyAsSimpleLines.SetInterpret(const Value: TInterpret);
begin
  ExchangeFieldElement(TElement(FInterpret),Value);
end;

procedure TCopyAsSimpleLines.SetInterpretName(const Value: string);
begin
  FInterpretName := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSelectAreasBorders);
   RegisterOrbisClass(TLineFindAreaForLayer);
   RegisterOrbisClass(TAreaLayerReconstruction);
   RegisterOrbisClass(TFindLinesForLayerName);
   RegisterOrbisClass(TIntegrateAreaLayers);
   RegisterOrbisClass(TCreateAreasFromLinesAndPoints);
   RegisterOrbisClass(TGeneralizeLines);
   RegisterOrbisClass(TCopyAsSimpleLines);
END.

