unit t_RepairTools;

interface
uses
   Elements,
   Tools;

type

TSafeTransferLinesFromTo = class(TTool)
  private
    FDestinationLayer: TEntity;
    FSourceLayer: TEntity;
    FRefAreasLayer: TEntity;
    FErrorCount: integer;
    FRefAreaClassName: string;
    FRefAreaClass: TElementClass;
    procedure SetDestinationLayer(const Value: TEntity);
    procedure SetRefAreasLayer(const Value: TEntity);
    procedure SetSourceLayer(const Value: TEntity);
    procedure SetErrorCount(const Value: integer);
    procedure SetRefAreaClass(const Value: TElementClass);
    procedure SetRefAreaClassName(const Value: string);
public
   procedure DoIt; override;
   property  ErrorCount: integer read FErrorCount write SetErrorCount;
   property  RefAreaClass: TElementClass read FRefAreaClass write SetRefAreaClass;
published
   property SourceLayer: TEntity read FSourceLayer write SetSourceLayer;
   property DestinationLayer: TEntity read FDestinationLayer write SetDestinationLayer;
   property RefAreasLayer: TEntity read FRefAreasLayer write SetRefAreasLayer;
   property RefAreaClassName: string read FRefAreaClassName write SetRefAreaClassName;
end;

implementation
uses
   Forms,
   Dialogs,
   SysUtils,
   m_Orbis,
   e_GeometryElement,
   e_Geometry,
   e_Class;

{ TSafeTransferLinesFromTo }

procedure TSafeTransferLinesFromTo.DoIt;
var
   li: integer;
   i: integer;
   En: TEntity;
   E: TElement;
   S: string;
   G: TGeometryElement;
begin
   ErrorCount := 0;
   if Input = nil then
   begin
      ShowMessage('Bohuel, Chyb vstupn entita lini');
      exit;
   end;
   if SourceLayer = nil then
   begin
      ShowMessage('Bohuel, zdrojov vrstva lini nebyla nalezena');
      exit;
   end;
   if DestinationLayer = nil then
   begin
      ShowMessage('Bohuel, clov vrstva lini nebyla nalezena');
      exit;
   end;
   if not(Input is TEntity) then exit;
   En := TEntity(Input);
   li := En.LastIndex;
   if not En.NoElements then
      StartProgress('Penos vybranch lini z '+SourceLayer.Name+' do '+DestinationLayer.Name,0,li);
   if RefAreasLayer = nil then S := 'Bez referennch ploch'
                          else S := 'Referenn plochy: '+RefAreasLayer.Name;
   ProgressText := S;
   Application.ProcessMessages;
   for i := li downto 0 do
   begin
      Progress := li-i;
      Application.ProcessMessages;
      E := TElement(En.Elements[i]);
      if not (E is TGeometryElement) then
      begin
         ErrorCount := ErrorCount + 1;
         continue;
      end;
      G := TGeometryElement(E);
      if G.NoGeometry then
      begin
         ErrorCount := ErrorCount + 1;
         continue;
      end;
      if RefAreasLayer <> nil then
      if G.Geometry.FindClassWithOwnerInOwners(RefAreaClass,RefAreasLayer) <> nil then
      begin
         ErrorCount := ErrorCount + 1;
         continue;
      end;
      SourceLayer.RemoveElement(G);
      DestinationLayer.AddElement(G);
      En.RemoveElement(G);
   end;
   EndProgress;
   if ErrorCount > 0 then
      ShowMessage('Nebylo peneseno '+IntToStr(ErrorCount)+' lini jsou patrn soust ploch.');
   PostMsg(tm_ModelChanged,nil,nil,false);
end;

procedure TSafeTransferLinesFromTo.SetDestinationLayer(const Value: TEntity);
begin
  FDestinationLayer := Value;
end;

procedure TSafeTransferLinesFromTo.SetErrorCount(const Value: integer);
begin
  FErrorCount := Value;
end;

procedure TSafeTransferLinesFromTo.SetRefAreaClass(const Value: TElementClass);
begin
  FRefAreaClass := Value;
end;

procedure TSafeTransferLinesFromTo.SetRefAreaClassName(const Value: string);
begin
   FRefAreaClassName := Value;
   RefAreaClass := GetOrbisClass(Value);
end;

procedure TSafeTransferLinesFromTo.SetRefAreasLayer(const Value: TEntity);
begin
  FRefAreasLayer := Value;
end;

procedure TSafeTransferLinesFromTo.SetSourceLayer(const Value: TEntity);
begin
  FSourceLayer := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSafeTransferLinesFromTo);
END.
