unit t_SpaceFilters;

interface
uses
   Elements,
   Tools;

type
// filtrovani vrstvy geom elementu podle vrstvy ploch
TSelectGeometryElementsByAreas = class(TTool)
  private
    FAreas: TEntity;
    FComment: string;
    FStrictlyInside: boolean;
    function GetResultElements: TEntity;
    procedure SetAreas(const Value: TEntity);
    procedure SetComment(const Value: string);
    procedure SetStrictlyInside(const Value: boolean);
public
   function  NumberToPropertyName(aNumber: longInt): string;override;
   procedure DoIt; override;
   property  ResultElements: TEntity read GetResultElements;
published
   property Areas: TEntity read FAreas write SetAreas;
   property Comment: string read FComment write SetComment;
   property StrictlyInside: boolean read FStrictlyInside write SetStrictlyInside;
end;

TSelectGeometryElementsByRect = class(TTool)
  private
    FMaxY: double;
    FMaxX: double;
    FMinX: double;
    FMinY: double;
    FRectElement: TElement;
    FComment: string;
    function GetResultElements: TEntity;
    procedure SetMaxX(const Value: double);
    procedure SetMaxY(const Value: double);
    procedure SetMinX(const Value: double);
    procedure SetMinY(const Value: double);
    procedure SetRectElement(const Value: TElement);
    procedure SetComment(const Value: string);
public
   function  NumberToPropertyName(aNumber: longInt): string;override;
   procedure DoIt; override;
   property  ResultElements: TEntity read GetResultElements;
published
    property Comment: string read FComment write SetComment;
   property RectElement: TElement read FRectElement write SetRectElement;
   property MinX: double read FMinX write SetMinX;
   property MinY: double read FMinY write SetMinY;
   property MaxX: double read FMaxX write SetMaxX;
   property MaxY: double read FMaxY write SetMaxY;
end;

implementation
uses
   Dialogs,
   FOrms,
   e_Class,
   G_Elems,
   e_Geometry,
   e_GeometryElement,
   e_QTree,
   t_SpatialAnalysis,
   t_QTreeTools;

{ TSelectGeometryElementsByAreas }

procedure TSelectGeometryElementsByAreas.DoIt;
var
   QTree: TRectEntityQTree;
   TQ: TAddEntityToQTree;
   QTW: TQTreeWalker;
   SA: TSpatialAnalysisTool;
   IE: TEntity;
   i,j,k: integer;
   e: TElement;
   GE,FGE: TGeometryElement;
   G: TGeometry;
   FA: TAreaElement;
   NearAreas: TQuickEntity;
   A: TAreaElement;
   MA: TMultiAreaElement;
   Lin: TPolyLineElement;
   Pnt: TPointElement;
   Txt: TTextElement;
   Found: boolean;
begin
   if Input = nil then
   begin
      ShowMessage(Comment+' Chyb Input!');
      exit;
   end;
   if not (Input is TEntity) then
   begin
      ShowMessage(Comment+' Input nen entita!');
      exit;
   end;
   IE := TEntity(Input);
   if IE.NoElements then exit;

   if Areas = nil then
   begin
      ShowMessage(Comment+'Chyb Plochy pro filtrovn!');
      exit;
   end;
   if Areas.NoElements then exit;
   if Comment = '' then
      Comment := 'Filtrovn '+ Input.Name + ' podle '+ Areas.Name;
   StartProgress(Comment,0,IE.LastIndex);
   ProgressText := 'Pprava...';
   Application.ProcessMessages;

   QTree :=TRectEntityQTree.Create;
   QTree.Name := 'Filter QT';
   AddElement(QTree);
   TQ := TAddEntityToQTree.Create;
   TQ.NoProgress := NoProgress;
   TQ.Input := Areas;
   TQ.Output := QTree;
   TQ.DoIt;
   TQ.Free;
   QTW := TQTreeWalker.Create;
   SA := TSpatialAnalysisTool.Create;
   NearAreas := TQuickEntity.Create;
   ProgressText := 'Filtrovn...';
   Application.ProcessMessages;
   for i := 0 to IE.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := IE.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      if GE.NoGeometry then continue;
      G := GE.Geometry;
      NearAreas.Clear;
      e := QTW.GetFirstElementForRectangle(QTree,G.Minx,G.MinY,G.MaxX,G.MaxY,0);
      while e <> nil do
      begin
         if NearAreas.IndexOf(e) < 0 then
         begin
            NearAreas.AddElement(e);
         end;
         e := QTW.GetNextElement;
      end;
      Found := false;
      if NearAreas.NoElements then continue; // uplne mimo
      for j := 0 to NearAreas.LastIndex do
      begin
         e := NearAreas.Elements[j];
         if not (e is TGeometryElement) then continue;
         FGE := TGeometryElement(e);
         if FGE.NoGeometry then continue;
         if not (FGE.Geometry is TAreaElement) then continue;
         FA := TAreaElement(FGE.Geometry);
         if (G is TAreaElement) then
         begin
            A := TAreaElement(G);
            if StrictlyInside then
            begin
               if A is TMultiAreaElement then
               begin
                  MA := TMultiAreaElement(A);
                  for k := 0 to MA.LastIndex do
                  begin
                     A := MA.AreaElement[k];
                     if FA.IsHitByXY(A.X,A.Y,0) then
                     begin
                        Found := true;
                        Break;
                     end;
                  end;
               end
               else
                  Found := FA.IsHitByXY(A.X,A.Y,0);
            end
            else
               Found := SA.Relate_Area_Area(FA, A);
         end
         else
         if (G is TPolyLineElement) then
         begin
            Lin := TPolyLineElement(G);
            if StrictlyInside then
            begin
               if FA.IsHitByXY(Lin.X,Lin.Y,0) then
                  Found := true;
            end
            else
            if SA.Relate_Line_Area(Lin,FA) then
            begin
               {
               if StrictlyInside then
               begin
                  if SA.Inside then
                     Found := true;
               end
               else
               }
                  Found := true;
            end;
         end
         else
         if (G is TPointElement) then
         begin
            Pnt := TPointElement(G);
            if FA.IsHitByXY(Pnt.X,Pnt.Y,0) then
               Found := true;
         end
         else
         if (G is TTextElement) then
         begin
            Txt := TTextElement(G);
            if FA.IsHitByXY(Txt.X,Txt.Y,0) then
               Found := true;
         end;
         if Found then
         begin
            ResultElements.AddElement(GE);
            Break;
         end;
      end;
   end;
   if ResultElements is TRectEntity then
      TRectEntity(ResultElements).ResetAndUpdate;
   NearAreas.Clear;
   NearAreas.Free;
   QTW.Free;
   SA.Free;
end;

function TSelectGeometryElementsByAreas.GetResultElements: TEntity;
begin
   if Output = nil then
   begin
      Output := TOwnerEntity.Create;
   end;
   result := TEntity(Output);
end;

function TSelectGeometryElementsByAreas.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Areas';
      1: result := 'Output';
      2: result := 'StrictlyInside';
   else
      result := '';
   end;
end;

procedure TSelectGeometryElementsByAreas.SetAreas(const Value: TEntity);
begin
  FAreas := Value;
end;

procedure TSelectGeometryElementsByAreas.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TSelectGeometryElementsByAreas.SetStrictlyInside(
  const Value: boolean);
begin
  FStrictlyInside := Value;
end;

{ TSelectGeometryElementsByRect }

procedure TSelectGeometryElementsByRect.DoIt;
var
   IE: TEntity;
   RE: TRectEntity;
   e: TElement;
   i: integer;
begin
   if Input = nil then
   begin
      ShowMessage(Comment+' Chyb Input!');
      exit;
   end;
   if not (Input is TEntity) then
   begin
      ShowMessage(Comment+' Input nen entita!');
      exit;
   end;
   IE := TEntity(Input);
   if IE.NoElements then exit;
   if RectElement <> nil then
   begin
      if not (RectElement is TRectEntity) then exit;
      RE := TRectEntity(RectElement);
      MinX := RE.MinX;
      MinY := RE.MinY;
      MaxX := RE.MaxX;
      MaxY := RE.MaxY;
   end;
   if (MinX = 0) and (MinY = 0) and (MaxX = 0) and (MaxY = 0) then exit;
   for i := 0 to IE.LastIndex do
   begin
      e := IE.Elements[i];
      if not (e is TRectEntity) then
         continue;
      RE := TRectEntity(e);
      if RE.IsOverRegion(MInX,MinY,MaxX,MaxY) then
         ResultElements.AddElement(RE);
   end;
   if (ResultElements is TRectEntity) then
      TRectEntity(ResultElements).Update;
end;

function TSelectGeometryElementsByRect.GetResultElements: TEntity;
begin
   if Output = nil then
   begin
      Output := TOwnerEntity.Create;
   end;
   result := TEntity(Output);
end;

function TSelectGeometryElementsByRect.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'RectElement';
      1: if RectElement <> nil then
            result := 'Output'
         else
            result := 'MinX';
      2: result := 'MinY';
      3: result := 'MaxX';
      4: result := 'MaxY';
      5: result := 'Output';
   end;
end;

procedure TSelectGeometryElementsByRect.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TSelectGeometryElementsByRect.SetMaxX(const Value: double);
begin
  FMaxX := Value;
end;

procedure TSelectGeometryElementsByRect.SetMaxY(const Value: double);
begin
  FMaxY := Value;
end;

procedure TSelectGeometryElementsByRect.SetMinX(const Value: double);
begin
  FMinX := Value;
end;

procedure TSelectGeometryElementsByRect.SetMinY(const Value: double);
begin
  FMinY := Value;
end;

procedure TSelectGeometryElementsByRect.SetRectElement(const Value: TElement);
begin
  FRectElement := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSelectGeometryElementsByAreas);
   RegisterOrbisClass(TSelectGeometryElementsByRect);
END.

