unit E_iscrpt;
{
[Unit_Description]
Description=Interpretan pedpis
Comment=Pedlat na potomka InheritanceModel
Admin=Petr
TechTopic=elements
Topic=Zobrazen dat
Version=1.0
}

interface
uses
   // KERNEL
   Elements,
   // DEVICE
   e_Device,
   // INTERPRET
   i_Basic;

   { consts }
const

   err_NoItem              = 1;
   err_ClassNotFound       = 2;
   err_WrongItem           = 3;
   err_WrongElement        = 4;
   err_NoInterpret         = 5;
   err_NoInstance          = 6;
   err_NoClass             = 7;
   err_PassIndexOutOfRange = 8;


type


TISInstance = class(TOwnerEntity)
 private
    FInterpret: TInterpret;
    FElement: TElement;
    procedure SetIElement(aValue: TElement);
 protected
    function  GetInterpret: TInterpret; virtual;
    procedure SetInterpret(aInterpret: TInterpret); virtual;
 public
    function  ItsMyElement(aElement: TElement): boolean; virtual;
    function  ItsMyClass(aValue: TElementClass): boolean; virtual;
    function  InterpretForElement(aElement: TElement): TInterpret; virtual;
    procedure Reset(aDevice: TDevice); virtual;
    procedure Close; virtual;
    procedure SetCombineName; virtual;
 published
    property  Interpret: TInterpret read GetInterpret write SetInterpret;
    property  Element: TElement read FElement write SetIElement;
end;

TISClass = class;
TISInstances = class(TOwnerEntity);


TISClass = class(TISInstance)
 private
    FInstances: TISInstances;
    FClass: TElementClass;
 protected
    procedure SetIClass(aValue: TElementClass);
    function  GetInstance(Index: longInt): TISInstance;
    function  GetInstances: TISInstances; virtual;
    function  GetNoInstance: boolean;
    procedure PutInstance(Index: longInt; aInstance: TISInstance);
    //procedure SetInstances(aInstances: TISInstances);
    procedure SetNextInstance(aValue: TISInstance); virtual;
    procedure SetNextClass(aValue: TISClass); virtual;
    procedure SetNextInstClass(aValue: TISInstance); virtual;
    function  GetClassName: string;
    procedure SetClassName(aValue: string);
    function  GetItemForClass(aValue: TElementClass): TISClass;
    function  GetItemForElement(aValue: TElement): TISInstance;
    function  GetItemForInterpretName(const aValue: string): TISInstance;
 public
    procedure Smaz;
    procedure SetCombineName; override;
    function  ItsMyElement(aElement: TElement): boolean; override;
    function  ItsMyClass(aValue: TElementClass): boolean; override;
    function  InterpretForElement(aElement: TElement): TInterpret; override;
    procedure Reset(aDevice: TDevice); override;
    procedure Close; override;
    procedure AddInstance(aInstance: TISInstance);
    function  CreateAndAddInstance(aElement: TElement; aInterpret: TInterpret): TISInstance;
    function  CreateAndAddClass(aClass: TElementClass; aInterpret: TInterpret): TISClass;

    property  Instances: TISInstances read GetInstances {write SetInstances};
    property  NoInstance: boolean read GetNoInstance;
    property  Instance[Index: longint]: TISInstance read GetInstance write PutInstance;
    property  ElementClass: TElementClass read FClass write SetIClass;
published
    property  NextInstance: TISInstance write SetNextInstance;
    property  NextClass: TISInstance write SetNextInstClass;
    property  ElementClassName: string read GetClassName write SetClassName;
end;

TInterpretScript = class(TISClass)
private
   FError: longInt;
   FActualItem: TISInstance;
   FPass: integer;
    FNoRect: boolean;
   function GetActualElement: TElement;
   function GetActualClassName: string;
   function GetActualInterpret: TInterpret;
   function  GetAIName: string;
   function  GetIsError: boolean;
   procedure SetAIName(const aValue: string);
   procedure SetNewElement(aValue: TElement);
   procedure SetNewClassName(const aValue: string);
   procedure SetActualElement(aValue: TElement);
   procedure SetActualClassName(const aValue: string);
   procedure SetActualIntrepret(aValue: TInterpret);
   procedure SetFindElement(aValue: TElement);
   procedure SetFindClassName(const aValue: string);
   procedure SetFindIntepretName(const aValue: string);
   procedure SetDeleteAI(aValue: boolean);
   function GetPassesCount: integer;
   procedure SetPass(const Value: integer);
   procedure SetPassesCount(const Value: integer);
    procedure SetNewClass(const Value: TElementClass);
    procedure SetNoRect(const Value: boolean);
protected
   function  GetInstances: TISInstances; override;
public
   constructor Create; override;
   procedure   SetCombineName; override;
   procedure   SetNewClassAndInterpret(aClass: TElementClass; aInterpret: TInterpret);
   property    NewClass: TElementClass write SetNewClass;
published
   property PassesCount: integer read GetPassesCount write SetPassesCount;
   property Pass: integer read FPass write SetPass;
   property ActualItem: TISInstance     read FActualItem                                  stored false;
   property NewElement: TElement                                write SetNewElement       stored false;
   property NewClassName: string                                write SetNewClassName     stored false;
   property FindElement: TElement                               write SetFindElement      stored false;
   property FindClassName: string                               write SetFindClassName    stored false;
   property FindInterpretName: string                           write SetFindIntepretName stored false;
   property ActualElement: TElement     read GetActualElement   write SetActualElement    stored false;
   property ActualClassName: string     read GetActualClassName write SetActualClassName  stored false;
   property ActualInterpret: TInterpret read GetActualInterpret write SetActualIntrepret  stored false;
   property ActualInterpretName: string read GetAIName          write SetAIName           stored false;
   property DeleteActualItem: boolean                           write SetDeleteAI         stored false;
   property IsError: boolean            read GetIsError                                   stored false;
   property Error: longInt              read FError             write FError              stored false;
   property NoRect: boolean read FNoRect write SetNoRect;
end;


implementation
uses
  Controls,
  Dialogs,
  Forms,
  SysUtils,
  WinProcs,
  {!t_Main,}
  // KERNEL
  e_Class;



{ TISInstance }


procedure TISInstance.SetIElement(aValue: TElement);
begin
   FElement := aValue;
   SetCombineName;
end;


function  TISInstance.GetInterpret: TInterpret;
begin
   result := FInterpret;
end;

procedure TISInstance.SetInterpret(aInterpret: TInterpret);
begin
   if assigned(FInterpret)then
   begin
      FreeElement(FInterpret);
      FInterpret := nil;
   end;
   FInterpret := aInterpret;
   AddElement(FInterpret);
   SetCombineName;
end;

function  TISInstance.ItsMyElement(aElement: TElement): boolean;
begin
   result := aElement = FElement;
end;

function  TISInstance.ItsMyClass(aValue: TElementClass): boolean;
begin
   result := aValue = FElement.ClassType;
end;

function  TISInstance.InterpretForElement(aElement: TElement): TInterpret;
begin
   if aElement = FElement then result := Interpret
                          else result := nil;
end;

procedure TISInstance.Reset(aDevice: TDevice);
begin
   if Interpret = nil then exit;
      Interpret.Reset(aDevice);
end;


procedure TISInstance.Close;
begin
   if Interpret = nil then exit;
      Interpret.Close;
end;

procedure TISInstance.SetCombineName;
var
   S: string;
begin
   S := '';
   if Element <> nil then
   try
      S := Element.Name+'('+Element.ClassName+'):';
   except
      S := '';
   end;
   if Interpret <> nil then
   try
      S := S + Interpret.Name;
   except
   end;
   if S <> '' then Name := S;
end;


{ TISClass }


procedure TISClass.SetIClass(aValue: TElementClass);
begin
   FClass := aValue;
   SetCombineName;
end;


function  TISClass.GetClassName: string;
begin
   result := ElementClass.ClassName;
end;


procedure TISClass.SetClassName(aValue: string);
begin
   ElementClass := GetOrbisClass(aValue);
end;



function  TISClass.GetInstance(Index: longInt): TISInstance;
begin
   result := TISInstance(Instances.Elements[Index]);
end;

function  TISClass.GetInstances: TISInstances;
begin
   if not assigned(FInstances) then
   begin
      FInstances := TISInstances.Create;
      AddElement(FInstances);
   end;
   result := FInstances;
end;


function  TISClass.GetNoInstance: boolean;
begin
   result := true;
   if not assigned(FInstances) then exit
                               else result := FInstances.Count = 0;
end;


procedure TISClass.PutInstance(Index: longInt; aInstance: TISInstance);
begin
   Instances.Elements[Index] := aInstance;
end;

{
procedure TISClass.SetInstances(aInstances: TISInstances);
begin
   if assigned(FInstances) then
   begin
      FreeElement(FInstances);
      FInstances := nil;
   end;
   FInstances := aInstances;
end;
}

function  TISClass.ItsMyElement(aElement: TElement): boolean;
begin
   result :=  aElement is FClass{ElementClass;}
end;


function  TISClass.ItsMyClass(aValue: TElementClass): boolean;
begin
   result :=  aValue.InheritsFrom(FClass){ElementClass;}
end;


procedure  TISClass.Smaz;
begin
  MessageBeep(0)
end;


function  TISClass.InterpretForElement(aElement: TElement): TInterpret;
var
   i : longInt;
   ElementInstance: TISInstance;
begin
   result := nil;
   { Zrevidovat !!!!!!!!!!!!}
   if aElement is ElementClass then {tato kontrola je zbyten .... asi }
   begin
      result := Interpret;
      if (aElement.ClassType = ElementClass) and NoInstance then exit;
      //if NoInstance then exit;
      for i := 0 to Instances.Count - 1 do
      begin
         ElementInstance := Instance[i];
         if ElementInstance.ItsMyElement(aElement) then
         begin
            result := ElementInstance.InterpretForElement(aElement);
            exit;
         end;
      end;
   end;
end;

procedure TISClass.Reset(aDevice: TDevice);
var
   i: longInt;
begin
   inherited Reset(aDevice);
   if NoInstance then exit;
   for i := 0 to Instances.Count - 1 do
   begin
   try
     if Instance[i] <> nil then
        Instance[i].Reset(aDevice);
   except
     on E:Exception do
     begin
        ShowMessage(E.Message);
        Screen.Cursor:= crDefault;
     end
   end
   end;
end;


procedure TISClass.Close;
var
   i: longInt;
begin
   inherited Close;
   if NoInstance then exit;
   for i := 0 to Instances.Count - 1 do
   begin
      Instance[i].Close;
   end;
end;



procedure  TISClass.AddInstance(aInstance: TISInstance);
begin
   Instances.AddElement(aInstance);
end;


function TISClass.CreateAndAddInstance(aElement: TElement; aInterpret: TInterpret): TISInstance;
begin
   result := TISInstance.Create;
   result.Element := aElement;
   result.Interpret := aInterpret;
   result.AddElement(aInterpret);
   NextInstance := result;
end;


function TISClass.CreateAndAddClass(aClass: TElementClass; aInterpret: TInterpret): TISClass;
begin
   result := TISClass.Create;
   result.ElementClass := aClass;
   result.Interpret := aInterpret;
   result.AddElement(aInterpret);
   NextClass := result;
end;

procedure TISClass.SetNextInstance(aValue: TISInstance);
var
   i: longInt;
   ISC: TISClass;
   IsClass: boolean;
begin
   for i := Instances.Count - 1 downto 0  do
   begin
      IsClass := Instance[i] is TISClass;
      if ISClass then
      begin
         ISC := TISClass(Instance[i]);
         if aValue.Element is ISC.ElementClass then
         begin
            ISC.SetNextInstance(aValue);
         end;
      end;
   end;
   if aValue.NoOwners then AddInstance(aValue);
end;



procedure TISClass.SetNextClass(aValue: TISClass);
var
   i: longInt;
   ISC: TISClass;
   ISI: TISInstance;
   IsClass: boolean;
begin
   for i := Instances.Count - 1 downto 0  do
   begin
      IsClass := Instance[i] is TISClass;
      if ISClass then
      begin
         ISC := TISClass(Instance[i]);
         if ISC.ElementClass.InheritsFrom(aValue.ElementClass) then
         begin
            Instances.RemoveElement(ISC);
            aValue.AddInstance(ISC);
         end
         else
         if aValue.ElementClass.InheritsFrom(ISC.ElementClass) then
         begin
            ISC.SetNextClass(aValue);
         end;
      end
      else
      begin
         ISI := TISInstance(Instance[i]);
         if ISI.Element is aValue.ElementClass then
         begin
            Instances.RemoveElement(ISI);
            aValue.AddInstance(ISI);
         end                  ;
      end;
   end;
   if aValue.NoOwners then AddInstance(aValue);
end;

procedure TISClass.SetNextInstClass(aValue: TISInstance);
begin
   SetNextClass(TISClass(aValue));
end;

procedure TISClass.SetCombineName;
var
   S: string;
begin
   S := '';
   if ElementClass <> nil then
   try
      S := '('+ElementClass.ClassName+'):';
   except
      S := '';
   end;
   if Interpret <> nil then
   try
      S := S + Interpret.Name;
   except
   end;
   if S <> '' then Name := S;
end;

function  TISClass.GetItemForClass(aValue: TElementClass): TISClass;
var
   i : longInt;
   ElementInstance: TISInstance;
begin
   result := Self;
   if aValue = ElementClass then exit;
   if NoInstance then exit;
   for i := 0 to Instances.Count - 1 do
   begin
      ElementInstance := Instance[i];
      if not (ElementInstance.ClassType = TISClass) then continue;
      if ElementInstance.ItsMyClass(aValue) then
      begin
         result := TISClass(ElementInstance).GetItemForClass(aValue);
         exit;
      end;
   end;
end;

function  TISClass.GetItemForElement(aValue: TElement): TISInstance;
var
   i : longInt;
   ElementInstance: TISInstance;
begin
   result := nil;
   if NoInstance then exit;
   for i := 0 to Instances.Count - 1 do
   begin
      ElementInstance := Instance[i];
      if ElementInstance.ClassType = TISClass then
      begin
         if ElementInstance.ItsMyElement(aValue) then
         begin
            result := TISClass(ElementInstance).GetItemForElement(aValue);
         end;
      end
      else
      if ElementInstance.ItsMyElement(aValue) then
      begin
         result := ElementInstance;
         exit;
      end;
   end;
end;

function  TISClass.GetItemForInterpretName(const aValue: string): TISInstance;
var
   i : longInt;
   ElementInstance: TISInstance;
   S: string;
begin
   result := nil;
   if NoInstance then exit;
   for i := 0 to Instances.Count - 1 do
   begin
      ElementInstance := Instance[i];

      if ElementInstance.Interpret <> nil then
      begin
         S := ElementInstance.Interpret.Name;
         if AnsiUpperCase(S) = AnsiUpperCase(aValue) then
         begin
            result := ElementInstance;
            exit;
         end;
      end;
      if ElementInstance.ClassType = TISClass then
      begin
         result := TISClass(ElementInstance).GetItemForInterpretName(aValue);
         if result <> nil then exit;
      end;
   end;
end;


{ TInterpretScript }

constructor TInterpretScript.Create;
begin
   inherited Create;
   ElementClass := TElement;
   FActualItem := Self;
end;


procedure TInterpretScript.SetCombineName;
begin
   Name := 'Interpret Script'
end;

function TInterpretScript.GetActualElement: TElement;
begin
   if ActualItem = Self then Error := err_WrongItem;
   result := ActualItem.Element;
end;

function TInterpretScript.GetActualClassName: string;
begin
   if ActualItem = Self then Error := err_WrongItem;
   if ActualItem is TISClass then result := TISClass(ActualItem).ElementClass.ClassName
                             else result := 'NIL';
end;

function TInterpretScript.GetActualInterpret: TInterpret;
begin
   if ActualItem = Self then Error := err_WrongItem;
   result := ActualItem.Interpret;
end;



procedure TInterpretScript.SetNewElement(aValue: TElement);
begin
   if aValue = nil then exit;
   FActualItem := TISInstance.Create;
   FActualItem.Element := aValue;
   NextInstance := FActualItem;
end;

procedure TInterpretScript.SetNewClassName(const aValue: string);
var
   c: TElementClass;
begin
   Error := 0;
   if aValue = '' then exit;
   c := GetOrbisClass(aValue);
   if c = nil then
   begin
      Error := err_ClassNotFound;
      exit;
   end;
   FActualItem := TISClass.Create;
   TISClass(FActualItem).ElementClass := c;
   NextClass := FActualItem;
end;

procedure TInterpretScript.SetActualElement(aValue: TElement);
begin
   Error := 0;
   if aValue = nil then
   begin
      Error := err_WrongElement;
      exit;
   end;
   if (ActualItem = Self) or (ActualItem.ClassType <> TISINstance) then
   begin
      Error := err_WrongItem;
      exit;
   end;
   ActualItem.Element := aValue;
   Instances.RemoveElement(ActualItem);
   NextInstance := ActualItem;
end;

procedure TInterpretScript.SetActualClassName(const aValue: string);
var
   ec: TElementClass;
begin
   Error := 0;
   if (ActualItem = Self) or (ActualItem.ClassType <> TISClass) then
   begin
      Error := err_WrongItem;
      exit;
   end;
   ec := GetOrbisClass(aValue);
   if ec = nil then
   begin
      Error := err_ClassNotFound;
      exit;
   end;
   TISClass(ActualItem).ElementClass := ec;
   Instances.RemoveElement(ActualItem);
   NextClass := ActualItem;
end;

procedure TInterpretScript.SetActualIntrepret(aValue: TInterpret);
begin
   Error := 0;
   if ActualItem = Self then
   begin
      Error := err_WrongItem;
      exit;
   end;
   ActualItem.Interpret := aValue;
   if aValue <> nil then
      aValue.NoRect := NoRect;
end;

function  TInterpretScript.GetAIName: string;
begin
   Error := 0;
   result := '';
   if ActualItem = Self then
   begin
      Error := err_WrongItem;
      exit;
   end;
   if ActualItem.Interpret = nil then
   begin
      Error := err_NoInterpret;
      exit;
   end;
   result := ActualItem.Interpret.Name;
end;

function  TInterpretScript.GetIsError: boolean;
begin
   result := Error > 0;
end;

procedure TInterpretScript.SetAIName(const aValue: string);
var
   ec: TElementClass;
   i: TInterpret;
begin
   Error := 0;
   if ActualItem = Self then
   begin
      Error := err_WrongItem;
      exit;
   end;
   ec := GetOrbisClass(aValue);
   if ec = nil then
   begin
      Error := err_ClassNotFound;
      i := nil;
   end
   else
      i := TInterpret(ec.Create);
   ActualItem.Interpret := i;
   if i <> nil then
      i.NoRect := NoRect;
end;



procedure TInterpretScript.SetFindElement(aValue: TElement);
var
   e: TISInstance;
begin
   Error := 0;
   e := GetItemForElement(aValue);
   if e = nil then
   begin
      Error := err_NoItem;
      FActualItem := Self;
   end
   else
   begin
      FActualItem := e;
   end;
end;



procedure TInterpretScript.SetFindClassName(const aValue: string);
var
   e: TISClass;
   ec: TElementClass;
begin
   Error := 0;
   ec := GetOrbisClass(aValue);
   if ec = nil then
   begin
      Error := err_ClassNotFound;
      FActualItem := Self;
      exit;
   end;
   e := GetItemForClass(ec);
   if (e = nil) or (e = Self) then
   begin
      Error := err_NoItem;
      FActualItem := Self;
   end
   else
   begin
      FActualItem := e;
   end;
end;



procedure TInterpretScript.SetFindIntepretName(const aValue: string);
var
   e: TISInstance;
begin
   Error := 0;
   e := GetItemForInterpretName(aValue);
   if e = nil then
   begin
      Error := err_NoItem;
      FActualItem := Self;
   end
   else
   begin
      FActualItem := e;
   end;
end;


procedure TInterpretScript.SetDeleteAI(aValue: boolean);
begin
   if ActualItem = Self then exit;
   ActualItem.Free;
   FActualItem := Self;
   Error := err_WrongItem;
end;



function TInterpretScript.GetPassesCount: integer;
begin
   result := Count;
end;

procedure TInterpretScript.SetPass(const Value: integer);
var
   i: integer;
begin
   Error := 0;
   i := Value;
   if PassesCount = 0 then PassesCount := 1;
   if (i < 0) or (i > LastIndex) then
   begin
      Error := err_PassIndexOutOfRange;
      i := 0
   end;
   FInstances := TISInstances(Elements[i]);
   FPass := i;
end;

procedure TInterpretScript.SetPassesCount(const Value: integer);
var
   i, di: integer;
   Inst: TISInstances;
begin
   i := Count;
   if Value < 0 then exit;
   if i = Value then exit;
   di := Abs(Value - i);
   if Value > i then
   begin
      for i := 1 to di do
      begin
         Inst := TISInstances.Create;
         Inst.Name := 'Pass ' + intToStr(LastIndex + i);
         AddElement(Inst);
      end;
   end
   else
   begin
      di := i - di;
      for i := LastIndex Downto di do
      begin
         FreeElement(Elements[i]);
      end;
   end;
end;

function TInterpretScript.GetInstances: TISInstances;
begin
   if PassesCount = 0 then Pass := 0;
   result := FInstances;
end;

procedure TInterpretScript.SetNewClass(const Value: TElementClass);
begin
   Error := 0;
   if Value = nil then
   begin
      Error := err_ClassNotFound;
      exit;
   end;
   FActualItem := TISClass.Create;
   TISClass(FActualItem).ElementClass := Value;
   NextClass := FActualItem;
end;

procedure TInterpretScript.SetNewClassAndInterpret(aClass: TElementClass;
  aInterpret: TInterpret);
begin
   NewClass := aClass;
   ActualInterpret := aInterpret;
   if aInterpret <> nil then
      aInterpret.NoRect := NoRect;
end;

procedure TInterpretScript.SetNoRect(const Value: boolean);
begin
  FNoRect := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TISInstance);
   RegisterOrbisClass(TISClass);
   RegisterOrbisClass(TInterpretScript);
END.
