unit e_InterpretContainer;

interface
uses
// KERNEL
   Elements,
   Tools,

// GLIB
   G_Elems,
// INTERPRET
   I_Basic;

type

TInterpretLayer = class(TRectEntity)
private
  FPath: string;
  FInterpret: TInterpret;
  FSwitchedOff: Boolean;
  FLayerClass: TElementClass;
  FMinScaleNumber: Double;
  FMaxScaleNumber: Double;
  FLayerType: string;
  FAsText: Boolean;
    FUseColor: boolean;
    FUseSymbol: boolean;
    FColor: Integer;
    FSymbol: integer;
    FTransparency: integer;
    FVirtualScale: double;
    FBackVirtualScale: double;
  function  GetInterpret: TInterpret;
  procedure SetLayClassName(const Value: string);
  function GetInterpretName: string;
  function GetLayClassName: string;
  procedure SetInterpretName(const Value: string);
    procedure SetColor(const Value: Integer);
    procedure SetSymbol(const Value: integer);
    procedure SetUseColor(const Value: boolean);
    procedure SetUseSymbol(const Value: boolean);
    procedure SetTransparency(const Value: integer);
public
  destructor Destroy; override;
  procedure SetLayerInterpretForType;
  procedure SetInterpret(const Value: TInterpret); virtual;
  procedure SetAsText(const Value: Boolean); virtual;
published
  property LayerClassName: string read GetLayClassName write SetLayClassName;
  property LayerType: string read FLayerType write FLayerType;
  property Path: string read FPath write FPath;
  property Color: Integer read FColor write SetColor;
  property UseColor: boolean read FUseColor write SetUseColor;
  property Symbol: integer read FSymbol write SetSymbol;
  property UseSymbol: boolean read FUseSymbol write SetUseSymbol;
  property SwitchedOff: Boolean read FSwitchedOff write FSwitchedOff;
  property Interpret: TInterpret read GetInterpret write SetInterpret stored false;
  property InterpretName: string read GetInterpretName write SetInterpretName;
  property LayerClass: TElementClass read FLayerClass write FLayerClass stored false;
  property MinScaleNumber: Double read FMinScaleNumber write FMinScaleNumber;
  property MaxScaleNumber: Double read FMaxScaleNumber write FMaxScaleNumber;
  property VirtualScale: double read FVirtualScale write FVirtualScale;
  property BackVirtualScale: double read FBackVirtualScale write FBackVirtualScale;
  property AsText: Boolean read FAsText write SetAsText;
  property Transparency: integer read FTransparency write SetTransparency;
end;

TInterpretContainer = class(TRectEntity)
public
  procedure  AddElement(aElement: TElement); override;
published
end;

implementation

uses
  SysUtils,
  e_Class;


{ TInterpretLayer }

destructor TInterpretLayer.Destroy;
begin
  if Interpret <> nil then
    Interpret.Free;
  inherited;
end;

function TInterpretLayer.GetInterpret: TInterpret;
begin
  result:= FInterpret;
end;

function TInterpretLayer.GetInterpretName: string;
begin
  result := '';
  if Interpret <> nil then
     result := Interpret.ClassName;
end;

function TInterpretLayer.GetLayClassName: string;
begin
  result := '';
  if LayerClass <> nil then result := LayerClass.ClassName;
end;

procedure TInterpretLayer.SetAsText(const Value: Boolean);
begin
   FAsText := Value;
end;

procedure TInterpretLayer.SetColor(const Value: Integer);
begin
   FColor := Value;
   if Interpret <> nil then
   begin
      Interpret.SetPropertyLongIntByName('COLOR',Color);
   end;
end;

procedure TInterpretLayer.SetInterpret(const Value: TInterpret);
begin
  if FInterpret <> nil then
  begin
    FInterpret.Free;
    FInterpret := nil;
  end;
  FInterpret := Value;
  if FInterpret <> nil then
  begin
     FInterpret.SetPropertyLongIntByName('COLOR',Color);
     FInterpret.SetPropertyLongIntByName('SYMBOL',Symbol);
     FInterpret.SetPropertyBooleanByName('USECOLOR',UseColor);
     FInterpret.SetPropertyBooleanByName('USESYMBOL',UseSymbol);
  end;
end;

procedure TInterpretLayer.SetInterpretName(const Value: string);
var
  IC: TElementClass;
begin
  if (Interpret = nil) or (CompareText(Interpret.Name,Value)<> 0) then
  begin
    IC := GetOrbisClass(Value);
    if IC <> nil then
       Interpret := TInterpret(IC.Create)
    else
       Interpret := nil;
  end;
end;

procedure TInterpretLayer.SetLayClassName(const Value: string);
begin
  FLayerClass:= GetOrbisClass(Value);
end;

procedure TInterpretLayer.SetLayerInterpretForType;
begin

end;


procedure TInterpretLayer.SetSymbol(const Value: integer);
begin
   FSymbol := Value;
   if Interpret <> nil then
      Interpret.SetPropertyLongIntByName('SYMBOL',Symbol);
end;

procedure TInterpretLayer.SetTransparency(const Value: integer);
begin
  FTransparency := Value;
end;

procedure TInterpretLayer.SetUseColor(const Value: boolean);
begin
   FUseColor := Value;
   if Interpret <> nil then
      Interpret.SetPropertyBooleanByName('USECOLOR',UseColor);
end;

procedure TInterpretLayer.SetUseSymbol(const Value: boolean);
begin
   FUseSymbol := Value;
   if Interpret <> nil then
      Interpret.SetPropertyBooleanByName('USESYMBOL',UseSymbol);
end;

{ TInterpretContainer }

procedure TInterpretContainer.AddElement(aElement: TElement);
begin
  inherited;
  if not (aElement is TRectEntity) then
     exit;
  UpdateRect(aElement);
end;

INITIALIZATION
  RegisterOrbisClass(TInterpretLayer);
  RegisterOrbisClass(TInterpretContainer);
END.

