{$D+}
unit E_IniFile;

interface
uses
  IniFiles, Classes,
// KERNEL
  Elements,
  E_Type;

type

TIniFileElement = class(TElement)
private
  FFileName: string;
  FSection: string;
  FIdent: string;
  FDefaultValue: string;
  FPreSetStringListElement: TStringListElement;
  procedure SetFileName(aValue: string);
  function GetValue: string;
  procedure SetValue(aValue: string);
  procedure SetSection(aValue: string);

protected
  procedure SetValues(aVal: TStringListElement);
public
  function GetIdents: TStringListElement;
  function GetValues: TStringListElement;
  procedure SetIdentsByStringList(aStringList: TStringList);
  function  GetPropertyStringByName(AName: string): string; override;
  procedure SetPropertyStringByName(AName: string; aValue: string); override;

published
  property FileName: string read FFileName write SetFileName;
  property Section: string read FSection write SetSection;
  property Ident: string read FIdent write FIdent;
  property DefaultValue: string read FDefaultValue write FDefaultValue;
  property Value: string read GetValue write SetValue;
  property IdentsElement: TStringListElement read GetIdents;
  property ValuesElement: TStringListElement read GetValues write SetValues;
  property PreSetStringListElement: TStringListElement read FPreSetStringListElement
                                                       write FPreSetStringListElement;
end;





implementation

uses
  SysUtils,
// KERNEL
  E_Class,
  {!F_Start,}
  U_String;



procedure TIniFileElement.SetFileName(aValue: string);
var
  aPath: string;
begin
  FFileName:= aValue;

  if fileExists(aValue) then
     exit;

 { kdy neexistuje, zejm chci zakldat nov ale pokud m przdnou cestu
   tak mon ho chci st z win adrese }

  aPath:= ExtractFilePath(aValue);
  if aPath <> '' then exit;

  if fileExists(WinDirStr + '\' + aValue) then
     FFileName:= WinDirStr + '\' + aValue;
end;



function TIniFileElement.GetValue: string;
var
  aIF: TIniFile;
begin
  result:= '';
  if not FileExists(FileName) then exit;
  aIF:= TIniFile.Create(FileName);
  result:= aIF.ReadString(Section, Ident, DefaultValue);
  aIF.Free
end;

procedure TIniFileElement.SetValue(aValue: string);
var
  aIF: TIniFile;
begin
  if not FileExists(FileName) then exit;
  aIF:= TIniFile.Create(FileName);
  aIF.WriteString(Section, Ident, aValue);
  aIF.Free
end;


procedure TIniFileElement.SetIdentsByStringList(aStringList: TStringList);
var
  aIF: TIniFile;
begin
  if not FileExists(FileName) then exit;
  aIF:= TIniFile.Create(FileName);
{  aIF.WriteSection(Section, aStringList); }
  aIF.Free;
end;


function TIniFileElement.GetIdents: TStringListElement;
var
  aIF: TIniFile;
  SL: TStringList;
  I: integer;
begin
  result:= nil;
  if not FileExists(FileName) then exit;
  SL:= TStringList.Create;


  aIF:= TIniFile.Create(FileName);
  aIF.ReadSection(Section, SL);

  if SL.Count > 0 then
  begin
    if PreSetStringListElement = nil then
    begin
      { zaloen StringElement a vloen jmen }
      result:= TStringListElement.CreateFromStringList(SL);
    end
    else
    begin
      result:= PreSetStringListElement;
      result.AddStringList(SL)
    end;

    { naplnn obsahu poloky }
    for I:= 0 to result.Count-1 do
    begin
      result.Strings[I]:= aIF.ReadString(Section, result.Names[I], '')
    end;
  end;

  SL.Free;
  aIF.Free;
end;


function TIniFileElement.GetValues: TStringListElement;
var
  aIF: TIniFile;
  SL: TStringList;
  I: integer;
begin
  result:= nil;
  if not FileExists(FileName) then exit;
  SL:= TStringList.Create;

  aIF:= TIniFile.Create(FileName);
  aIF.ReadSection(Section, SL);

  if SL.Count > 0 then
  begin
    for I:= 0 to SL.Count-1 do
    begin
      SL.Strings[I]:= aIF.ReadString(Section, SL.Strings[I], '')
    end;

    if PreSetStringListElement = nil then
       result:= TStringListElement.CreateFromStringList(SL)
    else
    begin
      result:= PreSetStringListElement;
      result.AddStringList(SL)
    end
  end;

  SL.Free;
  aIF.Free;
end;



procedure TIniFileElement.SetValues(aVal: TStringListElement);
var
  aIF: TIniFile;
  I: integer;
begin
  aIF:= TIniFile.Create(FileName);

  if aVal.Count > 0 then
  begin
    for I:= 0 to aVal.Count-1 do
    begin
      aIF.WriteString(Section, aVal.Names[I], aVal.Strings[I]);
    end;
  end;

  aIF.Free;
end;



function TIniFileElement.GetPropertyStringByName(AName: string): string;
begin
  Ident:= aName;
  Result:= Value
end;

procedure TIniFileElement.SetPropertyStringByName(AName, aValue: string);
begin
  Ident:= aName;
  Value:= aValue
end;


procedure TIniFileElement.SetSection(aValue: string);
var
  aIF: TIniFile;
begin
  aIF:= TIniFile.Create(FileName);
  FSection:= aValue;
  if not aIF.SectionExists(FSection) then
     FSection:= '';
  aIF.Free;
end;



BEGIN
   RegisterOrbisClass(TIniFileElement);
end.
