unit t_CreateHRK;

interface
uses
   Elements,
   e_Geometry,
   Tools;

type

TCreateHRK = class(TTool)
  private
    FTolerance: double;
    FComment: string;
    FReferenceClassName: string;
    FLines_HRK: TEntity;
    FLines_Source: TEntity;
    FReferenceClass: TElementClass;
    FLin_Forest: TEntity;
    FLin_Dilec: TEntity;
    FLin_Oddeleni: TEntity;
    FLin_HRKNew: TEntity;
    procedure SetComment(const Value: string);
    procedure SetLines_HRK(const Value: TEntity);
    procedure SetLines_Source(const Value: TEntity);
    procedure SetReferenceClassName(const Value: string);
    procedure SetTolerance(const Value: double);
    procedure SetReferenceClass(const Value: TElementClass);
    procedure SetLin_Dilec(const Value: TEntity);
    procedure SetLin_Forest(const Value: TEntity);
    procedure SetLin_Oddeleni(const Value: TEntity);
    procedure SetLin_HRKNew(const Value: TEntity);
public
   procedure DoIt; override;
   procedure SelectDilec;
   procedure SelectOddeleni;
   procedure SelectForest;
   procedure CreateLinesCopy;
   function  CopyLin(LinesEntity: TEntity): TEntity;
   procedure SetupLines;
   procedure JoinLines(aLines: TEntity);
   procedure Finish;
   procedure FinishAndGoHome;
   function  TestLine(aLine: TPolyLineElement): boolean;
   property  ReferenceClass: TElementClass read FReferenceClass write SetReferenceClass;
   property  Lin_Dilec: TEntity read FLin_Dilec write SetLin_Dilec;
   property  Lin_Oddeleni: TEntity read FLin_Oddeleni write SetLin_Oddeleni;
   property  Lin_Forest: TEntity read FLin_Forest write SetLin_Forest;
   property  Lin_HRKNew: TEntity read FLin_HRKNew write SetLin_HRKNew;
published
   property Lines_HRK: TEntity read FLines_HRK write SetLines_HRK;
   property Lines_Source: TEntity read FLines_Source write SetLines_Source;
   property ReferenceClassName: string read FReferenceClassName write SetReferenceClassName;
   property Tolerance: double read FTolerance write SetTolerance;
   property Comment: string read FComment write SetComment;
end;

implementation
uses
   Forms,
   e_Class,
   e_GeometryElement,
   e_Node,
   e_QTree,
   e_TopolElements,
   t_FindBreakLineConnections,
   t_LHCUtils,
   t_QTreeTools;

{ TCreateHRK }

function TCreateHRK.CopyLin(LinesEntity: TEntity): TEntity;
var
   LC: TPolyLineCollection;
   i,j: integer;
   E: TElement;
   TL,TL1: TTopolLineElement;
   L,L1: TPolyLineElement;
begin
   result := nil;
   if LinesEntity = nil then exit;
   LC := TPolyLineCollection.Create;
   result := LC;
   AddElement(LC);
   MaxProgress := LinesEntity.LastIndex;
   for i := 0 to LinesEntity.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      E := LinesEntity.Elements[i];
      if not (E is TTopolLineElement) then continue;
      TL := TTopolLineElement(E);
      if TL.NoGeometry then continue;
      L := TPolyLineElement(TL.Geometry);
      TL1 := TTopolLineElement(CreateElementCopy(TL));
      L1 := TPolyLineElement(CreateElementCopy(L));
      if L.Xi[0]<L.Xi[L.LastIndex] then
      begin
         for j := 0 to L.LastIndex do
         begin
            L1.AddXY(L.Xi[j],L.Yi[j]);
         end;
      end
      else
      begin
         for j := L.LastIndex DownTo 0 do
         begin
            L1.AddXY(L.Xi[j],L.Yi[j]);
         end;
      end;
      TL1.Geometry := L1;
      TL.AfterCopy(TL1);
      LC.AddAndUpdateRect(TL1);
      {
      if ChangeLinesCopy then
      begin
         TL1.Druh := LineCopyGroup;
         TL1.Barva := LineCopyColor;
         TL1.Znacka := LineCopySymbol;
      end;
      }
   end;
   FreeElement(LinesEntity);
   Progress := 0;
   MaxProgress := 0;
end;

procedure TCreateHRK.CreateLinesCopy;
begin
   Lin_Dilec := CopyLin(Lin_Dilec);
   Lin_Oddeleni := CopyLin(Lin_Oddeleni);
   Lin_Forest := CopyLin(Lin_Forest);
end;

procedure TCreateHRK.DoIt;
begin
   if ReferenceClass = nil then exit;
   if Lines_HRK = nil then exit;
   if Lines_Source = nil then exit;
   StartProgress(Comment,0,0);
   Lin_Dilec := TPolyLineCollection.Create;
   AddElement(Lin_Dilec);
   Lin_Oddeleni := TPolyLineCollection.Create;
   AddElement(Lin_Oddeleni);
   Lin_Forest := TPolyLineCollection.Create;
   AddElement(Lin_Forest);
   Lin_HRKNew := TPolyLineCollection.Create;
   AddElement(Lin_HRKNew);
   ProgressText := 'Hledn hranic dlc';
   Application.ProcessMessages;
   SelectDilec;
   ProgressText := 'Hledn hranic oddlen';
   Application.ProcessMessages;
   SelectOddeleni;
   ProgressText := 'Hledn hranic lesa';
   Application.ProcessMessages;
   SelectForest;
   ProgressText := 'Koprovn lini';
   Application.ProcessMessages;
   CreateLinesCopy;
   ProgressText := 'Reset lini';
   Application.ProcessMessages;
   SetupLines;
   ProgressText := '... skoro fini';
   Application.ProcessMessages;
   Finish;
   ProgressText := 'Jo .. jet spojen lini';
   Application.ProcessMessages;
   JoinLines(Lin_HRKNew);
   JoinLines(Lin_Oddeleni);
   ProgressText := '... a vtzn fini';
   Application.ProcessMessages;
   FinishAndGoHome;
end;

procedure TCreateHRK.Finish;
begin
   Lin_HRKNew.AddAllFrom := Lin_Oddeleni;
   Lin_HRKNew.AddAllFrom := Lin_Dilec;
   Lin_HRKNew.AddAllFrom := Lin_Forest;
end;

procedure TCreateHRK.FinishAndGoHome;
var
   QTFinder: TFindRectEntityInQTreeByXY;
   TQ: TAddEntityToQTree;
   LevelQT: TRectEntityQTree;
   i: integer;
   Ex: TOwnerEntity;
   e: TElement;
   TL: TTopolLineElement;
begin
   TPolyLineCollection(Lin_HRKNew).ResetAndUpdate;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Lin_HRKNew;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   QTFinder := TFindRectEntityInQTreeByXY.Create;
   QTFinder.Input := LevelQT;
   QTFinder.Tolerance := Tolerance;
   Ex := TOwnerEntity.Create;
   MaxProgress := Lines_HRK.LastIndex;
   for i := 0 to Lines_HRK.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      e := Lines_HRK.Elements[i];
      if not (e is TTopolLineElement) then continue;
      TL := TTopolLineElement(e);
      QTFinder.X := TL.X;
      QTFinder.Y := TL.Y;
      QTFinder.DoIt;
      if QTFinder.Output <> nil then
         Ex.AddElement(QTFinder.Output);
   end;
   LevelQT.Free;
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT1';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Lines_HRK;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   QTFinder.Input := LevelQT;
   MaxProgress := Lin_HRKNew.LastIndex;
   for i := 0 to Lin_HRKNew.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      e := Lin_HRKNew.Elements[i];
      if e.IsMyOwner(Ex) then
         continue;
      if not (e is TTopolLineElement) then continue;
      TL := TTopolLineElement(e);
      QTFinder.X := TL.X;
      QTFinder.Y := TL.Y;
      QTFinder.DoIt;
      if QTFinder.Output <> nil then
         Ex.AddElement(TL);
   end;
   MaxProgress := Lin_HRKNew.LastIndex;
   for i := Lin_HRKNew.LastIndex DownTo 0 do
   begin
      Progress := i;
      Application.ProcessMessages;
      e := Lin_HRKNew.Elements[i];
      if not e.IsMyOwner(Ex) then
         Lines_HRK.AddElement(e);
   end;
   if Lines_HRK is TGeometryElementCollection then
      TGeometryElementCollection(Lines_HRK).ResetAndUpdate;
   Ex.Free;
   QTFinder.Free;
   LevelQT.Free;
end;

procedure TCreateHRK.JoinLines(aLines: TEntity);
var
  FB: TFindBreakLineConnections;
  E: TEntity; // entita uzl
  El: TElement;
  N,NN: TNode;
  i,j: integer;
  TL1,TL2,TL: TTopolLineElement;
  L1,L2,L: TPolyLineElement;
  aX,aY,X1,Y1: double;
  B1,B2: boolean;
  NB1,NB2,NB: TNodeBranch;
begin
  FB := TFindBreakLineConnections.Create;
  FB.ConnectionsNumber := 2; //budu hledat uzly dvou lini
  FB.Input := aLines;
  FB.DoIt;
  E := TEntity(FB.output);
  FB.Free;
  MaxProgress := E.LastIndex;
  for i := 0 to E.LastIndex do
  begin
     Progress := i;
     Application.ProcessMessages;
     N := TNode(E.Elements[i]);
     NN := N;
     L1 := nil;
     L2 := nil;
     TL1 := nil;
     TL2 := nil;
     NB1 := nil;
     NB2 := nil;
     for j := 0 to N.LastIndex do
     begin
        El := N.Elements[j];
        if not (El is TNodeBranch) then continue;
        NB := TNodeBranch(El);
        if L1 = nil then
        begin
           NB1 := NB;
           L1 := NB1.Line;
           TL1 := TTopolLineElement(L1.FindClassWithOwnerInOwners(TTopolLineElement,aLines));
        end
        else
        begin
           NB2 := NB;
           L2 := NB2.Line;
           TL2 := TTopolLineElement(L2.FindClassWithOwnerInOwners(TTopolLineElement,aLines));
           Break;
        end;
     end;
     if (L1 = nil) or (L2 = nil) then continue;
     if L1 = L2 then continue;
     if (TL1 = nil) or (TL2 = nil) then continue;
     if (TL1.Druh <> TL2.Druh) or (TL1.Presnost <> TL2.Presnost) then continue;
     if (TL1.Symbol > 0) and (TL2.Symbol > 0) then continue;
     if (TL1.Symbol <= 0) and (TL2.Symbol > 0) then
     begin
        TL := TL2;
        L := L2;
        NB := NB2;
        TL2 := TL1;
        L2 := L1;
        NB2 := NB1;
        TL1 := TL;
        L1 := L;
        NB1 := NB;
     end;
     aX := TL1.AbsRefX;
     aY := TL1.AbsRefY;
     for j := 1 to L2.LastIndex do
     begin
        L2.GetOrientedCoord(j,NB2.LineBegin,X1,Y1);
        if NB1.LineBegin then //bude se pidvat na zatek
           L1.InsertXY(0,X1,Y1)
        else                  //nebo na konec
           L1.AddXY(X1,Y1);
     end;
     L1.RecalculateRect;
     L1.CalculateLabelProps;
     //L1.AbsRefX := aX;
     //L1.AbsRefY := aY;
     NN.FreeAll;
     GiveNodeBranchForLine(L2, not(NB2.LineBegin),N,NB);
     if (N <> nil) and (NB <> nil) then
     begin
        N.RemoveElement(L2);
        N.AddElement(L1);
        NB.Line := L1;
        NB.LineBegin := NB1.LineBegin;
        //NB.Free;
     end;
     //if NB2 <> nil then NB2.Free;
     TL2.Free;
  end;
  E.Free;
  Progress := 0;
end;

procedure TCreateHRK.SelectDilec;
var
   SB: TSelectForestBorderLines;
begin
   SB := TSelectForestBorderLines.Create;
   SB.RefFrameClassName := ReferenceClassName;
   SB.FrameClassName := 'Tgis_Dilec';
   SB.Input := Lines_Source;
   SB.Output := Lin_Dilec;
   SB.DoIt;
   SB.Free;
end;

procedure TCreateHRK.SelectForest;
var
   i: integer;
   e: TElement;
   GE: TGeometryElement;
   G: TGeometry;
   L: TPolyLineElement;
begin
   MaxProgress := Lin_Oddeleni.LastIndex;
   Application.ProcessMessages;
   for i := Lin_Oddeleni.LastIndex DownTo 0 do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e :=  Lin_Oddeleni.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      G := GE.Geometry;
      if G = nil then continue;
      if not (G is TPolyLineElement) then continue;
      L := TPolyLineElement(G);
      if not TestLine(L) then
      begin
         Lin_Oddeleni.RemoveElement(e);
         Lin_Forest.AddElement(e);
      end;
   end;
end;

procedure TCreateHRK.SelectOddeleni;
var
   SB: TSelectForestBorderLines;
   i: integer;
   e: TElement;
begin
   SB := TSelectForestBorderLines.Create;
   SB.RefFrameClassName := ReferenceClassName;
   SB.FrameClassName := 'Tgis_Oddeleni';
   SB.Input := Lin_Dilec;
   SB.Output := Lin_Oddeleni;
   SB.DoIt;
   SB.Free;
   MaxProgress := Lin_Dilec.LastIndex;
   Application.ProcessMessages;
   for i := Lin_Dilec.LastIndex DownTo 0 do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Lin_Dilec.Elements[i];
      if e.IsMyOwner(Lin_Oddeleni) then
      begin
         Lin_Dilec.RemoveElement(e);
      end;
   end;
end;

procedure TCreateHRK.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCreateHRK.SetLines_HRK(const Value: TEntity);
begin
  FLines_HRK := Value;
end;

procedure TCreateHRK.SetLines_Source(const Value: TEntity);
begin
  FLines_Source := Value;
end;

procedure TCreateHRK.SetLin_Dilec(const Value: TEntity);
begin
  FLin_Dilec := Value;
end;

procedure TCreateHRK.SetLin_Forest(const Value: TEntity);
begin
  FLin_Forest := Value;
end;

procedure TCreateHRK.SetLin_HRKNew(const Value: TEntity);
begin
  FLin_HRKNew := Value;
end;

procedure TCreateHRK.SetLin_Oddeleni(const Value: TEntity);
begin
  FLin_Oddeleni := Value;
end;

procedure TCreateHRK.SetReferenceClass(const Value: TElementClass);
begin
  FReferenceClass := Value;
end;

procedure TCreateHRK.SetReferenceClassName(const Value: string);
begin
   FReferenceClassName := Value;
   ReferenceClass := GetOrbisClass(Value);
end;

procedure TCreateHRK.SetTolerance(const Value: double);
begin
  FTolerance := Value;
end;

procedure TCreateHRK.SetupLines;
   procedure SetupLin(En: TEntity;Druh,Barva,Znacka: integer);
   var
      i: integer;
      e: TElement;
      TL: TTopolLineElement;
   begin
      MaxProgress := En.LastIndex;
      for i := 0 to En.LastIndex do
      begin
         if i mod 50 = 0 then
         begin
            Progress := i;
            Application.ProcessMessages;
         end;
         e := En.Elements[i];
         if not (e is TTopolLineElement) then continue;
         TL := TTopolLineElement(e);
         TL.Druh := Druh;
         TL.Barva := Barva;
         TL.Znacka := Znacka;
         TL.Presnost := 1;
      end;
      MaxProgress := 0;
   end;
begin
   SetupLin(Lin_Oddeleni,10592, 13, 10592);
   SetupLin(Lin_Dilec,10602, 13, 10602);
   SetupLin(Lin_Forest,0,9,1);
end;

function TCreateHRK.TestLine(aLine: TPolyLineElement): boolean;
var
   L: TPolyLineElement;
   i: integer;
   E: TEntity;
   P1,P2: TElement;
   Are: TPolyLineAreaElement;
begin
   Result := True;
   if aLine = nil then exit;
   L := aLine;
   P1 := nil;
   P2 := nil;
   for i := 0 to L.OwnersCount - 1 do
   begin
      E := L.Owners[i];
      if not (E is TPolyLineAreaElement) then continue;
      Are := TPolyLineAreaElement(E);
      if Are.IsHole then
      begin
         Are := TPolyLineAreaElement(Are.HoleOwnerArea);
         if Are <> nil then
            P2 := Are.FindClassInOwners(ReferenceClass);
      end
      else
         P2 := E.FindClassInOwners(ReferenceClass);
      if P2 <> nil then
      begin
         if P1 = nil then
         begin
            P1 := P2;
            P2 := nil;
         end
         else
            Break;
      end;
   end;
   if P1 = nil then exit;
   if P2 = nil then
   begin
      result := false;
      exit;
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TCreateHRK);
END.
