unit t_ParcelyTestTools;

interface
uses
   Tools;

type
// input - TPolyLineElement
// result = false jestlie linie tvo! hranici Skupin PAR (nebo okraj)
THraniceSKPTestTool = class(TBooleanTool)
  private
    FUseParZar: boolean;
    procedure SetUseParZar(const Value: boolean);
public
   procedure DoIt; override;
published
   property UseParZar: boolean read FUseParZar write SetUseParZar;
end;

TPolyLineISFromPARTool = class(TBooleanTool)
public
   procedure DoIt; override;
end;

implementation
uses
   Elements,
   e_Class,
   e_Geometry,
   e_KatastrModel,
   e_TopolElements;

{ TPolyLineISFromPARTool }

procedure TPolyLineISFromPARTool.DoIt;
var
   L: TPolyLineElement;
   i: integer;
   E: TEntity;
   TL: TTopolLineElement;
begin
   Result := True;
   if not (Input is TPolyLineElement) then exit;
   L := TPolyLineElement(Input);
   for i := 0 to L.OwnersCount - 1 do
   begin
      E := L.Owners[i];
      if not (E is TTopolLineElement) then continue;
      TL := TTopolLineElement(E);
      E := TL.FindOwnerName('LIN_PAR');
      if E <> nil then
      begin
         Result := false;
         exit;
      end;
   end;
end;

{ THraniceSKPTestTool }

procedure THraniceSKPTestTool.DoIt;
var
   L: TPolyLineElement;
   i: integer;
   E: TEntity;
   P1,P2: Tgis_ParcelaCast;
   Are: TPolyLineAreaElement;
begin
   Result := True;
   if not (Input is TPolyLineElement) then exit;
   L := TPolyLineElement(Input);
   P1 := nil;
   P2 := nil;
   for i := 0 to L.OwnersCount - 1 do
   begin
      E := L.Owners[i];
      if not (E is TPolyLineAreaElement) then continue;
      Are := TPolyLineAreaElement(E);
      if Are.IsHole then
      begin
         Are := TPolyLineAreaElement(Are.HoleOwnerArea);
         if Are <> nil then
            P2 := Tgis_ParcelaCast(Are.FindClassInOwners(Tgis_ParcelaCast));
      end
      else
         P2 := Tgis_ParcelaCast(E.FindClassInOwners(Tgis_ParcelaCast));
      if (P2 <> nil) and UseParZar and not P2.ParZar then
         P2 := nil;
      if P2 <> nil then
      begin
         if P1 = nil then
         begin
            P1 := P2;
            P2 := nil;
         end
         else
            Break;
      end;
   end;
   if P1 = nil then
   begin
      exit;
   end;
   if P2 = nil then
   begin
      result := false;
      exit;
   end;
   if not ((P1.KU_KOD = P2.KU_KOD) and (P1.SKUPAR_CIS = P2.SKUPAR_CIS)) then
   begin
      result := false;
      exit;
   end;
end;


procedure THraniceSKPTestTool.SetUseParZar(const Value: boolean);
begin
  FUseParZar := Value;
end;

INITIALIZATION
   RegisterOrbisClass(THraniceSKPTestTool);
   RegisterOrbisClass(TPolyLineISFromPARTool);
END.
