unit t_TAXUtils;

interface
uses
   Tools;

type

TCreateLesniCelek = class(TTool)
  private
    FDirectory: string;
    FRW: boolean;
    procedure SetDirectory(const Value: string);
    procedure SetRW(const Value: boolean);
public
   procedure DoIt; override;
   // este by to chtelo nastaveni, zda se otvira jen pro cteni nebo jak
   function NumberToPropertyName(aNumber: longInt): string;override;
published
   property Directory: string read FDirectory write SetDirectory;
   property RW: boolean read FRW write SetRW;
end;

TConvert_DB_to_DBF_InDirectory = class(TTool)
  private
    FDirectory: string;
    FOutDirectory: string;
    FComment: string;
    procedure SetDirectory(const Value: string);
    procedure SetOutDirectory(const Value: string);
    procedure SetComment(const Value: string);
public
   procedure DoIt; override;
   function NumberToPropertyName(aNumber: longInt): string;override;
published
   property Directory: string read FDirectory write SetDirectory;
   property OutDirectory: string read FOutDirectory write SetOutDirectory;
   property Comment: string read FComment write SetComment;
end;

implementation
uses
   Forms,
   SysUtils,
   Dialogs,
   DBTables,
   DTBLes98,
   LesCisOr,
   e_LesCel,
   LesCisel,
   eLesObec,
   e_Class,
   Elements,
   t_OilPro1,
   u_String,
   Worlds;

{ TCreateLesniCelek }

procedure TCreateLesniCelek.DoIt;
var
   LesniCelek: TLesniCelek;
   cisTB: TCiselnik_TB;
begin
   Output := nil;
   LesniCelek := TLesniCelek.create;
   //LesniCelek.LHC.VypMzdDleZezuly1 := true;
   World.objectmodel.AddElement(LesniCelek);
   LesniCelek.NajdiTypDatLHP(false,Directory,false,RW,true,false); // nalezen dat TAXu
   if LesniCelek.TypDatLHP=NeniLHP then
   begin
      showmessage('V adresi "" nejsou data LHP');
      LesniCelek.Free;
      LesniCelek := nil;
      exit;
   end;
   LesniCelek.nactiPrvniOddeleni;
   LesniCelek.CiselOrbis.nactiSeZOrbu(LesniCelek.PHSNazev,false);
   NactiCiselOrbis_DB98(LesniCelek.CiselOrbis,LesniCelek.LOvr98);
   PriradCiselnik_DB98(LesniCelek.Ciselnik_TB,LesniCelek.LOvr98);
   cisTB := LesniCelek.Ciselnik_TB;
   LesniCelek.LHC.ciselOrbis.MZDHospSoubMN.SmazVsechnyPrvkyVMnozine;
   vytvaretMZDHospSOUB := true;
   Output := LesniCelek;
end;

function TCreateLesniCelek.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Directory';
      1: result := 'RW';
   end;
end;

procedure TCreateLesniCelek.SetDirectory(const Value: string);
begin
  FDirectory := Value;
end;

procedure TCreateLesniCelek.SetRW(const Value: boolean);
begin
  FRW := Value;
end;

{ TConvert_DB_to_DBF_InDirectory }

procedure TConvert_DB_to_DBF_InDirectory.DoIt;
var
   i,V:integer;
   SR: TSearchRec;
   TAP: TGetAbsoluteFilePath;
   Table1,Table2: TTable;
   FileCount: integer;
begin
   Error := 0;
   if not DirectoryExists(Directory) then
   begin
      Error := 1;
      exit;
   end;
   StartProgress(Comment + ' '+Directory,0,0);
   if OutDirectory ='' then
      OutDirectory := 'DBF';
   TAP := TGetAbsoluteFilePath.Create;
   TAP.RootPath := Directory;
   TAP.Path := OutDirectory;
   TAP.DoIt;
   OutDirectory := TAP.Result;
   TAP.Free;
   CheckBackSlash(FDirectory);
   V:=FindFirst(Directory+'*.DB',faAnyFile,SR);
   FileCount := 0;
   while V = 0 do
   begin
      Inc(FileCount);
      V:=FindNext(SR);
   end;
   MaxProgress := FileCount;
   V:=FindFirst(Directory+'*.DB',faAnyFile,SR);
   Forcedirectories(OutDirectory);
   CheckBackSlash(FOutDirectory);
   Table1 := TTable.Create(Application);
   Table2 := TTable.Create(Application);
   while V = 0 do
   begin
      Table1.Active:=false;
      Table1.TableName:=Directory+SR.name;
      ProgressText := SR.Name;
      Progress := FileCount;
      Dec(FileCount);
      Application.ProcessMessages;
      Table1.Active:=true;
      Table2.Active:=false;
      Table2.TableName:=OutDirectory+SR.name+'f';

      Table1. First;
      Table2.FieldDefs.clear;
      for i:=0 to Table1.Fields.Count-1 do
      begin
        table2.fieldDefs.Add( table1.Fields.Fields[i].FieldName,
                              table1.Fields.Fields[i].DataType,
                              table1.Fields.Fields[i].Size,
                              false);
      end;
      Table2.exclusive:=true;
      Table2.CreateTable;
      Table2.EmptyTable;
      Table2.Open;
      Table1.First;
      while not Table1.Eof do
      begin
         Table1.edit;
         Table2.Insert;
         Table2.Edit;
         for i := 0 to Table1.Fields.Count-1 do
         begin
            Table2.Fields.Fields[i].AsString:=Table1.Fields.Fields[i].AsString;
         end;
         Table2.Post;
         Table1.Next;
      end;
      table2.close;
      //Datasource2.dataset:=table2;
      //table2.open;
      V:=FindNext(SR);
   end;
   Table1.Active := false;
   Table2.Active := false;
   Table1.Free;
   Table2.Free;
end;

function TConvert_DB_to_DBF_InDirectory.NumberToPropertyName(aNumber: Integer): string;
begin
   case aNumber of
      0: result := 'Directory';
      1: result := 'OutDirectory';
   end;
end;

procedure TConvert_DB_to_DBF_InDirectory.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TConvert_DB_to_DBF_InDirectory.SetDirectory(const Value: string);
begin
  FDirectory := Value;
end;

procedure TConvert_DB_to_DBF_InDirectory.SetOutDirectory(const Value: string);
begin
  FOutDirectory := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCreateLesniCelek);
   RegisterOrbisClass(TConvert_DB_to_DBF_InDirectory);
END.

