unit E_DESKTP.PAS_kdyby_to_nahodou_nebylo_ulozeny_E_DESKTP;
{****************************************************************************}
{                                                                            }
{          ORBIS ... Pracovn plocha - Desktop                               }
{                    (c) 1996                                                }
{                                                                            }
{****************************************************************************}

interface
uses
   Graphics,
// KERNEL
   Elements,
   e_Files,
// GLIB
   g_Elems,
// MAPPER
   e_MSpace;


type

TDesktop = class(TDrawRectangle) {(TRectEntity) }
private
   FMapSpace: TMapSpace;
   function  GetMapSpace: TMapSpace;
   procedure SetMapSpace(aValue: TMapSpace);
public
   constructor Create; override;
   function  CreateEnvelope: TElement; override;
   property  MapSpace: TMapSpace read GetMapSpace write SetMapSpace;
end;

TDeskTopFile = class(TFileElement)
private
  FDeltaX, FDeltaY: double;
published
  property DeltaX: double read FDeltaX write FDeltaX;
  property DeltaY: double read FDeltaY write FDeltaY;
end;

TMultiPageDesktop = class(TRectEntity)
  private
    FActualPage: TDesktop;
    FLeftNumberTextName: string;
    FRightNumberTextName: string;
    FNumberTextName: string;
    FTopNumberTextName: string;
    FBottomNumberTextName: string;
    FNoNumberText: string;
    FOverlap: double;
    FRealOverlap: double;
    procedure SetActualPage(const Value: TDesktop);
    procedure SetFIndPageNumber(const Value: integer);
    function GetDesktopPage(aNumber: integer): TDesktop;
    procedure SetDesktopPage(aNumber: integer; const Value: TDesktop);
    function GetActualBottomNumber: integer;
    function GetActualLeftNumber: integer;
    function GetActualNumber: integer;
    function GetActualRightNumber: integer;
    function GetActualTopNumber: integer;
    procedure SetBottomNumberTextName(const Value: string);
    procedure SetLeftNumberTextName(const Value: string);
    procedure SetNumberTextName(const Value: string);
    procedure SetRightNumberTextName(const Value: string);
    procedure SetTopNumberTextName(const Value: string);
    procedure SetNoNumberText(const Value: string);
    procedure SetFillNumbers(const Value: boolean);
    procedure SetPageNumberToDelete(const Value: integer);
    procedure SetOverlap(const Value: double);
    procedure SetRealOverlap(const Value: double);
public
   function FindDesktopPageByXY(aX,aY: double): TDesktop;
   function GetpageNumberByXY(aX,aY: double):integer;
   property DesktopPage[aNumber: integer]: TDesktop read GetDesktopPage write SetDesktopPage;
published
   property ActualPage: TDesktop read FActualPage write SetActualPage stored false;
   property FIndPageNumber: integer write SetFIndPageNumber stored false;
   property ActualNumber: integer read GetActualNumber stored false;
   property ActualLeftNumber: integer read GetActualLeftNumber stored false;
   property ActualRightNumber: integer read GetActualRightNumber stored false;
   property ActualTopNumber: integer read GetActualTopNumber stored false;
   property ActualBottomNumber: integer read GetActualBottomNumber stored false;
   property PageNumberToDelete: integer write SetPageNumberToDelete;
   property Overlap: double read FOverlap write SetOverlap;
   property RealOverlap: double read FRealOverlap write SetRealOverlap;
   property NumberTextName: string read FNumberTextName write SetNumberTextName;
   property LeftNumberTextName: string read FLeftNumberTextName write SetLeftNumberTextName;
   property RightNumberTextName: string read FRightNumberTextName write SetRightNumberTextName;
   property TopNumberTextName: string read FTopNumberTextName write SetTopNumberTextName;
   property BottomNumberTextName: string read FBottomNumberTextName write SetBottomNumberTextName;
   property NoNumberText: string read FNoNumberText write SetNoNumberText;
   property FillNumbers: boolean write SetFillNumbers stored false;
end;

implementation

uses
   SysUtils,
// KERNEL
   e_Class;



constructor TDesktop.Create;
begin
  inherited Create;
  BrushColor:= clWhite;
end;

function  TDesktop.GetMapSpace: TMapSpace;
begin
   result := FMapSpace;
end;

procedure TDesktop.SetMapSpace(aValue: TMapSpace);
begin
   if FMapSpace <> nil then
   begin
      FreeElement(FMapSpace);
      FMapSpace := nil;
   end;
   FMapSpace := aValue;
   AddElement(FMapSpace); { nechci nastavit obdelnik }
end;


function  TDesktop.CreateEnvelope: TElement;
begin
   result := inherited CreateEnvelope;
end;


{ TMultiPageDesktop }

function TMultiPageDesktop.FindDesktopPageByXY(aX, aY: double): TDesktop;
var
   E: TElement;
   D: TDesktop;
   i: integer;
begin
   result := nil;
   for i := 0 to LastIndex do
   begin
      E := Elements[i];
      if not (E is TDesktop) then continue;
      D := TDesktop(E);
      if D.PointIsInside(aX,aY) then
      begin
         result := D;
         exit;
      end;
   end;
end;

function TMultiPageDesktop.GetActualBottomNumber: integer;
begin
   result := GetPageNumberByXY(ActualPage.CenterX, ActualPage.CenterY-ActualPage.Height);
end;

function TMultiPageDesktop.GetActualLeftNumber: integer;
begin
   result := GetPageNumberByXY(ActualPage.CenterX-ActualPage.Width, ActualPage.CenterY);
end;

function TMultiPageDesktop.GetActualNumber: integer;
begin
   result := 0;
   if ActualPage = nil then exit;
   result := IndexOf(ActualPage) + 1;
end;

function TMultiPageDesktop.GetActualRightNumber: integer;
begin
   result := GetPageNumberByXY(ActualPage.CenterX+ActualPage.Width, ActualPage.CenterY);
end;

function TMultiPageDesktop.GetActualTopNumber: integer;
begin
   result := GetPageNumberByXY(ActualPage.CenterX, ActualPage.CenterY+ActualPage.Height);
end;

function TMultiPageDesktop.GetDesktopPage(aNumber: integer): TDesktop;
begin
   if aNumber > LastIndex then
      aNumber := LastIndex;
   result := TDesktop(Elements[aNumber]);
end;

function TMultiPageDesktop.GetpageNumberByXY(aX, aY: double): integer;
var
   D: TDesktop;
begin
   result := 0;
   if ActualPage = nil then exit;
   D := FindDesktopPageByXY(aX, aY);
   if D = nil then exit;
   result := IndexOf(D) + 1;
end;

procedure TMultiPageDesktop.SetPageNumberToDelete(const Value: integer);
begin
   if (Value < 1) or (Value > Count) then exit;
   if Value = ActualNumber then FindPageNumber := Value -1;
   FreeElement(DesktopPage[Value-1]);
   FillNumbers := true;
end;

procedure TMultiPageDesktop.SetActualPage(const Value: TDesktop);
begin
   FActualPage := Value;
end;

procedure TMultiPageDesktop.SetBottomNumberTextName(const Value: string);
begin
  FBottomNumberTextName := Value;
end;

procedure TMultiPageDesktop.SetDesktopPage(aNumber: integer;
  const Value: TDesktop);
begin

end;

procedure TMultiPageDesktop.SetFillNumbers(const Value: boolean);
var
   i: integer;
   T: TText;
   procedure SetTextValue(aTextName: string; aNumber: integer);
   var
      E: TElement;
   begin
      E := ActualPage.FindElementName(aTextName);
      if E = nil then exit;
      if not (E is TText) then exit;
      T := TText(E);
      if aNumber > 0 then T.Text := IntToStr(aNumber)
                     else T.Text := NoNumberText;
   end;

begin
   for i := 0 to lastIndex do
   begin
      ActualPage := DesktopPage[i];
      if ActualPage = nil then Continue;
      ActualPage.Name := 'Page '+ IntToStr(ActualNumber);
      SetTextValue(NumberTextName,ActualNumber);
      SetTextValue(LeftNumberTextName,ActualLeftNumber);
      SetTextValue(RightNumberTextName,ActualRightNumber);
      SetTextValue(TopNumberTextName,ActualTopNumber);
      SetTextValue(BottomNumberTextName,ActualBottomNumber);
   end;
end;

procedure TMultiPageDesktop.SetFIndPageNumber(const Value: integer);
var
   i: integer;
begin
   i := Value;
   if i < 1 then i := 1;
   if i > Count then i := Count;
   ActualPage := DesktopPage[i-1];
end;

procedure TMultiPageDesktop.SetLeftNumberTextName(const Value: string);
begin
   FLeftNumberTextName := Value;
end;

procedure TMultiPageDesktop.SetNoNumberText(const Value: string);
begin
   FNoNumberText := Value;
end;

procedure TMultiPageDesktop.SetNumberTextName(const Value: string);
begin
   FNumberTextName := Value;
end;

procedure TMultiPageDesktop.SetRightNumberTextName(const Value: string);
begin
   FRightNumberTextName := Value;
end;

procedure TMultiPageDesktop.SetTopNumberTextName(const Value: string);
begin
   FTopNumberTextName := Value;
end;

procedure TMultiPageDesktop.SetOverlap(const Value: double);
begin
   FOverlap := Value;
end;

procedure TMultiPageDesktop.SetRealOverlap(const Value: double);
begin
   FRealOverlap := Value;
end;

INITIALIZATION
  RegisterOrbisClass(TDesktop);
  RegisterOrbisClass(TMultiPageDesktop);
  RegisterOrbisClass(TDeskTopFile);
END.
