unit I_krizku;

{
********************************************************
**                                                    **
**                    ORBIS                           **
**              interprety Krizku mapy                **
**                                                    **
********************************************************
}

interface

uses
   Graphics,
   Elements,
   g_Elems,
   i_Basic,
   Tools;
   //E_TopMdl;



type


TCross = class(TElement)
private
  FWidth: double;
  FHeight: double;
  FBarva: TColor;
  FSirkaKrizku: double;
  FPenWidth: double; { v m }
  procedure SetStep(aValue: double);
published
  property Step: double write SetStep;
  property Width: double read FWidth write FWidth;
  property Height: double read FHeight write FHeight;
  property Size: double read FSirkaKrizku write FSirkaKrizku;
  property Color: TColor read FBarva write FBarva;
  property PenWidth: double read FPenWidth write FPenWidth;
end;




TInterpretKrizku = class(TInterpret)
private
   Rect: TDoubleRect;
   PolomerKrizku: double;

   procedure ZobrazKrizek(X, Y: double);
 protected
   procedure Process(Element: TElement); override;
   function  IsItsClass(AObject: TObject): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
 public
   procedure   DoIt; override;
end;


TLines = class(TElement)
private
  FWidth,
  FHeight: double;
  FColor: TColor;
  FPenWidth: longInt; { v 1/1000 mm }
published
  property Width: double read FWidth write FWidth;
  property Height: double read FHeight write FHeight;
  property Color: TColor read FColor write FColor;
  property PenWidth: longInt read FPenWidth write FPenWidth;
end;




TLinesInterpret = class(TInterpret)
private
   Rect: TDoubleRect;
 protected
   procedure Process(Element: TElement); override;
   function  IsItsClass(AObject: TObject): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
 public
   procedure   DoIt; override;
end;



implementation

uses
   SysUtils, Dialogs, Forms,
   e_Class,
   E_Color;




procedure TCross.SetStep(aValue: double);
begin
  FWidth:= aValue;
  FHeight:= aValue;
end;


{ TInterpretTopTextu }

procedure TInterpretKrizku.ZobrazKrizek(X, Y: double);
begin
  with Device do
  begin
    MoveTo(X-PolomerKrizku, Y);
    LineTo(X+PolomerKrizku, Y);
    MoveTo(X, Y-PolomerKrizku);
    LineTo(X, Y+PolomerKrizku)
  end;
end;


procedure TInterpretKrizku.Process(Element: TElement);
var
  X, Y, PocY: double;
  Krizky: TCross;
begin
  { pokryje plochu krizky }
  if Element is TCross then
  begin
    Krizky:= TCross(Element);
    with Rect do
    begin
      if (Krizky.Width <= 0) or (Krizky.Height <= 0) or
         ((MaxX-MinX)/Krizky.Width > 1E3) or ((MaxY - MinY)/Krizky.Height > 1E3) then
      begin
        ShowMessage('Objekt '+ Element.ClassName + ':' + Element.Name +
          ' m vzhledem k zobrazovan ploe nevhodn nastavenou ku i vku');
        exit
      end
    end;

    PolomerKrizku:= Krizky.Size/2;
    Device.Pen.Style:= psSolid;
    Device.Pen.Color:= Krizky.Color;
    Device.PenWidth:= Krizky.PenWidth;

    with Krizky do
    begin
      X:= trunc(Rect.MinX / Width) * Width;
      Y:= trunc(Rect.MinY / Height) * Height;
      PocY:= Y;

      while (X <= Rect.MaxX) do
      begin
        while (Y <= Rect.MaxY) do
        begin
          ZobrazKrizek(X, Y);
          Y:= Y + Height;
        end;

        Y:= PocY;
        X:= X + Width
      end;

      Application.ProcessMessages
    end
  end
end;



function  TInterpretKrizku.IsItsClass(AObject: TObject): boolean;
begin
  Result := (AObject is TCross)
end;

function  TInterpretKrizku.QProcessBefore(Element: TElement): boolean;
begin
  Result:= (Element.ClassType = TCross);
end;


procedure TInterpretKrizku.DoIt;
begin
   Rect := Device.Projection.ModelView;
   inherited DoIt;
end;








procedure TLinesInterpret.Process(Element: TElement);
var
  X, Y: double;
  Lines: TLines;
begin
  { pokryje plochu krizky }
  if (Element is TLines) then
  begin
    Lines:= TLines(Element);
    with Rect do
    begin
      if (Lines.Width <= 0) or (Lines.Height <= 0) or
         ((MaxX-MinX)/Lines.Width > 1E3) or ((MaxY - MinY)/Lines.Height > 1E3) then
      begin
        ShowMessage('Objekt '+ Element.ClassName + ':' + Element.Name +
          ' m vzhledem k zobrazovan ploe nevhodn nastavenou ku i vku');
        exit
      end
    end;

    Device.Pen.Style:= psSolid;
    Device.Pen.Color:= Lines.Color;
    Device.PenWidth:= Lines.PenWidth;

    with Lines do
    begin
      X:= trunc(Rect.MinX / Width) * Width;
      Y:= trunc(Rect.MinY / Height) * Height;

      while (X <= Rect.MaxX) do
      begin
        Device.MoveTo(X, Rect.MinY);
        Device.LineTo(X, Rect.MaxY);

        X:= X + Width
      end;

      while (Y <= Rect.MaxY) do
      begin
        Device.MoveTo(Rect.MinX, Y);
        Device.LineTo(Rect.MaxX, Y);
        Y:= Y + Height;
      end;
      Application.ProcessMessages
    end
  end
end;


function  TLinesInterpret.IsItsClass(AObject: TObject): boolean;
begin
  Result := (AObject is TLines)
end;

function  TLinesInterpret.QProcessBefore(Element: TElement): boolean;
begin
  Result:= (Element.ClassType = TLines);
end;


procedure TLinesInterpret.DoIt;
begin
   Rect := Device.Projection.ModelView;
   inherited DoIt;
end;


begin
   RegisterOrbisClass(TCross);
   RegisterOrbisClass(TLines);
   RegisterOrbisClass(TInterpretKrizku);
   RegisterOrbisClass(TLinesInterpret);
end.
