unit T_setec;

interface
uses
  Graphics,
// KERNEL
  Elements,
  e_Type,
  Tools,
// GLIB
  e_Pale,
//
  t_ITable;

type

TSetUserSymbolColor = class(TTool)
private
   FFillBackground: boolean;
   FColorElement: TIntegerElement;
   FColor: TColor;
   FUseColor: boolean;
   FColorByTopol: boolean;
   FPropColor: string;
   FToolColor: TResultTool;
   FPalette: TColorPalette;
   FColorByPalette: boolean;
   FElem: TElement;
   function  GetColor(C: longInt): TColor;
   function  GetColorElement: TIntegerElement;
   procedure SetLinePattern;
   procedure SetAreaPattern;
public
   procedure DoIt; override;
   property  ColorElement: TIntegerElement read GetColorElement;
published
   property   Palette: TColorPalette read FPalette write FPalette;
   property   PropColor: string read FPropColor write FPropColor;
   property   ToolColor: TResultTool read FToolColor write FToolColor;
   property   Color: TColor read FColor write FColor;
   property   UseColor: boolean read FUseColor write FUseColor;
   property   ColorByTopol: boolean read FColorByTopol write FColorByTopol;
   property   ColorByPalette: boolean read FColorByPalette write FColorByPalette;
   property   Elem: TElement read FElem write FElem;
   property   FillBackground: boolean read FFillBackground write FFillBackground;
end;

implementation
uses
  SysUtils,
  e_Class,
  e_color,
  e_LinZn,
  e_PlZn,
  e_Geometry,
  e_TopolElements,
  U_Props,
  U_String;

procedure TSetUserSymbolColor.SetLinePattern;
var
   L: TLiniovaZnacka;
   R: TRetezovaZnacka;
   TL: TTopolLineElement;
   C: integer;
begin
   if not (Elem is TTopolLineElement) then exit;
   TL := TTopolLineELement(Elem);
   C := TL.Barva;
   L := TLiniovaZnacka(Input);
   L.ByTrueColor := true;
   R := L.Znacky;
   while R <> nil do
   begin
      R.ByTrueColor := true;
      if R.Barva = -1 then R.TrueColor := GetColor(C)
                      else R.TrueColor := GetColor(R.Barva);
      R := TRetezovaZnacka(R.Dalsi);
   end;
end;
{
procedure TSetUserSymbolColor.SetLinePattern;
var
   L: TLiniovaZnacka;
   R: TRetezovaZnacka;
   P: TTopolProps;
   C: integer;
begin
   if not (Elem is TPolyLineElement) then exit;
   P := TTopolProps(TPolyLineElement(Elem).Properties);
   if TElement(P) is TTopolProps then C := P.Barva
                                 else C := 0;
   L := TLiniovaZnacka(Input);
   L.ByTrueColor := true;
   R := L.Znacky;
   while R <> nil do
   begin
      R.ByTrueColor := true;
      if R.Barva = -1 then R.TrueColor := GetColor(C)
                      else R.TrueColor := GetColor(R.Barva);
      R := TRetezovaZnacka(R.Dalsi);
   end;
end;
}

procedure TSetUserSymbolColor.SetAreaPattern;
var
   P: TPlosnaZnacka;
   V: TVrstvaSraf;
   S: TSrafa;
   i: longInt;
begin
   P := TPlosnaZnacka(Input);
   P.ByTrueColor := true;
   P.TrueColor := GetColor(P.Color);
   P.FillBackGround := FillBackGround;
   V := P.Znacky;
   while V <> nil do
   begin
      S := V.Srafy;
      for i := 1 to V.PocetSraf do
      begin
         if S = nil then Continue;
         S.ByTrueColor := true;
         S.TrueColor := GetColor(S.Barva);
         S := S.Dalsi;
      end;
      V := V.Dalsi;
   end;
end;

function  TSetUserSymbolColor.GetColor(C: longInt): TColor;
var
   i: longInt;
   aColor: TColor;
   cc: integer;
   TP: TTopolElement;
begin
   result := -1;
   if C = -1 then exit;

   if UseColor then result := Color
   else
   if ToolColor <> nil then
   begin
      if not (Elem is TTopolElement) then exit;
      TP := TTopolELement(Elem);
      ColorElement.Value := TP.Barva;
      TP.Barva := C;
      ToolColor.Input := Elem;
      ToolColor.DoIt;
      TP.Barva := ColorElement.Value;
      result:= StrToInt(ToolColor.Result)
   end
   else
   if PropColor = '' then result:= clWhite
                     else result:= C;

   if ColorByPalette and (result > -1) then
      result:= Palette.Color[result]
   else
      if ColorByTopol or (result < 0) then result := DejBarvicku(result);
end;


function  TSetUserSymbolColor.GetColorElement: TIntegerElement;
begin
   if FColorElement = nil then
   begin
      FColorElement := TIntegerElement.Create;
      AddElement(FColorElement);
   end;
   result := FColorElement;
end;


procedure TSetUserSymbolColor.DoIt;
begin
   if Input is TLiniovaZnacka then SetLinePattern else
   if Input is TPlosnaZnacka  then SetAreaPattern;
end;


BEGIN
   RegisterOrbisClass(TSetUserSymbolColor);
END.

