unit T_msedit;
{
*********************************************************************
*                                                                   *
*  ORBIS ... Nstroj pro zobrazen prvodce                         *
*                                                                   *
*                       Topol Pro 1996                             *
*********************************************************************
}


interface
uses
   Controls,
   Menus,
   Elements,
   e_Desktp,
   e_Guide,
   m_InterA,
   f_MSEdit,
   f_RectEd,
   t_InterA,
   t_SelG;
type


{ TMSEditorFormTool }

TMSEditorFormTool = class(TFormToolManager)
 private
   FForm: TMSEditorForm;
   FSizePanel: TF_rectedForm;
   FFileName: String;
   function  GetFileName: string;
   procedure SetFileName(aValue: string);
   function  GetModified: boolean;
   procedure SetModified(aValue: boolean);
 protected
   function    GetLeft: longInt; override;
   function    GetTop: longInt; override;
   function    GetWidth: longInt; override;
   function    GetHeight: longInt; override;
   procedure   SetLeft(aValue: longInt); override;
   procedure   SetTop(aValue: longInt); override;
   procedure   SetWidth(aValue: longInt); override;
   procedure   SetHeight(aValue: longInt); override;
   procedure   ShowScale(aVal: string); override;

   procedure   SetInput(aInput: TElement); override;
   procedure   CreateForm; override;
   procedure   LinkDeviceToForm; override;
   procedure   CreateInterpret; override;
   procedure   CreateTools; override;
   function    GetVisible: boolean; override;
   procedure   SetVisible(aVisible: boolean); override;
   function    GetMainMenu: TMainMenu; override;
   function    GetCaption: string; override;
   procedure   SetCaption(const aValue: string); override;
   function    GetMapsheet: TDesktop; virtual;
   procedure   SetMapsheet(aValue: TDesktop); virtual;
   function    GetFormCursor: TCursor; override;
   procedure   SetFormCursor(aValue: TCursor); override;
   function    GetSizePanel: TF_rectedForm; virtual;
 public

   constructor Create; override;
   destructor  Destroy; override;
   procedure   DoIt; override;
   procedure   AddTemplateToLibrary;
   procedure   On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   On_tm_RePaint(var Handled: boolean); override;
   procedure   on_tm_QuitTask; override;
   procedure   on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_Viewer(var Handled: boolean); virtual;
   procedure   on_tm_MSEdit(var Handled: boolean); virtual;
   procedure   on_tm_NewRect(var Handled: boolean); virtual;
   procedure   on_tm_NewView(var Handled: boolean); virtual;
   procedure   on_tm_NewText(var Handled: boolean); virtual;
   procedure   on_tm_NewTextBlock(var Handled: boolean); virtual;
   procedure   on_tm_NewBMP(var Handled: boolean); virtual;
   procedure   on_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   on_tm_ViewBySelect(var Handled: boolean); virtual;

   procedure   on_tm_New(var Handled: boolean); virtual;
   procedure   on_tm_Open(var Handled: boolean); override;
   procedure   on_tm_Save(var Handled: boolean); virtual;
   procedure   on_tm_SaveAs(var Handled: boolean); override;
   procedure   on_tm_SetupSelect(var Handled: boolean); virtual;
   procedure   on_tm_SelectToTop(var Handled: boolean); virtual;
   procedure   on_tm_SelectToBottom(var Handled: boolean); virtual;
   procedure   on_tm_DeleteSelect(var Handled: boolean); virtual;
   procedure   on_tm_SelectSizeChange(var Handled: boolean); virtual;

   procedure   on_tm_PenWidthChange(var Handled: boolean); virtual;
   procedure   on_tm_PenColorChange(var Handled: boolean); virtual;
   procedure   on_tm_BrushColorChange(var Handled: boolean); virtual;
   procedure   on_tm_BrushStyleChange(var Handled: boolean); virtual;
   procedure   on_tm_FontChange(var Handled: boolean); virtual;
   procedure   on_tm_SetPropertyEditors(var Handled: boolean); virtual;
   procedure   on_tm_SizePanelOn (var Handled: boolean); virtual;
   procedure   on_tm_SizePanelOff (var Handled: boolean); virtual;

   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   function    CloseTemplate: word;
   procedure   SaveTemplate;

   property    GForm: TMSEditorForm read FForm;
   property    SizePanel: TF_rectedForm read GetSizePanel;
   property    MapSheet: TDesktop read GetMapSheet write SetMapSheet;
   property    FileName: string read GetFileName write SetFileName;
published
   property    Modified: boolean read GetModified write SetModified;
end;




implementation


uses
   Dialogs,
   IniFiles,
   SysUtils,
   Graphics,
   Forms,
   e_Class,
   e_Map,
   //e_PWorld,
   f_GCrea,
   D_STDImg,
   e_MSpace,
   e_Type,
   m_Orbis,
   m_Proj,
   g_Elems,
   i_Desktp,
   t_Editor,
   t_LHPMap,
   t_Loader,
   t_Main,
   t_Mapsh,
   t_MSEd,
   t_Saver,
   u_String,
   Worlds;

{ TMSEditorFormTool }






constructor TMSEditorFormTool.Create;
begin
   inherited Create;
   Device.Page.Color := clSilver;
end;


destructor  TMSEditorFormTool.Destroy;
begin
   SizePanel.DontInformTool := true;
   FSizePanel.Free;
   if FForm <> nil then
      FForm.Free;
   FileName := '';
   inherited Destroy;
end;


procedure  TMSEditorFormTool.DoIt;
var
   MS: TDesktop;
   Pan: boolean;
   E: TRectEntityEditor;
begin
   Pan := false;
   if Input = nil then
   begin
      MS := TDesktop.Create;
      MS.Rect.MinX := 0;
      MS.Rect.MinY := 0;
      MS.Rect.MaxX := 0.297;
      MS.Rect.MaxY := 0.210;
      MS.BrushColor := clWhite;
      MS.BrushStyle := bsSolid;
      MS.PenStyle   := psClear;
      Input := MS;
      Pan := true;
      E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
      if E <> nil then E.SelectedElement := MS;
   end;
   PostMsg(tm_Review, Self, nil, DontKill);
   if Pan then PostMsg(tm_SizePanelOn, Self, nil, DontKill);
end;

function    TMSEditorFormTool.GetLeft: longInt;
begin
   result := GForm.Left;
end;

function    TMSEditorFormTool.GetTop: longInt;
begin
   result := GForm.Top;
end;

function    TMSEditorFormTool.GetWidth: longInt;
begin
   result := GForm.Width;
end;

function    TMSEditorFormTool.GetHeight: longInt;
begin
   result := GForm.Height;
end;

procedure   TMSEditorFormTool.SetLeft(aValue: longInt);
begin
   GForm.Left := aValue;
end;

procedure   TMSEditorFormTool.SetTop(aValue: longInt);
begin
   GForm.Top := aValue;
end;

procedure   TMSEditorFormTool.SetWidth(aValue: longInt);
begin
   GForm.Width := aValue;
end;

procedure   TMSEditorFormTool.SetHeight(aValue: longInt);
begin
   GForm.Height := aValue;
end;

procedure   TMSEditorFormTool.ShowScale(aVal: string);
begin
  GForm.l_M.Caption:= aVal
end;


function    TMSEditorFormTool.GetFormCursor: TCursor;
begin
   result := GForm.Image.Cursor;
end;


procedure   TMSEditorFormTool.SetFormCursor(AValue: TCursor);
begin
   GForm.Image.Cursor := aValue;
end;



function    TMSEditorFormTool.GetSizePanel: TF_rectedForm;
begin
   if FSizePanel = nil then
   begin
      Application.CreateForm(TF_rectedForm, FSizePanel);
      FSizePanel.Tool := Self;
      FSizePanel.Editor := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   end;
   result := FSizePanel;
end;

procedure  TMSEditorFormTool.AddTemplateToLibrary;
var
   ShortName: string;
   Index: longInt;
begin
   try
      ShortName := Input.Name;
      if ShortName = 'TDesktop' then
         ShortName:= ExtractFileName(ChangeFileExt(FileName, ''));
   except
      ShortName:= ExtractFileName(ChangeFileExt(FileName, '')); { prvn nastaven }
   end;
   if (FileName = '') or (ShortName = '') then exit;
   { ten jmna mapovho listu v MSD souboru }

   Index:= TMainTool(World.MainTool).TemplateForm.ListBox.Items.Add(ShortName);
   TMainTool(World.MainTool).TemplateForm.ListBox.ItemIndex:= Index;

   TMainTool(World.MainTool).TemplateLibrary.Add(ShortName + '=' + FileName);

   TMainTool(World.MainTool).SaveTemplateLibrary;
   PostMsg(tm_TemplateTouched, World.MainTool, nil, DontKill);
end;


procedure   TMSEditorFormTool.on_tm_QuitTask;
var
   eh: boolean;
begin
   if FileName = 'NONAME.MSD' then on_tm_Save(eh);
   if CloseTemplate = mrCancel then
   begin
      GForm.IsPossibleToClose := false;
      exit;
   end;
   GForm.Free;
   FForm:= nil;
   inherited on_tm_QuitTask;
end;


procedure   TMSEditorFormTool.On_tm_ReView(var aMessage: TOrbisMessage; var Handled: boolean);
var
   ShowSizePanel: boolean;
begin
   ShowSizePanel := false;
   {
   if SizePanel.Visible = true then
   begin
      ShowSizePanel := true;
      SizePanel.Visible := false;
   end;
   }
   Application.ProcessMessages;
   Interpret.Reset(Device);
   Device.Clear;
   inherited On_tm_ReView(aMessage, Handled);
   Device.Close;
   Interpret.Close;
   if ShowSizePanel then SizePanel.Visible := true;
end;


procedure TMSEditorFormTool.On_tm_RePaint(var Handled: boolean);
var
   i: longInt;
begin
   Handled := true;
   {
   PostMsg(tm_BreakInterpret, Self, nil, DontKill);
   }
   GForm.Image.Refresh;
end;




procedure   TMSEditorFormTool.on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean);
var
   aClass: TClass;
begin
   {
   GForm.Panel1.Visible := false;
   }
   inherited on_tm_ToolSelected(aMessage, Handled);
   Handled := true;
   aClass := aMessage.Data.ClassType;
   if aClass = TDrawRectTool then GForm.b_Rect.Down := true else
   if aClass = TViewer then GForm.b_Viewer.Down := true;
end;


procedure   TMSEditorFormTool.on_tm_Viewer(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TViewer').OnClick(nil);
   except
   end;
end;


procedure   TMSEditorFormTool.on_tm_MSEdit(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TMapSheetEditor').OnClick(nil);
   except
   end;
end;


procedure   TMSEditorFormTool.on_tm_NewRect(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TNewDrawRect').OnClick(nil);
   except
   end;
end;


procedure   TMSEditorFormTool.on_tm_NewView(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TNewView').OnClick(nil);
   except
   end;
end;


procedure   TMSEditorFormTool.on_tm_NewText(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TNewText').OnClick(nil);
   except
   end;
end;


procedure   TMSEditorFormTool.on_tm_NewTextBlock(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TNewTextBlock').OnClick(nil);
   except
   end;
end;


procedure   TMSEditorFormTool.on_tm_NewBMP(var Handled: boolean);
begin
   try
      FindMenuItemIn('Tools','TNewBMP').OnClick(nil);
   except
   end;
end;



procedure   TMSEditorFormTool.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   Coord: TDoubleCoord;
begin
    Handled := true;
    begin
       Coord := Device.Projection.GetModelPoint(aMessage.x, aMessage.y);
       GForm.l_x.Caption := FloatToStrF(Coord.x*1000, ffFixed, 5, 0);
       GForm.l_y.Caption := FloatToStrF(Coord.y*1000, ffFixed, 5, 0);
    end;
end;


procedure   TMSEditorFormTool.on_tm_ViewBySelect(var Handled: boolean);
begin
{
   T := TRecGuideSelect.Create;
   T.Select10 := not TGuideInterpret(Interpret).Show_5;
   T.Input := Guide;
   T.DoIt;
   R := TRectangle(T.Output);
   if R <> nil then
      Device.Projection.ChangeProjectionByRectangle(R, Device.Page);
   PostMsg(tm_Review, Self, nil, DontKill);
   R.Free;
   T.Free;
 }
end;



procedure   TMSEditorFormTool.on_tm_New(var Handled: boolean);
var
   MS: TDesktop;
begin
    if CloseTemplate = mrCancel then exit;
    MS := TDesktop.Create;
    MS.Rect.MinX := 0;
    MS.Rect.MinY := 0;
    MS.Rect.MaxX := 0.297;
    MS.Rect.MaxY := 0.210;
    MS.BrushColor := clWhite;
    MS.BrushStyle := bsSolid;
    MS.PenStyle   := psClear;
    Input := MS;
    FileName := 'NONAME.MSD';
    Modified := true;
    PostMsg(tm_Review, Self, nil, DontKill);
end;


procedure   TMSEditorFormTool.on_tm_Open(var Handled: boolean);
var
  Map: TMapSheet;
  MapSheetFiller: TMapSheetFiller;
begin
   Handled := true;
   if CloseTemplate = mrCancel then exit;
   GForm.OpenDialog.InitialDir := TMainTool(World.MainTool).Options.TemplateDirectory;
   if not GForm.OpenDialog.Execute then exit;
   GForm.SaveDialog.FileName := GForm.OpenDialog.FileName;
   MapSheetFiller := TMainTool(World.MainTool).MapSheetFiller;
   {
   MapSheetFiller.Input:= TMainTool(World.MainTool).SelectedMapSpace;
   }
   MapSheetFiller.Input:= TStringElement.CreateAll('TEMPLATE', GForm.OpenDialog.FileName);
   MapSheetFiller.Input:= TStringElement.CreateAll('DATADIR', TMainTool(World.MainTool).Options.DataDirectory);
   MapSheetFiller.FillGraphics:= false;
   MapSheetFiller.DoIt;
   Map:= TMapSheet(MapSheetFiller.Output);
   Input := Map;
   FileName := TMainTool(World.MainTool).SelectedTemplate;
   Modified := false;
   PostMsg(tm_Review, Self, nil, DontKill);

{
   if Map <> nil then
      World.PostMsg(tm_AddView, World.MainTool, Map, DontKill);
}
end;

procedure   TMSEditorFormTool.on_tm_Save(var Handled: boolean);
begin
   Handled := true;
   if FileName = 'NONAME.MSD' then
   begin
      GForm.SaveDialog.InitialDir := TMainTool(World.MainTool).Options.TemplateDirectory;
      GForm.SaveDialog.FileName := FileName;
      if GForm.SaveDialog.execute then
      begin
         FileName := GForm.SaveDialog.FileName;
         AddTemplateToLibrary;
      end;
   end;
   SaveTemplate;
end;

procedure   TMSEditorFormTool.on_tm_SaveAs(var Handled: boolean);
begin
   Handled := true;
   GForm.SaveDialog.InitialDir := TMainTool(World.MainTool).Options.TemplateDirectory;
   GForm.SaveDialog.FileName := FileName;
   if GForm.SaveDialog.execute then
   begin
      FileName := GForm.SaveDialog.FileName;
      AddTemplateToLibrary;
      SaveTemplate;
   end;
end;

procedure   TMSEditorFormTool.on_tm_SetupSelect(var Handled: boolean);
var
   E: TRectEntityEditor;
begin
   E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   if not(TElement(E) is TRectEntityEditor) then exit;
   E.SetupSelect;
end;

procedure   TMSEditorFormTool.on_tm_SelectToTop(var Handled: boolean);
begin
end;

procedure   TMSEditorFormTool.on_tm_SelectToBottom(var Handled: boolean);
begin
end;

procedure   TMSEditorFormTool.on_tm_DeleteSelect(var Handled: boolean);
begin
end;

procedure   TMSEditorFormTool.on_tm_SelectSizeChange(var Handled: boolean);
var
   E: TRectEntityEditor;
   El: TRectEntity;
   D: TDoublerect;
begin
   E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor
   ));
   if E = nil then exit;
   El := E.SelectedElement;
   if TElement(El) is TRectEntity then
   begin
      D := E.Device.Projection.GetModelRect(E.ShiftRect.Left,E.ShiftRect.Top,E.ShiftRect.Right,E.ShiftRect.Bottom);
      SizePanel.SetXMin(D.Minx);
      SizePanel.SetYMin(D.Miny);
      SizePanel.SetXMax(D.Maxx);
      SizePanel.SetYMax(D.Maxy);
   end;
end;


procedure   TMSEditorFormTool.on_tm_PenWidthChange(var Handled: boolean);
var
   E: TRectEntityEditor;
   El: TElement;
begin
   E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   if E = nil then exit;
   El := E.SelectedElement;
   if El is TDrawRectangle then
   begin
      TDrawRectangle(El).PenWidth := GForm.PenWidth;
      PostMsg(tm_ReView, Self, nil, DontKill);
   end;
end;

procedure   TMSEditorFormTool.on_tm_PenColorChange(var Handled: boolean);
var
   E: TRectEntityEditor;
   El: TElement;
begin
   E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   if E = nil then exit;
   El := E.SelectedElement;
   if El is TDrawRectangle then
   begin
      TDrawRectangle(El).PenColor := GForm.ColorPanel.Color;
      PostMsg(tm_ReView, Self, nil, DontKill);
   end
   else
   if El is TText then
   begin
      TText(El).Color := GForm.ColorPanel.Color;
      PostMsg(tm_ReView, Self, nil, DontKill);
   end
end;

procedure   TMSEditorFormTool.on_tm_BrushColorChange(var Handled: boolean);
var
   E: TRectEntityEditor;
   El: TElement;
begin
   E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   if E = nil then exit;
   El := E.SelectedElement;
   if El is TDrawRectangle then
   begin
      TDrawRectangle(El).BrushColor := GForm.BrushColorPanel.Color;
      PostMsg(tm_ReView, Self, nil, DontKill);
   end;
end;

procedure   TMSEditorFormTool.on_tm_BrushStyleChange(var Handled: boolean);
var
   E: TRectEntityEditor;
   El: TElement;
begin
   E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   if E = nil then exit;
   El := E.SelectedElement;
   if El is TDrawRectangle then
   begin
      if GForm.b_Fill.Down then TDrawRectangle(El).BrushStyle := bsSolid
                                 else TDrawRectangle(El).BrushStyle := bsClear;
      PostMsg(tm_ReView, Self, nil, DontKill);
   end;
end;

procedure   TMSEditorFormTool.on_tm_FontChange(var Handled: boolean);
var
   E: TRectEntityEditor;
   El: TElement;
begin
   E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   if E = nil then exit;
   El := E.SelectedElement;
   if El is TText then
   begin
      TText(El).Font:= GForm.FontDialog.Font;
      PostMsg(tm_ReView, Self, nil, DontKill);
   end;
end;

procedure   TMSEditorFormTool.on_tm_SetPropertyEditors(var Handled: boolean);
var
   E: TRectEntityEditor;
   El: TRectEntity;
begin
   E := TRectEntityEditor(Tools.FindElementClass(TRectEntityEditor));
   if E = nil then exit;
   El := E.SelectedElement;
   if El = nil then exit;
   if TElement(El) is TRectEntity then
   begin
      SizePanel.SetXMin(El.Minx);
      SizePanel.SetYMin(El.Miny);
      SizePanel.SetXMax(El.Maxx);
      SizePanel.SetYMax(El.Maxy);
      SizePanel.ed_Name.Text := El.Name;
   end;
   GForm.l_Comment.Caption := 'Vybran objekt: '+El.Name + ' (tda '+El.ClassName+')';
   if El is TDrawRectangle then
   begin
      GForm.ColorPanel.Color := TDrawRectangle(El).PenColor;
      GForm.PenWidth := TDrawRectangle(El).PenWidth;
      GForm.BrushColorPanel.Color := TDrawRectangle(El).BrushColor;
      if TDrawRectangle(El).BrushStyle = bsClear then GForm.b_Fill.Down := false
                                                 else GForm.b_Fill.Down := true;
   end
   else
   if El is TText then
   begin
      GForm.ColorPanel.Color := TText(El).Color;
      GForm.FontDialog.Font.Name := TText(El).FontName;
      GForm.FontDialog.Font.Color := TText(El).Color;
      GForm.FontDialog.Font.Style := TText(El).FontStyle;
      GForm.FontDialog.Font.CharSet := TText(El).CharSet;
      GForm.FontPanel.Text := TText(El).FontName;
   end
end;

procedure   TMSEditorFormTool.on_tm_SizePanelOn (var Handled: boolean);
begin
   SizePanel.Visible := true;
   GForm.b_Size.Down := true;
end;


procedure   TMSEditorFormTool.on_tm_SizePanelOff (var Handled: boolean);
begin
   SizePanel.Visible := false;
   GForm.b_Size.Down := false;
end;

function  TMSEditorFormTool.GetFileName: string;
begin
   result := FFileName;
   if result = '' then result := 'NONAME.MSD'
end;


procedure TMSEditorFormTool.SetFileName(aValue: string);
begin
   FFileName := aValue;
   GForm.Caption := aValue;
end;

function  TMSEditorFormTool.GetModified: boolean;
var
   E: TMapSheetEditor;
begin
   result := false;
   E := TMapSheetEditor(Tools.FindElementClass(TMapSheetEditor));
   if E = nil then exit;
   result := E.Modified;
end;

procedure TMSEditorFormTool.SetModified(aValue: boolean);
var
   E: TMapSheetEditor;
begin
   E := TMapSheetEditor(Tools.FindElementClass(TMapSheetEditor));
   if E = nil then exit;
   E.Modified := aValue;
end;



procedure   TMSEditorFormTool.SetInput(aInput: TElement);
var
   i: longInt;
begin
   if FInput <> nil then
   begin
      FInput.Free;
      FInput := nil;
   end;
   FInput := aInput;
   try
      if aInput is TRectEntity then
      begin
         Device.Projection.ChangeProjectionByRectangle(TRectEntity(aInput).Rect, Device.Page);
      end;
      Interpret.Input := Input;
   except
   end;
   for i := 0 to Tools.Count -1 do Tool[i].Input := aInput;
end;


procedure   TMSEditorFormTool.CreateForm;
begin
   Application.CreateForm(TMSEditorForm, FForm);
   GForm.Tool := Self;
   GForm.IsPossibleToClose := false;
   {
   Form := TStandardForm(GForm);
   }
end;


procedure   TMSEditorFormTool.LinkDeviceToForm;
begin
//   with TCanvasToBMPDevice(Device) do
//   with TCanvasBMPDoubleDevice(Device) do
   with TImageCanvasDevice(Device) do
   begin
//      Canvas := Form.PaintBox.Canvas;
      Canvas := GForm.Image.Canvas;
      Bitmap := GForm.Image.Picture.Bitmap;
   end;
end;

procedure   TMSEditorFormTool.CreateInterpret;
begin
   Interpret := TDesktopInterpret.Create;
end;


procedure   TMSEditorFormTool.CreateTools;
var
   E: TMapSheetEditor;
   EI: TEditorIATool;
begin
   E := TMapSheetEditor.Create;
   NextTool := E;

   EI := TNewBMP.Create;
   EI.Editor := E;
   NextTool := EI;

   EI := TNewDrawRect.Create;
   EI.Editor := E;
   NextTool := EI;

   EI := TNewView.Create;
   EI.Editor := E;
   NextTool := EI;

   EI := TNewText.Create;
   EI.Editor := E;
   NextTool := EI;

   EI := TNewTextBlock.Create;
   EI.Editor := E;
   NextTool := EI;

   NextTool := TViewer.Create;
end;


function    TMSEditorFormTool.GetVisible: boolean;
begin
   try
      result := GForm.Visible;
   except
      result := false;
   end;
end;



procedure   TMSEditorFormTool.SetVisible(aVisible: boolean);
begin
   try
      GForm.Visible := aVisible;
      Device.SetupPage(nil);
   except
   end;
end;


function    TMSEditorFormTool.GetMainMenu: TMainMenu;
begin
   result := GForm.MainMenu;
end;


function    TMSEditorFormTool.GetCaption: string;
begin
   result := GForm.Caption;
end;

procedure   TMSEditorFormTool.SetCaption(const aValue: string);
begin
   GForm.Caption := aValue;
end;


function    TMSEditorFormTool.GetMapSheet: TDesktop;
begin
   result := TDesktop(FInput);
end;



procedure   TMSEditorFormTool.SetMapSheet(aValue: TDesktop);
begin
   SetInput(aValue);
end;



procedure TMSEditorFormTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_Viewer         : on_tm_Viewer(Handled);
      tm_MSEdit         : on_tm_MSEdit(Handled);
      tm_NewRect        : on_tm_NewRect(Handled);
      tm_NewView        : on_tm_NewView(Handled);
      tm_NewText        : on_tm_NewText(Handled);
      tm_NewTextBlock   : on_tm_NewTextBlock(Handled);
      tm_NewBMP         : on_tm_NewBMP(Handled);
      tm_ViewBySelect   : on_tm_ViewBySelect(Handled);

      tm_New            : on_tm_New(Handled);
      tm_Open           : on_tm_Open(Handled);
      tm_Save           : on_tm_Save(Handled);
      tm_SaveAs         : on_tm_SaveAs(Handled);

      tm_SetupSelect    : on_tm_SetupSelect(Handled);
      tm_SelectToTop    : on_tm_SelectToTop(Handled);
      tm_SelectToBottom : on_tm_SelectToBottom(Handled);
      tm_DeleteSelect   : on_tm_DeleteSelect(Handled);
      tm_SelectSizeChange: on_tm_SelectSizeChange(Handled);

      tm_PenWidthChange   : on_tm_PenWidthChange(Handled);
      tm_PenColorChange   : on_tm_PenColorChange(Handled);
      tm_BrushColorChange : on_tm_BrushColorChange(Handled);
      tm_BrushStyleChange : on_tm_BrushStyleChange(Handled);
      tm_FontChange       : on_tm_FontChange(Handled);
      tm_SetPropertyEditors:on_tm_SetPropertyEditors(Handled);
      tm_SizePanelOn      : on_tm_SizePanelOn(Handled);
      tm_SizePanelOff     : on_tm_SizePanelOff(Handled);
   end;
end;



function    TMSEditorFormTool.CloseTemplate: word;
begin
   result := mrYes;
   if input = nil then exit;
   if not Modified then exit;
   result := MessageDlg('Uloit otevenou ablonu ?', mtConfirmation, [mbYes,mbNo,mbCancel], 0);
   if result = mrYes then
        SaveTemplate;
end;


procedure   TMSEditorFormTool.SaveTemplate;
var
  Ukladac: TDeskTopSaveToText;
begin
{  }
   Ukladac:= TDeskTopSaveToText.Create;
   Ukladac.MSDFileName:= FileName;
   Ukladac.Input:= MapSheet;
   Ukladac.DoIt;
   Ukladac.Free;
   Modified := false;
end;


INITIALIZATION
   RegisterOrbisClass(TMSEditorFormTool);
END.
