unit E_sptree;
{****************************************************************************}
{                                                                            }
{          ORBIS ... Obecn prostorov strom - Space tree                    }
{                ... zatm pro kontinuln 2D prostor                        }
{                                                                            }
{                       Topol Pro 1996                                      }
{                                                                            }
{****************************************************************************}

interface
uses
   Classes,
// KERNEL
   Elements,
// GLIB   
   g_Elems;

type


TSpaceTree = class;


TSpaces = class(TRectEntity)  { One column of the Space matrix }
private
   function  GetSpace(aIndex: longInt): TSpaceTree;
   procedure PutSpace(aIndex: longInt; aSpaceTree: TSpaceTree);
   function  GetNoSpaces: boolean;
public
   procedure UpdateRect;
   procedure FreeEmptySpaces;
   function  FindSpaceByCoord(x, y: double): TSpaceTree;
   function  IsIn(aCoord: TDoubleCoord): boolean;
   procedure AddSpace(aSpace: TSpaceTree); virtual;
   property  Space[Index: longInt]: TSpaceTree read GetSpace write PutSpace;
   property  NoSpaces: boolean read GetNoSpaces;
end;


TSpaceMatrix = class(TRectEntity)
private
   function  GetColumn(aIndex: longInt): TSpaces;
   procedure PutColumn(aIndex: longInt; aColumn: TSpaces);
   function  GetNoSpaces: boolean;
protected
   function  FindColumnBySpace(aSpace: TSpaceTree): TSpaces;
   function  AddColumnBySpace(aSpace: TSpaceTree): TSpaces;
public
   procedure UpdateRect;
   procedure FreeEmptySpaces;
   function  FindColumnByCoord(x, y: double): TSpaces;
   procedure AddSpace(aSpace: TSpaceTree); virtual;
   property  Column[Index: longInt]: TSpaces read GetColumn write PutColumn;
   property  NoSpaces: boolean read GetNoSpaces;
end;


TSpaceTree = class(TRectEntity)
private
   FMatrix: TSpaceMatrix;
   function  GetMatrix: TSpaceMatrix;
   procedure SetMatrix(aMatrix: TSpaceMatrix);
   function  StoreMatrix: boolean;
   function  GetNoSpaces: boolean;
   function  FindSpaceByCoord(x, y: double): TSpaceTree;
   function  GetColumn(aIndex: longInt): TSpaces;
   function  GetColumnCount: longInt;
   function  GetSpace(aCol, aRow: longInt): TSpaceTree;

protected
   function  FindSpaceByCoordAndClass(x, y: double; aClass: TClass): TSpaceTree;

public
   procedure Load(aReader: TReader; aIndex: TEntity); override;
   procedure Save(aWriter: TWriter); override;
   function  FindLevelSpace(x, y: double; aLevel: longInt): TSpaceTree;
   procedure AddSpace(aSpace: TSpaceTree); virtual;
   procedure UpdateRect;
   procedure FreeEmptySpaces; virtual;
   property  NoSpaces: boolean read GetNoSpaces;
   property  SpaceByCoord[x,y: double]: TSpaceTree read FindSpaceByCoord;
   property  Space[aCol, aRow: longInt]: TSpaceTree read GetSpace;
   property  Column[Index: longInt]: TSpaces read GetColumn;
   property  ColumnCount: longInt read GetColumnCount;
published
   property  Matrix: TSpaceMatrix read GetMatrix write SetMatrix stored StoreMatrix;
end;


implementation


uses
// KERNEL
   e_Class;


{ TSpaces }


procedure TSpaces.UpdateRect;
var
   i: longInt;
begin
   Rect.Reset;
   for i := Count -1 downto 0 do
   begin
      Rect.UpdateByRectangle(Space[i].Rect);
   end;
end;




procedure TSpaces.FreeEmptySpaces;
var
   i: longInt;
   aSpace: TSpaceTree;
begin
   for i := Count -1 downto 0 do
   begin
      aSpace := Space[i];
      if aSpace.NoSpaces then FreeElement(aSpace);
   end;
end;


function  TSpaces.GetSpace(aIndex: longInt): TSpaceTree;
begin
   result := TSpaceTree(Elements[aIndex]);
end;

procedure TSpaces.PutSpace(aIndex: longInt; aSpaceTree: TSpaceTree);
begin
   Elements[aIndex] := aSpaceTree;
end;

function  TSpaces.GetNoSpaces: boolean;
begin
   result := NoElements;
end;

function  TSpaces.FindSpaceByCoord(x, y: double): TSpaceTree;
var
   i: longInt;
   Sp: TSpaceTree;
   R: TRectangle;
begin
   result := nil;
   if NoElements then exit;
   for i := 0 to Count -1 do
   begin
      Sp := Space[i];
      R  := Sp.Rect;
      if (y > R.MinY) and (y < R.MaxY) then
      begin
         result := Sp;
         exit;
      end;
   end;
end;


function  TSpaces.IsIn(aCoord: TDoubleCoord): boolean;
begin
   result := (aCoord.X > Rect.MinX) and (aCoord.X < Rect.MaxX);
end;

procedure TSpaces.AddSpace(aSpace: TSpaceTree);
var
   i: longInt;
   Coord: TDoubleCoord;
begin
   Coord := aSpace.Rect.CenterCoord;
   Rect.UpdateByRectangle(aSpace.Rect);
   for i := 0 to Count - 1 do
   begin
      if Coord.Y < Space[i].Rect.MinY then
      begin
         InsertElement(i, aSpace);
         exit;
      end;
   end;
   AddElement(aSpace);
end;


{ TSpaceMatrix }

procedure TSpaceMatrix.UpdateRect;
var
   i: longInt;
   aColumn: TSpaces;
begin
   Rect.Reset;
   for i := Count -1 downto 0 do
   begin
      aColumn := Column[i];
      aColumn.UpdateRect;
      Rect.UpdatebyRectangle(aColumn.Rect)
   end;
end;


procedure TSpaceMatrix.FreeEmptySpaces;
var
   i: longInt;
   aColumn: TSpaces;
begin
   for i := Count -1 downto 0 do
   begin
      aColumn := Column[i];
      aColumn.FreeEmptySpaces;
      if aColumn.NoSpaces then FreeElement(aColumn);
   end;
end;


function  TSpaceMatrix.GetColumn(aIndex: longInt): TSpaces;
begin
   result := TSpaces(Elements[aIndex]);
end;


procedure TSpaceMatrix.PutColumn(aIndex: longInt; aColumn: TSpaces);
begin
   Elements[aIndex] := aColumn;
end;

function  TSpaceMatrix.GetNoSpaces: boolean;
var
   //Col: TSpaces;
   i: longInt;
begin
   result := NoElements;
   if not result then
   begin
      for i := 0 to Count -1 do
      begin
         result := Column[i].NoSpaces;
         if not result then exit;
      end;
   end;
end;



function  TSpaceMatrix.FindColumnBySpace(aSpace: TSpaceTree): TSpaces;
var
   i: longInt;
   Coord: TDoubleCoord;
   Col: TSpaces;
begin
   result := nil;
   Coord := aSpace.Rect.CenterCoord;
   for i := 0 to Count - 1 do
   begin
      Col := Column[i];
      if Col.IsIn(Coord) then
      begin
         result := Col;
         exit;
      end;
   end;
end;


function  TSpaceMatrix.AddColumnBySpace(aSpace: TSpaceTree): TSpaces;
var
   i: longInt;
   Coord: TDoubleCoord;
   Col: TSpaces;
begin
   result := TSpaces.Create;
   result.Rect.Assign(aSpace.Rect);
   Coord := aSpace.Rect.CenterCoord;
   for i := 0 to Count - 1 do
   begin
      Col := Column[i];
      if (Coord.X < Col.Rect.MinX) then
      begin
         InsertElement(i, result);
         exit;
      end;
   end;
   AddElement(result);
end;


function  TSpaceMatrix.FindColumnByCoord(x, y: double): TSpaces;
var
   i: longInt;
   Col: TSpaces;
   R: TRectangle;
begin
   result := nil;
   if NoElements then exit;
   for i := 0 to Count -1 do
   begin
      Col := Column[i];
      R := Col.Rect;
      if (x > R.MinX) and (x < R.MaxX) then
      begin
         result := Col;
         exit;
      end;
   end;
end;


procedure TSpaceMatrix.AddSpace(aSpace: TSpaceTree);
var
   aColumn: TSpaces;
begin
   aColumn := FindColumnBySpace(aSpace);
   if not assigned(aColumn) then aColumn := AddColumnBySpace(aSpace);
   aColumn.AddSpace(aSpace);
   Rect.UpdateByRectangle(aColumn.Rect);
end;




{ TSpaceTree }



procedure TSpaceTree.UpdateRect;
begin
   Rect.Reset;
   if NoSpaces then exit;
   Matrix.UpdateRect;
   Rect.UpdateByRectangle(Matrix.Rect);
end;



procedure TSpaceTree.FreeEmptySpaces;
begin
   if NoSpaces then exit;
   Matrix.FreeEmptySpaces;
end;



procedure  TSpaceTree.Load(aReader: TReader; aIndex: TEntity);
var
   i: longInt;
begin
   inherited Load(aReader, aIndex);
   i := aReader.ReadInteger;
   if i > -1 then Matrix := TSpaceMatrix(aIndex.Elements[i])
             else Matrix := nil;
end;


procedure  TSpaceTree.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   if Matrix <> nil then aWriter.Writeinteger(Matrix.ID)
                    else aWriter.Writeinteger(-1);
end;


function  TSpaceTree.FindSpaceByCoord(x, y: double): TSpaceTree;
var
   Column: TSpaces;
begin
   result := nil;
   try
      Column := Matrix.FindColumnByCoord(x, y);
      if Column = nil then exit;
      result := Column.FindSpaceByCoord(x, y);
   except
      result := nil;
   end;
end;


function  TSpaceTree.FindSpaceByCoordAndClass(x, y: double; aClass: TClass): TSpaceTree;
var
  I: longInt;
  ST: TSpaceTree;
begin
   result := nil;
   try
     for I := Count-1 Downto 0 do
     begin
       if not (Elements[I] is TSpaceTree) then continue;
       ST:= TSpaceTree(Elements[I]);
       if not ST.PointIsInside(X, Y) then continue;

       if ST.Count > 0 then
       begin
          result:= ST.FindSpaceByCoordAndClass(x,y, aClass);
          if result <> nil then exit
       end;

       if (TElement(ST) is aClass) and ST.PointIsInside(X,Y) then
       begin
         result:= ST;
         exit
       end
     end
   except
   end;
end;





function  TSpaceTree.GetColumn(aIndex: longInt): TSpaces;
begin
   result := Matrix.Column[aIndex];
end;



function  TSpaceTree.GetColumnCount: longInt;
begin
   result := Matrix.Count;
end;



function  TSpaceTree.GetSpace(aCol, aRow: longInt): TSpaceTree;
begin
   try
      result := Column[aCol].Space[aRow];
   except
      result := nil;
   end;
end;


function  TSpaceTree.StoreMatrix: boolean;
begin
   result := FMatrix <> nil;
end;


function  TSpaceTree.GetMatrix: TSpaceMatrix;
begin
   if not assigned(FMatrix) then
   begin
      FMatrix := TSpacematrix.Create;
      AddElement(FMatrix);
   end;
   result := FMatrix;
end;



procedure TSpaceTree.SetMatrix(aMatrix: TSpaceMatrix);
begin
   if assigned(FMatrix) then
   begin
       FreeElement(FMatrix);
       FMatrix := nil;
   end;
   FMatrix := aMatrix;
end;



function  TSpaceTree.GetNoSpaces: boolean;
begin
   result := not Assigned(FMatrix) or (Matrix.NoSpaces);
end;


function  TSpaceTree.FindLevelSpace(x, y: double; aLevel: longInt): TSpaceTree;
begin
   result := nil;
   if PointIsInside(x, y) then result := Self;
   try
      while longBool(aLevel) do
      begin
         result := result.SpaceByCoord[x, y];
         dec(aLevel);
      end;
   except
      result := nil;
   end;
end;


procedure TSpaceTree.AddSpace(aSpace: TSpaceTree);
begin
   Matrix.AddSpace(aSpace);
   Rect.UpdateByRectangle(Matrix.Rect);
end;



INITIALIZATION
   RegisterOrbisClass(TSpaces);
   RegisterOrbisClass(TSpaceMatrix);
   RegisterOrbisClass(TSpaceTree);
END.
