unit I_view;
{****************************************************************************}
{                                                                            }
{          ORBIS ... Interpret Viewu                                         }
{                    (c) 1996                                                }
{                                                                            }
{****************************************************************************}

interface
uses
   Wintypes,
   Elements,
   e_IScrpt,
   e_View,
   g_Elems,
   i_Uni,
   i_Basic;

type
TViewInterpret = class(TInterpret)
private
   FUI: TUniversalInterpret;
   function  GetInterpretScript: TInterpretScript;
   function  GetUI: TUniversalInterpret;
   procedure SetInterpretScript(aValue: TInterpretScript);
protected
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   function  IsItsClass(AObject: TObject): boolean; override;
published
   property  InterpretScript: TInterpretScript read GetInterpretScript write SetInterpretScript;
   property  Interpret: TUniversalInterpret read GetUI;
end;



implementation
uses
   SysUtils,
   Graphics,
   M_Orbis,
   e_Class,
   {!t_Main,}
   t_TisCre,
   Tools,
   Worlds;

function TViewInterpret.GetInterpretScript: TInterpretScript;
var
   aTool: TUniversalTopolScriptCreator;
begin
   if Interpret.Script = nil then
   begin
       aTool := TUniversalTopolScriptCreator.Create;
       aTool.DoIt;
       Interpret.Script := TInterpretScript(aTool.Output);
       aTool.Free;
   end;
   result := Interpret.Script;
end;


procedure TViewInterpret.SetInterpretScript(aValue: TInterpretScript);
begin
   if (TElement(aValue) is TInterpretScript) then Interpret.Script := aValue;
end;


function  TViewInterpret.GetUI: TUniversalInterpret;
begin
   if FUI = nil then
   begin
      FUI := TUniversalInterpret.Create;
      NextInterpret := FUI;
   end;
   result := FUI;
end;


function  TViewInterpret.QGoInside(Element: TElement): boolean;
begin
  Result:= (Element is TEntity) and not ((Element is TView)or(Element is TText));
end;

function  TViewInterpret.QProcessBefore(Element: TElement): boolean;
begin
  Result:= (Element is TView);
end;


procedure TViewInterpret.Process(Element: TElement);
var
  IX, IY: double;
  I: longInt;
  oAngle: double;
  oScale,NScale: double;
  oCenterPoint: TDoubleCoord;
  oVirtualScale: double;
  oClipModelView: TDoubleRect;
  oClipping: boolean;
  oCenterToClipRegion: boolean;
  R: TDoubleRect;
  RClip,oClipRegion: TRect;
  PCenter: TPoint;
  PCenterD: TDoubleCoord;
  iShift: TPoint;
  DShift: TDoubleCoord;
begin
  if (Element is TView) then
  begin
     Device.Brush.Color := clWhite;
     Device.Brush.Style := bsSolid;
     Device.Pen.Width := 0;
     Device.Pen.Style:= psDot;
     Device.Pen.Color:= clSilver;
     with TView(Element).Rect do
     begin
{        Device.Rectangle(Min.x,Min.Y,Max.X,Max.Y); }
        // Zjitn prniku view a zazen
        oClipRegion := Device.Page.ClipRegion;
        R := Device.Projection.ClipModelView;
        oClipModelView := R;
        oClipping := Device.Page.Clipping;
        oAngle := Device.Projection.Angle;
        oCenterToClipRegion := Device.Page.CenterToClipRegion;

        if not TView(Element).IsOverRegion(R.Minx, R.MinY, R.MaxX, R.MaxY) then exit;
        R := TView(Element).IntersectByRect(R);
        RClip := Device.Projection.GetPageRect(R.Minx, R.MinY, R.MaxX, R.MaxY);
        //oClipRegion := RClip;
        PCenter := Device.Projection.GetPagePoint(R.Minx+(R.MaxX-R.MinX)/2,R.Miny+(R.Maxy-R.Miny)/2);

        Device.Page.ClipRegion := Device.Projection.GetPageRect(Min.x,Min.Y,Max.X,Max.Y);
        Device.Page.Clipping := true;
        Device.Page.CenterToClipRegion := true;
        if InterpretScript = nil then;
        Interpret.Printing := Printing;
        Interpret.Input := TView(Element).Model;
        Interpret.Device := Device;
        oScale := Device.Projection.Scale;
        oCenterPoint := Device.Projection.CenterPoint;
        oVirtualScale:= Device.Projection.VirtualScale;

        TView(Element).ModelCenterX:= TView(Element).ModelCenterX;
        TView(Element).ModelCenterY:= TView(Element).ModelCenterY;
        NScale := TView(Element).Projection.Scale*OScale;
        Device.Projection.ChangeProjection(TView(Element).Projection.CenterPoint,
                                           NScale, Device.Page);
        Device.Projection.Angle := TView(Element).Projection.Angle;
        Device.Projection.RotateCenter_x := TView(Element).Projection.RotateCenter_x;
        Device.Projection.RotateCenter_y := TView(Element).Projection.RotateCenter_y;
        Device.Projection.VirtualScale:= TView(Element).Projection.VirtualScale;
        // prava oezn
        PCenterD := Device.Projection.GetModelPoint(PCenter.X,PCenter.Y);
        Device.Page.ClipRegion := RClip;
        Device.Projection.ChangeProjection(PCenterD, NScale, Device.Page);
        {
        PostTextMsg(tm_AddToProtocol,'VIEW: '+Element.Name,nil);
        PostTextMsg(tm_AddToProtocol,' Center: '+ FloatToStr(oCenterPoint.X)+' : '+ FloatToStr(oCenterPoint.Y),nil);
        PostTextMsg(tm_AddToProtocol,' Scale: '+ FloatToStr(oScale),nil);
        PostTextMsg(tm_AddToProtocol,' Clip: '+ FloatToStr(oClipModelView.MinX)+' '+FloatToStr(oClipModelView.MinY)+' '+FloatToStr(oClipModelView.MaxX)+' '+FloatToStr(oClipModelView.MaxY),nil);
        }

        try
          if Interpret.Input <> nil then
          begin
             Interpret.Reset(Device);
             Interpret.DoIt;
          end;
        finally
          Device.Projection.Page.ClipRegion := oClipRegion;
          Device.Page.Clipping := oClipping;
          Device.Page.CenterToClipRegion := oCenterToClipRegion;
          Device.Projection.ChangeProjection(oCenterPoint, oScale, Device.Page);
          Device.Projection.Angle := oAngle;
          Device.Projection.VirtualScale:= oVirtualScale;
          //Device.Projection.ClipModelView:= oClipModelView;
          {
          PostTextMsg(tm_AddToProtocol,' Center: '+ Device.Projection.GetPropertyStringByName('Center_X')+' : '+ Device.Projection.GetPropertyStringByName('Center_Y'),nil);
          PostTextMsg(tm_AddToProtocol,' Scale: '+ FloatToStr(Device.Projection.Scale),nil);
          R := Device.Projection.ClipModelView;
          PostTextMsg(tm_AddToProtocol,' Clip: '+ FloatToStr(R.MinX)+' '+FloatToStr(R.MinY)+' '+FloatToStr(R.MaxX)+' '+FloatToStr(R.MaxY),nil);
          PostTextMsg(tm_AddToProtocol,'************ END ',nil);
          }
        end
     end;
  end;
end;



function TViewInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := AObject is TView;
end;

BEGIN
   RegisterOrbisClass(TViewInterpret);
END.
