unit T_tiscre;
{
************************************************************************
**                                                                    **
**  ORBIS ... Nstroj pro vrobu obecnho Interpretanho pedpisu    **
**            pro Topolsk model                                      **
**                                                                    **
**                         Topol Pro 1996                            **
**                                                                    **
************************************************************************
}


interface
uses
// KERNEL
   Elements,
   Tools,
// INTERPRET
   e_IScrpt;


type

TUniversalTopolScriptCreator = class(TTool)
  private
    FNoRect: boolean;
    procedure SetNoRect(const Value: boolean);
 public
    procedure DoIt; override;
    property NoRect: boolean read FNoRect write SetNoRect;
end;

implementation
uses
   e_TileLayer,
   i_TileLayer,
   e_WMS,
   i_WMS,
// KERNEL
   e_Class,
// GLIB
   g_Elems,
// SPACE
   e_Annotation,
   i_Annotation,
   e_Geometry,
   e_GeometryElement,
   e_GeometryEx,
   i_GeometryEx,
   //e_Space,
   e_MSpace,
// INTERPRET
   i_Basic,
// MAPPER
   e_Desktp,
   i_Desktp,
   i_MSpace,
   I_Krizku,
// RASTER
{$IFDEF RAS}
   e_Raster,
   i_Raster,
{$ENDIF}
   e_LHCModel,
   e_LHCElements,
   e_KatastrModel,
   i_LHCElements,
   i_KatastrElements,
   i_PolyLineElement,
// TOPOL
   e_TopoLElements,
   i_Top_B,
   i_Top_L,
   i_Top_P,
   i_Top_T;


procedure TUniversalTopolScriptCreator.DoIt;
var
   Scr: TInterpretScript;
   i: TInterpret;
begin
   inherited DoIt;
   { Script }
   Scr := TInterpretScript.Create;
   Scr.NoRect := NoRect;
   Scr.ElementClass := TElement;
   Output := Scr;

   { TRectEntity }
   Scr.NewClass := TRectEntity;
   Scr.SetNewClassAndInterpret(TCross, TInterpretKrizku.Create);
   Scr.SetNewClassAndInterpret(TLines, TLinesInterpret.Create);
{$IFDEF RAS}
   if UseRasters then
   begin
   try
      Scr.SetNewClassAndInterpret(TRaster, TRasterInterpret.Create);
   except
      UseRasters := false;
   end;
   end;
{$ENDIF}

     i := TInterpretTopTextu.Create;
     Scr.SetNewClassAndInterpret(TTopoLTextElement, i);
     Scr.SetNewClassAndInterpret(TTextCollection, i);

     i := TInterpretTopBodu.Create;
     Scr.SetNewClassAndInterpret(TTopoLPointElement, i);
     Scr.SetNewClassAndInterpret(TPointCollection, i);

     i := TInterpretTopPloch.Create;
     Scr.SetNewClassAndInterpret(TTopoLAreaElement, i);
     Scr.SetNewClassAndInterpret(TAreaCollection, i);

     i := TInterpretTopLinii.Create;
     Scr.SetNewClassAndInterpret(TTopoLLineElement, i);
     Scr.SetNewClassAndInterpret(TPolyLineCollection, i);

     Scr.SetNewClassAndInterpret(TGeometryTextAnnotation, TGeometryTextAnnotationInterpret.Create);

     Scr.SetNewClassAndInterpret(Tgis_Note, Tgis_NoteInterpret.Create);

     Scr.SetNewClassAndInterpret(Tgis_Parcela, Tgis_ParcelaInterpret.Create);

     Scr.SetNewClassAndInterpret(Tgis_ParcelaCast, Tgis_ParcelaCastInterpret.Create);

     Scr.SetNewClassAndInterpret(Tgis_SkupinaParcel, Tgis_SkupinaParcelInterpret.Create);

     Scr.SetNewClassAndInterpret(Tgis_LS, Tgis_LSInterpret.Create);

     Scr.SetNewClassAndInterpret(Tgis_Oddeleni, Tgis_OddeleniInterpret.Create);

     Scr.SetNewClassAndInterpret(Tgis_Porost, Tgis_PorostInterpret.Create);

     Scr.SetNewClassAndInterpret(Tgis_PorostniSkupina, Tgis_PorostniSkupinaInterpret.Create);

     Scr.SetNewClassAndInterpret(TSRFPlocha, TSRFPlochaInterpret.Create);

     Scr.SetNewClassAndInterpret(TTEZPlocha, TTEZPlochaInterpret.Create);

     Scr.SetNewClassAndInterpret(Tgis_LT, Tgis_LTInterpret.Create);

     Scr.SetNewClassAndInterpret(TPolyLineElement, TPolyLineElementInterpret.Create);

     Scr.SetNewClassAndInterpret(TDesktop, TDesktopInterpret.Create);

     Scr.SetNewClassAndInterpret(TSMO_5, TSMO_Interpret.Create);

     Scr.SetNewClassAndInterpret(TGis_OrbisTilesLayer, TGis_OrbisTilesLayerInterpret.Create);

     Scr.SetNewClassAndInterpret(Tgis_WMSLayer, Tgis_WMSLayerInterpret.Create);
end;
(*
procedure TUniversalTopolScriptCreator.DoIt;
var
   ElemClass, EC: TISClass;
   i: TInterpret;
begin
   inherited DoIt;
   { Script }
   ElemClass := TInterpretScript.Create;
   ElemClass.ElementClass := TElement;
   Output := ElemClass;
   { TRectEntity }
   EC := ElemClass.CreateAndAddClass(TRectEntity, nil);
   //ElemClass.CreateAndAddClass(TTopolText, TInterpretTopTextu.Create);
   ElemClass.CreateAndAddClass(TCross, TInterpretKrizku.Create);
   {
   ElemClass.CreateAndAddClass(TMapsZM, TInterpretZM.Create);
   }
   ElemClass.CreateAndAddClass(TLines, TLinesInterpret.Create);
   { Iside RectEntity }
     ElemClass := EC;
{$IFDEF RAS}
     if UseRasters then
        ElemClass.CreateAndAddClass(TRaster, TRasterInterpret.Create);
{$ENDIF}
{ !!!!!!!!! }
     //ElemClass.CreateAndAddClass(TLayerScript, TInterpretLayerScript.Create);
     //ElemClass.CreateAndAddClass(TBMPFileElement, TBMPFileInterpret.Create);

     i := TInterpretTopTextu.Create;
     ElemClass.CreateAndAddClass(TTopoLTextElement, i);
     ElemClass.CreateAndAddClass(TTextCollection, i);

     i := TInterpretTopBodu.Create;
     ElemClass.CreateAndAddClass(TTopoLPointElement, i);
     ElemClass.CreateAndAddClass(TPointCollection, i);

     i := TInterpretTopPloch.Create;
     ElemClass.CreateAndAddClass(TTopoLAreaElement, i);
     ElemClass.CreateAndAddClass(TAreaCollection, i);

     i := TInterpretTopLinii.Create;
     ElemClass.CreateAndAddClass(TTopoLLineElement, i);
     ElemClass.CreateAndAddClass(TPolyLineCollection, i);
{$IFDEF BINRAS}
     //ElemClass.CreateAndAddClass(TTopRas, TInterpretTopRastru.Create);
{$ENDIF}

     ElemClass.CreateAndAddClass(TDesktop, TDesktopInterpret.Create);
     //ElemClass.CreateAndAddClass(TDrawRectangle, TDrawRectInterpret.Create);
     //EC := ElemClass.CreateAndAddClass(TMapSpace, nil);
     { Inside TMapSpace }
       //ElemClass := EC;
    {
       Interpret := TGuideInterpret.Create;
       TGuideInterpret(Interpret).Show_5 := true;
       TGuideInterpret(Interpret).Show_Text := true;
       TGuideInterpret(Interpret).Fill_Page  := true;
    }
       (-
       EC := ElemClass.CreateAndAddClass(TSMO_10, TSMO_10_Interpret.Create);
       { Inside TSMO_10 }
       ElemClass := EC;
       -)

       ElemClass.CreateAndAddClass(TSMO_5, TSMO_Interpret.Create);
end;
*)
procedure TUniversalTopolScriptCreator.SetNoRect(const Value: boolean);
begin
  FNoRect := Value;
end;

initialization
   RegisterOrbisClass(TUniversalTopolScriptCreator);
end.

