unit T_OilOper;

interface
uses
  E_type,
  Tools;

type
TTypeList=array[TType] of boolean;
PTypeList=^TTypeList;

TOilOperator=class(TTechnology)
protected
 FTypeList:PTypeList;
private
 function GetTypeItem(index:TType):boolean;virtual;
 procedure SetTypeItem(index:TType;aValue:boolean);virtual;
 function GetTypeList:PTypeList;virtual;
 procedure SetTypeList(aList:PTypeList);virtual;
 property Typelist:PTypeList read GetTypeList write SetTypeList;
protected
 procedure ClearResults;virtual;
public
 procedure CleanData;virtual;
 property TypeItem[index:TType]:boolean read GetTypeItem write SetTypeItem;
 procedure GetTypeInfo;virtual;
 procedure ClearTypeInfo;virtual;
 function TypeCheck:boolean;virtual;
end;


TOilPtrOperator=class(TOilOperator)
private
 FFirstOperator:TOilOperator;
 FFirstResult:TTypeElement;
 procedure SetFirstOperator(aOperator:TOilOperator);virtual;
 function GetFirstOperator:TOilOperator;virtual;
 procedure SetFirstResult(aResult:TTypeElement);virtual;
 function GetFirstResult:TTypeElement;virtual;
protected
 procedure ClearResults;override;
 procedure ReadFirstResult;virtual;
 property FirstResult:TTypeElement read GetFirstResult write SetFirstResult;
public
 constructor create;override;
 property FirstOperator:TOilOperator read GetFirstOperator write SetFirstOperator;
end;

TOilTwoPtrOperator=class(TOilPtrOperator)
private
 FSecondOperator:TOilOperator;
 FSecondResult:TTypeElement;
 procedure SetSecondOperator(aOperator:TOilOperator);virtual;
 function GetSecondOperator:TOilOperator;virtual;
 procedure SetSecondResult(aResult:TTypeElement);virtual;
 function GetSecondResult:TTypeElement;virtual;
protected
 procedure ClearResults;override;
 procedure ReadSecondResult;virtual;
 property SecondResult:TTypeElement read GetSecondResult write SetSecondResult;
public
 constructor create;override;
 property SecondOperator:TOilOperator read GetSecondOperator write SetSecondOperator;
end;

TOilConst=class(TOilOperator)
 private
  FConst:TTypeElement;
  procedure SetConst(aConst:TTypeElement);virtual;
 public
  procedure DoIt;override;
  constructor Create ;override;
  procedure GetTypeInfo;override;
  property Constant:TTypeElement read FConst write SetConst;
 end;

TOilUnMinus=class(TOilPtrOperator)
 public
  procedure GetTypeInfo;override;
  procedure DoIt;override;
 end;

 TOilNot=class(TOilPtrOperator)
 public
  procedure GetTypeInfo;override;
  procedure Doit;override;
 end;

 TOilPlus=class(TOilTwoPtrOperator)
  public
   procedure GetTypeInfo;override;
   procedure Doit;override;
  end;

TOilMinus=class(TOilTwoPtrOperator)
  public
   procedure GetTypeInfo;override;
   procedure Doit;override;
  end;

TOilKrat=class(TOilTwoPtrOperator)
  public
   procedure GetTypeInfo;override;
   procedure Doit;override;
  end;

TOilDeleno=class(TOilTwoPtrOperator)
  public
   procedure GetTypeInfo;override;
   procedure Doit;override;
  end;

TOilMod=class(TOilTwoPtrOperator)
  public
   procedure GetTypeInfo;override;
   procedure Doit;override;
  end;

TOilDiv=class(TOilTwoPtrOperator)
  public
   procedure GetTypeInfo;override;
   procedure Doit;override;
  end;


TOilCompare=class(TOilTwoPtrOperator)
  public
  procedure GetTypeInfo;override;
  end;

TOilVetsi=class(TOilCompare)
  public
   procedure Doit;override;
  end;

TOilMensi=class(TOilCompare)
  public
   procedure Doit;override;
  end;

TOilVetsiRovno=class(TOilCompare)
  public
   procedure Doit;override;
  end;

TOilMensiRovno=class(TOilCompare)
  public
   procedure Doit;override;
  end;

TOilRovno=class(TOilCompare)
  public
   procedure Doit;override;
  end;

TOilRuzne=class(TOilCompare)
  public
   procedure Doit;override;
  end;


TOilAnd=class(TOilTwoPtrOperator)
 public
  procedure GetTypeInfo;override;
  procedure DoIt;override;
 end;

TOilOr=class(TOilTwoPtrOperator)
 public
  procedure GetTypeInfo;override;
  procedure DoIt;override;
 end;

 
implementation
uses T_Errors;

{TOilOperator}
procedure TOilOperator.ClearResults;
  begin
  end;

function TOilOperator.GetTypeList:PTypeList;
  begin
  result:=FTypeList;
  end;

procedure TOilOperator.SetTypeList(aList:PTypeList);
  begin
  FTypeList:=aList;
  end;

function TOilOperator.GetTypeItem(index:TType):boolean;
  begin
  result:=TypeList^[index];
  end;

procedure TOilOperator.SetTypeItem(index:TType;aValue:boolean);
  begin
  TypeList^[index]:=aValue;
  end;

procedure TOilOperator.CleanData;
  {pokud je vymazani zakazano, pak je nutno tuto proc. predefinovat}
  begin
   if Output<>nil then if TTypeElement(Output).IsPossibleToFree then Output.free;
   Output:=nil;          {pokud bude Output vzdy novy prvek, nemusel by se kontolovat count}
  end;

procedure TOilOperator.GetTypeInfo;
  begin {pouze pro poradek-jinak by byla abstract}
  new(FTypeList);
  end;

procedure TOilOperator.ClearTypeInfo;
 begin
  dispose(FTypeList);
  FTypeList:=nil;
 end;

function TOilOperator.TypeCheck:boolean;
 var i:TType;
 begin
  GetTypeInfo;
  result:=false;
  for i:=_Boolean to _Other do result:=result or TypeList^[i];
  ClearTypeInfo;
 end;

{OilPtrOperator}
 procedure TOilPtrOperator.SetFirstOperator(aOperator:TOilOperator);
  begin
   if assigned(FFirstOperator) then FreeTool(FFirstOperator);
   if aOperator<>nil then FFirstOperator:=TOilOperator(AddTool(aOperator))
    else FFirstOperator:=nil;
  end;

 function TOilPtrOperator.GetFirstOperator:TOilOperator;
  begin
   result:=FFirstOperator;
  end;

 procedure TOilPtrOperator.SetFirstResult(aResult:TTypeElement);
  begin
   FFirstResult:=aResult;
  end;

 function TOilPtrOperator.GetFirstResult:TTypeElement;
  begin
   result:=FFirstResult;
  end;

constructor TOilPtrOperator.create;
{asi jeto zbytecne Delphi to asi delaj sami}
   begin
    inherited create;
    FFirstOperator:=nil;
   end;

 procedure TOilPtrOperator.ReadFirstResult;
  begin
   if FirstOperator=nil then FirstResult:=nil
   else
    begin
     FirstOperator.DoIt;
     FirstResult:=TTypeElement(FirstOperator.Output);
    end;
  end;

 procedure TOilPtrOperator.ClearResults;
   begin
    if FirstOperator<>nil then FirstOperator.CleanData;
   end;

{OilTwoPtr}
procedure TOilTwoPtrOperator.SetSecondOperator(aOperator:TOilOperator);
   begin
   if assigned(FSecondOperator) then FreeTool(FSecondOperator);
   if aOperator<>nil then FSecondOperator:=TOilOperator(AddTool(aOperator))
    else FSecondOperator:=nil;
   end;

function TOilTwoPtrOperator.GetSecondOperator:TOilOperator;
   begin
   result:=FSecondOperator;
   end;

procedure TOilTwoPtrOperator.SetSecondResult(aResult:TTypeElement);
   begin
   FSecondResult:=aResult;
   end;

function TOilTwoPtrOperator.GetSecondResult:TTypeElement;
   begin
   result:=FSecondResult;
   end;

constructor TOilTwoPtrOperator.create;
   begin
   inherited create;
   FSecondOperator:=nil;
   end;

procedure TOilTwoPtrOperator.ReadSecondResult;
   begin
   if (SecondOperator=nil) or (FirstResult=nil) then SecondResult:=nil
   else
    begin
     SecondOperator.DoIt;
     Secondresult:=TTypeElement(SecondOperator.Output);
    end;
   end;

procedure TOilTwoPtrOperator.ClearResults;
   begin
   if FirstOperator<>nil then FirstOperator.CleanData;
   if SecondOperator<>nil then SecondOperator.CleanData;
   end;


{Const}
procedure TOilConst.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   For i:=_boolean to _other do TypeItem[i]:=false;
   TypeItem[TTypeElement(Constant).Typ]:=true;
  end;

  procedure TOilConst.SetConst(aConst:TTypeElement);
   begin
    if assigned(FConst) then FreeElement(FConst);
    AddElement(aConst);
    FConst:=aConst;
   end;


procedure TOilConst.DoIt;
   begin
    Output:=FConst.SelfCopy;
   end;

 constructor TOilConst.Create;
   begin
    inherited create;
    FConst:=nil;
   end;


{Unarni minus}
procedure TOilUnMinus.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;
   if FirstOperator.TypeItem[_byte] or
     FirstOperator.TypeItem[_integer] then TypeItem[_integer]:=true;
   if FirstOperator.TypeItem[_float] then TypeItem[_float]:=true;
   FirstOperator.ClearTypeInfo;
  end;

procedure TOilUnMinus.Doit;
  var Tmp:TTypeElement;
   begin
     inherited DoIt;
     Tmp:=nil;
     Output:=nil;
     ReadFirstResult;
     if (FirstResult=nil) then
         begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
         end;
     if not (FirstResult.Typ in [_Byte,_Float,_Integer]) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
     case FirstResult.Typ of
        _Byte,_Integer:
          begin
           Tmp:=TIntegerElement.create;
           TIntegerElement(Tmp).Value:=-(TTypeElement(FirstResult).AsInteger);
          end;
        _Float:
          begin
           Tmp:=TFloatElement.create;
           TFloatElement(Tmp).Value:=-(TFloatElement(FirstResult).Value);
          end;
       end;
     Output:=Tmp;
     ClearResults;
   end;


 {Not}
 procedure TOilNot.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;
   if FirstOperator.TypeItem[_boolean] then TypeItem[_boolean]:=true;
   FirstOperator.ClearTypeInfo;
  end;

 procedure TOilNot.Doit;
  var Tmp:TBooleanElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    if (FirstResult=nil) then
        begin
        {kill}
        ClearResults;
        raise TLangError.CreateError(12);
        end;
    if (FirstResult.Typ<>_boolean) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
    Tmp:=TBooleanElement.create;
    Tmp.Value:=not TBooleanElement(FirstResult).Value;
    Output:=Tmp;
    ClearResults;
   end;

 {Plus}
 procedure TOilPlus.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   SecondOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;

   if (FirstOperator.TypeItem[_byte] or FirstOperator.TypeItem[_integer])
    and (SecondOperator.TypeItem[_byte] or SecondOperator.TypeItem[_integer])
      then TypeItem[_integer]:=true;

   if (FirstOperator.TypeItem[_Float]and(SecondOperator.TypeItem[_Byte] or
       SecondOperator.TypeItem[_Integer] or SecondOperator.TypeItem[_Float]))
    or((FirstOperator.TypeItem[_Byte] or FirstOperator.TypeItem[_Integer] or
      FirstOperator.TypeItem[_Float]) and(SecondOperator.TypeItem[_Float]))
     then TypeItem[_float]:=true;

   if (FirstOperator.TypeItem[_string] or FirstOperator.TypeItem[_char])
      and(SecondOperator.TypeItem[_string] or FirstOperator.TypeItem[_char])
     then TypeItem[_string]:=true;

   FirstOperator.ClearTypeInfo;
   SecondOperator.ClearTypeInfo;
  end;

  procedure TOilPlus.Doit;
  var Tmp:TTypeElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
    if (FirstResult=nil) or (SecondResult=nil)then
        begin
        {kill}
         ClearResults;
         raise TLangError.CreateError(12);
        end;
   if (FirstResult.Typ in[_Byte,_Integer])and(SecondResult.Typ in[_Byte,_Integer]) then
    begin
    Tmp:=TIntegerElement.create;
    TIntegerElement(Tmp).Value:=FirstResult.AsInteger+SecondResult.AsInteger;
    end
   else
   if ((FirstResult.Typ=_Float)and(SecondResult.Typ in[_Byte,_Integer,_Float]) or
       (FirstResult.Typ in[_Byte,_Integer,_Float])and(SecondResult.Typ=_Float)) then
   begin
    Tmp:=TFloatElement.create;
    TFloatElement(Tmp).Value:=FirstResult.AsFloat+SecondResult.AsFloat;
   end
   else
   if (FirstResult.Typ in [_String,_Char])and(SecondResult.Typ in [_String,_Char]) then
   begin
    Tmp:=TStringElement.create;
    TStringElement(Tmp).Value:=FirstResult.AsString+SecondResult.AsString;
   end
   else
    begin
     {kill}
     ClearResults;
     raise TLangError.CreateError(3);
    end;
   Output:=Tmp;
   ClearResults;
  end;

{Minus}
procedure TOilMinus.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   SecondOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;

   if (FirstOperator.TypeItem[_byte] or FirstOperator.TypeItem[_integer])
    and (SecondOperator.TypeItem[_byte] or SecondOperator.TypeItem[_integer])
      then TypeItem[_integer]:=true;

   if (FirstOperator.TypeItem[_Float]and(SecondOperator.TypeItem[_Byte] or
    SecondOperator.TypeItem[_Integer] or SecondOperator.TypeItem[_Float]))
    or((FirstOperator.TypeItem[_Byte] or FirstOperator.TypeItem[_Integer] or
     FirstOperator.TypeItem[_Float]) and(SecondOperator.TypeItem[_Float]))
     then TypeItem[_float]:=true;

   FirstOperator.ClearTypeInfo;
   SecondOperator.ClearTypeInfo;
  end;


procedure TOilMinus.Doit;
  var Tmp:TTypeElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
    if (FirstResult=nil) or (SecondResult=nil)then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
        end;
   if (FirstResult.Typ in[_Byte,_Integer])and(SecondResult.Typ in[_Byte,_Integer]) then
    begin
    Tmp:=TIntegerElement.create;
    TIntegerElement(Tmp).Value:=TTypeElement(FirstResult).AsInteger-
                                TTypeElement(SecondResult).AsInteger;
    end
   else
   if ((FirstResult.Typ=_Float)and(SecondResult.Typ in[_Byte,_Integer,_Float]) or
       (FirstResult.Typ in[_Byte,_Integer,_Float])and(SecondResult.Typ=_Float)) then
   begin
    Tmp:=TFloatElement.create;
    TFloatElement(Tmp).Value:=TTypeElement(FirstResult).AsFloat-TTypeElement(SecondResult).AsFloat;
   end
   else
    begin
      {kill}
      ClearResults;
      raise TLangError.CreateError(3);
    end;
   Output:=Tmp;
   ClearResults;
  end;

{Krat}
procedure TOilKrat.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   SecondOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;
   if (FirstOperator.TypeItem[_Byte] or FirstOperator.TypeItem[_Integer] or FirstOperator.TypeItem[_Float])
     and (SecondOperator.TypeItem[_Byte] or SecondOperator.TypeItem[_Integer] or SecondOperator.TypeItem[_Float])
     then
     begin
      TypeItem[_float]:=true;
      if (FirstOperator.TypeItem[_Byte] or FirstOperator.TypeItem[_Integer] )
       and (SecondOperator.TypeItem[_Byte] or SecondOperator.TypeItem[_Integer])
       then TypeItem[_integer]:=true;
     end;
   FirstOperator.ClearTypeInfo;
   SecondOperator.ClearTypeInfo;
  end;


procedure TOilKrat.Doit;
  var Tmp:TTypeElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
    if (FirstResult=nil) or (SecondResult=nil) then
       begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
       end;
    if not ((FirstResult.Typ in [_Byte,_Integer,_Float])
       and (SecondResult.Typ in [_Byte,_Integer,_Float]))then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
   if (FirstResult.Typ =_Float) or (SecondResult.Typ =_Float) then
   begin
   Tmp:=TFloatElement.create;
   TFloatElement(Tmp).Value:=TTypeElement(FirstResult).AsFloat*TTypeElement(SecondResult).AsFloat;
   end
   else
   begin
   if abs(TTypeElement(FirstResult).AsFloat*TTypeElement(SecondResult).AsFloat)>maxlongint then
   begin
   Tmp:=TFloatElement.create;
   TFloatElement(Tmp).Value:=TTypeElement(FirstResult).AsFloat*TTypeElement(SecondResult).AsFloat;
   end
   else
   begin
   Tmp:=TIntegerElement.create;
   TIntegerElement(Tmp).Value:=TTypeElement(FirstResult).AsInteger*TTypeElement(SecondResult).AsInteger;
   end;
   end;
   Output:=Tmp;
   ClearResults;
  end;

 {Deleno}
procedure TOilDeleno.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   SecondOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;
   if (FirstOperator.TypeItem[_Byte] or FirstOperator.TypeItem[_Integer] or FirstOperator.TypeItem[_Float])
     and (SecondOperator.TypeItem[_Byte] or SecondOperator.TypeItem[_Integer] or SecondOperator.TypeItem[_Float])
    then TypeItem[_float]:=true;
   FirstOperator.ClearTypeInfo;
   SecondOperator.ClearTypeInfo;
  end;

procedure TOilDeleno.Doit;
  var Tmp:TFloatElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
    if (FirstResult=nil) or (SecondResult=nil) then
       begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
       end;
    if not ((FirstResult.Typ in [_Byte,_Integer,_Float])
       and (SecondResult.Typ in [_Byte,_Integer,_Float]))then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
   Tmp:=TFloatElement.create;
   Tmp.Value:=TTypeElement(FirstResult).AsFloat/TTypeElement(SecondResult).asFloat;
   Output:=Tmp;
   ClearResults;
  end;

 {Mod}
procedure TOilMod.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   SecondOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;
   if (FirstOperator.TypeItem[_Byte] or FirstOperator.TypeItem[_Integer])
     and (SecondOperator.TypeItem[_Byte] or SecondOperator.TypeItem[_Integer])
    then TypeItem[_integer]:=true;
   FirstOperator.ClearTypeInfo;
   SecondOperator.ClearTypeInfo;
  end;


procedure TOilMod.Doit;
  var Tmp:TIntegerElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
    if (FirstResult=nil) or (SecondResult=nil) then
       begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
       end;
    if not((FirstResult.Typ in [_Byte,_Integer])
       and (SecondResult.Typ in [_Byte,_Integer])) or
       (TTypeElement(Secondresult).AsInteger=0) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
   Tmp:=TIntegerElement.create;
   Tmp.Value:=TTypeElement(FirstResult).AsInteger mod TTypeElement(SecondResult).asInteger;
   Output:=Tmp;
   ClearResults;
  end;

 {Div}
procedure TOilDiv.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   SecondOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;
   if (FirstOperator.TypeItem[_Byte] or FirstOperator.TypeItem[_Integer])
     and (SecondOperator.TypeItem[_Byte] or SecondOperator.TypeItem[_Integer])
    then TypeItem[_integer]:=true;
   FirstOperator.ClearTypeInfo;
   SecondOperator.ClearTypeInfo;
  end;


procedure TOilDiv.Doit;
  var Tmp:TIntegerElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
    if (FirstResult=nil) or (SecondResult=nil) then
       begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
       end;
    if not((FirstResult.Typ in [_Byte,_Integer])
       and (SecondResult.Typ in [_Byte,_Integer]))  or
       (TTypeElement(Secondresult).AsInteger=0)then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
   Tmp:=TIntegerElement.create;
   Tmp.Value:=TTypeElement(FirstResult).AsInteger div TTypeElement(SecondResult).asInteger;
   Output:=Tmp;
   ClearResults;
  end;


 {Compare}
procedure TOilCompare.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   SecondOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;
   if (FirstOperator.TypeItem[_Byte] or FirstOperator.TypeItem[_Integer] or FirstOperator.TypeItem[_Float])
     and (SecondOperator.TypeItem[_Byte] or SecondOperator.TypeItem[_Integer] or SecondOperator.TypeItem[_Float])
     then TypeItem[_boolean]:=true;
   if FirstOperator.TypeItem[_char] and SecondOperator.TypeItem[_char]
     then TypeItem[_boolean]:=true;
   if FirstOperator.TypeItem[_pointer] and SecondOperator.TypeItem[_pointer]
     then TypeItem[_boolean]:=true;
   if FirstOperator.TypeItem[_string] and SecondOperator.TypeItem[_string]
     then TypeItem[_boolean]:=true;
   FirstOperator.ClearTypeInfo;
   SecondOperator.ClearTypeInfo;
  end;

{vetsi}
procedure TOilVetsi.DoIt;
 var Tmp:TBooleanElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
    if (FirstResult=nil) or (SecondResult=nil) then
       begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
       end;
    if ((FirstResult.typ<>SecondResult.typ) and not
        ((FirstResult.Typ in [_Byte,_Integer,_Float])
         and (SecondResult.Typ in [_Byte,_Integer,_Float]))) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
   Tmp:=TBooleanElement.create;
   case FirstResult.Typ of
      _Char:Tmp.Value:=TCharElement(FirstResult).Value>TCharElement(SecondResult).Value;
      _String:Tmp.Value:=TStringElement(FirstResult).Value>TStringElement(SecondResult).Value;
      _Byte,_Float,_Integer:Tmp.Value:=TTypeElement(FirstResult).AsFloat>TTypeElement(SecondResult).AsFloat;
      _Pointer:Tmp.Value:=TTypeElement(FirstResult).AsNativeInt>TTypeElement(SecondResult).AsNativeInt;
         else
           begin
           {kill}
           Tmp.free;
           ClearResults;
           raise TLangError.CreateError(3);
          end;
       end;
   Output:=Tmp;
   ClearResults;
  end;

{vetsirovno}
 procedure TOilVetsiRovno.DoIt;
 var Tmp:TBooleanElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
    if (FirstResult=nil) or (SecondResult=nil) then
       begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
       end;
    if ((FirstResult.typ<>SecondResult.typ) and not
        ((FirstResult.Typ in [_Byte,_Integer,_Float])
         and (SecondResult.Typ in [_Byte,_Integer,_Float]))) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
   Tmp:=TBooleanElement.create;
   case FirstResult.Typ of
      _Char:Tmp.Value:=TCharElement(FirstResult).Value>=TCharElement(SecondResult).Value;
      _String:Tmp.Value:=TStringElement(FirstResult).Value>=TStringElement(SecondResult).Value;
      _Byte,_Float,_Integer:Tmp.Value:=TTypeElement(FirstResult).AsFloat>=TTypeElement(SecondResult).AsFloat;
      _Pointer:Tmp.Value:=TTypeElement(FirstResult).AsNativeInt>=TTypeElement(SecondResult).AsNativeInt;
         else
           begin
           {kill}
           Tmp.free;
           ClearResults;
           raise TLangError.CreateError(3);
          end;
       end;
   Output:=Tmp;
   ClearResults;
  end;

 {Mensi}
 procedure TOilMensi.DoIt;
 var Tmp:TBooleanElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
   if (FirstResult=nil) or (SecondResult=nil) then
       begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
       end;
   if ((FirstResult.typ<>SecondResult.typ) and not
        ((FirstResult.Typ in [_Byte,_Integer,_Float])
         and (SecondResult.Typ in [_Byte,_Integer,_Float]))) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
   Tmp:=TBooleanElement.create;
   case FirstResult.Typ of
      _Char:Tmp.Value:=TCharElement(FirstResult).Value<TCharElement(SecondResult).Value;
      _String:Tmp.Value:=TStringElement(FirstResult).Value<TStringElement(SecondResult).Value;
      _Byte,_Float,_Integer:Tmp.Value:=TTypeElement(FirstResult).AsFloat<TByteElement(SecondResult).AsFloat;
      _Pointer:Tmp.Value:=TTypeElement(FirstResult).AsNativeInt<TTypeElement(SecondResult).AsNativeInt;
         else
           begin
           {kill}
           Tmp.free;
           ClearResults;
           raise TLangError.CreateError(3);
          end;
       end;
   Output:=Tmp;
   ClearResults;
  end;

 {MensiRovno}
 procedure TOilMensiRovno.DoIt;
 var Tmp:TBooleanElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
   if (FirstResult=nil) or (SecondResult=nil) then
      begin
       {kill}
       ClearResults;
       raise TLangError.CreateError(12);
      end;
   if ((FirstResult.typ<>SecondResult.typ) and not
        ((FirstResult.Typ in [_Byte,_Integer,_Float])
         and (SecondResult.Typ in [_Byte,_Integer,_Float]))) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
   Tmp:=TBooleanElement.create;
   case FirstResult.Typ of
      _Char:Tmp.Value:=TCharElement(FirstResult).Value<=TCharElement(SecondResult).Value;
      _String:Tmp.Value:=TStringElement(FirstResult).Value<=TStringElement(SecondResult).Value;
      _Byte,_Float,_Integer:Tmp.Value:=TTypeElement(FirstResult).AsFloat<=TByteElement(SecondResult).AsFloat;
      _Pointer:Tmp.Value:=TTypeElement(FirstResult).AsNativeInt<=TTypeElement(SecondResult).AsNativeInt;
         else
           begin
           {kill}
           Tmp.free;
           ClearResults;
           raise TLangError.CreateError(3);
          end;
       end;
   Output:=Tmp;
   ClearResults;
  end;


{Rovno}
 procedure TOilRovno.DoIt;
 var Tmp:TBooleanElement;
     dif:double;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
    if (FirstResult=nil) or (SecondResult=nil) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
        end;
    if ((FirstResult.typ<>SecondResult.typ) and not
        ((FirstResult.Typ in [_Byte,_Integer,_Float])
         and (SecondResult.Typ in [_Byte,_Integer,_Float]))) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
   Tmp:=TBooleanElement.create;
   case FirstResult.Typ of
      _Boolean:Tmp.Value:=TBooleanElement(FirstResult).Value=TBooleanElement(SecondResult).Value;
      _Char:Tmp.Value:=TCharElement(FirstResult).Value=TCharElement(SecondResult).Value;
      _String:Tmp.Value:=TStringElement(FirstResult).Value=TStringElement(SecondResult).Value;
      _Pointer:Tmp.Value:=TTypeElement(FirstResult).AsNativeInt=TTypeElement(SecondResult).AsNativeInt;
      _Byte,_Float,_Integer:
             begin
             dif:=TTypeElement(FirstResult).AsFloat-TTypeElement(SecondResult).AsFloat;
             Tmp.Value := abs(dif) < 1e-13;
             {
             Tmp.Value:=(abs(dif/TTypeElement(FirstResult).AsFloat)<1e-13) and
              (abs(dif/TTypeElement(SecondResult).AsFloat)<1e-13);
             }
             end;
         else
           begin
           {kill}
           Tmp.free;
           ClearResults;
           raise TLangError.CreateError(3);
          end;
       end;
   Output:=Tmp;
   ClearResults;
  end;
{Ruzne}
procedure TOilRuzne.DoIt;
 var Tmp:TBooleanElement;
  begin
    inherited DoIt;
    Output:=nil;
    ReadFirstResult;
    ReadSecondResult;
    if (FirstResult=nil) or (SecondResult=nil) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(12);
        end;
    if ((FirstResult.typ<>SecondResult.typ) and not
        ((FirstResult.Typ in [_Byte,_Integer,_Float])
         and (SecondResult.Typ in [_Byte,_Integer,_Float]))) then
        begin
         {kill}
         ClearResults;
         raise TLangError.CreateError(3);
        end;
   Tmp:=TBooleanElement.create;
   case FirstResult.Typ of
      _Boolean:Tmp.Value:=TBooleanElement(FirstResult).Value<>TBooleanElement(SecondResult).Value;
      _Char:Tmp.Value:=TCharElement(FirstResult).Value<>TCharElement(SecondResult).Value;
      _String:Tmp.Value:=TStringElement(FirstResult).Value<>TStringElement(SecondResult).Value;
      _Byte,_Float,_Integer:Tmp.Value :=TTypeElement(FirstResult).AsFloat<>TTypeElement(SecondResult).AsFloat;
      _Pointer:Tmp.Value:=TTypeElement(FirstResult).AsNativeInt<>TTypeElement(SecondResult).AsNativeInt;
      else
           begin
           {kill}
           Tmp.free;
           ClearResults;
           raise TLangError.CreateError(3);
          end;
       end;
   Output:=Tmp;
   ClearResults;
  end;


{And}
 procedure TOilAnd.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   SecondOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;
    if FirstOperator.TypeItem[_boolean] and SecondOperator.TypeItem[_boolean]
     then TypeItem[_boolean]:=true;
   FirstOperator.ClearTypeInfo;
   SecondOperator.ClearTypeInfo;
  end;

procedure TOilAnd.DoIt;
var Tmp:TBooleanElement;
 begin
  inherited DoIt;
  Output:=nil;
  Tmp:=nil;
  ReadFirstResult;
  if (FirstResult<>nil) then
     begin
      {kill}
      ClearResults;
      raise TLangError.CreateError(12);
     end;
  if (FirstResult.Typ=_boolean) then
   begin
     Tmp:=TBooleanElement.create;
     if TBooleanElement(FirstResult).Value then
       begin
        ReadSecondResult;
        if (SecondResult<>nil) then
           begin
            {kill}
            ClearResults;
            raise TLangError.CreateError(12);
           end; 
        if (FirstResult.Typ=_boolean) then
             begin
              if TBooleanElement(SecondResult).Value then Tmp.Value:=true
                                                     else Tmp.Value:=false;
             end
             else
             begin
              {kill}
              Tmp.free;
              ClearResults;
              raise TLangError.CreateError(3);
             end;
       end
       else
       Tmp.Value:=false;
    end;
   Output:=Tmp;
   ClearResults;
 end;

{Or}
procedure TOilOr.GetTypeInfo;
  var i:TType;
  begin
   new(FTypeList);
   FirstOperator.GetTypeInfo;
   SecondOperator.GetTypeInfo;
   For i:=_boolean to _other do TypeItem[i]:=false;
    if FirstOperator.TypeItem[_boolean] and SecondOperator.TypeItem[_boolean]
     then TypeItem[_boolean]:=true;
   FirstOperator.ClearTypeInfo;
   SecondOperator.ClearTypeInfo;
  end;

procedure TOilOr.DoIt;
var Tmp:TBooleanElement;
 begin
  inherited DoIt;
  Output:=nil;
  Tmp:=nil;
  ReadFirstResult;
  if (FirstResult<>nil) then
     begin
       {kill}
       ClearResults;
       raise TLangError.CreateError(12);
     end;
  if (FirstResult.Typ=_boolean) then
   begin
     Tmp:=TBooleanElement.create;
     if not TBooleanElement(FirstResult).Value then
       begin
        ReadSecondResult;
        if (SecondResult<>nil) then
           begin
             {kill}
             ClearResults;
             raise TLangError.CreateError(12);
           end;
        if(FirstResult.Typ=_boolean) then
             begin
              if TBooleanElement(SecondResult).Value then Tmp.Value:=true
                                                     else Tmp.Value:=false;
             end
             else
             begin
              {kill}
              Tmp.free;
              ClearResults;
              raise TLangError.CreateError(3);
             end;
       end
       else
       Tmp.Value:=true;
    end;
    Output:=Tmp;
    ClearResults;
 end;

end.
