unit e_RASTER;

interface
uses
   // KERNEL
   Elements,
   // GLIB
   G_Elems,
   e_GFile,
   // RASTER
   Rasdata_TLB,
   TOPOCXLib_TLB;

type
TRasterCollection = class;

TRasterRepository = class(TOwnerEntity)
  private
    FComment: string;
    FActualCollection: TRasterCollection;
    procedure SetComment(const Value: string);
    procedure SetActualCollection(const Value: TRasterCollection);
    procedure SetFindDirectoryAndFilter(const Value: string);
public
   function GetCollectionByDirectoryAndFilter(const Value: string): TRasterCollection; overload;
   function GetCollectionByDirectoryAndFilter(const aDirectory,aFilter: string): TRasterCollection; overload;
published
   property Comment: string read FComment write SetComment;
   property FindDirectoryAndFilter: string write SetFindDirectoryAndFilter stored false;
   property ActualCollection: TRasterCollection read FActualCollection write SetActualCollection stored false;
end;

TRasterCollection = class(TRectEntity)
private
   FColor: integer;
   FFilter: string;
   FDirectory: string;
    FMaxScale: double;
    FVisible: boolean;
    FComment: string;
    FInvert: boolean;
   procedure SetColor(const Value: integer);
   procedure SetDirectory(const Value: string);
   procedure SetFilter(const Value: string);
    procedure SetMaxScale(const Value: double);
    procedure SetComment(const Value: string);
    procedure SetVisible(const Value: boolean);
    function GetDirectoryAndFilter: string;
    procedure SetDirectoryAndFilter(const Value: string);
    procedure SetInvert(const Value: boolean);
published
   property Directory: string read FDirectory write SetDirectory;
   property Filter: string read FFilter write SetFilter;
   property Color: integer read FColor write SetColor;
   property MaxScale: double read FMaxScale write SetMaxScale;
   property Visible: boolean read FVisible write SetVisible;
   property Comment: string read FComment write SetComment;
   property DirectoryAndFilter: string read GetDirectoryAndFilter write SetDirectoryAndFilter stored false;
   property Invert: boolean read FInvert write SetInvert;
end;

TRaster = class(TGraphicFileElement)
private
   FColor: integer;
   FSrvRas: IRaster;
   FViewEntry: IViewEntry;
   FHorizDPI: integer;
   FVertDPI: integer;
    FError: integer;
    FActive: boolean;
    FInvert: boolean;
    function GetErrorText: string;
    procedure SetError(const Value: integer);
    procedure SetActive(const Value: boolean);
    function GetWarmUp: boolean;
    procedure SetWarmUp(const Value: boolean);
    procedure SetInvert(const Value: boolean);
protected
   function GetFileName: string; override;
   procedure SetFileName(aValue: string); override;
   function GetColor: integer;
   procedure SetColor(aValue: integer);
   function GetRCS: integer;
   procedure SetRCS(aValue: integer);
   function  GetFileSize: integer;
   function  GetRasTyp: SmallInt;
   function  GetCompres: SmallInt;
   function  GetTiled: boolean;
   function  GetPixelWidth: integer;
   function  GetPixelHeight: integer;
   function  GetVertRes: double;
   function  GetHorizRes: double;
   function  GetGeoRaster: boolean;
   procedure SetHorizDPI(aValue: integer);
   procedure SetVertDPI(aValue: integer);
public
   constructor  Create; override;
   destructor   Destroy; override;
   property SrvRas: IRaster read FSrvRas write FSrvRas;
   property ViewEntry: IViewEntry read FViewEntry write FViewEntry;
published
   property Active: boolean read FActive write SetActive stored false;
   property WarmUp: boolean read GetWarmUp write SetWarmUp stored false;
   property Color: integer read GetColor write SetColor;
   property RCS: integer read GetRCS write SetRCS;
   property FileSize: integer read GetFileSize stored false;
   property RasTyp: SmallInt read GetRasTyp stored false;
   property Compres: SmallInt read GetCompres stored false;
   property Tiled: boolean read GetTiled stored false;
   property PixelWidth: integer read GetPixelWidth stored false;
   property PixelHeight: integer read GetPixelHeight stored false;
   property VertRes: double read GetVertRes stored false;
   property HorizRes: double read GetHorizRes stored false;
   property GeoRaster: boolean read GetGeoRaster stored false;
   property HorizDPI: integer read FHorizDPI write SetHorizDPI;
   property VertDPI: integer read FVertDPI write SetVertDPI;
   property Error: integer read FError write SetError;
   property ErrorText: string read GetErrorText stored false;
   property Invert: boolean read FInvert write SetInvert;
end;

const
  UseRasters: Boolean = True;
  re_NoRASLib      = 1;
  re_FileNotExists = 2;
  re_FileOpenError = 3;



implementation
uses
   Math,
   SysUtils,
// KERNEL
   e_Class,
   u_String,
// RASTER
   UseLic;

constructor  TRaster.Create;
begin
   inherited;
end;

destructor   TRaster.Destroy;
begin
   {
   try
   if Error <> re_NoRasLib then
      if longBool(SrvRas.IsOpen) then SrvRas.CloseRas(0);
   except
   end;
   SrvRas := nil;
   ViewEntry := nil;
   }
   Active := false;
   inherited;
end;

function  TRaster.GetFileSize: integer;
begin
   if Active then
      result := SrvRas.FileSize
   else
      result := 0;
end;

function  TRaster.GetRasTyp: SmallInt;
begin
   if Active then
      result := SrvRas.RasTyp
   else
      result := 0;
end;

function  TRaster.GetCompres: SmallInt;
begin
   if Active then
      result := SrvRas.Compres
   else
      result := 0;
end;

function  TRaster.GetTiled: boolean;
begin
   if Active then
      result := longBool(SrvRas.Tiled)
   else
      result := false;
end;

function  TRaster.GetPixelWidth: integer;
begin
   if Active then
      result := SrvRas.Width
   else
      result := 0;
end;

function  TRaster.GetPixelHeight: integer;
begin
   if Active then
      result := SrvRas.Height
   else
      result := 0;
end;

function  TRaster.GetVertRes: double;
begin
   if Active then
      result := SrvRas.VertRes
   else
      result := 0;
end;

function  TRaster.GetHorizRes: double;
begin
   if Active then
      result := SrvRas.HorzRes
   else
      result := 0;
end;

function  TRaster.GetGeoRaster: boolean;
begin
   if Active then
      result := not(((SrvRas.y1 = 0) and (SrvRas.x1 = 0))and(((HorizRes = 1) and (VertRes = 1))
                or((HorizRes = 0) and (VertRes = 0))))
   else
      result := false;
end;

procedure TRaster.SetHorizDPI(aValue: integer);
begin
   if not Active then exit;
   if GeoRaster then exit;
   FHorizDPI := aValue;
   if aValue = 0 then Width := SrvRas.dy
   else
   begin
      Width := 0.0254*(PixelWidth/aValue);
   end;
end;

procedure TRaster.SetVertDPI(aValue: integer);
begin
   if not Active then exit;
   if GeoRaster then exit;
   FVertDPI := aValue;
   if aValue = 0 then Height := SrvRas.dx
   else
   begin
      Height := 0.0254*(PixelHeight/aValue);
   end;
end;


function  TRaster.GetFileName: string;
begin
   result := inherited GetFileName;
end;

procedure TRaster.SetFileName(aValue: string);
var
   Act: boolean;
begin
   Act := Active;
   Active := false;
   inherited SetFileName(aValue);
   Active := Act;
end;

function TRaster.GetColor: integer;
begin
   result := FColor;
   //result := ViewEntry.P1;
end;

procedure TRaster.SetColor(aValue: integer);
begin
   FColor := aValue;
   //ViewEntry.P1 := aValue;
end;

function TRaster.GetRCS: integer;
begin
   if Active then
      result := SrvRas.CoordOri
   else
      result := 0;
end;

procedure TRaster.SetRCS(aValue: integer);
begin
   if Active then
      SrvRas.CoordOri := aValue;
end;


function TRaster.GetErrorText: string;
begin
   case Error of
      re_NoRASLib:      result := 'RASLIB error';
      re_FileNotExists: result := 'File not exists';
      re_FileOpenError: result := 'File open error';
   end;
end;

procedure TRaster.SetError(const Value: integer);
begin
  FError := Value;
end;

procedure TRaster.SetActive(const Value: boolean);
var
  S: string;
begin
   if Value then     // aktivace
   begin
      if FActive then exit;
      if not UseRasters then exit;
      if not FileExists(FileName) then
      begin
         Error := re_FileNotExists;
         exit;
      end;
      // faze 1 prevzata z Create
      WarmUp := true;
      if Error > 0 then exit;
      // faze 2 prevzata z SetFileName
      //if longBool(SrvRas.IsOpen) then SrvRas.CloseRas(0);
      SrvRas.FileName := FileName;
      try
      SrvRas.OpenRas(FileName, 1,2);
      except
         Error := re_FileOpenError;
         SrvRas := nil;
         ViewEntry := nil;
         exit;
      end;
      S := SrvRas.ErrorStr;
      FActive := true;
      if GeoRaster then
      begin
         Minx := -SrvRas.y1-SrvRas.dy;
         MinY := -SrvRas.x1-SrvRas.dx;
         Width := SrvRas.dy;
         Height := SrvRas.dx;
          { TODO -oPPK -cRASTER : Tohle je tu jen kvli dotahovn rastr ... jinak mi nesedly }

         Miny := Miny - SrvRas.VertRes/2;
         Minx := Minx - SrvRas.HorzRes/2;
         Maxy := Maxy + SrvRas.VertRes/2;
         Maxx := Maxx + SrvRas.HorzRes/2;
          
      end
      else
      begin
         Minx := SrvRas.y1;
         MinY := SrvRas.x1;
         Width := SrvRas.Width;
         Height := SrvRas.Height;
      end;
   end
   else              // deaktivace
   begin
      WarmUp := false;
      FActive  := false;
   end;
end;

function TRaster.GetWarmUp: boolean;
begin
   result := SrvRas <> nil;
end;

procedure TRaster.SetWarmUp(const Value: boolean);
var
  hr: hresult;
const
  TLRasterLicKey: widestring        = '=0FJA=0C4kVIB4AE0URF=T[:';
begin
   if Value then
   begin
      if SrvRas <> nil then exit; // uz to tam je
      try
      hr:= CreateInprocObjLic('TopoL.Raster', TLRasterLicKey,
                              IRaster, IUnknown(FSrvRas));
      //SrvRas.SoSys := 2;
      ViewEntry := CoViewEntry.Create;
      ViewEntry.Type_ := 1;
      ViewEntry.ViewObj := SrvRas;
      except
         Error := re_NoRASLib;
         SrvRas := nil;
         ViewEntry := nil;
         exit;
      end;
   end
   else
   begin
      if SrvRas <> nil then
      begin
         try
         if Error <> re_NoRasLib then
            if longBool(SrvRas.IsOpen) then SrvRas.CloseRas(0);
         except
         end;
      end;
      SrvRas := nil;
      ViewEntry := nil;
   end;
end;

{ TRasterCollection }

function TRasterCollection.GetDirectoryAndFilter: string;
begin
   result := Directory + '*.'+ Filter;
end;

procedure TRasterCollection.SetColor(const Value: integer);
begin
  FColor := Value;
end;

procedure TRasterCollection.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TRasterCollection.SetDirectory(const Value: string);
begin
   FDirectory := Value;
   CheckBackSlash(FDirectory)
end;

procedure TRasterCollection.SetDirectoryAndFilter(const Value: string);
var
   S, S1: string;
begin
   S := Value;
   S1 := ExtractFileName(S);
   S := ExtractFilePath(S);
   CheckBackSlash(S);
   Directory := S;
   if StringAsTwoParams(S1,'.',S,S1) then
      Filter := UpperCase(S1)
   else
      Filter := '*';
end;

procedure TRasterCollection.SetFilter(const Value: string);
begin
  FFilter := Value;
end;

procedure TRasterCollection.SetInvert(const Value: boolean);
begin
  FInvert := Value;
end;

procedure TRasterCollection.SetMaxScale(const Value: double);
begin
  FMaxScale := Value;
end;

procedure TRasterCollection.SetVisible(const Value: boolean);
begin
  FVisible := Value;
end;

{ TRasterRepository }

function TRasterRepository.GetCollectionByDirectoryAndFilter(const Value: string): TRasterCollection;
var
   i: integer;
   e: TElement;
   RC: TRasterCollection;
begin
   result := nil;
   if ActualCollection <> nil then
   begin
      if AnsiSameText(Value,ActualCollection.DirectoryAndFilter) then
      begin
         result := ActualCollection;
         exit;
      end;
   end;
   ActualCollection := nil;
   for i := 0 to LastIndex do
   begin
      e := Elements[i];
      if not (e is TRasterCollection) then continue;
      RC := TRasterCollection(e);
      if AnsiSameText(Value,RC.DirectoryAndFilter) then
      begin
         ActualCollection := RC;
         result := RC;
         exit;
      end;
   end;
end;

function TRasterRepository.GetCollectionByDirectoryAndFilter(const aDirectory, aFilter: string): TRasterCollection;
var
   i: integer;
   e: TElement;
   RC: TRasterCollection;
   SDir: string;
begin
   SDir := aDirectory;
   CheckBackSlash(SDir);
   result := nil;
   if ActualCollection <> nil then
   begin
      if AnsiSameText(SDir,ActualCollection.Directory) and SameText(aFilter,ActualCollection.Filter) then
      begin
         result := ActualCollection;
         exit;
      end;
   end;
   ActualCollection := nil;
   for i := 0 to LastIndex do
   begin
      e := Elements[i];
      if not (e is TRasterCollection) then continue;
      RC := TRasterCollection(e);
      if AnsiSameText(SDir,RC.Directory) and SameText(aFilter,RC.Filter) then
      begin
         ActualCollection := RC;
         result := RC;
         exit;
      end;
   end;
end;

procedure TRasterRepository.SetActualCollection(
  const Value: TRasterCollection);
begin
  FActualCollection := Value;
end;

procedure TRasterRepository.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TRasterRepository.SetFindDirectoryAndFilter(const Value: string);
begin
  ActualCollection := GetCollectionByDirectoryAndFilter(Value);
end;

procedure TRaster.SetInvert(const Value: boolean);
begin
  FInvert := Value;
end;

initialization
   RegisterOrbisClass(TRaster);
   RegisterOrbisClass(TRasterCollection);
   RegisterOrbisClass(TRasterRepository);
end.
