unit t_CHeck_ARE_LIN;
// kontroly hranic ploch - uzavreni, ztracene linie

interface
uses
   Elements,
   t_Chk,
   t_ChkSp;

type

{
  Hledn hranic ploch, kter nemaj odpovdajc
  TopolLine (resp.GeometryElement) ve vrstv LIN
  ... jsou to ztracen linie tzv. LostLines
}
TCheck_ARE_LIN_LostLines = class (TCheckTool)
private
    FAreas: TEntity;
    FLines: TEntity;
    procedure SetAreas(const Value: TEntity);
    procedure SetLines(const Value: TEntity);
public
   procedure DoIt; override;
   procedure ProcessLine(Le: TElement); virtual;
published
   property Lines: TEntity read FLines write SetLines;
   property Areas: TEntity read FAreas write SetAreas;
end;

// hranice ploch, kter vlastn vcekrt stejnou plochu,
// patrn se vrac samy po sob

TCheck_ARE_LIN_SameAre = class(TCheck_ARE_LIN_LostLines)
public
   procedure ProcessLine(Le: TElement); override;
end;

// hranice ploch, kter jsou hranic vce jak 3 ploch v Areas

TCheck_ARE_LIN_TooManyAres = class(TCheck_ARE_LIN_LostLines)
public
   procedure ProcessLine(Le: TElement); override;
end;

// oteven hranice liniovch ploch ... linie nenavazuj
TCheck_ARE_LIN_OpenShape = class(TCheckSpaceEntity)
  private
    FAreas: TEntity;
    procedure SetAreas(const Value: TEntity);
public
   procedure DoIt; override;
   procedure ProcessArea(Are: TElement); virtual;
published
   property Areas: TEntity read FAreas write SetAreas;
end;

implementation
uses
   Forms,
   Dialogs,
   SysUtils,
   //MathLib,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   t_LineExtractor;

{ TCheck_ARE_LIN_LostLines }

procedure TCheck_ARE_LIN_LostLines.DoIt;
var
   i: integer;
   e: TElement;
   LEX: TLineExtractor;
   WLines: TEntity;
begin

   if (Lines = nil) or Lines.NoElements then
   begin
      ShowMessage(Name + ': vrstva Lines nen definovna nebo je przdn!');
      exit;
   end;
   if Areas = nil then
   begin
      ShowMessage(Name + ': Areas nen definovno!');
      exit;
   end;
   Input := TGeometryElement.Create;
   AddElement(Input);
   StartProgress(Name,0,0);
   progressText := 'hledn hranic ploch...';
   Application.ProcessMessages;
   LEX := TLineExtractor.Create;
   LEX.Input := Areas;
   LEX.NoDuplicates := true;
   LEX.MaxLevel := 5;// maximalni uroven v niz lze najit vlastni linii
   LEX.DoIt;
   WLines := TEntity(LEX.Output);
   AddElement(WLines);
   LEX.Free;
   MaxProgress := WLines.LastIndex;
   progressText := 'Line Checking...';
   Application.ProcessMessages;
   for i := 0 to WLines.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := WLines.Elements[i];
      ProcessLine(e);
   end;
end;


procedure TCheck_ARE_LIN_LostLines.ProcessLine(Le: TElement);
var
   E: TEntity;
   GE: TGEometryElement;
begin
   GE := TGeometryElement(Input);
   E :=  Le.FindClassWithOwnerInOwners(TGeometryElement,Lines);
   if E = nil then  // nen ve vrstv lini
   begin
      GE.Geometry := TGeometry(Le);
      AddErrorElement;
   end;
end;

procedure TCheck_ARE_LIN_LostLines.SetAreas(const Value: TEntity);
begin
  FAreas := Value;
end;


procedure TCheck_ARE_LIN_LostLines.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

{ TCheck_ARE_LIN_SameAre }

procedure TCheck_ARE_LIN_SameAre.ProcessLine(Le: TElement);
var
   O,O1: TEntity;
   GE: TGEometryElement;
   i,j: integer;
   A1,A2: TAreaElement;
   oc: integer;
   E: TEntity;
begin
   GE := TGeometryElement(Input);
   oc := Le.OwnersCount;
   if  oc < 2 then exit;
   for i := 0 to oc - 2 do
   begin
      O := Le.Owners[i];
      if not (O is TAreaELement) then continue;
      A1 := TAreaElement(O);
      if A1.IsHole then
      begin
         A1 := TAreaElement(A1.HoleOwnerArea);
         if A1 = nil then continue;
      end;
      E :=  A1.FindClassWithOwnerInOwners(TGeometryElement,Areas);
      if E = nil then continue;
      for j := i + 1 to oc -1 do
      begin
         O1 := Le.Owners[j];
         if not (O1 is TAreaElement) then continue;
         A2 := TAreaElement(O1);
         if A2.IsHole then
            A2 := TAreaElement(A2.HoleOwnerArea);
         if A1 = A2 then
         begin
            GE.Geometry := TGeometry(Le);
            AddErrorElement;
            exit;
         end;
      end;
   end;
end;

{ TCheck_ARE_LIN_TooManyAres }

procedure TCheck_ARE_LIN_TooManyAres.ProcessLine(Le: TElement);
var
   E,E1,E2,O: TEntity;
   GE: TGEometryElement;
   i: integer;
   A: TAreaElement;
   oc: integer;
begin
   GE := TGeometryElement(Input);
   oc := Le.OwnersCount;
   E1 := nil;
   E2 := nil;
   if  oc < 3 then exit;
   for i := 0 to oc - 1 do
   begin
      O := Le.Owners[i];
      if not (O is TAreaELement) then continue;
      A := TAreaElement(O);
      E := A.FindClassWithOwnerInOwners(TGeometryElement,Areas);
      if E = nil then continue;
      if E1 = nil then
      begin
         E1 := E;
      end
      else
      if E2 = nil then
      begin
         E2 := E;
      end
      else
      begin
         GE.Geometry := TGeometry(Le);
         AddErrorElement;
         exit;
      end;
   end;
end;

{ TCheck_ARE_LIN_OpenShape }

procedure TCheck_ARE_LIN_OpenShape.DoIt;
var
   i,j: integer;
   GE: TGeometryElement;
   e: TElement;
   G: TGeometry;
   A: TAreaElement;
   MA: TMultiAreaElement;
begin
   if Areas = nil then
   begin
      ShowMessage(Name + ': Areas nen definovno!');
      exit;
   end;
   Input := TGeometryElement.Create;
   AddElement(Input);
   StartProgress(Name,0,Areas.LastIndex);
   progressText := 'hledn neuzavench hranic ploch...';
   Application.ProcessMessages;
   for i := 0 to Areas.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      try
      e := Areas.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      if GE.NoGeometry then continue;
      G := GE.Geometry;
      if not (G is TAreaElement) then continue;
      A := TAreaElement(G);
      if (A is TMultiAreaElement) then
      begin
         MA := TMultiAreaElement(A);
         for j := 0 to MA.LastIndex do
         begin
            ProcessArea(MA.AreaElement[j]);
         end;
      end
      else
         ProcessArea(A);
     except
        ShowMessage('Exception at index '+ IntToStr(i));
     end;
   end;
end;

procedure TCheck_ARE_LIN_OpenShape.ProcessArea(Are: TElement);
var
   i,inext: integer;
   GE: TGeometryElement;
   A: TPolyLineAreaElement;
   L,L1: TPolyLineElement;
   B,B1: boolean;
   LX,LY,LX1,LY1: double;
begin
   GE := TGeometryElement(Input);
   if not (Are is TPolyLineAreaElement) then exit;
   A := TPolyLineAreaElement(Are);
   for i := 0 to A.LastIndex do
   begin
      if i = A.LastIndex then
         inext := 0
      else
         inext := i + 1;
      L := TPolyLineElement(A.Elements[i]);
      B := A.LinesOrigines[i];
      L1 := TPolyLineElement(A.Elements[inext]);
      B1 := A.LinesOrigines[inext];
      L.GetOrientedCoord(L.LastIndex,B,LX,LY);
      L1.GetOrientedCoord(0,B1,LX1,LY1);
      if not ((LX = LX1) and (LY = LY1)) then
      begin
         GE.Geometry := A;
         X := LX;
         Y := LY;
         AddErrorElement;
      end;
   end;

end;

procedure TCheck_ARE_LIN_OpenShape.SetAreas(const Value: TEntity);
begin
  FAreas := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCheck_ARE_LIN_LostLines);
   RegisterOrbisClass(TCheck_ARE_LIN_SameAre);
   RegisterOrbisClass(TCheck_ARE_LIN_TooManyAres);
   RegisterOrbisClass(TCheck_ARE_LIN_OpenShape);
END.

