unit t_Check_AreaExists;

interface
uses
   Elements,
   e_Geometry,
   e_GeometryElement,
   e_QTree,
   t_QTreeTools,
   t_Chk;

type

TCHeckAreaExists = class (TCheckTool)
  private
    FAreas: TEntity;
    FLines: TEntity;
    FRefAreas: TEntity;
    FRefQT: TRectEntityQTree;
    FLevelQT: TRectEntityQTree;
    FQTFinder: TFindRectEntityInQTreeByXY;
    FNewAreas: TAreaString;
    FGE: TGeometryElement;
    FMaxArea: double;
    FRefCompareOnly: boolean;
    FEnhStr: string;
    procedure SetAreas(const Value: TEntity);
    procedure SetLevelQT(const Value: TRectEntityQTree);
    procedure SetLines(const Value: TEntity);
    procedure SetRefAreas(const Value: TEntity);
    procedure SetRefQT(const Value: TRectEntityQTree);
    function GetQTFinder: TFindRectEntityInQTreeByXY;
    procedure SetNewAreas(const Value: TAreaString);
    function GetGE: TGeometryElement;
    procedure SetMaxArea(const Value: double);
    procedure SetRefCompareOnly(const Value: boolean);
    procedure SetEnhStr(const Value: string);
protected
  function GetErrorMsg: string; override;
  property EnhStr: string read FEnhStr write SetEnhStr;
public
   procedure DoIt; override;
   procedure PrepareQTree;
   procedure MakeNewAreas;
   procedure ProcessArea(Are: TAreaELement);
   procedure ProcessCompare(aRef: boolean);
   procedure ProcessCompareArea(Are: TAreaELement; aQT: TRectEntityQTree);
   property LevelQT: TRectEntityQTree read FLevelQT write SetLevelQT;
   property RefQT: TRectEntityQTree read FRefQT write SetRefQT;
   property QTFinder: TFindRectEntityInQTreeByXY read GetQTFinder;
   property NewAreas: TAreaString read FNewAreas write SetNewAreas;
   property GE: TGeometryElement read GetGE;
published
   property Lines: TEntity read FLines write SetLines;
   property Areas: TEntity read FAreas write SetAreas;
   property RefAreas: TEntity read FRefAreas write SetRefAreas;
   property MaxArea: double read FMaxArea write SetMaxArea;
   property RefCompareOnly: boolean read FRefCompareOnly write SetRefCompareOnly;
end;

implementation
uses
   Forms,
   Dialogs,
   SysUtils,
   e_Class,
   m_Orbis,
   t_AreMak;

{ TCHeckAreaExists }

procedure TCHeckAreaExists.DoIt;
var
   i: integer;
   e: TElement;
   MA: TMultiAreaElement;
   Are: TAreaElement;
   S: string;
begin
   if not RefCompareOnly then
   if (Lines = nil) or Lines.NoElements then
   begin
      ShowMessage(Name + ': vrstva Lines nen definovna nebo je przdn!');
      exit;
   end;
   if Areas = nil then
   begin
      if MaxArea = 0 then
         PostTextMsg(tm_AddToProtocol,Name + ': Areas nen definovno!',nil);
      //exit;
   end;
   if RefCompareOnly then
   if RefAreas = nil then
   begin
      PostTextMsg(tm_AddToProtocol,Name + ':Ref.Areas nen definovno!',nil);
      //exit;
   end;
   StartProgress(Name,0,0);
   progressText := 'Pracovn struktura ploch';
   Application.ProcessMessages;
   PrepareQTree;
   S := 'Hledn chybjcch ploch ...';
   if not RefCompareOnly then
   begin
      progressText := 'Pracovn peplochovn...';
      Application.ProcessMessages;
      MakeNewAreas;
      MaxProgress := NewAreas.LastIndex;
      progressText := S;
      Application.ProcessMessages;
   end;
   if RefCompareOnly then
   begin
      progressText := S+RefAreas.Name;
      Application.ProcessMessages;
      EnhStr := RefAreas.Name;
      ProcessCompare(true);
      progressText := S+Areas.Name;
      Application.ProcessMessages;
      EnhStr := Areas.Name;
      ProcessCompare(false);
   end
   else
   for i := 0 to NewAreas.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := NewAreas.Elements[i];
      if not (e is TAreaElement) then continue;
      Are := TAreaElement(e);
      GE.Geometry := Are;
      ProcessArea(Are);
   end;
end;

function TCHeckAreaExists.GetErrorMsg: string;
begin
   if GE.NoGeometry then
      result := 'Missing Are'
   else
      result := EnhStr+' Are: '+GE.Geometry.GetPropertyStringByName('Area');
end;

function TCHeckAreaExists.GetGE: TGeometryElement;
begin
   if FGE = nil then
   begin
      FGE := TGeometryElement.Create;
      FGE.Name := 'G_E';
      AddElement(FGE);
      Input := FGE;
   end;
   result := FGE;
end;

function TCHeckAreaExists.GetQTFinder: TFindRectEntityInQTreeByXY;
begin
   if FQTFinder = nil then
   begin
      FQTFinder := TFindRectEntityInQTreeByXY.Create;
      AddElement(FQTFinder);
   end;
   result := FQTFinder;
end;

procedure TCHeckAreaExists.MakeNewAreas;
var
   AFL: TAreasFromClearLines;
begin
   AFL := TAreasFromClearLines.Create;
   AFL.Input := Lines;
   AFL.DoIt;
   NewAreas := TAreaString(AFL.Output);
   AFL.Free;
end;

procedure TCHeckAreaExists.PrepareQTree;
var
   TQ: TAddEntityToQTree;
begin
   if Areas <> nil then
   begin
      LevelQT :=TRectEntityQTree.Create;
      LevelQT.Name := 'LEVEL QT';
      TQ := TAddEntityToQTree.Create;
      TQ.Input := Areas;
      TQ.Output := LevelQT;
      TQ.DoIt;
      TQ.Free;
   end;
   if RefAreas = nil then exit;
   if RefAreas.NoElements then exit;
   RefQT :=TRectEntityQTree.Create;
   RefQT.Name := 'Ref QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := RefAreas;
   TQ.Output := RefQT;
   TQ.DoIt;
   TQ.Free;
end;

procedure TCHeckAreaExists.ProcessArea(Are: TAreaELement);
var
   RefFound: boolean;
   aGE: TGeometryElement;
begin
   RefFound := false;
   EnhStr := '';
   aGE := nil;
   if LevelQT <> nil then // normalni kontrola ploch
   begin
      QTFinder.Input := LevelQT;
      QTFinder.X := Are.X;
      QTFInder.Y := Are.Y;
      QTFinder.Tolerance := 0;
      QTFinder.DoIt;
      if QTFinder.Output <> nil then exit;
      if RefQT <> nil then
      begin
         QTFinder.Input := RefQT;
         QTFinder.X := Are.X;
         QTFInder.Y := Are.Y;
         QTFinder.Tolerance := 0;
         QTFinder.DoIt;
         if QTFinder.Output = nil then exit;
      end;
   end
   else                 // male plochy s komparaci
   if RefQT <> nil then
   begin
      QTFinder.Input := RefQT;
      QTFinder.X := Are.X;
      QTFInder.Y := Are.Y;
      QTFinder.Tolerance := 0;
      QTFinder.DoIt;
      if QTFinder.Output <> nil then  // nejaka plocha uz tam je
      begin
         RefFound := true;
         aGE := TGeometryElement(QTFinder.Output);
      end;
   end;
   if MaxArea > 0 then
   begin
      if Are.Area > MaxArea then exit;
      if RefFound then
      begin
         GE.Geometry := aGE.GetSimpleGeometryByXY(Are.X,Are.Y,0);
         EnhStr := 'Exists';
      end
      else
         EnhStr := 'Hole ';
   end;
   AddErrorElement;
   ProgressText := IntToStr(Errors.Count);
   Application.ProcessMessages;
end;

procedure TCHeckAreaExists.ProcessCompare(aRef: boolean);
var
   i,j: integer;
   e: TElement;
   aGE: TGeometryElement;
   A: TAreaElement;
   MA: TMultiAreaElement;
   Lay: TEntity;
   QT: TRectEntityQTree;
begin
   if aRef then
   begin
      Lay := Areas;
      QT := RefQT;
   end
   else
   begin
      Lay := RefAreas;
      QT := LevelQT;
   end;
   Progress := 0;
   MaxProgress := Lay.LastIndex;
   Application.ProcessMessages;
   for i := 0 to Lay.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Lay.Elements[i];
      if not (e is TGeometryElement) then continue;
      aGE := TGeometryElement(e);
      if aGE.NoGeometry then continue;
      if not (aGE.Geometry is TAreaElement) then continue;
      A := TAreaElement(aGE.Geometry);
      if A is TMultiAreaElement then
      begin
         MA := TMultiAreaElement(A);
         for j := 0 to MA.LastIndex do
         begin
            A := MA.AreaElement[j];
            ProcessCompareArea(A,QT);
         end;
      end
      else
         ProcessCompareArea(A,QT);
   end;
end;

procedure TCHeckAreaExists.ProcessCompareArea(Are: TAreaELement; aQT: TRectEntityQTree);
begin
   if aQT <> nil then
   begin
      QTFinder.Input := aQT;
      QTFinder.X := Are.X;
      QTFInder.Y := Are.Y;
      QTFinder.Tolerance := 0;
      QTFinder.DoIt;
      if QTFinder.Output <> nil then exit;
   end;
   if MaxArea > 0 then
      if Are.Area > MaxArea then exit;
   GE.Geometry := Are;
   AddErrorElement;
   ProgressText := EnhStr+' '+ IntToStr(Errors.Count);
   Application.ProcessMessages;
end;

procedure TCHeckAreaExists.SetAreas(const Value: TEntity);
begin
  FAreas := Value;
end;

procedure TCHeckAreaExists.SetEnhStr(const Value: string);
begin
  FEnhStr := Value;
end;

procedure TCHeckAreaExists.SetLevelQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLevelQT), Value);
end;

procedure TCHeckAreaExists.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

procedure TCHeckAreaExists.SetMaxArea(const Value: double);
begin
  FMaxArea := Value;
end;

procedure TCHeckAreaExists.SetNewAreas(const Value: TAreaString);
begin
   ExchangeFieldElement(TElement(FNewAreas), Value);
end;

procedure TCHeckAreaExists.SetRefAreas(const Value: TEntity);
begin
  FRefAreas := Value;
end;

procedure TCHeckAreaExists.SetRefCompareOnly(const Value: boolean);
begin
  FRefCompareOnly := Value;
end;

procedure TCHeckAreaExists.SetRefQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FRefQT), Value);
end;

INITIALIZATION
   RegisterOrbisClass(TCHeckAreaExists);
END.

