unit T_paral;

interface

uses
// KERNEL
     Elements,
     Tools,
// GLIB
     g_Elems,
     MathLib;


type

  TParalel = class(TTool)
   protected
     Index: longInt;
     MaxIndex: longInt;
     FRozestup: double;
     FVpravo  : boolean;
     FProcRozs: double;
     A, B, C, D: TDoubleCoord;
     procedure VypocitejKrajniBod (AA ,BB: TDoubleCoord; VP: boolean);
     function  TupyUhel: boolean;
     procedure ZpracujABC;
   public
     procedure  Setup(InitRozestup: double; InitVpravo: boolean; PresnostVProcRozest: double);
     procedure  DoIt; override;

     property Rozestup: double read FRozestup write FRozestup;
     property Vpravo: boolean  read FVpravo   write FVpravo;
     property ProcRozs: double read FProcRozs write FProcRozs;
  end;





implementation


procedure TParalel.Setup ( InitRozestup: double; InitVpravo: boolean;
                                PresnostVProcRozest: double);
begin
  Rozestup := InitRozestup;
  Vpravo   := InitVpravo;
  ProcRozs := PresnostVProcRozest;
end;



procedure  TParalel.DoIt;
//var
   //PomCoord: T2DCoordinate;
   //ParCoord: T2DCoordinate;

begin
   if ProcRozs < 0 then ProcRozs:= 0;
   if ProcRozs > 99 then ProcRozs:= 99;

   Index := 0;
   MaxIndex := TEntity(Input).Count -1;

   Output   := TRectEntity.Create;

   A.x := TCoordinate(TEntity(Input).Elements[0]).x;
   A.y := TCoordinate(TEntity(Input).Elements[0]).y;

   B.x := TCoordinate(TEntity(Input).Elements[1]).x;
   B.y := TCoordinate(TEntity(Input).Elements[1]).y;


   VypocitejKrajniBod (A, B, Vpravo);

   while (MaxIndex - Index) > 1 do
   begin
      C.x := TCoordinate(TEntity(Input).Elements[index+2]).x;
      C.y := TCoordinate(TEntity(Input).Elements[index+2]).y;
      ZpracujABC;
      Inc(Index);
      A := B;
      B := C;
   end;
   VypocitejKrajniBod ( B, A, not Vpravo);
end;


procedure TParalel.VypocitejKrajniBod (AA, BB: TDoubleCoord; VP: boolean);
var
   K: integer;
   CC: T2DCoordinate;
begin
   if VP
   then K := 1
   else K :=-1;
   VratPosunutyBod (AA, Atan2M (AA.X -BB.X, AA.Y -BB.Y) +K*PiPul, Rozestup, D);
   CC   := T2DCoordinate.Create;
   CC.x := D.x;
   CC.y := D.y;
   TEntity(Output).AddElement(CC);
end;




function TParalel.TupyUhel: boolean;
begin
   TupyUhel := Vpravo <> BodVpravoOdOrientPrimky (A, B, C)
end;





procedure TParalel.ZpracujABC;
var
  Posun   : double;
  Omega   : double;
  Mii     : double;
  Alfa    : double;
  MaxDelta: double;
  Gama    : double;
  Delta   : double;

  PocetH  : integer;
  i       : integer;
  Sg      : integer;
  Draha   : double;
  CC      : T2DCoordinate;



begin{zpracuj_ABC}
   Alfa  := ATan2M (A.X -B.X, A.Y -B.Y);
   Omega := Uhel2Usecek (A, B, C);


   if Vpravo
   then Sg := 1
   else Sg :=-1;


   if TupyUhel then
   begin
      Gama    := Pi -Omega;
      if ProcRozs = 0
      then
      begin
         if Omega = 0 then
         begin
            Mii   := Alfa + Sg * PiPul;
            Draha := Rozestup;
         end
         else
         begin
            Omega := DvaPi - Omega;
            Mii   := Alfa +Sg*Omega/2;
            Gama  := Omega/2 - PiPul;
            Draha := Rozestup/Cos(Gama);
         end;

         VratPosunutyBod (B, Mii, Draha, D);
      end
      else if 2*Rozestup*sin(Gama/2) < Rozestup*ProcRozs/100 then
      begin
         Mii   := Alfa +Sg*Omega/2;
         VratPosunutyBod (B, Mii, Rozestup, D);
      end
      else
      begin
         Delta   := 1 -ProcRozs/100;
         MaxDelta:= 2*acos(Delta);
         PocetH  := trunc (Gama/MaxDelta) +2;

         for i := 1 to PocetH do
         begin
            Mii := Alfa +Sg*(i -1)*Gama/(PocetH -1) +Sg*Pi/2;
            VratPosunutyBod (B, Mii, Rozestup, D);
            CC   := T2DCoordinate.Create;
            CC.x := D.x;
            CC.y := D.y;
            TEntity(Output).AddElement(CC);
         end;
         exit;
      end;
   end
   else
   begin
      if Omega < 1E-6 then
      begin
         Posun := Rozestup;
         Mii   := Alfa +Sg*PiPul
      end
      else
      begin
         Posun := Rozestup/sin(Omega/2);
         Mii   := Alfa +Sg*Omega/2;
      end;

      VratPosunutyBod (B, Mii, Posun, D);
   end;
   CC   := T2DCoordinate.Create;
   CC.x := D.x;
   CC.y := D.y;
   TEntity(Output).AddElement(CC);
end; {zpracuj_ABC}










END.


