unit E_Space;

{
[Unit_Description]
Description=Zkladn tdy prostorovch element
Comment=zrevidovat
Admin=Olda
TechTopic=Space Elements
Topic=Grafika
Version=1.0

[Edit]
 28.4.1998 9:50 = Registrace
 20.5.1998 12:30 = Nhrada PString
}

interface
uses
  Classes,
// KERNEL
  Elements,
// GLIB
  G_Elems;

const
  MaxCountInCell = 64;

type


TSpaceEntity = class(TRectEntity)
private
  FProperties: TElement;
  FAngle: double;
  procedure SetProperties(aProps: TElement);
  procedure SetIsDone(aVal: boolean);
  function GetIsDone: boolean;
public
  function   GetSpaceID: longInt; virtual;
  procedure  SetSpaceID(aVal: longInt); virtual;
  function   GetGroup: longInt; virtual;
  function   CreateEnvelope: TElement; override;
  procedure  Assign(Source: TPersistent); override;
  destructor Destroy; override;
  function   IsIsolatedFrom(aBiggerSpaceEntity: TSpaceEntity): boolean;
  function   IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; virtual;
  function   HasPropertyByName(aName: string): boolean; override;
  function   GetPropertyStringByName(AName: string): string; override;
  procedure  SetPropertyStringByName(AName: string; aValue: string); override;
published
  property SpaceID: longInt read GetSpaceID write SetSpaceID stored true;
  property Group: longInt read GetGroup stored false;
  property IsDone: boolean read GetIsDone write SetIsDone stored false;
  property Angle: double read FAngle write FAngle stored true;
  property Properties: TElement read FProperties write SetProperties stored true;
end;


TSpaceEntityString = class(TSpaceEntity)
public
  procedure  SetPropertyStringByName(AName: string; aValue: string); override;
  procedure  on_tm_DivideSpaceEntity(var aMessage: TOrbisMessage; var Handled: boolean);
  procedure  HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
end;


TPointElement = class(TSpaceEntity)
end;

TPointString = class(TSpaceEntityString)
end;


TTextElement = class(TSpaceEntity)
private
  FText: string;
  procedure SetText(aVal: string);
  function  GetText: string;
published
  property Text: string read GetText write SetText stored true;
end;



TTextString = class(TSpaceEntityString)
end;



TPolyLineElement = class;

TPolyElement = class(TSpaceEntity)
private
  function  GetCoordsCount: longInt; virtual;
  function  GetCoordElement(aIndex: longInt): T2DCoordinate; virtual;

  function  GetLength: double;
  procedure SetX(aIndex: longInt; aX: double);
  function  GetX(aIndex: longInt): double;
  procedure SetY(aIndex: longInt; aY: double);
  function  GetY(aIndex: longInt): double;
  function  GetArea: double; virtual;
  {
  procedure SetOrientedX(aIndex: longInt; aBegin: boolean; aCoord: double);
  procedure SetOrientedY(aIndex: longInt; aBegin: boolean; aCoord: double);
  }
public
  procedure GetOrientedCoord(aIndex: longInt; aBegin: boolean; var aX, aY: double);
  function  GetOrientedCoordElement(aIndex: longInt; aBegin: boolean): T2DCoordinate;
  procedure GivePointPropsForLength(Length: double; var pX, pY, pAngle: double);
  procedure GiveLabelProps(var aX, aY, Angle: double);
  procedure CalculateLabelProps;
  procedure RecalculateRect;
  function  GetSubAreaValue(FromBegin: boolean): double;
  procedure AddXY(aX, aY: double);
  procedure InsertXY(anIndex: longInt; aX, aY: double);
  procedure AddPolyLine(aPoly: TPolyLineElement; aBegin: boolean); virtual;
  property  Xi[aIndex: longInt]: double read GetX write SetX;
  property  Yi[aIndex: longInt]: double read GetY write SetY;
published
  property CoordsCount: longInt read GetCoordsCount stored false;
  property Length      : double  read GetLength stored false;
  property Area1: double read GetArea stored false;
end;



TPolyLineElement = class(TPolyElement)
public
  function IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; override;
  function GetAreaFromBegin(aBegin: Boolean): double;
end;


TPolyLineString = class(TSpaceEntityString)
end;

{TAreaElement = class; }




TAreaElement = class(TPolyElement)
{ plocha obklopen souadnicemi }
private
  FSavedArea: double;
  FHoles: TRectEntity;
  FIsHole: boolean;

  function GetArea: double; override;
  function GetWholeArea: double;
  function GetHoleOwnerArea: TPolyElement;
public
  destructor Destroy; override;
  procedure AddHole(aHole: TAreaElement);
  function  GetHoleByXY(aX, aY: double): TAreaElement;
  function XYIsInside(aX, aY: double): boolean;
  function XYIsInsideNoHoles(aX, aY: double): boolean; virtual;
  function IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; override;
published
  property Holes: TRectEntity read FHoles write FHoles stored true;
  property IsHole: boolean read FIsHole write FIsHole stored true;
  property HoleOwnerArea: TPolyElement read GetHoleOwnerArea stored false;
  property Area1: double read GetArea stored false;
  property WholeArea: double read GetWholeArea stored false;
  property SavedArea: double read FSavedArea stored false;
end;

THolesEntity = class(TRectEntity)
private
  FOwnerArea: TAreaElement;
published
  property OwnerArea: TAreaElement read FOwnerArea stored false;
end;


TAreaString=class(TSpaceEntityString)
end;



TPolyLineAreaElement = class(TAreaElement)
{ plocha obklopen liniemi }
private
  FLinesOrigines: string;
  function GetOrigin(aIndex: longInt): boolean;
  procedure RemoveOrigin(aIndex: longInt);
  procedure InsertOrigin(aIndex: Longint; aVal: boolean);
  procedure SetOrigin(aIndex: longInt; Item: boolean);
  procedure InsertPolyLine(aIndex: integer; aPoly: TPolyLineElement; aBegin: boolean);

  function  GetCoordsCount: longInt; override;
  function  GetCoordElement(aIndex: longInt): T2DCoordinate; override;
    function GetLinesOriginesHex: string;
    procedure SetLinesOriginesHex(const Value: string);
public
  procedure  on_tm_DivideSpaceEntity(var aMessage: TOrbisMessage; var Handled: boolean);

  procedure AddPolyLine(aPoly: TPolyLineElement; aBegin: boolean); override;
  destructor Destroy; override;
  procedure  RemoveElement(aElement: TElement); override;
{
  procedure  Load(aReader: TReader; aIndex: TEntity); override;
  procedure  Save(aWriter: TWriter); override;
}
  function   HasLine(aPoly: TPolyLineElement; aBegin: boolean): boolean;
{  function   CreateEnvelope: TElement; override; }
  function   CreateAreaElementNoHole: TAreaElement;
  function   CreateAreaElement: TAreaElement;
  function   IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; override;
  function   XYIsInsideNoHoles(aX, aY: double): boolean; override;

  property   LinesOrigines[aIndex: longInt]: boolean read GetOrigin write SetOrigin;

  procedure  HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
published
  property   LinesOriginesHex: string read GetLinesOriginesHex write SetLinesOriginesHex;
end;


TSpaceRect = class(TRectEntity)
private
  HX, HY: double; { polovicni hodnoty }
  FLUQ, FLBQ, FRUQ, FRBQ: TSpaceRect;
  FIsLast: boolean;
  function GetCount: longInt; virtual; abstract;
protected
  procedure PrepareQTree;
public
  constructor Create; override;
published
  property Last: boolean read FIsLast default true;
end;




TSpaceLineBegEndCell = class(TSpaceRect)
private

  function GetCount: longInt; override;
  procedure AddToQTree
           (Line: TPolyLineElement; aX, aY: double; aBegin: boolean);
  procedure AddToQTreeDirectly
           (Line: TPolyLineElement; aX, aY: double; aBegin: boolean);
  procedure AddBegEnd(Line: TPolyLineElement; aBegin: boolean);
  procedure MoveLineToQTree(Lin: TPolyLineElement; aBegin: boolean);
  procedure MoveToQTree;
  procedure AddLineXY(Lin: TPolyLineElement; aX, aY: double; aBegin: boolean);
public
  FLineBegins, FLineEnds: TPolyLineString;

  constructor Create; override;
  procedure AddLine(Lin: TPolyLineElement; aBegin: boolean);
published
  property Count: longInt read GetCount;
end;



implementation
uses
  SysUtils, Dialogs,
// KERNEL
  e_Class,
  M_Orbis,
  U_Props,
  U_String,
// GLIB
  AnalInt,
  MathLib,
// SPACE
  E_SpaPrp,
  E_Props;




procedure   TSpaceEntity.Assign(Source: TPersistent);
begin
  inherited Assign(Source);
  if not (Source is TSpaceEntity) then exit;
{
  Start:= GetPropIndex(Element, 'ANGLE');
  for I:= Start to GetPropCount(Element)-1 do
  begin
    SetPropString(Self, I, GetPropString(Element, I))
  end;
}
  if not (TSpaceEntity(Source).Properties is TSpaceEntityProps) then exit;
  Properties:= TSpaceEntityProps(TSpaceEntity(Source).Properties).CreateDuplicate;
end;




function   TSpaceEntity.CreateEnvelope: TElement;
begin
  result:= self
{
  result := inherited CreateEnvelope;
  TSpaceEntity(result).Angle:= Angle;
}
end;



destructor TSpaceEntity.Destroy;
begin
  if FProperties <> nil then
     FProperties.Free;
  inherited Destroy
end;


procedure TSpaceEntity.SetProperties(aProps: TElement);
begin
  if FProperties <> nil then
     FProperties.Free;
  FProperties:= aProps;
try
  if (aProps is TSpaceEntityProps) then
     TSpaceEntityProps(aProps).SpaceEntity:=  self;
except
  FProperties:= nil;
end
end;


function   TSpaceEntity.GetSpaceID: longInt;
begin
  result:= -1;
  if FProperties = nil then exit;
  if FProperties is TSpaceEntityProps then
     result:= TSpaceEntityProps(FProperties).SpaceId
  else
     result:= Properties.GetPropertyLongIntByName('SPACEID');
end;


procedure  TSpaceEntity.SetSpaceID(aVal: longInt);
begin
  if FProperties = nil then exit;
  Properties.SetPropertyLongIntByName('SPACEID', aVal);
end;


function   TSpaceEntity.GetGroup: longInt;
begin
  result:= -1;
  if FProperties = nil then exit;
  result:= Properties.GetPropertyLongIntByName('GROUP');
end;



procedure TSpaceEntity.SetIsDone(aVal: boolean);
begin
  if aVal then
     ID:= 0
  else
     ID:= -1
end;




function TSpaceEntity.GetIsDone: boolean;
begin
  result:= not (ID = -1)
end;

function TSpaceEntity.IsIsolatedFrom(aBiggerSpaceEntity: TSpaceEntity): boolean;
var
  J: longInt;
  aSE: TSpaceEntity;
begin
  result:= true;

  for J:= 0 to Count-1 do
  begin
    aSE:= TSpaceEntity(Elements[J]);
    if not (TElement(aSE) is TSpaceEntity) then continue;
    if aSE.IsMyOwner(aBiggerSpaceEntity) then
    begin
      result:= false;
      exit
    end
  end;
end;




function  TSpaceEntity.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
begin
  result:= false;
  if CoordEl = nil then exit;
  result:= Distance(CoordEl.X, CoordEl.Y, X, Y) < abs(Tolerance)
end;



function  TSpaceEntity.HasPropertyByName(aName: string): boolean;
begin
  result:= inherited HasPropertyByName(aName);
  if result then exit;
  result:= Properties.HasPropertyByName(aName);
end;



function   TSpaceEntity.GetPropertyStringByName(AName: string): string;
begin
  result:= inherited GetPropertyStringByName(aName);
  if result <> '' then exit;
  if Properties = nil then exit;
  result:= Properties.GetPropertyStringByName(AName)
end;


procedure   TSpaceEntity.SetPropertyStringByName(AName: string; aValue: string);
begin
  if GetPropIndex(Self, aName) > -1 then
     inherited SetPropertyStringByName(aName, aValue)
  else
  begin
    if Properties = nil then exit;
    Properties.SetPropertyStringByName(AName, aValue)
  end
end;




function  TPolyElement.GetArea: double;
var
  CrdP, CrdI: T2DCoordinate;
  X1, Y1: double;
  I: longInt;
begin
  result:= 0;
  try
    CrdP:= GetCoordElement(0);
    if not (CrdP is T2DCoordinate) then
    begin
      exit;
    end;
    X1:= CrdP.X; Y1:= CrdP.Y;
    result:= 0;
    for I:= 1 to CoordsCount - 1 do
    begin
      CrdI:= GetCoordElement(I);
      result:= result + (CrdI.X-X1) * (CrdI.Y+Y1)/2;
      X1:= CrdI.X; Y1:= CrdI.Y;
    end;
    result:= result + (CrdP.X-X1) * (CrdP.Y+Y1)/2
  except
    ShowMessage('GetCalculatedArea Error')
  end;
end;





         { TSpaceEntityString }



procedure   TSpaceEntityString.SetPropertyStringByName(AName: string; aValue: string);
begin
  if GetPropIndex(Self, aName) > -1 then
     inherited SetPropertyStringByName(aName, aValue)
  else
  begin
    if Properties = nil then
       Properties:= TDynamicProperties.Create;
    if Properties <> nil then
       Properties.SetPropertyStringByName(AName, aValue)
  end
end;





procedure TSpaceEntityString.on_tm_DivideSpaceEntity(var aMessage: TOrbisMessage; var Handled: boolean);
begin
  AddElement(aMessage.Data)
end;


procedure   TSpaceEntityString.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_DivideSpaceEntity: on_tm_DivideSpaceEntity(aMessage, Handled);
   end;

end;



                   { TTextElement }

procedure TTextElement.SetText(aVal: string);
begin
  FText:= aVal;
end;



function  TTextElement.GetText: string;
begin
  Result := FText
end;



               { TPolyElement }






function TPolyElement.GetCoordsCount: longInt;
begin
  Result:= Count;
end;


function TPolyElement.GetLength: double;
var
  I: longInt;
{  X1, Y1, Z1, X2, Y2, Z2: double; }
  Crd1, Crd2: TCoordinate;
begin
  result:= 0;
{  GetCoord(0, true, X1, Y1);
  for I:= 0 to Count-2 do
  begin
    GetCoord(I+1, true, X2, Y2);
    result:= result + Distance(X1, Y1, X2, Y2);
    X1:= X2; Y1:= Y2; Z1:= Z2
  end;
}
  Crd1:= GetCoordElement(0);
  for I:= 0 to CoordsCount-1 do
  begin
    Crd2:= GetCoordElement(I);
    result:= result + Distance(Crd1.X, Crd1.Y, Crd2.X, Crd2.Y);
    Crd1:= Crd2
  end
end;



procedure TPolyElement.SetX(aIndex: longInt; aX: double);
var
  aCrd: TCoordinate;
begin
  if aIndex >= CoordsCount then exit;
  aCrd:= GetCoordElement(aIndex);
  aCrd.X:= aX
end;


function  TPolyElement.GetX(aIndex: longInt): double;
var
  aCrd: TCoordinate;
begin
  result:= 0;
  if aIndex >= CoordsCount then exit;
  aCrd:= GetCoordElement(aIndex);
  result:= aCrd.X
end;


procedure TPolyElement.SetY(aIndex: longInt; aY: double);
var
  aCrd: TCoordinate;
begin
  if aIndex >= CoordsCount then exit;
  aCrd:= GetCoordElement(aIndex);
  aCrd.Y:= aY
end;


function  TPolyElement.GetY(aIndex: longInt): double;
var
  aCrd: TCoordinate;
begin
  result:= 0;
  if aIndex >= CoordsCount then exit;
  aCrd:= GetCoordElement(aIndex);
  result:= aCrd.Y
end;




procedure TPolyElement.GivePointPropsForLength(Length: double; var pX, pY, pAngle: double);
var
  AuxLen, AbscLen: double;
  X1, Y1, X2, Y2: double;
  Found: boolean;
  I: longInt;
begin
  pX:= 0;
  pY:= 0;
  pAngle:= 0;

  if Length > GetLength then
  begin
    GetOrientedCoord(Count-1, true, pX, pY);
    exit;
  end;

  if Count = 0 then exit;

  if Count = 1 then
  begin
    GetOrientedCoord(0, true, pX, pY);
    exit
  end;

  I:= 0;
  AuxLen:= 0;

  Found:= false;
  while (I < Count-1) and not Found do
  begin
    GetOrientedCoord(I, true, X1, Y1);
    GetOrientedCoord(I+1, true, X2, Y2);
    AbscLen:= Distance(X1, Y1, X2, Y2);

    if AuxLen + AbscLen > Length then
    begin
      GiveAbscPointByLength(X1, Y1, X2, Y2, Length-AuxLen, 0, pX, pY);
      pAngle:= Atan2M(X2-X1, Y2-Y1) + Pi/2;
      Found:= true;
    end
    else
    begin
      AuxLen:= AuxLen + AbscLen;
      inc(I)
    end
  end
end;



procedure TPolyElement.GiveLabelProps(var aX, aY, Angle: double);
var
  Half: double;
begin
  Half:= GetLength/2;

  GivePointPropsForLength(Half, aX, aY, Angle);
end;

procedure TPolyElement.CalculateLabelProps;
var
  aX, aY, aAngle: double;
begin
  GiveLabelProps(aX, aY, aAngle);
  SetAbsRefX(aX);
  SetAbsRefY(aY);
  Angle:= aAngle
end;


procedure TPolyElement.RecalculateRect;
var
  I: longInt;
  aX, aY: double;
begin
  MinX:= 1E30;
  MinY:= 1E30;
  MaxX:= -1E30;
  MaxY:= -1E30;
  for I:= 0 to Count-1 do
  begin
    GetOrientedCoord(I, true, aX, aY);
    FRect.UpdateByXY(aX, aY)
  end
end;


function  TPolyElement.GetCoordElement(aIndex: longInt): T2DCoordinate;
begin
  result:= nil;
  if aIndex >= CoordsCount then exit;
  result:= T2DCoordinate(Elements[aIndex]);
  if not (TElement(result) is T2DCoordinate) then
     result:= nil
end;



function TPolyElement.GetOrientedCoordElement(aIndex: longInt; aBegin: boolean): T2DCoordinate;
begin
  Result:= nil;
  if aIndex >= CoordsCount then exit;

  if not aBegin then aIndex:= CoordsCount-aIndex-1;
  result:= GetCoordElement(aIndex)
end;





procedure TPolyElement.GetOrientedCoord(aIndex: longInt; aBegin: boolean; var aX, aY: double);
var
  Crd: T2DCoordinate;
begin
  aX:= 0; aY:= 0;
  if aIndex >= CoordsCount then exit;
  Crd:= GetOrientedCoordElement(aIndex, aBegin);
  if Crd = nil then exit;
  aX:= Crd.X;
  aY:= Crd.Y
end;


{
procedure TPolyElement.SetOrientedX(aIndex: longInt; aBegin: boolean; aCoord: double);
begin
  GetOrientedCoordElement(aIndex, aBegin).X:= aCoord
end;

procedure TPolyElement.SetOrientedY(aIndex: longInt; aBegin: boolean; aCoord: double);
begin
  GetOrientedCoordElement(aIndex, aBegin).Y:= aCoord
end;
}

function TPolyElement.GetSubAreaValue(FromBegin: boolean): double;
var
  I: longInt;
  X1, Y1, X2, Y2: double;
begin
  result:= 0;
  if Count > 1 then
  begin
    GetOrientedCoord(0, FromBegin, X1, Y1);
    for I:= 1 to Count-1 do
    begin
      GetOrientedCoord(I, FromBegin, X2, Y2);
      result:= result + (X2-X1)*(Y1+Y2);
      X1:= X2;
      Y1:= Y2
    end;
    result:= result/2
  end
end;



procedure TPolyElement.AddXY(aX, aY: double);
var
  aCoord: T2DCoordinate;
begin
  aCoord:= T2DCoordinate.Create;
  aCoord.X:= aX;
  aCoord.Y:= aY;
  AddElement(aCoord)
end;



procedure TPolyElement.InsertXY(anIndex: longInt; aX, aY: double);
var
  aCoord: T2DCoordinate;
begin
  aCoord:= T2DCoordinate.Create;
  aCoord.X:= aX;
  aCoord.Y:= aY;
  InsertElement(anIndex, aCoord);
end;



procedure TPolyElement.AddPolyLine(aPoly: TPolyLineElement; aBegin: boolean);
var
  I: longInt;
begin
  if aBegin then
    for I:= 0 to aPoly.Count-1 do
    begin
      AddElement(TCoordinate(aPoly[I]))
    end
  else
    for I:= aPoly.Count-1 downTo 0 do
    begin
      AddElement(TCoordinate(aPoly[I]))
    end
end;





                      { TPolyLineElement }

function TPolyLineElement.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
var
  I: longInt;
  Uvnitr: boolean;
  XP, YP, Vzd: double;

begin
  Result:= false;
  for I:= 0 to Count-2 do
  begin
    DejVztahyBod_Usecka(CoordEl.X, CoordEl.Y, Xi[I], Yi[I], Xi[I+1], Yi[I+1],
                        Uvnitr, Xp, Yp, Vzd);

    if Uvnitr and (Vzd < Tolerance) then
    begin
      Result:= true;
      Break
    end
  end
end;


function  TPolyLineElement.GetAreaFromBegin(aBegin: boolean): double;
var
  CrdP, CrdI: T2DCoordinate;
  X1, Y1: double;
  I: longInt;
begin
  result:= 0;
  try
    CrdP:= GetOrientedCoordElement(0, aBegin);
    if not (CrdP is T2DCoordinate) then
    begin
      exit;
    end;
    X1:= CrdP.X; Y1:= CrdP.Y;
    result:= 0;
    for I:= 1 to CoordsCount - 1 do
    begin
      CrdI:= GetOrientedCoordElement(I, aBegin);
      result:= result + (CrdI.X-X1) * (CrdI.Y+Y1)/2;
      X1:= CrdI.X; Y1:= CrdI.Y;
    end;
    result:= result + (CrdP.X-X1) * (CrdP.Y+Y1)/2
  except
    ShowMessage('GetCalculatedArea Error')
  end;
end;





              { TAreaElement }


destructor TAreaElement.Destroy;
begin
  if FHoles <> nil then
  begin
    FHoles.Free
  end;
  inherited Destroy
end;


function TAreaElement.GetWholeArea: double;
var
  CrdP, CrdI: T2DCoordinate;
  X1, Y1: double;
  I: longInt;
begin
  result:= 0;
  try
    CrdP:= GetCoordElement(0);
    if not (TElement(CrdP) is T2DCoordinate) then
    begin
      exit;
    end;
    X1:= CrdP.X; Y1:= CrdP.Y;
    result:= 0;
    for I:= 1 to CoordsCount - 1 do
    begin
      CrdI:= GetCoordElement(I);
      result:= result + (CrdI.X-X1) * (CrdI.Y+Y1)/2;
      X1:= CrdI.X; Y1:= CrdI.Y;
    end;
    result:= result + (CrdP.X-X1) * (CrdP.Y+Y1)/2
  except
    ShowMessage('GetCalculatedArea Error')
  end;
  FSavedArea:= result
end;


function TAreaElement.GetArea: double;
var
  I: longInt;
  aHole: TAreaElement;
begin
  result:= GetWholeArea;
  if Holes = nil then exit;

  if Holes.Count < 1 then exit;

  for I:= 0 to Holes.Count-1 do
  begin
    aHole:= TAreaElement(Holes.Elements[I]);
    if not (TElement(aHole) is TAreaElement) then continue;
    result:= result-abs(aHole.WholeArea)
  end;
end;


function TAreaElement.GetHoleOwnerArea: TPolyElement;
var
  aHoles: THolesEntity;
begin
  result:= nil;
  if not IsHole then exit;
  aHoles:= THolesEntity(findOwnerClass(THolesEntity));
  if aHoles = nil then exit;
  result:= aHoles.OwnerArea;
end;



procedure TAreaElement.AddHole(aHole: TAreaElement);
begin
  if FHoles = nil then
  begin
    FHoles:= THolesEntity.Create;
    THolesEntity(FHoles).FOwnerArea:= self; { pro zajitn vazby nahoru }
  end;
  if FHoles = nil then exit;

  FHoles.AddAndUpdateRect(aHole);
  aHole.FIsHole:= true
end;


function TAreaElement.XYIsInsideNoHoles(aX, aY: double): boolean;
var
  Sour: T2DCoordinate;
begin
  Sour:= T2DCoordinate.Create;
  Sour.X:= aX;
  Sour.Y:= aY;
  result:= InsideTest(Self, Sour);
  Sour.Free
end;




function  TAreaElement.GetHoleByXY(aX, aY: double): TAreaElement;
var
  I: longInt;
  aHole: TAreaElement;
begin
  result:= nil;
  if FHoles = nil then exit;
  if FHoles.Count = 0 then exit;
  I:= 0;

  while I < FHoles.Count do
  begin
    aHole:= TAreaElement(FHoles.Elements[I]);
    if TElement(aHole) is TAreaElement then
    begin
      if aHole.XYIsInsideNoHoles(aX, aY) then
      begin
        result:= aHole;
        exit;
      end
    end;
    inc(I);
  end;
end;



function TAreaElement.XYIsInside(aX, aY: double): boolean;
begin
  //result:= false;
  result:= XYIsInsideNoHoles(aX, aY);

  if result then
     result:= GetHoleByXY(aX, aY) = nil
end;





function TAreaElement.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
begin
  Result:= false;
  if CoordEl = nil then exit;
  if RegionIsInside(CoordEl.X-Tolerance, CoordEl.Y-Tolerance,
                          CoordEl.X+Tolerance, CoordEl.Y+Tolerance) then
  begin
    result:= XYIsInside(CoordEl.X, CoordEl.Y);
  end;
end;




              { TPolyLineAreaElement }


procedure TPolyLineAreaElement.AddPolyLine(aPoly: TPolyLineElement; aBegin: boolean);
begin
  AddAndUpdateRect(aPoly);
  LinesOrigines[Count-1]:= aBegin;
end;



procedure GivePosition(aIndex: longInt; var ByteNo: longInt; var ByteMask: byte);
{ ByteNo: posice v retezci, 1.Byt ma ByteNo=1, 1.bit v Bytu ma BytePos=0}
begin
  ByteNo:= aIndex div 8 + 1;
  if aIndex > 7 then
  begin
    ByteMask:= aIndex mod 8
  end
  else
    ByteMask:= aIndex;
{ prevod na masku}
  ByteMask:= 1 shl ByteMask
end;



procedure TPolyLineAreaElement.RemoveOrigin(aIndex: longInt);
var
  I: longInt;
begin
  if FLinesOrigines = '' then exit;
  for I:= aIndex + 1 to Count-1 do
  begin
    SetOrigin(I-1, GetOrigin(I))
  end;
end;


procedure TPolyLineAreaElement.InsertOrigin(aIndex: Longint; aVal: boolean);
var
  I: longInt;
begin
  if aIndex > Count then Exit;
  if aIndex < 0 then Exit;

  if (FLinesOrigines = '') or (aIndex >= Count-1) then
  begin
    SetOrigin(aIndex, aVal);
    exit;
  end;

  for I:= Count-1 downto aIndex do
  begin
    SetOrigin(I+1, GetOrigin(I))
  end;

  SetOrigin(aIndex, aVal)
end;



procedure  TPolyLineAreaElement.RemoveElement(aElement: TElement);
var
  I: longInt;
begin
  if (aElement is TPolyLineElement) and
     (FLinesOrigines <> '') then
  begin
    I:= IndexOf(aElement);
    if I > -1 then RemoveOrigin(I)
  end;
  inherited RemoveElement(aElement);
end;



procedure TPolyLineAreaElement.InsertPolyLine(aIndex: integer; aPoly: TPolyLineElement; aBegin: boolean);
var
  I: longInt;
begin
  InsertElement(aIndex, aPoly);
  InsertOrigin(aIndex, aBegin);
end;



function TPolyLineAreaElement.GetOrigin(aIndex: longInt): boolean;
var
  ByteNo: longInt;
  ByteMask: byte;

begin
  Result:= false;
  if aIndex < 0 then exit;
  GivePosition(aIndex, ByteNo, ByteMask);
  if (system.length(FlinesOrigines) >= ByteNo) then
  begin
    Result:= (ord(FLinesOrigines[ByteNo]) and ByteMask) <> 0
  end;
end;


procedure TPolyLineAreaElement.SetOrigin(aIndex: longInt; Item: boolean);
var
  ByteNo, ByteCount: longInt;
  ByteMask: byte;
begin
  if aIndex < 0 then exit;
  GivePosition(aIndex, ByteNo, ByteMask);
  ByteCount:= system.length(FlinesOrigines);

  if ByteCount < ByteNo then
     SetLength(FLinesOrigines, ByteNo);

  if Item then
     FLinesOrigines[ByteNo]:= chr(ord(FLinesOrigines[ByteNo])and not ByteMask or ByteMask)
  else
     FLinesOrigines[ByteNo]:= chr(ord(FLinesOrigines[ByteNo])and not ByteMask)
end;





function TPolyLineAreaElement.HasLine(aPoly: TPolyLineElement; aBegin: boolean): boolean;
var
  I: longInt;
  Nasel: boolean;
  Lin: TPolyLineElement;
begin
  I:= 0;
  Nasel:= false;
  while (I < Count) and not Nasel do
  begin
    if Elements[I] is TPolyLineElement then
    begin
      Lin:= TPolyLineElement(Elements[I]);
      Nasel:= (Lin = aPoly) and (LinesOrigines[I] = aBegin)
    end;
    if not Nasel then inc(I)
  end;
  result:= Nasel
end;

{
function   TPolyLineAreaElement.CreateEnvelope: TElement;
begin
   result := inherited CreateEnvelope;
   SetPString(TPolyLineAreaElement(result).FLinesOrigines, FLinesOrigines^);
end;
}



procedure TPolyLineAreaElement.on_tm_DivideSpaceEntity(var aMessage: TOrbisMessage; var Handled: boolean);
var
  I: integer;
  OrigLine, NewLine: TPolyLineElement;
  Origin: Boolean;
begin
  OrigLine:= TPolyLineElement(aMessage.Sender);
  NewLine:= TPolyLineElement(aMessage.Data);
  if not (TObject(OrigLine) is TPolyLineElement) then
     Exit;
  if not (TObject(NewLine) is TPolyLineElement) then
     Exit;
  I:= IndexOf(OrigLine);
  if I < 0 then
     Exit;
  Origin:= GetOrigin(I);
  if Origin then
     InsertPolyLine(I, NewLine, Origin)
  else
     InsertPolyLine(I+1, NewLine, Origin)
end;




function TPolyLineAreaElement.CreateAreaElementNoHole: TAreaElement;
var
  I, J: longInt;
  Linie: TPolyLineElement;
  Crd: T2DCoordinate;
  Start: longInt;
begin
  Result:= TAreaElement.Create;

{  Result.Properties:= Properties; }

  Start:= 0;   { naten prvn souadnice prvn linie, u dalch lini a druh }
  { prirazeni souradnic }
  for I:= 0 to Count-1 do
  begin
    if TObject(Elements[I]) is TPolyLineElement then
    begin
      Linie:= TPolyLineElement(Elements[I]);
      if LinesOrigines[I] then
      for J:= Start to Linie.Count-1 do
      begin
        Crd:= T2DCoordinate(TCoordinate(Linie.Elements[J]).CreateDuplicate);
        Result.AddElement(Crd)
      end
      else
      for J:= Linie.Count-Start-1 downTo 0 do
      begin
        Crd:= T2DCoordinate(TCoordinate(Linie.Elements[J]).CreateDuplicate);
        Result.AddElement(Crd)
      end;
      Start:= 1
    end
  end;
  Result.Rect:= FRect.CreateDuplicate;
end;



function TPolyLineAreaElement.CreateAreaElement: TAreaElement;
var
  I: longInt;
  Hole: TAreaElement;
begin
  //result:= nil;
  result:= CreateAreaElementNoHole;
  if Holes = nil then exit;
  for I:= 0 to Holes.Count-1 do
  begin
    Hole:= TAreaElement(Holes.Elements[I]);
    if Hole is TPolyLineAreaElement then
    begin
       Hole:= TPolyLineAreaElement(Hole).CreateAreaElementNoHole;
       result.AddHole(Hole)
    end
  end;
end;



function TPolyLineAreaElement.GetCoordsCount: longInt;
var
  I: longInt;
  Linie: TPolyLineElement;
begin
  Result:= 0;
  for I:= 0 to Count-1 do
  begin
    if TObject(Elements[I]) is TPolyLineElement then
    begin
      Linie:= TPolyLineElement(Elements[I]);
      Result:= Result + Linie.Count-1;
    end
  end
end;




function TPolyLineAreaElement.GetCoordElement(aIndex: longInt): T2DCoordinate;
var
  I, J, AuxI: longInt;
  Linie: TPolyLineElement;
  Nasel: boolean;
begin
  I:= 0;
  AuxI:= 0;
  Nasel:= false;
  result:= nil;
  while (I < Count) and not Nasel do
  begin
    if TObject(Elements[I]) is TPolyLineElement then
    begin
      Linie:= TPolyLineElement(Elements[I]);
      if AuxI + Linie.Count-1 > aIndex then
      begin
        J:= aIndex - AuxI;
        Result:= T2DCoordinate(Linie.GetOrientedCoordElement(J, LinesOrigines[I]));
        Nasel:= true
      end
      else
        AuxI:= AuxI + Linie.Count-1
    end;
    if not Nasel then inc(I)
  end
end;




function TPolyLineAreaElement.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
begin
  Result:= false;
  if CoordEl = nil then exit;
  if RegionIsInside(CoordEl.X-Tolerance, CoordEl.Y-Tolerance,
                          CoordEl.X+Tolerance, CoordEl.Y+Tolerance) then
  begin
    result:= XYIsInside(CoordEl.X, CoordEl.Y);
  end;
end;



function TPolyLineAreaElement.XYIsInsideNoHoles(aX, aY: double): boolean;
var
  aArea: TAreaElement;
begin
  aArea:= CreateAreaElement;
  result:= aArea.XYIsInsideNoHoles(aX, aY);
  aArea.Free;
end;



destructor TPolyLineAreaElement.Destroy;
begin
  inherited Destroy
end;


function TPolyLineAreaElement.GetLinesOriginesHex: string;
var
  I: integer;
  aChar: char;
  aL: integer;
begin
  result:= '';
  for I:= 1 to system.length(FLinesOrigines) do
  begin
    aChar:= FLinesOrigines[i];
    result:= result + IntToHex(ord(aChar), 2);
  end;
end;

procedure TPolyLineAreaElement.SetLinesOriginesHex(const Value: string);
var
  I: integer;
  aChar: char;
  aLength: integer;

  function Char2ToInt(C1, C2: char): integer;
  var
    aRes: integer;
    aStr: string;
  begin
    aStr:= '$' + C1 + C2;
    val(aStr, Result, aRes);
  end;

  function HexToChr: char;
  begin
    Result:= chr(Char2ToInt(Value[2*I-1], Value[2*I]))
  end;

begin
  FLinesOrigines:= '';
  aLength:= system.length(Value);
  aLength:= aLength div 2;
  for I:= 1 to aLength do
  begin
    aChar:= HexToChr;
    FLinesOrigines:= FLinesOrigines + aChar
  end;
end;



procedure   TPolyLineAreaElement.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_DivideSpaceEntity: on_tm_DivideSpaceEntity(aMessage, Handled);
   end;
end;



{ TSpaceRect }

constructor TSpaceRect.Create;
begin
  inherited Create; { zahada}
  FIsLast:= true
end;


procedure TSpaceRect.PrepareQTree;
begin
  HX:= (maxX+minX)/2;
  HY:= (maxY+minY)/2;
  FLUQ:= TSpaceRect(TElementClass(self.ClassType).Create);
  FLBQ:= TSpaceRect(TElementClass(self.ClassType).Create);
  FRUQ:= TSpaceRect(TElementClass(self.ClassType).Create);
  FRBQ:= TSpaceRect(TElementClass(self.ClassType).Create);
  FLUQ.FIsLast:= true;
  FLBQ.FIsLast:= true;
  FRUQ.FIsLast:= true;
  FRBQ.FIsLast:= true;

  if (FLBQ<>nil) and (FLUQ<>nil) and (FRBQ<>nil) and (FRUQ<>nil) then
  begin
    FLUQ.Rect.minX:= Self.MinX;
    FLUQ.Rect.maxX:= HX;
    FLUQ.Rect.MinY:= HY;
    FLUQ.Rect.maxY:= Self.maxY;
    FLBQ.Rect.minX:= Self.MinX;
    FLBQ.Rect.maxX:= HX;
    FLBQ.Rect.MinY:= Self.MinY;
    FLBQ.Rect.maxY:= HY;
    FRUQ.Rect.minX:= HX;
    FRUQ.Rect.maxX:= Self.MaxX;
    FRUQ.Rect.MinY:= HY;
    FRUQ.Rect.maxY:= Self.maxY;
    FRBQ.Rect.minX:= HX;
    FRBQ.Rect.maxX:= Self.MaxX;
    FRBQ.Rect.MinY:= Self.MinY;
    FRBQ.Rect.maxY:= HY;

    FIsLast:= false;

    FLBQ.Name:= 'LEVY DOLNI';
    FLUQ.Name:= 'LEVY HORNI';
    FRBQ.Name:= 'PRAVY DOLNI';
    FRUQ.Name:= 'PRAVY HORNI';
    AddElement(FLBQ);
    AddElement(FLUQ);
    AddElement(FRBQ);
    AddElement(FRUQ);
  end
end;




                         { TSpaceLineBegEndCell }


constructor TSpaceLineBegEndCell.Create;
begin
  inherited Create;
  FIsLast:= true  { zahada}
end;


procedure TSpaceLineBegEndCell.AddToQTree
          (Line: TPolyLineElement; aX, aY: double; aBegin: boolean);
begin
  if (aX < HX) then
  begin
    if aY < HY then
      TSpaceLineBegEndCell(FLBQ).AddLineXY(Line, aX, aY, aBegin)
    else
      TSpaceLineBegEndCell(FLUQ).AddLineXY(Line, aX, aY, aBegin)
  end
  else
  begin
    if aY < HY then
      TSpaceLineBegEndCell(FRBQ).AddLineXY(Line, aX, aY, aBegin)
    else
      TSpaceLineBegEndCell(FRUQ).AddLineXY(Line, aX, aY, aBegin)
  end
end;


procedure TSpaceLineBegEndCell.AddToQTreeDirectly
          (Line: TPolyLineElement; aX, aY: double; aBegin: boolean);
begin
  if (aX < HX) then
  begin
    if aY < HY then
      TSpaceLineBegEndCell(FLBQ).AddBegEnd(Line, aBegin)
    else
      TSpaceLineBegEndCell(FLUQ).AddBegEnd(Line, aBegin)
  end
  else
  begin
    if aY < HY then
      TSpaceLineBegEndCell(FRBQ).AddLineXY(Line, aX, aY, aBegin)
    else
      TSpaceLineBegEndCell(FRUQ).AddLineXY(Line, aX, aY, aBegin)
  end
end;


procedure TSpaceLineBegEndCell.AddBegEnd(Line: TPolyLineElement; aBegin: boolean);
var
  Lines: TPolyLineString;
begin
  if aBegin then Lines:= FLineBegins
  else Lines:= FLineEnds;

  if Lines = nil then
  begin
    Lines:= TPolyLineString.Create;

    if aBegin then Lines.Name:= 'BEGINS'
    else Lines.Name:= 'ENDS';

    AddElement(Lines);
  end;

  if Lines <> nil then
  begin
    Lines.AddAndUpdateRect(Line)
  end;

  if aBegin then FLineBegins:= Lines
  else FLineEnds:= Lines;
end;


procedure TSpaceLineBegEndCell.MoveLineToQTree(Lin: TPolyLineElement; aBegin: boolean);
var
  aX, aY: double;
begin
  Lin.GetOrientedCoord(0, aBegin, aX, aY);
  AddToQTreeDirectly(Lin, aX, aY, aBegin)
end;


procedure TSpaceLineBegEndCell.MoveToQTree;
var
  I: longInt;
begin
  if FLineBegins <> nil then
  begin
    for I:= 0 to FLineBegins.Count-1 do
        MoveLineToQTree(TPolyLineElement(FLineBegins.Elements[I]), true);
{    FLineBegins.Clear; to by se pak linie nezbavily tohoto vlastnika }
    FreeElement(FLineBegins);
    FLineBegins:= nil
  end;

  if FLineEnds <> nil then
  begin
    for I:= 0 to FLineEnds.Count-1 do
        MoveLineToQTree(TPolyLineElement(FLineEnds.Elements[I]), false);
{    FLineEnds.Clear; }

    FreeElement(FLineEnds);
    FLineEnds:= nil
  end
end;


function TSpaceLineBegEndCell.GetCount: longInt;
var
  Sum: longInt;
begin
  Sum:= 0;
  if not FIsLast then
  begin
    Sum:= Sum+FLBQ.GetCount;
    Sum:= Sum+FLUQ.GetCount;
    Sum:= Sum+FRBQ.GetCount;
    Sum:= Sum+FRUQ.GetCount
  end
  else
  begin
    if FLineBegins <> nil then Sum:= Sum + FLineBegins.Count;
    if FLineEnds <> nil then Sum:= Sum + FLineEnds.Count;
  end;
  result:= Sum
end;


procedure TSpaceLineBegEndCell.AddLineXY
          (Lin: TPolyLineElement; aX, aY: double; aBegin: boolean);
begin
  if not FIsLast then
     AddToQTree(Lin, aX, aY, aBegin)
  else
  begin
    if GetCount < MaxCountInCell then
    begin
      AddBegEnd(Lin, aBegin)
    end
    else
    begin
      PrepareQTree;
      MoveToQTree;
      AddToQTree(Lin, aX, aY, aBegin)
    end
  end
end;





procedure TSpaceLineBegEndCell.AddLine
          (Lin: TPolyLineElement; aBegin: boolean);
var
  aX, aY: double;
begin
try
  Lin.GetOrientedCoord(0, aBegin, aX, aY);
  AddLineXY(Lin, aX, aY, aBegin);
except
  ShowMessage('Error: TSpaceLineBegEndCell.AddLine')
end;
end;




begin
   RegisterOrbisClass(TSpaceEntity);
   RegisterOrbisClass(TPointElement);
   RegisterOrbisClass(TPointString);
   RegisterOrbisClass(TTextElement);
   RegisterOrbisClass(TTextString);
   RegisterOrbisClass(TPolyElement);
   RegisterOrbisClass(TPolyLineElement);
   RegisterOrbisClass(TPolyLineString);
   RegisterOrbisClass(TAreaElement);
   RegisterOrbisClass(TAreaString);
   RegisterOrbisClass(TPolyLineAreaElement);
   RegisterOrbisClass(TSpaceLineBegEndCell);
   RegisterOrbisClass(THolesEntity);
end.