unit E_TopMDL;
{
*********************************************************************
*                                                                   *
*  ORBIS ... Tdy grafickch, t.j. souadnicov zvislch objekt  *
*            Objektov model TopoLu *********                       *
*                                                                   *
*********************************************************************
Natn ploch vetn enklvovitosti

Natn se provd sekvenn, avak pi naten enklvy se nataj i
odpovdajc nadplochy.
Nejdve se nataj vechny plochy, kter nemaj podplochy. Nadplochy se
nataj v rekurzi. Zrove se pracuje se seznamem.


}

interface
uses
   Classes,
   Graphics,
   DBTables,
   Elements,
   g_Elems,
   u_TopoLT, E_TplPrp;

const



  TTopolEntitySize = sizeOf(longInt) + 4*sizeOf(smallInt) + 2*sizeOf(boolean)
                       + sizeOf(byte) + 3*sizeOf(double);

  TTopolLinSize = 3*sizeOf(longInt) + sizeOf(TDrziSe) + sizeOf(boolean);



type

  TTopolObjects = class;



  TTopoLElement = class(T2DCoordinate)
   protected
     FOwners: TSuperList;         { ElementList }
     function     GetOwnerEntity(Index: longint): TEntity; override;
     procedure    PutOwner(Index: longint; Item: TEntity); override;
     function     GetOwnersCount: longint; override;
     function     GetObjectType: char; virtual; abstract;
     function     GetObjectsOwner: TTopolObjects; virtual; abstract;
{     function     GetDBF: TElement; virtual; }
   public
     FCislo   : longInt;
     FDruh    : smallInt;
     FBarva   : smallInt;
     FZnacka  : smallInt;
     FPresnost: byte;
     procedure   Load(aReader: TReader; aIndex: TEntity); override;
     procedure   Save(aWriter: TWriter); override;
     function    IsInsideRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
     function    IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; virtual;
     function    CreateEnvelope: TElement; override;
     procedure   AddOwner(aOwner: TEntity); override;
     procedure   RemoveOwner(aOwner: TEntity); override;
     function    IsMyOwner(aOwner: TEntity): boolean; override;
     function    NoOwners: boolean; override;
     function    FindOwnerClass(aClass: TClass): TEntity; override;
     function     GetPropertyStringByName(AName: string): string; override;
     function   GetBlockDirectory: string; virtual;
     function   GetDBFName: string; virtual;

     property ObjectOwner: TTopolObjects read GetObjectsOwner;
  published
{     property DBF: TElement read GetDBF; }
     property ObjectType: char read GetObjectType stored false;
     property Cislo:    longInt read FCislo    write FCislo;
     property Druh:     smallInt read FDruh     write FDruh;
     property Barva:    smallInt read FBarva    write FBarva;
     property Znacka:   smallInt read FZnacka   write FZnacka;
     property Presnost: byte    read FPresnost write FPresnost;

     property BlockDir: string read GetBlockDirectory stored false;
     property DBFName:  string read GetDBFName stored false;
  end;


  TTopoLEntity = class(TRectEntity)
   protected
     FCislo   : longInt;
     FDruh    : smallInt;
     FBarva   : smallInt;
     FZnacka  : smallInt;
     FVyber   : boolean;
     FPresnost: byte;
     FSymbol  : smallInt;
     FSymX, FSymY : double;
     FSymAngle    : double;
     FDone    : boolean;
     function     GetObjectType: char; virtual; abstract;
     function     GetObjectsOwner: TTopolObjects; virtual; abstract;
{     function     GetDBF: TElement; virtual; }
   public
     procedure  Assign(Source: TPersistent); override;
     procedure  Load(aReader: TReader; aIndex: TEntity); override;
     procedure  Save(aWriter: TWriter); override;
     function   CreateEnvelope: TElement; override;
     function   GetBlockDirectory: string; virtual;
     function   GetDBFName: string; virtual;
     function   IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; virtual;
     function   GetPropertyStringByName(AName: string): string; override;
     property ObjectOwner: TTopolObjects read GetObjectsOwner;
   published
     property ObjectType: char read GetObjectType stored false;
     property Cislo:    longInt read FCislo    write FCislo;
     property Druh:     smallInt read FDruh     write FDruh;
     property Barva:    smallInt read FBarva    write FBarva;
     property Znacka:   smallInt read FZnacka   write FZnacka;
     property Vyber:    boolean read FVyber    write FVyber;
     property Presnost: byte    read FPresnost write FPresnost;
     property Symbol     : smallInt read FSymbol  write FSymbol;
     property SymbolX    : double read FSymX     write FSymX;
     property SymbolY    : double read FSymY     write FSymY;
     property SymbolAngle: double read FSymAngle write FSymAngle;
     property IsDone: boolean   read FDone     write FDone stored false default false ;

     property BlockDir: string read GetBlockDirectory stored false;
     property DBFName:  string read GetDBFName stored false;
  end;


  TRectTopoLElement = class(TTopoLElement)
   protected
     FRect    : TRectangle;
     function  GetRect: TRectangle; virtual;
     procedure SetRect(aRect: TRectangle);
   public
     procedure   Load(aReader: TReader; aIndex: TEntity); override;
     procedure   Save(aWriter: TWriter); override;
     destructor  Destroy; override;
     function    GetRegion: TDoubleRect; override;
     function    IsInRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
     function    IsOverRegion(Minx, MinY, MaxX, MaxY: double): Boolean;

     property Rect: TRectangle read GetRect write SetRect;
  end;




 TTopoLBod = class(TTopoLElement)
  private
    FUhel  : double;
  protected
     function     GetObjectType: char; override;
  public
    procedure   Load(aReader: TReader; aIndex: TEntity); override;
    procedure   Save(aWriter: TWriter); override;
    function   GetObjectsOwner: TTopolObjects; override;
  published
    property Uhel: double read FUhel write FUhel;
 end;


  TTopoLText = class(TRectTopoLElement)
  private
     FCodePage: TCodePage;

     procedure SetCodePage(aCode: TCodePage);
     function GetCodePage: TCodePage;
   protected
     FVychoziBod : TCoordinate;
     FVeta: PString;
     function  GetVeta: string;
     procedure SetVeta(aVeta: string);
     function  GetVychoziBod: TCoordinate;
     procedure SetVychoziBod(aVychoziBod: TCoordinate);
     function  GetObjectType: char; override;
     function  GetObjectsOwner: TTopolObjects; override;
   public
     Font       : TTopolFont;
     Vyska      : double;
     KoefSirky  : double;
     Smer       : double; { Uhel v Radianech }
     Sklon      : double; { Uhel v Radianech }
     Rozestupy  : double; { Zvetseni default hodnoty v Bodech }
     Atributy   : TTextAtrib;
     RezimTextu : byte;

     destructor  Destroy; override;
     procedure   Load(aReader: TReader; aIndex: TEntity); override;
     procedure   Save(aWriter: TWriter); override;
     function    CreateEnvelope: TElement; override;

     property    VychoziBod: TCoordinate read GetVychoziBod write SetVychoziBod;
  published
     property CodePage: TCodePage read GetCodePage write SetCodePage;
     Property Veta : String read GetVeta write SetVeta;
  end;



  TTopoLLinieASymbol = class(TEntity);


  TTopolPlocha = class;

  TTopoLLin = class(TTopoLEntity)
  private
    FLevaPlocha  : longInt;
    FPravaPlocha : longInt;
    FPuvCislo      : longInt;
    function GetLength: double;
    procedure SetX(aIndex: longInt; aX: double);
    function  GetX(aIndex: longInt): double;
    procedure SetY(aIndex: longInt; aY: double);
    function  GetY(aIndex: longInt): double;
  protected
    function  GetObjectType: char; override;
    function  GetObjectsOwner: TTopolObjects; override;
  public
    DrziSe        : TDrziSe;
    ExportedToDBF : boolean;

    procedure Assign(Source: TPersistent); override;
    procedure Load(aReader: TReader; aIndex: TEntity); override;
    procedure Save(aWriter: TWriter); override;
    procedure GetCoord(aIndex: longInt; aBegin: boolean; var aX, aY: double);
    function  GetCoordElement(aIndex: longInt; aBegin: boolean): T2DCoordinate;
    procedure SetOrientedX(aIndex: longInt; aBegin: boolean; aCoord: double);
    procedure SetOrientedY(aIndex: longInt; aBegin: boolean; aCoord: double);
    procedure GivePointPropsForLength(var Length, pX, pY, pAngle: double);
    procedure GiveLabelProps(var X, Y, Angle: double);
    procedure CalculateLabelProps;
    procedure RecalculateRect;
    function  GetSubAreaValue(FromBegin: boolean): double;
    function  GetRightAreaForGroup(aGroup: smallInt; aRightArea: boolean): TTopolPlocha;
    function  IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; override;
    property  X[aIndex: longInt]: double read GetX write SetX;
    property  Y[aIndex: longInt]: double read GetY write SetY;
  published
    property LevaPlocha : longInt read FLevaPlocha  write FLevaPlocha;
    property PravaPlocha: longInt read FPravaPlocha write FPravaPlocha;
    property Delka      : double  read GetLength stored false;
    property PuvodniCislo: longInt read FPuvCislo write FPuvCislo stored false;
 end;



  TTopoLPlochaBoundary = class (TTopoLEntity)
   private
     function GetCoordsCount: longInt; virtual;
     function GetCoordinateElement(Index: longInt): T2DCoordinate; virtual;
     function GetCalculatedArea: double; virtual;
     function HasChildren: boolean; virtual;
  protected
     procedure GiveCoords(Index: longInt; var X, Y: double);
     function  GetObjectType: char; override;
     function  GetObjectsOwner: TTopolObjects; override;
  public
     FMaEnklavu : boolean;
     FCisloOtce : longInt;
     PrvniLinie : longInt;
     procedure   Load(aReader: TReader; aIndex: TEntity); override;
     procedure   Save(aWriter: TWriter); override;
     function    IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; override;
   published
     property Paternity: boolean read HasChildren stored false;
     property Pater: longInt read FCisloOtce write FCisloOtce;
     property CoordsCount: longInt read GetCoordsCount stored false;
     property Area: double read GetCalculatedArea stored false;
  end;

  TTopolNadPlocha = class;

  TTopoLPlocha = class (TTopoLPlochaBoundary)
  private
    FLinesOrigines: PString;
    function GetOrigin(Index: longInt): boolean;
    procedure SetOrigin(Index: longInt; Item: boolean);
    function GetCoordsCount: longInt; override;
    function GetCoordinateElement(Index: longInt): T2DCoordinate; override;
    function GetFather: TTopolNadPlocha;
    function GetLinesOrigines: string;
    procedure SetLinesOrigines(const aValue: string);
  public
    destructor Destroy; override;
    procedure  Load(aReader: TReader; aIndex: TEntity); override;
    procedure  Save(aWriter: TWriter); override;
    function   HasLine(aLine: TTopolLin; aBegin: boolean): boolean;
    function   CreateEnvelope: TElement; override;
    function   CreatePlochaBoundary: TTopolPlochaBoundary;
    function   IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; override;
    property   LinesOrigines[Index: longInt]: boolean read GetOrigin write SetOrigin;
    property   Father: TTopolNadPlocha read GetFather;
  published
    property LineOrigines: string read GetLinesOrigines write SetLinesOrigines;
  end;


  TSubAreas = class(TRectEntity);


  TTopolNadPlocha = class(TTopolPlocha)
  private
    FSavedArea: double;
    FSubAreas: TSubAreas;
    function   GetSubArea(Index: longInt): TTopolNadPlocha;
    function   GetSubAreasCount: longInt;
    function   GetCleanArea: double;
    function   HasChildren: boolean; override;
    function   GetCalculatedArea: double; override;
  public
    procedure Load(aReader: TReader; aIndex: TEntity); override;
    procedure Save(aWriter: TWriter); override;
    procedure AddArea(aPlocha: TTopolPlocha);
    property  SubArea[Index: longInt]: TTopolNadPlocha read GetSubArea;
{    property  Subareas: TSubAreas read FSubAreas write FSubAreas; }
  published
    property  SubAreasCount: longInt read GetSubAreasCount stored false;
    property  CleanArea: double read GetCleanArea stored false;
    property  SavedArea: double read FSavedArea stored false;
  end;


  TTopolRectEntity = class(TRectEntity)
  private
    FBlockDirectory: pString;
    FDBF: TTable;
    function GetBlockDir: string;
    procedure SetBlockDir(FN: string);
    function GetDBF: TTable;
  public
    procedure Load(aReader: TReader; aIndex: TEntity); override;
    procedure Save(aWriter: TWriter); override;
    destructor Destroy; override;
  published
    property  BlockDir: string read GetBlockDir write SetBlockDir;
    property  DBF: TTable read GetDBF stored false;
  end;


  TTopolObjects = class(TTopolRectEntity)
  private
     FColor: TColor;
     FUseColor: boolean;
     FSymbol: smallInt;
     FUseSymbol: boolean;
     FMinScale, FMaxScale: double;
  published
     property   Color: TColor read FColor write FColor;
     property   UseColor: boolean read FUseColor write FUseColor;
     property   Symbol: smallInt read FSymbol write FSymbol;
     property   UseSymbol: boolean read FUseSymbol write FUseSymbol;
     property   MinScale: double read FMinScale write FMinScale;
     property   MaxScale: double read FMaxScale write FMaxScale;
  end;

  TTopoLLinie        = class(TTopolObjects);
  TTopoLLinieDruhu   = class(TTopolLinie);
  TTopoLBody    = class(TTopolObjects);

  TTopoLPlochy  = class(TTopolObjects)
  public
     procedure AddAreaChildren(Area: TTopolPlocha);
  end;

  TTopoLTexty   = class(TTopolObjects);

  TTopoLOstatni = class(TTopolObjects);


  TTopoLBlok  = class(TTopolObjects)
  private
    FCodePage: TCodePage;
  published
    property CodePage: TCodePage read FCodePage write FCodePage;
  end;

  TTopolPredmetZajmu = class(TRectEntity);
{

Struktura (Vlastnick hierarchie) TopoLovskho modelu:
*****************************************************

TTopoLPredmetZajmu
      |
   TTopoLBLok - TTopoLBLok - TTopoLBLok ....
         |
      TTopoLLinie  -  TTopoLBody  -  TTopoLPlochy  -  TTopoLTexty
            |            |               |                   |
         TTopoLLin ...  TTopoLBod ...   TTopoLPlocha ...    TTopoLText ...

}


function GetDBFNameForObject(BlockDir: string; ObjType: char; Druh: smallInt): string;




implementation

uses
   SysUtils, Dialogs, Forms,
   e_Class,
   u_String, MathLib, Worlds,
   {!t_Main,}
{$IFDEF Databaze}
  C_TLDBF,
{$ENDIF}
   AnalInt;


function GetDBFNameForObject(BlockDir: string; ObjType: char; Druh: smallInt): string;
begin
  Result:= BlockDir + '\' + ObjType + Format('%4.4d', [Druh]) + '.DBF';
end;


{ TTopoLElement }

procedure TTopoLElement.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   aReader.Read(FCislo, SizeOf(FCislo)+
                        SizeOf(FDruh) +
                        SizeOf(FBarva)+
                        SizeOf(FZnacka)+
                        SizeOf(FPresnost));
end;


procedure TTopoLElement.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.Write(FCislo, SizeOf(FCislo)+
                         SizeOf(FDruh) +
                         SizeOf(FBarva)+
                         SizeOf(FZnacka)+
                         SizeOf(FPresnost));
end;




function    TTopoLElement.IsInsideRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
var
   ax, aY: double;
begin
   ax := x;
   ay := y;
   result:= (MinX <= ax) and (MaxX >= ax) and
            (MinY <= aY) and (MaxY >= ay);
end;



function TTopolElement.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
begin
  Result:= Distance(X, Y, CoordEl.X, CoordEl.Y) <= Tolerance
end;







function   TTopolElement.CreateEnvelope: TElement;
begin
   result := inherited CreateEnvelope;
   TTopolElement(result).Cislo   := Cislo;
   TTopolElement(result).Druh    := Druh;
   TTopolElement(result).Barva   := Barva;
   TTopolElement(result).Znacka  := Znacka;
   TTopolElement(result).Presnost:= Presnost;
end;

function     TTopolElement.GetOwnerEntity(Index: longint): TEntity;
begin
   Result := nil;
   if NoOwners or (Index >= FOwners.Count)then exit;
   Result := FOwners[Index];
end;


procedure    TTopolElement.PutOwner(Index: longint; Item: TEntity);
begin
   if not assigned(FOwners) then FOwners := TSuperList.Create;
   if (FOwners.Count >=0) and (Index = 0) then FOwners.Add(Item)
                                          else FOwners.Insert(Index, Item);
end;


function     TTopolElement.GetOwnersCount: longint;
begin
   if not assigned(FOwners) then Result := 0
                            else Result := FOwners.Count;
end;


procedure    TTopolElement.AddOwner(aOwner: TEntity);
begin
   if not assigned(FOwners) then FOwners := TSuperList.Create;
   FOwners.Add(aOwner);
end;


procedure    TTopolElement.RemoveOwner(aOwner: TEntity);
begin
   if NoOwners then exit;
   FOwners.Remove(aOwner);
end;


function     TTopolElement.IsMyOwner(aOwner: TEntity): boolean;
begin
   Result := false;
   if not NoOwners then Result := FOwners.IndexOf(aOwner) > -1;
end;


function     TTopolElement.NoOwners: boolean;
begin
   Result := OwnersCount = 0;
end;


function     TTopolElement.FindOwnerClass(aClass: TClass): TEntity;
var
   i: longInt;
begin
   if not NoOwners then
   for i := 0 to OwnersCount -1 do
   begin
      if Owners[i] is aClass then
      begin
         Result := Owners[i];
         exit;
      end;
   end;
   Result := nil;
end;



function TTopolElement.GetBlockDirectory: string;
var
  Vlastnik: TTopolObjects;
begin
  Result:= '';
  Vlastnik:= GetObjectsOwner;
{ hledani nadrizene entity TTopolPlochy primo nebo pres TTopolPlocha }
  if Vlastnik <> nil then
     Result:= Vlastnik.BlockDir
end;



function TTopolElement.GetDBFName: string;
begin
  result:= GetDBFNameForObject(BlockDir, ObjectType, Self.Druh);
end;




function   TTopolElement.GetPropertyStringByName(AName: string): string;
{$IFDEF Databaze}
var
  FTLDBF: TTopolDBFTable;
  Vlastnik: TTopolObjects;
  aDBFName: string;
{$ENDIF}
begin
  result:= inherited GetPropertyStringByName(aName);
{$IFDEF Databaze}

  if result = '' then
  begin
     try
(*
        try
           FTLDBF:= TTopolDBFTable.Create(Application);
           FTLDBF.SetForTopolObject(BlockDir, ObjectType, Druh, Cislo);
           result:= FTLDBF.FieldAsString(aName);
        finally
           FTLDBF.Free
        end;
*)
       Vlastnik:= GetObjectsOwner;
       if Vlastnik = nil then exit;
       aDBFName:= GetDBFNameForObject(Vlastnik.BlockDir, ObjectType, Druh);
       TTopolDBFTable(Vlastnik.DBF).SetForNameCislo(aDBFName, Cislo);
       result:= TTopolDBFTable(Vlastnik.DBF).GetFieldAsString(aName);
     except
        result := '';
     end;
  end;

{$ENDIF}
end;


{ TRectTopoLElement }

procedure TRectTopoLElement.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   Rect.MinX := aReader.ReadFloat;
   Rect.MinY := aReader.ReadFloat;
   Rect.MaxX := aReader.ReadFloat;
   Rect.MaxY := aReader.ReadFloat;
end;

procedure TRectTopoLElement.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteFloat(Rect.MinX);
   aWriter.WriteFloat(Rect.MinY);
   aWriter.WriteFloat(Rect.MaxX);
   aWriter.WriteFloat(Rect.MaxY);
end;

function  TRectTopoLElement.GetRect: TRectangle;
begin
   if not assigned(FRect) then FRect := TRectangle.Create;
   result := FRect;
end;

procedure TRectTopoLElement.SetRect(aRect: TRectangle);
begin
   if Assigned(FRect) then FRect.Free;
   FRect := nil;
   FRect := aRect;
end;


destructor  TRectTopoLElement.Destroy;
begin
   Rect := nil;
   inherited Destroy;
end;

function   TRectTopoLElement.IsInRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
begin
   result:= (MinX <= Rect.Min.X) and (MaxX >= Rect.Max.x) and
            (MinY <= Rect.Min.Y) and (MaxY >= Rect.Max.y)
end;


function   TRectTopoLElement.IsOverRegion(Minx, MinY, MaxX, MaxY: double): Boolean;
begin
   result := false;
   if (MinX > Rect.Max.X) or (MaxX < Rect.Min.x) or
      (MinY > Rect.Max.Y) or (MaxY < Rect.Min.y) then exit;
   result := true;
end;



function   TRectTopoLElement.GetRegion: TDoubleRect;
begin
   result := Rect.AsDoubleRect;
end;


{ TTopolEntity }

procedure TTopolEntity.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   aReader.Read(FCislo, TTopolEntitySize);
end;

procedure TTopolEntity.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.Write(FCislo, TTopolEntitySize);
end;

procedure  TTopolEntity.Assign(Source: TPersistent);
var
  TopEnt: TTopolEntity;
begin
  if Source is TTopolEntity then
  begin
    inherited Assign(Source);
    TopEnt:= TTopolEntity(Source);
    Cislo   := TopEnt.Cislo;
    Druh    := TopEnt.Druh;
    Barva   := TopEnt.Barva;
    Znacka  := TopEnt.Znacka;
    Vyber   := TopEnt.Vyber;
    Presnost:= TopEnt.Presnost;
    Symbol  := TopEnt.Symbol;
    SymbolX := TopEnt.SymbolX;
    SymbolY := TopEnt.SymbolY;
    SymbolAngle := TopEnt.SymbolAngle;
    IsDone  := TopEnt.IsDone
  end;
end;

function TTopolEntity.CreateEnvelope: TElement;
begin
   result := inherited CreateEnvelope;
   TTopolEntity(result).Cislo   := Cislo;
   TTopolEntity(result).Druh    := Druh;
   TTopolEntity(result).Barva   := Barva;
   TTopolEntity(result).Znacka  := Znacka;
   TTopolEntity(result).Vyber   := Vyber;
   TTopolEntity(result).Presnost:= Presnost;
   TTopolEntity(result).Symbol  := Symbol;
   TTopolEntity(result).SymbolX := SymbolX;
   TTopolEntity(result).SymbolY := SymbolY;
   TTopolEntity(result).SymbolAngle:= SymbolAngle;
end;



function TTopolEntity.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
begin
  Result:= RegionIsInside(CoordEl.X-Tolerance, CoordEl.Y-Tolerance,
                          CoordEl.X+Tolerance, CoordEl.Y+Tolerance)
end;



function TTopolEntity.GetBlockDirectory: string;
var
  Vlastnik: TTopolObjects;
begin
  Result:= '';
  Vlastnik:= GetObjectsOwner;
{ hledani nadrizene entity TTopolPlochy primo nebo pres TTopolPlocha }
  if Vlastnik <> nil then
     Result:= Vlastnik.BlockDir
end;

function TTopolEntity.GetDBFName: string;
begin
  result:= GetDBFNameForObject(BlockDir, ObjectType, Self.Druh);
end;





function   TTopolEntity.GetPropertyStringByName(AName: string): string;
{$IFDEF Databaze}
var
  Vlastnik: TTopolObjects;
{  FTLDBF: TTopolDBFTable; }
  aDBFName: string;
{$ENDIF}
begin
  result:= inherited GetPropertyStringByName(aName);
{$IFDEF Databaze}
  if result = '' then
  begin
     try
(*
        try
           FTLDBF:= TTopolDBFTable.Create(Application);
           FTLDBF.SetForTopolObject(BlockDir, ObjectType, Druh, Cislo);
           result:= FTLDBF.FieldAsString(aName);
        finally
           FTLDBF.Free
        end;
*)
       Vlastnik:= GetObjectsOwner;
       if Vlastnik = nil then exit;
       aDBFName:= GetDBFNameForObject(Vlastnik.BlockDir, ObjectType, Druh);
       TTopolDBFTable(Vlastnik.DBF).SetForNameCislo(aDBFName, Cislo);
       result:= TTopolDBFTable(Vlastnik.DBF).GetFieldAsString(aName);
     except
        result := '';
     end;
  end;
{$ENDIF}
end;




{ TTopolBod }

procedure TTopoLBod.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   FUhel := aReader.ReadFloat;
end;

procedure TTopoLBod.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteFloat(FUhel);
end;


function TTopolBod.GetObjectsOwner: TTopolObjects;
var
  Vlastnik: TTopolObjects;
begin
  Result:= nil;
{ hledani nadrizene entity TTopolPlochy primo nebo pres TTopolPlocha }
  Vlastnik:= TTopolObjects(FindOwnerClass(TTopolBody));
  if TElement(Vlastnik) is TTopolBody then
     Result:= TTopolObjects(Vlastnik)
end;





function TTopolBod.GetObjectType: char;
begin
  result:= 'B'
end;


{ TTopoLText }

destructor  TTopoLText.Destroy;
begin
   Veta := '';
   inherited Destroy;
end;

procedure TTopoLText.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   Veta         := aReader.ReadString;
   VychoziBod.x := aReader.ReadFloat;
   VychoziBod.y := aReader.ReadFloat;
   aReader.Read( Font, SizeOf(Font)      +
                       SizeOf(Vyska)     +
                       SizeOf(KoefSirky) +
                       SizeOf(Smer)      +
                       SizeOf(Sklon)     +
                       SizeOf(Rozestupy) +
                       SizeOf(Atributy)  +
                       SizeOf(RezimTextu));
end;

procedure TTopoLText.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteString(Veta);
   aWriter.WriteFloat(VychoziBod.x);
   aWriter.WriteFloat(VychoziBod.y);
   aWriter.Write(Font, SizeOf(Font)      +
                       SizeOf(Vyska)     +
                       SizeOf(KoefSirky) +
                       SizeOf(Smer)      +
                       SizeOf(Sklon)     +
                       SizeOf(Rozestupy) +
                       SizeOf(Atributy)  +
                       SizeOf(RezimTextu));
end;

function    TTopolText.CreateEnvelope: TElement;
begin
    result := inherited CreateEnvelope;
    TTopolText(result).Font         := Font;
    TTopolText(result).Vyska        := Vyska;
    TTopolText(result).KoefSirky    := KoefSirky;
    TTopolText(result).Smer         := Smer;
    TTopolText(result).Sklon        := Sklon;
    TTopolText(result).Rozestupy    := Rozestupy;
    TTopolText(result).Atributy     := Atributy;
    TTopolText(result).RezimTextu   := RezimTextu;
    TTopolText(result).VychoziBod.x := VychoziBod.x;
    TTopolText(result).VychoziBod.y := VychoziBod.y;
    TTopolText(result).Veta         := Veta;
end;




function TTopolText.GetObjectsOwner: TTopolObjects;
var
  Vlastnik: TTopolObjects;
begin
  Result:= nil;
{ hledani nadrizene entity TTopolPlochy primo nebo pres TTopolPlocha }
  Vlastnik:= TTopolObjects(FindOwnerClass(TTopolTexty));
  if Vlastnik is TTopolTexty then
     Result:= TTopolObjects(Vlastnik)
end;




procedure TTopolText.SetCodePage(aCode: TCodePage);
begin
  FCodePage:= aCode
end;

function TTopolText.GetCodePage: TCodePage;
begin
  result:= FCodePage
end;

function    TTopoLText.GetVeta: string;
begin
   result := GetPString(FVeta);
end;

procedure   TTopoLText.SetVeta(aVeta: string);
begin
   SetPString(FVeta, aVeta);
end;

function TTopoLText.GetVychoziBod: TCoordinate;
begin
   if not assigned(FVychoziBod) then FVychoziBod := T2DCoordinate.Create;
   result := FVychoziBod;
end;

procedure   TTopoLText.SetVychoziBod(aVychoziBod: TCoordinate);
begin
   if assigned(FVychoziBod) then FVychoziBod.Free;
   FVychoziBod := nil;
   FVychoziBod := aVychoziBod;
end;

function TTopolText.GetObjectType: char;
begin
  result:= 'T'
end;





{ TTopolLin }

procedure TTopolLin.Assign(Source: TPersistent);
var
  Lin: TTopolLin;
begin
  try
    if Source is TTopolLin then
    begin
      inherited Assign(Source);
      Lin:= TTopolLin(Source);

      LevaPlocha    := Lin.LevaPlocha;
      PravaPlocha   := Lin.PravaPlocha;
      DrziSe        := Lin.DrziSe;
      PuvodniCislo  := Lin.PuvodniCislo;
    end;
  except
    on E:Exception do
       ShowMessage('Chyba piazovn linie : '
                   + E.Message);
  end
end;

procedure TTopoLLin.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   aReader.Read(FLevaPlocha, 3* SizeOf(longInt));
   aReader.Read(DrziSe, sizeOf(TDrziSe) + sizeOf(boolean))
end;

procedure TTopoLLin.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.Write(FLevaPlocha, 3* SizeOf(longInt));
   aWriter.Write(DrziSe, sizeOf(TDrziSe) + sizeOf(boolean))
end;


function TTopolLin.GetObjectsOwner: TTopolObjects;
var
  Plocha: TTopolPlocha;
begin
  result:= TTopolObjects(FindOwnerClass(TTopolLinie));
  if result <> nil then exit;

  Plocha:= TTopolPlocha(FindOwnerClass(TTopolPlocha));
  if Plocha = nil then exit;
  result:= Plocha.GetObjectsOwner;
end;





function TTopolLin.GetLength: double;
var
  I: longInt;
  X1, Y1, X2, Y2: double;
begin
  result:= 0;
  GetCoord(0, true, X1, Y1);
  for I:= 0 to Count-2 do
  begin
    GetCoord(I+1, true, X2, Y2);
    result:= result + Distance(X1, Y1, X2, Y2);
    X1:= X2; Y1:= Y2;
  end;
end;

procedure TTopolLin.SetX(aIndex: longInt; aX: double);
begin
  if aIndex < Count then
  begin
    T2DCoordinate(Elements[aIndex]).X:= aX
  end
end;

function  TTopolLin.GetX(aIndex: longInt): double;
begin
  result:= 0;
  if aIndex < Count then
     result:= T2DCoordinate(Elements[aIndex]).X
end;

procedure TTopolLin.SetY(aIndex: longInt; aY: double);
begin
  if aIndex < Count then
  begin
    T2DCoordinate(Elements[aIndex]).Y:= aY
  end
end;

function  TTopolLin.GetY(aIndex: longInt): double;
begin
  result:= 0;
  if aIndex < Count then
     result:= T2DCoordinate(Elements[aIndex]).Y
end;

function TTopolLin.GetObjectType: char;
begin
  result:= 'L'
end;

procedure TTopolLin.GivePointPropsForLength(var Length, pX, pY, pAngle: double);
var
  AuxLen, AbscLen: double;
  X1, Y1, X2, Y2: double;
  Found: boolean;
  I: longInt;
begin
  pX:= 0;
  pY:= 0;
  pAngle:= 0;

  if Length > GetLength then
  begin
    GetCoord(Count-1, true, pX, pY);
    exit;
  end;

  if Count = 0 then exit;

  if Count = 1 then
  begin
    GetCoord(0, true, pX, pY);
    exit
  end;

  I:= 0;
  AuxLen:= 0;

  Found:= false;
  while (I < Count-1) and not Found do
  begin
    GetCoord(I, true, X1, Y1);
    GetCoord(I+1, true, X2, Y2);
    AbscLen:= Distance(X1, Y1, X2, Y2);

    if AuxLen + AbscLen > Length then
    begin
      GiveAbscPointByLength(X1, Y1, X2, Y2, Length-AuxLen, 0, pX, pY);
      pAngle:= Atan2M(X2-X1, Y2-Y1) + Pi/2;
      Found:= true;
    end
    else
    begin
      AuxLen:= AuxLen + AbscLen;
      inc(I)
    end
  end
end;

procedure TTopolLin.GiveLabelProps(var X, Y, Angle: double);
var
  Half: double;
begin
  Half:= GetLength/2;

  GivePointPropsForLength(Half, X, Y, Angle);
end;

procedure TTopolLin.CalculateLabelProps;
begin
  GiveLabelProps(FSymX, FSymY, FSymAngle)
end;

procedure TTopolLin.RecalculateRect;
var
  I: longInt;
  X, Y: double;
begin
  MinX:= 1E30;
  MinY:= 1E30;
  MaxX:= -1E30;
  MaxY:= -1E30;
  for I:= 0 to Count-1 do
  begin
    GetCoord(I, true, X, Y);
    FRect.UpdateByXY(X, Y)
  end
end;

function TTopolLin.GetCoordElement(aIndex: longInt; aBegin: boolean): T2DCoordinate;
begin
  Result:= nil;
  if aIndex < Count then
  begin
    if not aBegin then aIndex:= Count-aIndex-1;
    result:= T2DCoordinate(Elements[aIndex])
  end
end;



procedure TTopolLin.GetCoord(aIndex: longInt; aBegin: boolean; var aX, aY: double);
begin
  aX:= 0; aY:= 0;
  if aIndex < Count then
  begin
    try
      if not aBegin then aIndex:= Count-aIndex-1;
      aX:= X[aIndex];
      aY:= Y[aIndex]
    except
      aX:= aX
    end
  end
end;

procedure TTopolLin.SetOrientedX(aIndex: longInt; aBegin: boolean; aCoord: double);
begin
  GetCoordElement(aIndex, aBegin).X:= aCoord
end;

procedure TTopolLin.SetOrientedY(aIndex: longInt; aBegin: boolean; aCoord: double);
begin
  GetCoordElement(aIndex, aBegin).Y:= aCoord
end;

function TTopolLin.GetSubAreaValue(FromBegin: boolean): double;
var
  I: longInt;
  X1, Y1, X2, Y2: double;
begin
  result:= 0;
  if Count > 1 then
  begin
    GetCoord(0, FromBegin, X1, Y1);
    for I:= 1 to Count-1 do
    begin
      GetCoord(I, FromBegin, X2, Y2);
      result:= result + (X2-X1)*(Y1+Y2);
      X1:= X2;
      Y1:= Y2
    end;
    result:= result/2
  end
end;

function TTopolLin.GetRightAreaForGroup
                   (aGroup: smallInt; aRightArea: boolean): TTopolPlocha;
{Nalezen t plochy z mnoha monch(vlastncch uvedenou linii), kter je uvede-
 nho druhu (pedpokld se, e kad z ploch reprezentuje nco jinho a tud
 je i jinho druhu) a dr linii v uvedenm smru. }
var
  I: longInt;
  Nasel: boolean;
  Plocha: TTopolPlocha;
begin
  result:= nil;
  I:= 0;
  Nasel:= false;
  while (I < OwnersCount) and not Nasel do
  begin
    if (Owners[I] is TTopolPlocha) {and (TTopolPlocha(Owners[I]).Druh = aGroup)} then
    begin
      Plocha:= TTopolPlocha(Owners[I]);
      Nasel:= Plocha.HasLine(self, not aRightArea)
    end;
    if not Nasel then inc(I)
  end;
  if Nasel then
     result:= TTopolPlocha(Owners[I])
end;





function TTopolLin.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
var
  I: longInt;
  Uvnitr: boolean;
  XP, YP, Vzd: double;

begin
  Result:= false;
  for I:= 0 to Count-2 do
  begin
    DejVztahyBod_Usecka(CoordEl.X, CoordEl.Y, X[I], Y[I], X[I+1], Y[I+1],
                        Uvnitr, Xp, Yp, Vzd);
    if Uvnitr and (Vzd < Tolerance) then
    begin
      Result:= true;
      Break
    end
  end
end;



                        { TTopolPlochaBoundary }

procedure TTopolPlochaBoundary.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   aReader.Read( FMaEnklavu, SizeOf(FMaEnklavu) +
                             SizeOf(FCisloOtce) +
                             SizeOf(PrvniLinie))
end;

procedure TTopolPlochaBoundary.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.Write(FMaEnklavu, SizeOf(FMaEnklavu) +
                             SizeOf(FCisloOtce) +
                             SizeOf(PrvniLinie))
end;

function TTopolPlochaBoundary.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
begin
  Result:= false;
  if inherited IsHit(CoordEl, Tolerance) then
  begin
    Result:= InsideTest(Self, CoordEl)
  end;
end;



function TTopolPlochaBoundary.HasChildren: boolean;
begin
  result:= false
end;



function TTopolPlochaBoundary.GetObjectsOwner: TTopolObjects;
var
  Vlastnik: TTopolObjects;
  SubAreas: TSubAreas;
  Plocha: TTopolPlochaBoundary;
begin
  Result:= nil;
{ hledani nadrizene entity TTopolPlochy primo nebo pres TTopolPlocha }
  Vlastnik:= TTopolObjects(FindOwnerClass(TTopolPlochy));
  if Vlastnik = nil then
  begin
    SubAreas:= TSubAreas(FindOwnerClass(TSubAreas));
    if SubAreas <> nil then
    begin
      Plocha:= TTopolPlochaBoundary(SubAreas.FindOwnerClass(TTopolNadPlocha));
      if Plocha <> nil then
         result:= Plocha.GetObjectsOwner
    end
  end
  else
     Result:= Vlastnik
end;






function TTopolPlochaBoundary.GetObjectType: char;
begin
  result:= 'P'
end;




function TTopolPlochaBoundary.GetCoordsCount: longInt;
begin
  Result:= Count;
end;



function TTopolPlochaBoundary.GetCoordinateElement(Index: longInt): T2DCoordinate;
begin
  result:= nil;
  if Index < Count then
     result:= T2DCoordinate(Elements[Index])
end;


procedure TTopolPlochaBoundary.GiveCoords(Index: longInt; var X, Y: double);
var
  C2D: T2DCoordinate;
begin
  X:= 0; Y:= 0;
  C2D:= GetCoordinateElement(Index);
  if C2D <> nil then
  begin
    X:= C2D.X;
    Y:= C2D.Y
  end;
end;





function TTopolPlochaBoundary.GetCalculatedArea: double;
var
  XP, YP, X1, Y1, X2, Y2: double;
  CCount: longInt;
  I: longInt;
begin
  try
    CCount:= GetCoordsCount;
    GiveCoords(0, X1, Y1);
    XP:= X1; YP:= Y1;
    result:= 0;
    for I:= 1 to CCount - 1 do
    begin
      GiveCoords(I, X2, Y2);
      result:= result + (X2-X1) * (Y2+Y1)/2;
      X1:= X2; Y1:= Y2;
    end;
    result:= result + (XP-X1) * (YP+Y1)/2
  except
    ShowMessage('GetCalculatedArea Error')
  end
end;




{ TTopolPlocha }

procedure TTopolPlocha.Load(aReader: TReader; aIndex: TEntity);
begin
   inherited Load(aReader, aIndex);
   SetPString(FLinesOrigines, aReader.ReadString);
end;

procedure TTopolPlocha.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteString(GetPString(FLinesOrigines));
end;

procedure GivePosition(Index: longInt; var ByteNo: longInt; var ByteMask: byte);
{ ByteNo: posice v retezci, 1.Byt ma ByteNo=1, 1.bit v Bytu ma BytePos=0}
begin
  ByteNo:= Index div 8 + 1;
  if Index > 7 then
  begin
    ByteMask:= Index mod 8
  end
  else
    ByteMask:= Index;
{ prevod na masku}
  ByteMask:= 1 shl ByteMask
end;

function TTopolPlocha.GetOrigin(Index: longInt): boolean;
var
  ByteNo: longInt;
  ByteMask: byte;

begin
  Result:= false;
  GivePosition(Index, ByteNo, ByteMask);
  if (FLinesOrigines <> nil) and (length(FlinesOrigines^) >= ByteNo) then
  begin
    Result:= (ord(FLinesOrigines^[ByteNo]) and ByteMask) <> 0
  end;
end;


procedure TTopolPlocha.SetOrigin(Index: longInt; Item: boolean);
var
  ByteNo, ByteCount: longInt;
  ByteMask: byte;
  Buf: pointer;
begin
  GivePosition(Index, ByteNo, ByteMask);
  if FLinesOrigines <> nil then
     ByteCount:= length(FlinesOrigines^)
  else ByteCount:= 0;

  if ByteCount < ByteNo then
  begin
    GetMem(Buf, ByteNo + 4);
    if ByteCount > 0 then
    begin
      PString(Buf)^:= FLinesOrigines^;
      DisposeStr(FLinesOrigines)
    end;
    FLinesOrigines:= Buf;
   {$IFDEF WIN32}
      SetLength(FLinesOrigines^, ByteNo + 3);
   {$ELSE}
      FLinesOrigines^[0]:= chr(ByteNo + 3)
   {$ENDIF}
  end;

  if FLinesOrigines <> nil then
  begin
    if Item then
       FLinesOrigines^[ByteNo]:= chr(ord(FLinesOrigines^[ByteNo])and not ByteMask or ByteMask)
    else
       FLinesOrigines^[ByteNo]:= chr(ord(FLinesOrigines^[ByteNo])and not ByteMask)
  end
end;


function TTopolPlocha.HasLine(aLine: TTopolLin; aBegin: boolean): boolean;
var
  I: longInt;
  Nasel: boolean;
  Lin: TTopolLin;
begin
  I:= 0;
  Nasel:= false;
  while (I < Count) and not Nasel do
  begin
    if Elements[I] is TTopolLin then
    begin
      Lin:= TTopolLin(Elements[I]);
      Nasel:= (Lin = aLine) and (LinesOrigines[I] = aBegin)
    end;
    if not Nasel then inc(I)
  end;
  result:= Nasel
end;


function   TTopolPlocha.CreateEnvelope: TElement;
begin
   result := inherited CreateEnvelope;
   SetPString(TTopolPlocha(result).FLinesOrigines, FLinesOrigines^);
end;




function TTopolPlocha.CreatePlochaBoundary: TTopolPlochaBoundary;
var
  I, J: longInt;
  Linie: TTopolLin;
begin
  Result:= TTopolPlochaBoundary.Create;

  Result.Cislo   := Self.Cislo;
  Result.Druh    := Self.Druh;
  Result.Barva   := Self.Barva;
  Result.Znacka  := Self.Znacka;
  Result.Presnost:= Self.Presnost;

  { prirazeni souradnic }
  for I:= 0 to Count-1 do
  begin
    if TObject(Elements[I]) is TTopolLin then
    begin
      Linie:= TTopolLin(Elements[I]);
      if LinesOrigines[I] then
      for J:= 1 to Linie.Count-1 do
         Result.AddElement(Linie.Elements[J])
      else
      for J:= Linie.Count-2 downTo 0 do
         Result.AddElement(Linie.Elements[J])
    end
  end;
  result.AddOwner(ObjectOwner);
  Result.Rect:= FRect.CreateDuplicate;
end;





function TTopolPlocha.GetCoordsCount: longInt;
var
  I, J: longInt;
  Linie: TTopolLin;
begin
  Result:= 0;
  for I:= 0 to Count-1 do
  begin
    if TObject(Elements[I]) is TTopolLin then
    begin
      Linie:= TTopolLin(Elements[I]);
      Result:= Result + Linie.Count-1;
    end
  end
end;


function TTopolPlocha.GetFather: TTopolNadPlocha;
var
  SubAreas: TSubAreas;

begin
  result:= nil;
  SubAreas:= TSubAreas(FindOwnerClass(TSubAreas));
  if SubAreas <> nil then
     result:=  TTopolNadPlocha(FindOwnerClass(TTopolNadPlocha))
end;

function TTopolPlocha.GetLinesOrigines: string;
begin
   result := GetPString(FLinesOrigines);
end;

procedure TTopolPlocha.SetLinesOrigines(const aValue: string);
begin
   SetPString(FLinesOrigines, aValue);
end;


function TTopolPlocha.GetCoordinateElement(Index: longInt): T2DCoordinate;
var
  I, J, AuxI: longInt;
  Linie: TTopolLin;
  Nasel: boolean;
begin
  I:= 0;
  AuxI:= 0;
  Nasel:= false;
  result:= nil;
  while (I < Count) and not Nasel do
  begin
    if TObject(Elements[I]) is TTopolLin then
    begin
      Linie:= TTopolLin(Elements[I]);
      if AuxI + Linie.Count-1 > Index then
      begin
        J:= Index - AuxI;
        Result:= T2DCoordinate(Linie.GetCoordElement(J, LinesOrigines[I]));
        Nasel:= true
      end
      else
        AuxI:= AuxI + Linie.Count-1
    end;
    if not Nasel then inc(I)
  end
end;




function TTopolPlocha.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
var
  PlochaBoundary: TTopolPlochaBoundary;
begin
  Result:= false;
  if RegionIsInside(CoordEl.X-Tolerance, CoordEl.Y-Tolerance,
                          CoordEl.X+Tolerance, CoordEl.Y+Tolerance) then
  begin
    PlochaBoundary:= CreatePlochaBoundary;
    Result:= InsideTest(PlochaBoundary, CoordEl);
    PlochaBoundary.Clear;
    PlochaBoundary.Free
  end;
end;




destructor TTopolPlocha.Destroy;
begin
  if FLinesOrigines <> nil then DisposeStr(FLinesOrigines);
  inherited Destroy
end;



{  TTopolNadPlocha }

procedure TTopolNadPlocha.Load(aReader: TReader; aIndex: TEntity);
var
   i: longInt;
begin
   inherited Load(aReader, aIndex);
   i := aReader.ReadInteger;
   if i > -1 then FSubAreas := TSubAreas(aIndex.Elements[i])
             else FSubAreas := nil;
end;

procedure TTopolNadPlocha.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   if assigned(FSubAreas) then aWriter.WriteInteger(FSubAreas.ID)
                          else aWriter.WriteInteger(-1);
end;


function TTopolNadPlocha.HasChildren: boolean;
begin
  result:= SubAreasCount > 0
end;


function TTopolNadPlocha.GetCalculatedArea: double;
begin
  result:= inherited GetCalculatedArea;
  FSavedArea:= result
end;


function TTopolNadPlocha.GetSubAreasCount: longInt;
begin
  result:= 0;
  if FSubAreas <> nil then
     result:= FSubAreas.Count
end;


function TTopolNadPlocha.GetCleanArea: double;
var
  I: longInt;
begin
  result:= Area;
  if FSubAreas <> nil then
  begin
    for I:= 0 to FSubAreas.Count-1 do
    begin
      result:= result - SubArea[I].Area
    end
  end;
end;



function TTopolNadPlocha.GetSubArea(Index: longInt): TTopolNadPlocha;
begin
  result:= nil;
  if FSubAreas <> nil then
  begin
    if Index < FSubAreas.Count then
    begin
      result:= TTopolNadPlocha(FSubAreas.Elements[Index])
    end
  end
end;



procedure TTopolNadPlocha.AddArea(aPlocha: TTopolPlocha);
begin
  if FSubAreas = nil then
  begin
    FSubAreas:= TSubAreas.Create;
    AddElement(FSubAreas)
  end;
  if FSubAreas <> nil then
  begin
    FSubAreas.AddElement(aPlocha)
  end;
end;


{ TTopolRectEntity }

procedure TTopolRectEntity.Load(aReader: TReader; aIndex: TEntity);
var
   i: longInt;
begin
   inherited Load(aReader, aIndex);
   BlockDir := aReader.ReadString;
end;

procedure TTopolRectEntity.Save(aWriter: TWriter);
begin
   inherited Save(aWriter);
   aWriter.WriteString(BlockDir);
end;

function TTopolRectEntity.GetBlockDir: string;
begin
  Result:= GetPString(FBlockDirectory)
end;

function TTopolRectEntity.GetDBF: TTable;
begin
  if FDBF = nil then
{$IFDEF Databaze}
     FDBF:= TTopolDBFTable.Create(Application)
{$ENDIF}
     ;
  result:= FDBF
end;



procedure TTopolRectEntity.SetBlockDir(FN: string);
begin
  SetPString(FBlockDirectory, FN)
end;

destructor TTopolRectEntity.Destroy;
begin
  if FDBF <> nil then
     FDBF.Free;
  inherited Destroy
end;

{ TTopolPlochy }

procedure TTopoLPlochy.AddAreaChildren(Area: TTopolPlocha);
var
  I: longInt;
  InSide: boolean;

begin
  I:= 0;
  InSide:= false;
  while (I < Count) and not Inside do
  begin
    if TTopolPlocha(Elements[I]) is TTopolNadPlocha then
    begin
      TTopolNadPlocha(Elements[I]).AddArea(Area);

    end
  end
end;




BEGIN
   RegisterOrbisClass(TTopoLElement);
   RegisterOrbisClass(TTopoLEntity);

   RegisterOrbisClass(TTopoLBod);
   RegisterOrbisClass(TTopoLLin);
   RegisterOrbisClass(TTopoLText);
   RegisterOrbisClass(TTopoLPlochaBoundary);
   RegisterOrbisClass(TTopoLPlocha);
   RegisterOrbisClass(TTopoLNadPlocha);
   RegisterOrbisClass(TSubAreas);

   RegisterOrbisClass(TTopoLLinie);
   RegisterOrbisClass(TTopoLLinieDruhu);
   RegisterOrbisClass(TTopoLBody);
   RegisterOrbisClass(TTopoLPlochy);
   RegisterOrbisClass(TTopoLTexty);
   RegisterOrbisClass(TTopoLOstatni);

   RegisterOrbisClass(TTopoLBlok);
   RegisterOrbisClass(TTopoLPredmetZajmu);
END.
