unit T_arelin;
{
[Unit_Description]
Description=Nstroje pro hran s liniemi ve vlastnictv pnoch
Admin=Olda
TechTopic=Tools
Topic=Space analysis
Version=1.0

[Edit]
 8.8.1997 19:47=Registrace
}
interface
uses
// KERNEL
  Elements,
  E_Type,
  Tools,
// SPACE
  e_TopoLElements,
  e_GeometryElement,
  e_Geometry;
  //E_Space;

type

TExtractAnyLines = class(TTool)
{ pedek vyjma lini
  input - neobsloueno
  output - entita lini, kter me bt nastavena propertou Lines }
private
  FLines: TEntity;
  procedure SetLines(aLines: TEntity);
  function  GetLines: TEntity;
protected
  procedure AddLine(aLine: TTopoLLineElement); virtual;
published
  property Lines: TEntity read GetLines write SetLines;
end;


TExtractAnyLinesForArea = class(TExtractAnyLines)
{ pedek vyjmae lini z plochy
  input - plocha
  output - entita lini, kter me bt nastavena propertou Lines }
protected
  function  GetAreaFromInput: TTopoLAreaElement;
end;



TExtractBoundaryLinesForArea = class(TExtractAnyLinesForArea)
{ vyjme vechny linie obklopujc plochu
  input - plocha
  output - entita lini, kter me bt nastavena propertou Lines }
public
  procedure DoIt; override;
end;



TExtractHolesLinesForArea = class(TExtractAnyLinesForArea)
{ vyjme vechny linie obklopujc dry
  input - plocha
  output - entita lini }
public
  procedure DoIt; override;
end;



TExtractLinesForAreas = class(TExtractAnyLines)
{ vyjme vechny linie psluc plochm v entit
  input - entita ploch
  output - entita lini, kter me bt nastavene propertou Lines
}
public
  procedure ExtractLinesForArea(anArea: TTopoLAreaElement); virtual;
  procedure DoIt; override;
end;



TExtractSubAreasForArea = class(TTool)
{ vytvo entitu vech ploch, kter jsou bezprostedn obklopeny plochou na vstupu
  input - zkouman plocha
  output - entita podploch, me bt nastavena propertou SubAreas}
private
  FSubAreas: TEntity;
  FAreaCollection: TAreaCollection;
  procedure SetSubAreas(aSubAreas: TEntity);
  function  GetSubAreas: TEntity;
  procedure AddArea(anArea: TTopoLAreaElement);
public
  procedure DoIt; override;
published
  property SubAreas: TEntity read GetSubAreas write SetSubAreas;
  property AreaCollection: TAreaCollection read FAreaCollection write FAreaCollection;
end;



TExportElements = class(TTool)
private
  FExpProps: TPropertyList;
  function GetExpProps: TPropertyList;
  procedure SetNextPropName(aName: string);
  procedure SetDelimiter(aValue: string);
public
  destructor Destroy; override;
published
  property ExpProps: TPropertyList read GetExpProps;
  property NextPropName: string write SetNextPropName;
  property Delimiter: string write SetDelimiter;
end;



implementation
uses
// GLIB
  G_Elems;


                       { TExtractAnyLines }

procedure TExtractAnyLines.SetLines(aLines: TEntity);
begin
  FLines:= aLines;
end;


function  TExtractAnyLines.GetLines: TEntity;
begin
  result:= FLines;
  if FLines <> nil then exit;
  FLines:= TPolyLineCollection.Create;
  result:= FLines;
end;


procedure TExtractAnyLines.AddLine(aLine: TTopoLLineElement);
var
  I: longInt;
begin
  if Lines = nil then exit;
  I:= Lines.IndexOf(aLine);
  if I > -1 then exit;
  if Lines is TRectEntity then
     TRectEntity(Lines).AddAndUpdateRect(aLine)
  else
    Lines.AddElement(aLine)
end;


                      { TExtractAnyLinesForArea }


function TExtractAnyLinesForArea.GetAreaFromInput: TTopoLAreaElement;
begin
  result:= TTopoLAreaElement(input);
  if input=nil then exit;
  if TElement(result) is TTopoLAreaElement then exit;
  result:= nil;
end;



                     { TExtractBounderyLinesForArea }

{ vyjme vechny linie obklopujc plochu
  input - plocha
  output - entita lini, kter me bt nastavena propertou Lines }

procedure TExtractBoundaryLinesForArea.DoIt;
var
  anArea: TTopoLAreaElement;
  Are: TPolyLineAreaElement;
  I: longInt;
  aLine: TPolyLineElement;
  TopoLLine: TTopoLLineElement;
begin
  AnArea:= GetAreaFromInput;
  if anArea = nil then exit;
  if anArea.Geometry = nil then exit;
  if not (AnArea.Geometry is TPolyLineAreaElement) then exit;
  Are := TPolyLineAreaElement(AnArea.Geometry);
  for I:= 0 to Are.LastIndex do
  begin
    aLine:= TPolyLineElement(Are.Elements[I]);
    if not (TElement(aLine) is TPolyLineElement) then continue;
    //TopoLLine := TTopoLLineElement(aLine.FindOwnerClass(TTopoLLineElement));
    TopoLLine := TTopoLLineElement(aLine.FindClassWithOwnerInOwners(TTopoLLineElement,Lines));
    if TopoLline = nil then exit; //!!!!!!!!!!!!!chyba
    AddLine(TopoLLine);
  end;
  Output:= Lines
end;



                 { TExtractHolesLinesForArea }

{ vyjme vechny linie obklopujc dry
  input - plocha
  output - entita lini }

procedure TExtractHolesLinesForArea.DoIt;
var
  anArea: TPolyLineAreaElement;
  Are: TTopolAreaElement;
  aHole: TPolyLineAreaElement;
  EBLFA: TExtractBoundaryLinesForArea;
  I: longInt;
begin
  Are:= GetAreaFromInput;
  if Are = nil then exit;
  AnArea := TPolyLineAreaElement(Are.Geometry);
  if anArea = nil then exit;
  if anArea.Holes = nil then exit;
  if anArea.Holes.Count < 1 then exit;

  for I:= 0 to anArea.Holes.Count-1 do
  begin
    aHole:= TPolyLineAreaElement(anArea.Holes.Elements[I]);
    if not (aHole is TPolyLineAreaElement) then continue;
    Are := TTopoLAreaElement(aHole.FindOwnerClass(TTopoLAreaElement));
    if Are = nil then continue; // !!!!!!!!!!!! chyba ... nen topol plocha
    EBLFA:= TExtractBoundaryLinesForArea.Create;
    EBLFA.Input := Are;
    EBLFA.Lines := Lines;
    EBLFA.DoIt;
    EBLFA.Free;
  end;

  Output:= Lines
end;




                 { TExtractLinesForAreas }

procedure TExtractLinesForAreas.ExtractLinesForArea(anArea: TTopoLAreaElement);
var
  EBLFA: TExtractBoundaryLinesForArea;
begin
  EBLFA:= TExtractBoundaryLinesForArea.Create;
  EBLFA.Input:= anArea;
  EBLFA.Lines:= Lines;
  EBLFA.DoIt;
  EBLFA.Free;
end;



procedure TExtractLinesForAreas.DoIt;
var
  anArea: TTopoLAreaElement;
  I: longInt;
begin
  if Input = nil then exit;
  if not (Input is TEntity) then exit;

  for I:= 0 to TEntity(Input).LastIndex do
  begin
    anArea:= TTopoLAreaElement(TEntity(Input).Elements[I]);
    if not (TElement(anArea) is TTopoLAreaElement) then continue;
    ExtractLinesForArea(anArea);
  end;
  Output:= Lines
end;




                   { TExtractSubAreasForArea }


procedure TExtractSubAreasForArea.SetSubAreas(aSubAreas: TEntity);
begin
  FSubAreas:= aSubAreas;
end;


function  TExtractSubAreasForArea.GetSubAreas: TEntity;
begin
  result:= FSubAreas;
  if FSubAreas <> nil then exit;
  FSubAreas:= TAreaCollection.Create;
  result:= FSubAreas;
end;


procedure TExtractSubAreasForArea.AddArea(anArea: TTopoLAreaElement);
begin
  if SubAreas = nil then exit;
  if SubAreas is TRectEntity then
     TRectEntity(SubAreas).AddAndUpdateRect(anArea)
  else
     SubAreas.AddElement(anArea)
end;


procedure TExtractSubAreasForArea.DoIt;
var
  anArea: TTopoLAreaElement;
  Are: TAreaElement;
  Lines: TEntity;
  EHLA: TExtractHolesLinesForArea;
  I, J: longInt;
  aLine: TTopoLLineElement;
  Lin: TPolyLineElement;
  AreaOwner: TPolyLineAreaElement;
begin
  if Input = nil then exit;
  if not (Input is TTopoLAreaElement) then exit;
  Output:= nil;
  anArea := TTopoLAreaElement(Input);
  if anArea.Geometry = nil then exit;
  Are:= TAreaElement(anArea.Geometry);

  if Are.Holes = nil then exit;
  if Are.Holes.Count = 0 then exit;

  EHLA:= TExtractHolesLinesForArea.Create;
  EHLA.Input:= anArea;
  EHLA.DoIt;
  Lines:= TEntity(EHLA.Output);
  EHLA.Free;

  if Lines = nil then exit;

  for I:= 0 to Lines.Count-1 do
  begin
    aLine:= TTopoLLineElement(Lines.Elements[I]);
    if not (TElement(aLine) is TPolyLineElement) then continue;
    Lin := TPolyLineElement(aLine.Geometry);

    for J:= 0 to Lin.OwnersCount-1 do
    begin
      AreaOwner:= TPolyLineAreaElement(aLine.Owners[J]);
      if not (TElement(AreaOwner) is TPolyLineAreaElement) then continue;
      if AreaOwner.IsMyOwner(Are.Holes) then continue;
      anArea := TTopoLAreaElement(AreaOwner.FindElementClass(TTopoLAreaElement));
      if AreaCollection <> nil then
      begin
        if not anArea.IsMyOwner(AreaCollection) then continue
      end;

      AddArea(anArea);
    end;
  end;

  Lines.Free;

  Output:= SubAreas
end;



                   { TExportElements }


function TExportElements.GetExpProps: TPropertyList;
begin
  result:= FExpProps;
  if result <> nil then exit;
  FExpProps:= TPropertyList.Create;
  result:= FExpProps;
end;



procedure TExportElements.SetNextPropName(aName: string);
begin
  if aName = '' then exit;
  if ExpProps = nil then exit;
  ExpProps.NextName:= aName;
end;




procedure TExportElements.SetDelimiter(aValue: string);
begin
  if ExpProps = nil then exit;
  ExpProps.Delimiter:= aValue
end;



destructor TExportElements.Destroy;
begin
  if FExpProps <> nil then
     FExpProps.Free;
  inherited Destroy
end;



end.
