unit Nastroje;

interface

Uses DB, dbiProcs, dbiTypes, DBTables, SysUtils, Classes, IniFiles, WinProcs;

function CisloZaznamu(Data:TDataSet):LongInt;
function ZpakujTabulku(Tabulka : TTable): SmallInt;
function UlozBufferNaDisk(TabHandle : HDbiCur): SmallInt;
function VytvorAliasCfg(Jmeno, Cesta: String):SmallInt;
function ZkopirujSoubor(ZdrojovySoubor, CilovySoubor: String): Word;
function CestaKAliasu(JmenoAliasu : string; Lomitko : boolean): String;
procedure SpustProgram(CestakProgramu,Parametr : string);
function OdstranMezery(S : String):String;


implementation

function CisloZaznamu(Data:TDataSet):LongInt;
Var RP:RECProps;
    CP:CURProps;
    CisloZaznamu:LongInt;
begin
 Data.UpdateCursorPos;
 dbigetcursorprops(Data.handle,cp);
 Case CP.iSeqNums of
  0 : begin {pro tabulky dBASE}
       DbiGetRecord (Data.Handle,dbiNoLock,nil,@RP);
       CisloZaznamu:=RP.iPhyRecNum;
      end;
  1 : begin {pro tabulky Paradox}
       DbiGetSeqNo (Data.Handle,CisloZaznamu);
      end
  else
   Result := -1;
   exit;
{   raise EDatabaseError.Create('Tabulka nen typu dBASE nebo Paradox!');}
  end;
 Data.CursorPosChanged;
 Result := CisloZaznamu;
end;

function ZpakujTabulku(Tabulka : TTable): SmallInt;
var
  Exkluzivne,Otevrena : Boolean;
  JmenoTabulky : array [0..100] of char;
begin
  Exkluzivne := Tabulka.Exclusive;
  Otevrena := Tabulka.Active;

  StrPCopy(JmenoTabulky, Tabulka.TableName);
  try
    Tabulka.Close;
    Tabulka.Exclusive := true;
    Tabulka.Open;
  except
    Result := -1;
  end;

  try
    dbiPackTable(Tabulka.DBHandle, Tabulka.Handle, JmenoTabulky, nil, True);
  except
    result := -2;
  end;

  Tabulka.Close;
  Tabulka.Exclusive := Exkluzivne;
  Tabulka.Active := Otevrena;

  Result := 0;
end;

function UlozBufferNaDisk(TabHandle : HDbiCur): SmallInt;
begin
  try
    dbiSaveChanges(TabHandle);
    Result := 0;
  except
    Result := -1;
  end;
end;

function VytvorAliasCfg(Jmeno, Cesta: String):SmallInt;
var
  S1: array[0..100] of char;
  S2: array[0..100] of char;
begin
  try
    StrPCopy(S1, Jmeno);
    StrPCopy(S2, Cesta);
    DbiAddAlias(Nil, S1, Nil, S2, False);
    Result := 0;
  except
    Result := -1;
  end;
end;

function ZkopirujSoubor(ZdrojovySoubor, CilovySoubor: String): Word;
var
  ErrorCode,BytesRead,BytesWritten : Integer;
  Time : LongInt;
  Src,Dest : File;
  SaveFileMode : Word;
  SaveFAttr : Word;
  HSour, HDest: LongInt;
  Buffer: Pointer;

  procedure UnDo(CloseAndDeleteDest : Boolean);
  begin
    Close(Src);
    if IoResult <> 0 then ;
    if CloseAndDeleteDest then begin
      Close(Dest);
      if IoResult <> 0 then ;
      Erase(Dest);
      if IoResult <> 0 then ;
    end;
  end;

begin
  GetMem(Buffer, 8096);
  try
    SaveFileMode := FileMode;
    Assign(Src, ZdrojovySoubor);
    FileMode := FileMode and $F0;
    Reset(Src,1);
    FileMode := SaveFileMode;
    if IoResult <> 0 then begin
      ZkopirujSoubor := 1;                   {unable to open SrcFile}
      Exit;
    end;
    Assign(Dest, CilovySoubor);
    Rewrite(Dest,1);
    if IoResult <> 0 then begin
      ZkopirujSoubor := 2;                   {unable to open DestFile}
      Undo(False);
      Exit;
    end;
    while not EOF(Src) do begin
      BlockRead(Src,Buffer^,8096,BytesRead);
      if IoResult <> 0 then begin
        ZkopirujSoubor := 3;                 {error reading SrcFile}
        UnDo(True);
        Exit;
      end;
      BlockWrite(Dest,Buffer^,BytesRead,BytesWritten);
      if (IoResult <> 0) or (BytesWritten <> BytesRead) then begin
        ZkopirujSoubor := 4;                 {error reading SrcFile}
        UnDo(True);                    {error writing DestFile}
        Exit;
      end;
    end;
    Close(Dest);
    if IoResult <> 0 then begin
      ZkopirujSoubor := 7;
      Close(Src);
      if IoResult <> 0 then ;
    end
    else begin
      Close(Src);
      if IoResult <> 0 then ;
      ZkopirujSoubor := 0;
    end;
    HSour := FileOpen(ZdrojovySoubor, fmOpenRead);
    if HSour > 0 then begin
      HDest := FileOpen(CilovySoubor, fmOpenWrite);
      if HDest > 0 then begin
        FileSetDate(HDest, FileGetDate(HSour));
        FileClose(HDest);
      end;
      FileClose(HSour);
    end;
  finally
    FreeMem(Buffer, 8096);
  end;
end;

function CestaKAliasu(JmenoAliasu : string; Lomitko : boolean): String;
var
  MyString : TStringList;
  fg       : word;
  S        : String;
begin
  Result := '';
  MyString := TStringList.Create;
  try
    Session.GetAliasParams(JmenoAliasu,MyString);
  except
    MyString.Free;
    Exit;
  end;
  S := MyString.Strings[0];
  If Length(S) > 5 then
    begin
      System.Delete(S,1,5);
      If (S[Length(S)] <> '\') and (Lomitko) then S := S + '\';
      Result := S;
    end;
  MyString.Free;
end;

procedure SpustProgram(CestaKProgramu,Parametr : string);
var MyINI : TIniFile;
    CestaStr : string;
    CestaChar : array [0..240] of char;
begin
  CestaStr := CestaKProgramu;

  StrPCopy(CestaChar,CestaStr);

  WinExec(CestaChar,cmdShow);
end;

function OdstranMezery(S : String):String;
begin
  while S[1] = ' ' do begin
    Delete(S, 1, 1);
    if Length(S) = 0 then
      Break;
  end;
  if Length(S) > 0 then
    while S[Length(S)] = ' ' do begin
      Delete(S,1,Length(S));
      if Length(S) = 0 then
        Break;
    end;
  Result := S;
end;

end.
