unit e_GPS;

interface
uses
   GPSToolsXP_TLB,
   Elements;

type

TGPSTool = class(TOwnerEntity)
private
   FLicense: ILicense;
   FNMEAParser: TNMEAPArser;
   FParser: INmeaParser;
public
   Constructor Create; override;
   Destructor  Destroy; override;
   property License: ILicense read FLicense write FLicense;
   property NMEAParser: TNmeaParser read FNMEAParser write FNMEAParser;
   property Parser: INmeaParser read FParser write FParser;
   procedure _OnComStatus(Sender: TObject; var objComStatus: OleVariant);
   procedure _OnSatellites(Sender: TObject; var objSatellites: OleVariant);
end;

implementation
uses
   ComObj,
   Dialogs,
   Forms,
   SysUtils,
   e_Class,
   m_Orbis;

{ TGPSTool }

constructor TGPSTool.Create;
begin
   try
   License := CreateOLEObject( 'GpsToolsXP.License' ) as ILicense ;
   //result := True;
   except
   //result := False;
   ShowMessage('Component is not installed !');
   exit;
   end;
   //License := v as ILicense;
   License.LicenseKey := 'BF20A0E6287C1C2C4C30';
   //--------------------------------
   NMEAParser := TNmeaParser.Create(Application);
   Parser := CreateOLEObject( 'GpsToolsXP.NMEAParser' ) as INMEAParser ;
   NMEAParser.ConnectTo(Parser);
   NMEAParser.OnComStatus := _OnComStatus;
   NMEAParser.OnSatellites := _OnSatellites;
   Parser.ComPort := 0;
   Parser.PortEnabled := true;
end;

destructor TGPSTool.Destroy;
begin
   Parser.PortEnabled := false;
   NMEAParser.Disconnect;
   NMEAParser.Free;
   License := nil;
   //Parser := nil;
   inherited;
end;

procedure TGPSTool._OnComStatus(Sender: TObject;  var objComStatus: OleVariant);
var
   baud_rate: integer;
   port: integer;
begin
   if(objComStatus.ValidNmea) then
   begin
      baud_rate := objComStatus.BaudRate;
      port := objComStatus.ComPort;
      if(port = -1) then
         PostTextMsg(tm_AddToProtocol, 'GpsGate Direct!', nil)
      else
         PostTextMsg(tm_AddToProtocol, 'GPS OK! COM' + IntToStr(port) + ', ' + IntToStr(baud_rate), nil);
  end
  else
  begin
    PostTextMsg(tm_AddToProtocol, 'Trying to connect...',nil);
  end;
  Application.ProcessMessages;
end;

procedure TGPSTool._OnSatellites(Sender: TObject;  var objSatellites: OleVariant);
var
  inx: integer;
  sat_count: integer;
  sat: OleVariant;
begin
  sat_count := objSatellites.Count;

  for inx := 1 to sat_count- 1 do
  begin
    sat := objSatellites.Item(inx);
    PostTextMsg(tm_AddToProtocol,'PRN: ' + IntToStr(sat.ID) + ' SNR: ' + IntToStr(sat.SNR),nil);
  end;
  Application.ProcessMessages;
end;

INITIALIZATION
   RegisterOrbisClass(TGPSTool);
END.
